#ifndef I_ENGINECORE_H_INCLUDED
#define I_ENGINECORE_H_INCLUDED

#include <Windows.h>
#include "ICommonPlugin.h"
#include "IFileSystem.h"
#include "IEventSystem.h"
#include "ILua.h"
#include "ILog.h"

#pragma pack(1)

typedef struct
{	
	unsigned int Width;
	unsigned int Height;
	unsigned int BitDepth;
	bool VSync;
	bool FullScreen;
	unsigned int sample_count;
} TEngWindow;

typedef struct
{	
	char os_name[128];
	char cpu_name[128];
	int	 cpu_count;
	int  cpu_freq;
 __int64 ram_total;
 __int64 ram_free;
	char videocard_name[128];
	int	 videocard_count;
	int  videocard_ram;
} TSysInfo;

#pragma pack()

enum E_LOG_TYPE
{
	ELT_TEXT = 0,
	ELT_HTML,
	ELT_NONE
};

enum E_REG_PROCEDURE_TYPE
{
	PROC_PROCESS = 0,
	PROC_RENDER,
	PROC_INIT,
	PROC_FREE
};


// {E7A02BF6-3712-4c9d-8D10-4A4DB66E586B}
static const GUID IID_IENGCORE = 
{ 0xe7a02bf6, 0x3712, 0x4c9d, { 0x8d, 0x10, 0x4a, 0x4d, 0xb6, 0x6e, 0x58, 0x6b } };

//! Main Engine interface
interface __declspec(uuid("{E7A02BF6-3712-4c9d-8D10-4A4DB66E586B}")) IEngineCore: public IUnknown
{
	virtual HRESULT CALLBACK InitializeEngine( IN HWND Handle, IN const char* ApplicationName, IN TEngWindow WindowParam, IN int ProcessInterval, IN const char* modules, IN E_LOG_TYPE log_type = ELT_NONE ) = 0;
	virtual HRESULT CALLBACK GetSysInfo(OUT TSysInfo &info) = 0;
	virtual HRESULT CALLBACK GetCurrentScrSize(OUT int &width, OUT int &height) = 0;
	virtual HRESULT CALLBACK StartEngine() = 0;
	virtual HRESULT CALLBACK QuitEngine() = 0;
	virtual HRESULT CALLBACK AllowAutoPause (IN bool Enable) = 0;
	virtual HRESULT CALLBACK GetFPS(OUT int &FPS) = 0;
	virtual HRESULT CALLBACK GetLog(OUT ILog **Log) = 0;
	virtual HRESULT CALLBACK AddProcedure(IN E_REG_PROCEDURE_TYPE P_type, IN void (*Proc)()) = 0;
	virtual HRESULT CALLBACK RemoveProcedure(IN E_REG_PROCEDURE_TYPE P_type, IN void (*Proc)()) = 0;
	virtual HRESULT CALLBACK Lua_DoString(const char* text) = 0;
	virtual HRESULT CALLBACK Lua_ReInit() = 0;
	virtual HRESULT CALLBACK Lua_Get(OUT ILua **lua) = 0;
	virtual HRESULT CALLBACK Con_Visible(bool IsVisible) = 0;
	virtual HRESULT CALLBACK Con_Write(const char* text, bool toprevline = false) = 0;
	virtual HRESULT CALLBACK Con_Exec(const char* command) = 0;
	virtual HRESULT CALLBACK Con_RegComProc( const char* name, const char* help, void (*proc)(const char* param) ) = 0; 
	virtual HRESULT CALLBACK Con_RegComValue( const char* name, const char* help, int* value, int minvalue, int maxvalue, void (*proc)(const char* param) = NULL ) = 0;
	virtual HRESULT CALLBACK PluginCount(OUT ULONG *count, E_PluginType type) = 0;
	virtual HRESULT CALLBACK PluginInformation(OUT PluginInfo *info, E_PluginType type, ULONG index = 0) = 0;
	virtual HRESULT CALLBACK PluginByType(OUT ICommonPlugin **iplugin, E_PluginType type, ULONG index = 0) = 0;
	virtual HRESULT CALLBACK PluginByName(OUT ICommonPlugin **iplugin, const char* name) = 0;
	virtual HRESULT CALLBACK FileSystem(OUT IFileSystem **imain_fs) = 0;
	virtual HRESULT CALLBACK AddSubEventSystem(IN IEventSystem *EventSystem) = 0;
};

#endif//I_ENGINECORE_H_INCLUDED


