#ifndef I_INPUT
#define I_INPUT

#include "ICommonPlugin.h"

// {CC8B2654-76CA-41e6-BC7E-F2F392B77CDA}
static const GUID IID_IInput = 
{ 0xcc8b2654, 0x76ca, 0x41e6, { 0xbc, 0x7e, 0xf2, 0xf3, 0x92, 0xb7, 0x7c, 0xda } };

#define KEY_ESCAPE          0x01	// Escape
#define KEY_TAB             0x0F	// Tab
#define KEY_GRAVE           0x29    // accent grave "~"
#define KEY_CAPSLOCK        0x3A    // Caps Lock
#define KEY_BACKSPACE       0x0E    // Backspace
#define KEY_RETURN          0x1C    // Enter
#define KEY_SPACE           0x39	// Space
#define KEY_SLASH           0x35    // "/" 
#define KEY_BACKSLASH       0x2B	// "\"

#define KEY_SYSRQ           0xB7    // PtrScr (SysRq)
#define KEY_SCROLL          0x46    // Scroll Lock
#define KEY_PAUSE           0xC5    // Pause

#define KEY_INSERT          0xD2    // Insert
#define KEY_DELETE          0xD3    // Delete
#define KEY_HOME            0xC7    // Home
#define KEY_END             0xCF    // End
#define KEY_PGUP            0xC9    // PgUp
#define KEY_PGDN            0xD1    // PgDn

#define KEY_LSHIFT          0x2A	// Left Shift
#define KEY_RSHIFT          0x36	// Right Shift
#define KEY_LALT			0x38    // Left Alt
#define KEY_RALT            0xB8    // Right Alt
#define KEY_LWIN            0xDB    // Left Windows key 
#define KEY_RWIN            0xDC    // Right Windows key 
#define KEY_LCONTROL        0x1D	// Left Control
#define KEY_RCONTROL        0x9D	// Right Control

#define KEY_UP              0xC8    // UpArrow
#define KEY_RIGHT           0xCD    // RightArrow
#define KEY_LEFT            0xCB    // LeftArrow
#define KEY_DOWN            0xD0    // DownArrow

#define KEY_1               0x02
#define KEY_2               0x03
#define KEY_3               0x04
#define KEY_4               0x05
#define KEY_5               0x06
#define KEY_6               0x07
#define KEY_7               0x08
#define KEY_8               0x09
#define KEY_9               0x0A
#define KEY_0               0x0B

#define KEY_F1              0x3B
#define KEY_F2              0x3C
#define KEY_F3              0x3D
#define KEY_F4              0x3E
#define KEY_F5              0x3F
#define KEY_F6              0x40
#define KEY_F7              0x41
#define KEY_F8              0x42
#define KEY_F9              0x43
#define KEY_F10             0x44
#define KEY_F11             0x57
#define KEY_F12             0x58

#define KEY_Q               0x10
#define KEY_W               0x11
#define KEY_E               0x12
#define KEY_R               0x13
#define KEY_T               0x14
#define KEY_Y               0x15
#define KEY_U               0x16
#define KEY_I               0x17
#define KEY_O               0x18
#define KEY_P               0x19
#define KEY_A               0x1E
#define KEY_S               0x1F
#define KEY_D               0x20
#define KEY_F               0x21
#define KEY_G               0x22
#define KEY_H               0x23
#define KEY_J               0x24
#define KEY_K               0x25
#define KEY_L               0x26
#define KEY_Z               0x2C
#define KEY_X               0x2D
#define KEY_C               0x2E
#define KEY_V               0x2F
#define KEY_B               0x30
#define KEY_N               0x31
#define KEY_M               0x32

#define KEY_MINUS           0x0C    // "-"
#define KEY_PLUS            0x0D	// "+"
#define KEY_LBRACKET        0x1A    // "["
#define KEY_RBRACKET        0x1B    // "]" 

#define KEY_SEMICOLON       0x27    // ";"
#define KEY_APOSTROPHE      0x28    // '"' 

#define KEY_COMMA           0x33	// ","
#define KEY_PERIOD          0x34    // "."

#define KEY_NUMPAD0         0x52
#define KEY_NUMPAD1         0x4F
#define KEY_NUMPAD2         0x50
#define KEY_NUMPAD3         0x51
#define KEY_NUMPAD4         0x4B
#define KEY_NUMPAD5         0x4C
#define KEY_NUMPAD6         0x4D
#define KEY_NUMPAD7         0x47
#define KEY_NUMPAD8         0x48
#define KEY_NUMPAD9         0x49
#define KEY_NUMPADPERIOD    0x53    // "." on numpad
#define KEY_NUMPADENTER     0x9C    // Enter on numpad
#define KEY_NUMPADSTAR      0x37    // "*" on numpad
#define KEY_NUMPADPLUS      0x4E    // "+" on numpad
#define KEY_NUMPADMINUS     0x4A    // "-" on numpad
#define KEY_NUMPADSLASH     0xB5    // "/" on numpad
#define KEY_NUMLOCK         0x45	// Num Lock on numpad

typedef struct 
{
	long delta_x;
	long delta_y; 
	int world_x;	// screen position x 
	int world_y;	// screen position y 
	long delta_whell;
	bool left_button;
	bool right_button;
	bool middle_button;
} sMouseStates;

//! For OnMouseDown, OnMouseUp, OnMousePress events
enum DPE_MOUSE_BUTTONS
{
	DMB_NONE = 0,
	DMB_LEFT,
	DMB_RIGHT,
	DMB_MIDDLE
};

typedef struct 
{
	bool capslock;
	bool shiftL;
	bool shiftR;
	bool ctrlL;
	bool ctrlR;
	bool altL;
	bool altR;
} TKeyBoardStates;

typedef struct 
{
	int XAxes;		    // x-axis position
	int YAxes;		    // y-axis position
	int ZAxes;		    // z-axis position
	int XRotate;		// x-axis rotation
	int YRotate;		// y-axis rotation
	int ZRotate;		// z-axis rotation
	int VX;			    // x-axis velocity
	int VY;			    // y-axis velocity
	int VZ;			    // z-axis velocity
	int VXRotate;		// x-axis angular velocity
	int VYRotate;		// y-axis angular velocity
	int VZRotate;		// z-axis angular velocity
	int AX;			    // x-axis acceleration
	int AY;			    // y-axis acceleration
	int AZ;			    // z-axis acceleration
	int AXRotate;		// x-axis angular acceleration
	int AYRotate;		// y-axis angular acceleration
	int AZRotate;		// z-axis angular acceleration
	int FX;			    // x-axis force
	int FY;			    // y-axis force
	int FZ;			    // z-axis force
	int FXRotate;		// x-axis torque
	int FYRotate;		// y-axis torque
	int FZRotate;		// z-axis torque
	int Slider0;		// extra axes positions
	int Slider1;		//
	int VSlider0;		// extra axes velocities
	int VSlider1;		//
	int ASlider0;		// extra axes accelerations
	int ASlider1;		//
	int FSlider0;		// extra axes forces
	int FSlider1;		//
	int POV0;			// POV directions
	int POV1;			//
	int POV2;			//
	int POV3;			//
} sJoystickStates;

interface __declspec(uuid("{CC8B2654-76CA-41e6-BC7E-F2F392B77CDA}")) IInput: public ICommonPlugin
{
	virtual HRESULT CALLBACK CreateDevice( IN bool foreground, IN bool exclusive, IN bool winkey) = 0;
	virtual HRESULT CALLBACK ReleaseDevice() = 0;
	virtual HRESULT CALLBACK Update() = 0;
	virtual HRESULT CALLBACK GetMouseStates( OUT sMouseStates &ms ) = 0;
	virtual HRESULT CALLBACK GetInitialized( OUT bool &init ) = 0;
	virtual HRESULT CALLBACK GetKey( IN UINT key, OUT bool &pressed ) = 0;
	virtual HRESULT CALLBACK GetKeyName( IN UINT key, OUT char* &name ) = 0;
	virtual HRESULT CALLBACK GetJoystickStates( OUT sJoystickStates &js ) = 0;
	virtual HRESULT CALLBACK GetJButtonPressed( IN UINT num, OUT bool &pressed ) = 0; // 128 buttons [num 0..127]	
};

#endif