#ifndef I_CORE_RENDERER_H_INCLUDED
#define I_CORE_RENDERER_H_INCLUDED

#include <windows.h>
#include "ICommonPlugin.h"
#include "IEngineCore.h"
#include "enums.h"

#ifndef _MSC_VER
#define _In_
#define _In_opt_
#define _In_count_(size)
#define _Out_
#endif

// {DCA3DDE7-9C1A-4b3d-9C07-514D7FF15A0B}
static const GUID IID_ICoreRenderer = 
{ 0xdca3dde7, 0x9c1a, 0x4b3d, { 0x9c, 0x7, 0x51, 0x4d, 0x7f, 0xf1, 0x5a, 0xb } };

class __declspec(uuid("{DCA3DDE7-9C1A-4b3d-9C07-514D7FF15A0B}")) ICoreRenderer: public ICommonPlugin
{
public:
	struct DISP_MODE
	{
		unsigned int Width;
		unsigned int Height;
		unsigned int RefreshRate;
		RendererEnums::FRAME_BUFF_FORMAT Format;
	};
	struct ADAPTER_INFO
	{
		HMONITOR monitor;
		DISP_MODE cur_mode;
	};
	struct _2D_VERTEX
	{
		int x, y;
		float r, g, b, a;
	};
	struct _2D_TEX_VERTEX
	{
		int x, y;
		float u, v;
		float r, g, b, a;
	};
	struct _3D_VERTEX
	{
		float x, y, z;
		float r, g, b;
	};

	class __declspec(uuid("{846C553E-EE19-48db-9AB5-A2624BBB01B9}")) IAdaptersInfo: virtual public IUnknown
	{
	public:
		virtual HRESULT STDMETHODCALLTYPE GetAdapterCount(size_t &adapter_count) const throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE GetAdapterInfo(size_t adapter_ixd, ADAPTER_INFO &adapter_info) const throw() = 0;
	};

	class __declspec(uuid("{F2421ACD-B48B-4fbb-A9D4-572B7E2779CB}")) IAdapterDisplayModes: virtual public IUnknown
	{
	public:
		virtual HRESULT STDMETHODCALLTYPE GetModeCount(size_t &mode_count) const throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE GetMode(size_t mode_idx, DISP_MODE &mode) const throw() = 0;
	};

	virtual HRESULT STDMETHODCALLTYPE CheckDevice(unsigned int adapter, RendererEnums::FRAME_BUFF_FORMAT frame_buffer_format, bool windowed, unsigned int sample_count, unsigned int &quality_levels) const throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE CheckDeviceCaps(unsigned int adapter) const throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE CreateDevice(HWND hWnd, unsigned int adapter, const DISP_MODE &mode, bool vertical_sync = false, unsigned int buffer_count = 2, unsigned int sample_count = 1, unsigned int quality = 0, bool windowed = true) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE DestroyDevice() throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE Init(HWND hWnd, unsigned int adapter, const TEngWindow &WindowParam) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE GetSupportedTextureFileFormats(OUT const char *&formats) const throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE GetTextureFormatSupported(unsigned int adapter, RendererEnums::TEXTURE_FORMAT format) const throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE GetAdaptersInfo(IAdaptersInfo *&pAdaptersInfo) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE GetAdapterDisplayModes(unsigned int adapter_idx, IAdapterDisplayModes *&pAdapterDisplayModes) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE SetMode(const DISP_MODE &mode, bool vertical_sync = false) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE SetBufferCount(unsigned int buffer_count) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE SetMSAA(unsigned int sample_count, unsigned int quality) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE SetFullscreen(bool Fullscreen = true) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE DrawLine(const _2D_VERTEX (&vertices)[2]) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE DrawRect(const _2D_VERTEX (&vertices)[2]) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE DrawText(int x, int y, float r, float g, float b, float a, const char string[]) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE PresentFrame() throw() = 0;
};

#endif //I_CORE_RENDERER_H_INCLUDED