//Warning!
//Don't use this interface directly in your application if engine wasn't build by the same compiler as your application.
#ifndef I_RENDERER_H_INCLUDED
#define I_RENDERER_H_INCLUDED

#include <windows.h>
#include "ICoreRenderer.h"
#include "IFileSystem.h"
#include "IFTexture.h"
#include "IFBitmapFont.h"
#include "IFVectorImg.h"

class __declspec(uuid("{9DB42960-C13E-4abe-AEB0-4848D71440A6}")) IRenderer: public ICoreRenderer
{
public:
	// mesh interface
	class __declspec(uuid("{C18C3D08-C059-4c55-8001-586A634BC5DE}")) IMesh: virtual public IUnknown
	{
	public:
		virtual HRESULT STDMETHODCALLTYPE GetAABB(float (&MIN)[3], float (&MAX)[3]) const throw() = 0;
	};

	// texture interfaces
	//	forward declarations
	struct TEXTURE_2D_DESC;
	struct TEXTURE_2D_DATA;
	class __declspec(uuid("{2B25B72A-76AB-45f5-8BCA-D91579F3F6F4}")) ITexture2D: virtual public IUnknown
	{
	public:
		enum MAP_TYPE
		{
			READ,
			WRITE,
			READ_WRITE
		};
		virtual HRESULT STDMETHODCALLTYPE GetDesc(TEXTURE_2D_DESC &desc) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE GetCreationDesc(TEXTURE_2D_DESC &desc) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE Map(unsigned long level, MAP_TYPE map_type, TEXTURE_2D_DATA &mapped_data) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE Unmap(unsigned long level) throw() = 0;
	};

	class __declspec(uuid("{6C208935-FF3A-4e03-B23A-21EADA27BAAD}")) IMaterialTexture: virtual public IUnknown
	{
	public:
		virtual HRESULT STDMETHODCALLTYPE SetTexture(ITexture2D *pTexture) throw() = 0;
	};

	// material interfaces
	class __declspec(uuid("{F225620F-247A-436f-9F4C-8D8298A98BE4}")) IStandardMaterial: virtual public IUnknown
	{
	public:
		virtual HRESULT STDMETHODCALLTYPE SetLightingModel(RendererEnums::IStandardMaterial::LIGHTING_MODEL lighting_model) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE SetNormalTechnique(RendererEnums::IStandardMaterial::NORMAL_TECHNIQUE normal_technique) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE SetParallaxTechnique(RendererEnums::IStandardMaterial::PARALLAX_TECHNIQUE parallax_technique) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE SetMaterialAmbient(const float (&material_ambient)[4]) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE SetMaterialDiffuse(const float (&material_diffuse)[3]) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE SetMaterialSpecular(const float (&material_specular)[3]) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE SetMaterialShininess(float material_shininess) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE SetAmbientDiffuse(IUnknown *pAmbientDiffuse) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE SetSpecular(IUnknown *pSpecular) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE SetNormal(IUnknown *pNormal) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE SetHeight(IUnknown *pHeight) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE SetHeightDeltaScale(float height_delta_scale) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE SetHeightScale(float height_scale) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE SetAnisotropy(bool enable) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE SetAlphaTest(bool enable) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE SetShadowReceiving(bool receive_shadows) throw() = 0;
	};

	// transform interfaces
	class __declspec(uuid("{70BB5CE9-B4B6-49bf-B7CA-05D4F74C98EB}")) IAffineTransform: virtual public IUnknown
	{
	public:
		virtual HRESULT STDMETHODCALLTYPE SetTransform(const float (&matrix)[4][3]) throw() = 0;
	};

	class __declspec(uuid("{23A09E93-3FBA-4613-BBDF-5CF5C02689AA}")) ISetTransform: virtual public IUnknown
	{
	public:
		virtual HRESULT STDMETHODCALLTYPE SetTransform(IAffineTransform *pTransform) throw() = 0;
	};

	// instance interfaces
	class __declspec(uuid("{EE183F68-B0F7-49dd-89DA-85EDCFE48E8E}")) IInstanceVisible: virtual public IUnknown
	{
	public:
		virtual HRESULT STDMETHODCALLTYPE SetVisibleStatus(bool visible) throw() = 0;
	};

	class ICullTreeInstance;	// forward decl
	class __declspec(uuid("{F6A888D0-B786-4b26-AED4-BB9078983F82}")) IInstanceCulling: virtual public IUnknown
	{
	public:
		virtual HRESULT STDMETHODCALLTYPE AddToCullTree(ICullTreeInstance *pInstance, IUnknown *&pHandle) throw() = 0;
	};

	class __declspec(uuid("{C9357E7B-8F89-4f44-8894-4234C881E860}")) IInstanceFill: virtual public IUnknown
	{
	public:
		virtual HRESULT STDMETHODCALLTYPE SetFillMode(bool wireframe) throw() = 0;
	};

	class __declspec(uuid("{FFB91518-7225-4d21-937C-5CA98573C2BE}")) IInstanceFaceCull: virtual public IUnknown
	{
	public:
		virtual HRESULT STDMETHODCALLTYPE SetCullMode(RendererEnums::IInstance::CULL_MODE cull_mode) throw() = 0;
	};

	class __declspec(uuid("{E274DD5F-5D61-4cb7-9069-EDCD114BFD6E}")) IInstanceLayer: virtual public IUnknown
	{
	public:
		virtual HRESULT STDMETHODCALLTYPE SetLayer(signed short layer) throw() = 0;
	};

	class __declspec(uuid("{AFA8F668-CDD4-4913-A8F3-D01AB12C9AFE}")) IInstanceLODRange: virtual public IUnknown
	{
	public:
		virtual HRESULT STDMETHODCALLTYPE SetRange(float appear, float disappear) throw() = 0;
	};

	class __declspec(uuid("{F00ABB8C-892D-44a9-BB81-A2494B9E8E78}")) IInstanceAABB: virtual public IUnknown
	{
	public:
		virtual HRESULT STDMETHODCALLTYPE GetAABB(float (&MIN)[3], float (&MAX)[3]) const throw() = 0;
	};

	class __declspec(uuid("{AED1EA2E-FC2F-4bf1-8913-7C3EA48BD168}")) IInstance: virtual public ISetTransform, virtual public IInstanceCulling, virtual public IInstanceLayer, virtual public IInstanceLODRange
	{
	public:
	};

	class __declspec(uuid("{05A8B46C-5C82-457a-B800-E4C258CFB193}")) ISimpleSkyInstance: virtual public ISetTransform, virtual public IInstanceCulling, virtual public IInstanceAABB
	{
	public:
	};

	class __declspec(uuid("{4D573696-661B-4b01-83D4-2D75A668CD23}")) IVectorImageInstance2D: virtual public IInstanceLayer
	{
	public:
		virtual HRESULT STDMETHODCALLTYPE SetPosition(float x, float y) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE SetSize(float size) throw() = 0;
	};

	class __declspec(uuid("{5B7D59C2-0894-498d-ABD2-83B382C7323E}")) IVectorImageInstance3D: virtual public ISetTransform, virtual public IInstanceCulling, virtual public IInstanceLayer, virtual public IInstanceLODRange
	{
	public:
	};

	// light source interfaces
	class __declspec(uuid("{210DF685-26E3-45d1-A802-28C19A6AB11E}")) ILightBase: virtual public IUnknown
	{
	public:
		virtual HRESULT STDMETHODCALLTYPE SetDiffuse(float (&light_diffuse)[3]) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE SetSpecular(float (&light_specular)[3]) throw() = 0;
	};

	class __declspec(uuid("{85E19709-4F1C-4b24-BE96-9824F4F7447D}")) ILightDirection: virtual public IUnknown
	{
	public:
		virtual HRESULT STDMETHODCALLTYPE SetDirection(float (&direction)[3]) throw() = 0;
	};

	class __declspec(uuid("{208B811C-5073-4b8d-9903-FFFC9A8CB04B}")) ILightPosition: virtual public IUnknown
	{
	public:
		virtual HRESULT STDMETHODCALLTYPE SetPosition(float (&position)[3]) throw() = 0;
	};

	class __declspec(uuid("{406FD555-7A2E-45c6-BCDB-64F5EBD8FB06}")) ILightRange: virtual public IUnknown
	{
	public:
		virtual HRESULT STDMETHODCALLTYPE SetRange(float range) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE SetConstantAttenuationFactor(float factor) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE SetLinearAttenuationFactor(float factor) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE SetQuadraticAttenuationFactor(float factor) throw() = 0;
	};

	class __declspec(uuid("{90F9972E-786D-441d-B3F5-6F7612B8FB9F}")) IDirectionalLight: virtual public ILightBase, virtual public ILightDirection
	{
	public:
	};

	class __declspec(uuid("{71A735B3-CADC-46cd-B95C-D116234B4CDB}")) IOmniLight: virtual public ILightBase, virtual public ILightPosition, virtual public ILightRange
	{
	public:
	};

	class __declspec(uuid("{43563E6B-E730-4e69-9ED7-8B1C2773D9A1}")) INodeBase: virtual public IInstanceVisible, virtual public IInstanceFill, virtual public IInstanceFaceCull
	{
	public:
	};

	class __declspec(uuid("{DCAF46AD-F1A8-44bf-A959-07FC40511973}")) IInstanceNode: virtual public IInstanceAABB, virtual public INodeBase
	{
	public:
		virtual HRESULT STDMETHODCALLTYPE SetMaterial(IUnknown *pMaterial) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE SetShadowCasting(bool cast_shadows) throw() = 0;
	};

	class __declspec(uuid("{6E051729-48A9-4b8d-9261-63B5B1398BB7}")) ISimpleSkyNode: virtual public INodeBase
	{
	public:
		virtual HRESULT STDMETHODCALLTYPE SetSkyTexture(ITexture2D *pSkyTexture) throw() = 0;
	};

	class __declspec(uuid("{3D581DB7-523B-405a-976A-E89FC5AFACE3}")) IVectorImage2DNode: virtual public IInstanceVisible
	{
	public:
	};

	class __declspec(uuid("{9C7D61AF-CC43-49cd-BDD3-548965E71233}")) IVectorImage3DNode: virtual public IInstanceVisible
	{
	public:
	};

	class __declspec(uuid("{C7C26724-4781-4c37-AA7A-6C6C25A33D36}")) ILightNode: virtual public IUnknown
	{
	public:
		virtual HRESULT STDMETHODCALLTYPE Enable(bool enable) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE SetShadowCasting(bool cast_shadows) throw() = 0;
	};

	// vector image interface
	class __declspec(uuid("{0FB1FD77-44AB-4977-BFB8-B9B6FE0637C3}")) IVectorImage: virtual public IUnknown
	{
	public:
		virtual HRESULT STDMETHODCALLTYPE GetAABB(float (&MIN)[2], float (&MAX)[2]) const throw() = 0;
	};

	// cull tree interfaces
	class INode;

	class __declspec(uuid("{ADBE29F5-DC2C-4ecb-B7B3-BC7AD8163345}")) IAddChild: virtual public IUnknown
	{
	public:
		virtual HRESULT STDMETHODCALLTYPE AddChild(INode *pChild, IUnknown *&pHandle) throw() = 0;
	};

	class __declspec(uuid("{77294F97-1F23-4578-AB61-B82B2AE0A9D9}")) ICullTreeInstance: virtual public ISetTransform
	{
	public:
		virtual HRESULT STDMETHODCALLTYPE SetCulling(bool frustum_culling, bool occlusion_culling) throw() = 0;
	};

	class __declspec(uuid("{ED1D9265-7092-4b4f-AA79-12AC9E61EE01}")) ICullTreeInstance2DAABB: virtual public ICullTreeInstance
	{
	public:
		virtual HRESULT STDMETHODCALLTYPE SetAABB(float (&MIN)[2], float (&MAX)[2]) throw() = 0;
	};

	class __declspec(uuid("{C51C7517-C863-4ac3-BC2E-F58C47B4CA1A}")) ICullTreeInstance3DAABB: virtual public ICullTreeInstance
	{
	public:
		virtual HRESULT STDMETHODCALLTYPE SetAABB(float (&MIN)[3], float (&MAX)[3]) throw() = 0;
	};

	class __declspec(uuid("{52C52068-8A65-49a8-B3BB-84CF8668CA89}")) INode: virtual public IAddChild
	{
		struct PARTICLE_BASE
		{
			PARTICLE_BASE(float x, float y, float z):
				x(x), y(y), z(z)
			{
			}
			union
			{
				struct
				{
					float x, y, z;
				};
				float coords[3];
			};
		};
	public:
		struct PARTICLE_SPRITE: PARTICLE_BASE
		{
			PARTICLE_SPRITE(float x = 0, float y = 0, float z = 0, float width = 0, float height = 0, ITexture2D *pSprite = NULL, float r = 1, float g = 1, float b = 1, float a = 1, bool additive = false, float angle = 0):
				PARTICLE_BASE(x, y, z),
				width(width),
				height(height),
				pSprite(pSprite),
				r(r), g(g), b(b), a(a),
				additive(additive),
				angle(angle)
			{
			}
			ITexture2D *pSprite;
			float r, g, b, a;
			bool additive;
			float width, height;
			float angle;
		};
		virtual HRESULT STDMETHODCALLTYPE SetInstance(ICullTreeInstance *pInstance) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE AddInstance(IInstance *pInstance, IInstanceNode *&pSceneInstance) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE AddInstance(IVectorImageInstance3D *pInstance, IVectorImage3DNode *&pInstanceIterHandle) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE SetParticles(size_t particle_count, const PARTICLE_SPRITE particles[]) throw() = 0;
	};

 	// scene interface
	class __declspec(uuid("{66A43D0C-3AEF-4145-A743-49FD4DC1EB51}")) IScene: virtual public IAddChild
	{
	public:
		virtual HRESULT STDMETHODCALLTYPE AddInstance(_In_ ISimpleSkyInstance *pSimpleSkyInstance, ISimpleSkyNode *&pInstanceIterHandle) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE AddInstance(IVectorImageInstance2D *pInstance, IVectorImage2DNode *&pInstanceIterHandle) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE AddLight(ILightBase *pLight, ILightNode *&pLightNode) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE CreateCullTreeInstance(_In_ IAffineTransform *pTransform, ICullTreeInstance2DAABB *&pCullTreeInstance) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE CreateCullTreeInstance(_In_ IAffineTransform *pTransform, ICullTreeInstance3DAABB *&pCullTreeInstance) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE CreateCullTreeNode(INode *&pCullTreeNode) throw() = 0;
	};

	// viewport interface
	class __declspec(uuid("{348DB6A0-87BC-443e-B74E-7B17DE1362DA}")) IViewPort: virtual public IUnknown
	{
	public:
		virtual HRESULT STDMETHODCALLTYPE SetScene(IScene *pScene) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE SetViewMatrix(const float (&matrix)[4][3]) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE SetLookAtViewMatrix(float eye_x, float eye_y, float eye_z, float at_x, float at_y, float at_z, float up_x, float up_y, float up_z) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE SetOrthoProjectionMatrix(float w, float h, float zn, float zf) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE SetOrthoProjectionMatrix(float l, float r, float b, float t, float zn, float zf) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE SetPerspectiveProjectionMatrix(float w, float h, float zn, float zf) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE SetPerspectiveProjectionMatrix(float l, float r, float b, float t, float zn, float zf) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE SetPerspectiveFovProjectionMatrix(float fovy, float aspect, float zn, float zf) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE GetViewMatrix(float (&matrix)[4][3]) const throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE GetProjectionMatrix(float (&matrix)[4][4]) const throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE Project(const float (&world_pos)[3], float (&screen_pos)[2]) const throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE Unproject(const float (&screen_pos)[2], float (&world_near_pos)[3], float (&world_far_pos)[3]) const throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE DrawLine(size_t vcount, _In_count_(vcount) const _3D_VERTEX vertices[], bool AA, bool closed) throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE Draw() = 0;
	};

	// font interface
	class __declspec(uuid("{1BB2F662-6ED3-43f9-9359-A008B8C8E515}")) IFont: virtual public IUnknown
	{
	public:
		virtual HRESULT STDMETHODCALLTYPE GetFontTexture(ITexture2D *&pFontTexture) const throw() = 0;
		virtual HRESULT STDMETHODCALLTYPE GetStringSize(const char *string, unsigned int &width, unsigned int &height) const throw() = 0;
	};

	// In/Out structures
	//	 2D
	struct _2D_TEX_ROT_CENTER
	{
		float cx, cy;
	};
	struct _2D_ELLIPSE_DESC
	{
		_2D_VERTEX center;
		float rx, ry;
		size_t vcount;
	};
private:
	struct _2D_DRAW_COLOR
	{
		inline _2D_DRAW_COLOR():
			r(1), g(1), b(1), a(1)
		{
		}
		float r, g, b, a;
	};
	struct _2D_DRAW_BASE_DESC
	{
		inline _2D_DRAW_BASE_DESC():
			X(0), Y(0),
			angle(0),
			pRotCenter(NULL)
		{
		}
		int X, Y;
		float angle;
		const _2D_TEX_ROT_CENTER *pRotCenter;
	};
	struct _2D_DRAW_TEXTURE_SPRITE_BASE_DESC: _2D_DRAW_BASE_DESC
	{
		inline _2D_DRAW_TEXTURE_SPRITE_BASE_DESC():
			pTexture(NULL),
			alpha_modulate(false),
			additive(false),
			width(0), height(0),
			flipX(false), flipY(false)
		{
		}
		ITexture2D *pTexture;
		bool alpha_modulate;
		bool additive;
		unsigned int width, height;
		bool flipX, flipY;
	};
	struct _2D_DRAW_TEXTURE_BASE_DESC: _2D_DRAW_TEXTURE_SPRITE_BASE_DESC
	{
		inline _2D_DRAW_TEXTURE_BASE_DESC():
			pRect(NULL)
		{
		}
		LPCRECT pRect;
	};
	struct _2D_DRAW_SPRITE_BASE_DESC: _2D_DRAW_TEXTURE_SPRITE_BASE_DESC
	{
		inline _2D_DRAW_SPRITE_BASE_DESC():
			frames_x_count(0), frames_y_count(0), cur_frame(0)
		{
		}
		unsigned int frames_x_count, frames_y_count, cur_frame;
	};
public:
	struct _2D_DRAW_TEXTURE_DESC: _2D_DRAW_TEXTURE_BASE_DESC
	{
		_2D_DRAW_COLOR color;
	};
	struct _2D_DRAW_TEXTURE_SEPARATE_DESC: _2D_DRAW_TEXTURE_BASE_DESC
	{
		_2D_DRAW_COLOR color[4];
	};
	struct _2D_DRAW_SPRITE_DESC: _2D_DRAW_SPRITE_BASE_DESC
	{
		_2D_DRAW_COLOR color;
	};
	struct _2D_DRAW_SPRITE_SEPARATE_DESC: _2D_DRAW_SPRITE_BASE_DESC
	{
		_2D_DRAW_COLOR color[4];
	};
	struct _2D_TEXT_STRING: _2D_DRAW_BASE_DESC
	{
		inline _2D_TEXT_STRING():
			pString(NULL),
			scaleX(1), scaleY(1),
			pFont(NULL)
		{
		}
		const char *pString;
		float scaleX, scaleY;
		const IFont *pFont;
		_2D_DRAW_COLOR color;
	};
	//	 textures descriptions
	struct TEXTURE_2D_DATA
	{
		inline TEXTURE_2D_DATA():
			pData(NULL),
			pitch(0)
		{
		}
		void *pData;
		unsigned int pitch;
	};
	struct TEXTURE_2D_DESC
	{
		inline TEXTURE_2D_DESC():
			width(0), height(0),
			level_count(0),
			format(RendererEnums::TEX_UNKNOWN)
		{
		}
		inline TEXTURE_2D_DESC(unsigned int width, unsigned int height, unsigned long level_count, RendererEnums::TEXTURE_FORMAT format):
			width(width), height(height),
			level_count(level_count),
			format(format)
		{
		}
		unsigned int width, height;
		unsigned long level_count;
		RendererEnums::TEXTURE_FORMAT format;
	};
	struct TEXTURE_2D_FROM_FILE_DESC: TEXTURE_2D_DESC
	{
		inline TEXTURE_2D_FROM_FILE_DESC() {}
		inline TEXTURE_2D_FROM_FILE_DESC(const TEXTURE_2D_DESC &texture_2d_desc): TEXTURE_2D_DESC(texture_2d_desc) {}
		// fill this later
	};
	struct LOAD_TEXTURES_2D_FROM_FILES_IN_MEMORY_DESC
	{
		const TEXTURE_2D_FROM_FILE_DESC *pTextureDesc;
		IFile *pFile;
	};
	//	vector image
	struct VECTOR_PRIMITIVE
	{
		enum PRIMITIVE_TYPE
		{
			POINT,
			LINE,
			BEZIER,
			POLYGON,
			TEXT
		} primitive_type;
		enum BILLBOARD_ALIGNMENT
		{
			PLANE,
			SCREEN,
			SCREEN_SCALED
		};
		struct STIPPLE
		{
			void *pattern;
			unsigned short width, height;
			float scale[2];
		};
		union
		{
			struct
			{
				float vertex[2];
				float color[3];
				ITexture2D *texture;
				float size;
				BILLBOARD_ALIGNMENT alignment;
			} point;
			struct
			{
				size_t vertex_count;
				const float (*vertices)[2];
				float color[3], back_color[3];
				float width;
				STIPPLE stipple;
				bool closed;
				bool fixed_width;
			} line;
			struct
			{
				float control_points[4][3];
				float color[3], back_color[3];
				float width;
				STIPPLE stipple;
			} bezier;
			struct
			{
				size_t vertex_count;
				const float (*vertices)[2];
				float color[3], back_color[3];
				STIPPLE stipple;
			} polygon;
			struct
			{
				float position[2];
				const char *str;
				IFont *font;
				float color[3];
				float scaleX, scaleY;
				float angle;
				BILLBOARD_ALIGNMENT alignment;
			} text;
		};
	};
	//	file transfer
	struct FILE_TRANSFER_IDX_DESC
	{
		size_t *idx_array, idx_count;
	};
	//	font
	struct FONT_DESC
	{
		ITexture2D *pFontTexture;
		IFBitmapFont::LETTER_FRAME *letters;
		size_t letter_count;
	};

	virtual HRESULT STDMETHODCALLTYPE SetRenderTarget(ITexture2D *p_render_target = NULL) throw() = 0;

	virtual HRESULT STDMETHODCALLTYPE CreateMesh(const unsigned long *const*IndexBufferArray, const unsigned int *VBLengthArray, unsigned int IBLength, const float (*VertexCoordsVB)[3], unsigned int VertexCoordsIB, IMesh *&pMesh) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE CreateMesh(const unsigned long *const*IndexBufferArray, const unsigned int *VBLengthArray, unsigned int IBLength, const float (*VertexCoordsVB)[3], unsigned int VertexCoordsIB, const float (*TexCoordsVB)[2], unsigned int TexCoordsIB, IMesh *&pMesh) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE CreateMesh(const unsigned long *const*IndexBufferArray, const unsigned int *VBLengthArray, unsigned int IBLength, const float (*VertexCoordsVB)[3], unsigned int VertexCoordsIB, const float (*NormalVB)[3], unsigned int NormalIB, const float (*TexCoordsVB)[2], unsigned int TexCoordsIB, IMesh *&pMesh) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE CreateMesh(const unsigned long *const*IndexBufferArray, const unsigned int *VBLengthArray, unsigned int IBLength, const float (*VertexCoordsVB)[3], unsigned int VertexCoordsIB, const float (*TangentVB)[3], unsigned int TangentIB, const float (*BitangentVB)[3], unsigned int BitangentIB, const float (*NormalVB)[3], unsigned int NormalIB, const float (*TexCoordsVB)[2], unsigned int TexCoordsIB, IMesh *&pMesh) throw() = 0;

	virtual HRESULT STDMETHODCALLTYPE CreateTextures2D(size_t texture_count, const TEXTURE_2D_DESC texture_desc_array[], const TEXTURE_2D_DATA init_data_array[], ITexture2D *texture_pointer_array[]) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE LoadTextures2D(size_t texture_count, const LOAD_TEXTURES_2D_FROM_FILES_IN_MEMORY_DESC LoadDescArray[], ITexture2D *texture_pointer_array[]) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE LoadTextures2D(IFileTransfer *pFileTransfer, const FILE_TRANSFER_IDX_DESC *p_idx_desc, ITexture2D *texture_pointer_array[]) throw() = 0;

	virtual HRESULT STDMETHODCALLTYPE CreateMaterialTexture(IMaterialTexture *&pMaterialTexture) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE CreateSingleColorTextureForMaterial(IMaterialTexture *&pSingleColorTextureForMaterial) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE CreateStandardMaterial(IStandardMaterial *&pMaterial) throw() = 0;

	virtual HRESULT STDMETHODCALLTYPE CreateVectorImage(size_t primitive_count, const VECTOR_PRIMITIVE primitives[], IVectorImage *&pVectorImage) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE LoadVectorImages(IFileTransfer *pFileTransfer, const FILE_TRANSFER_IDX_DESC *p_idx_desc, IVectorImage *vector_images[]) throw() = 0;

	virtual HRESULT STDMETHODCALLTYPE CreateLight(IDirectionalLight *&pLight) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE CreateLight(IOmniLight *&pLight) throw() = 0;

	virtual HRESULT STDMETHODCALLTYPE SetBackgroundColor(float r, float g, float b) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE CreateAffineTransform(IAffineTransform *&pTransform) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE CreateInstance(_In_ IMesh *pMesh, _In_ IAffineTransform *pTransform, IInstance *&pInstance) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE CreateSimpleSkyInstance(_In_ IMesh *pMesh, _In_ IAffineTransform *pTransform, ISimpleSkyInstance *&pInstance) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE CreateVectorImageInstance(_In_ IVectorImage *pVectorImage, IVectorImageInstance2D *&pInstance) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE CreateVectorImageInstance(_In_ IVectorImage *pVectorImage, _In_ IAffineTransform *pTransform, IVectorImageInstance3D *&pInstance) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE CreateScene(IScene *&pScene) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE CreateViewPort(IViewPort *&pViewPort) throw() = 0;

	virtual HRESULT STDMETHODCALLTYPE _2D_CreateFont(const FONT_DESC &font_desc, IFont *&pFont) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE _2D_LoadFonts(_In_ IFileTransfer *pFileTransfer, _In_opt_ const FILE_TRANSFER_IDX_DESC *p_idx_desc, _Out_ IFont *font_pointer_array[]) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE _2D_DrawPoint(const ICoreRenderer::_2D_VERTEX &vertex, LPCRECT pScissorRect = NULL) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE _2D_DrawLine(size_t vcount, _In_count_(vcount) const _2D_VERTEX vertices[], bool AA, bool closed, _In_opt_ LPCRECT pScissorRect = NULL) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE _2D_DrawPolygon(size_t vcount, _In_count_(vcount) const _2D_VERTEX vertices[], _In_opt_ LPCRECT pScissorRect = NULL) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE _2D_DrawPolygon(size_t vcount, _In_count_(vcount) const _2D_TEX_VERTEX vertices[], _In_ ITexture2D *pTexture, bool alpha_modulate, bool additive, _In_opt_ LPCRECT pScissorRect = NULL) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE _2D_DrawEllipsePoints(const _2D_ELLIPSE_DESC &ellipse_desc, _In_opt_ LPCRECT pScissorRect = NULL) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE _2D_DrawEllipseWireframe(const _2D_ELLIPSE_DESC &ellipse_desc, bool AA, _In_opt_ LPCRECT pScissorRect = NULL) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE _2D_DrawEllipseSolid(const _2D_ELLIPSE_DESC &ellipse_desc, _In_opt_ LPCRECT pScissorRect = NULL) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE _2D_DrawTexture(const _2D_DRAW_TEXTURE_DESC &desc, _In_opt_ LPCRECT pScissorRect = NULL) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE _2D_DrawTexture(const _2D_DRAW_TEXTURE_SEPARATE_DESC &desc, _In_opt_ LPCRECT pScissorRect = NULL) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE _2D_DrawSprite(const _2D_DRAW_SPRITE_DESC &sprite_desc, _In_opt_ LPCRECT pScissorRect = NULL) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE _2D_DrawSprite(const _2D_DRAW_SPRITE_SEPARATE_DESC &sprite_desc, _In_opt_ LPCRECT pScissorRect = NULL) throw() = 0;
	virtual HRESULT STDMETHODCALLTYPE _2D_DrawTextString(const _2D_TEXT_STRING &string_desc, _In_opt_ LPCRECT pScissorRect = NULL) throw() = 0;
};

#endif //I_RENDERER_H_INCLUDED