/*-----------------------------------------------------------------------
This source file is a part of DPEngine.

Created:		15.07.2008
LastModified:	12.10.2008

Revision history:
	* Created by Mike
	* Revised by ...

Disctription:
-----------------------------------------------------------------------*/
#include "DPETypes.h"

namespace DPE_Scenes
{
//-----------------------------------------------------------------------
// Point2D_Int
//-----------------------------------------------------------------------
Point2D_Int &Point2D_Int::operator += (const Point2D_Int &other)
{
	this->x+= other.x;
	this->y+= other.y;
	return *this;
}
Point2D_Int &Point2D_Int::operator -= (const Point2D_Int &other)
{
	this->x-= other.x;
	this->y-= other.y;
	return *this;
}
Point2D_Int operator + (const Point2D_Int &first, const Point2D_Int &second)
{
	return Point2D_Int(first)+= second;
}
Point2D_Int operator - (const Point2D_Int &first, const Point2D_Int &second)
{
	return Point2D_Int(first)-= second;
}
Point2D_Int &Point2D_Int::operator *= (int scalar)
{
	this->x*= scalar;
	this->y*= scalar;
	return *this;
}
Point2D_Int &Point2D_Int::operator /= (int scalar)
{
	this->x/= scalar;
	this->y/= scalar;
	return *this;
}
Point2D_Int operator * (const Point2D_Int &first, int scalar)
{
	return Point2D_Int(first)*= scalar;
}
Point2D_Int operator / (const Point2D_Int &first, int scalar)
{
	return Point2D_Int(first)/= scalar;
}
Point2D_Int operator * (int scalar, const Point2D_Int &first)
{
	return Point2D_Int(first)*= scalar;
}
Point2D_Int operator / (int scalar, const Point2D_Int &first)
{
	return Point2D_Int(first)/= scalar;
}

bool operator ==(const Point2D_Int &first, const Point2D_Int &second)
{
	return (first.x == second.x) && (first.y == second.y);
}
bool operator !=(const Point2D_Int &first, const Point2D_Int &second)
{
	return (first.x != second.x) || (first.y != second.y);
}
//-----------------------------------------------------------------------
// Vector2D_Float
//-----------------------------------------------------------------------
Vector2D_Float &Vector2D_Float::operator += (const Vector2D_Float &other)
{
	this->x+= other.x;
	this->y+= other.y;
	return *this;
}
Vector2D_Float &Vector2D_Float::operator -= (const Vector2D_Float &other)
{
	this->x-= other.x;
	this->y-= other.y;
	return *this;
}
Vector2D_Float operator + (const Vector2D_Float &first, const Vector2D_Float &second)
{
	return Vector2D_Float(first)+= second;
}
Vector2D_Float operator - (const Vector2D_Float &first, const Vector2D_Float &second)
{
	return Vector2D_Float(first)-= second;
}
bool operator ==(const Vector2D_Float &first, const Vector2D_Float &second)
{
	return (first.x == second.x) && (first.y == second.y);
}
bool operator !=(const Vector2D_Float &first, const Vector2D_Float &second)
{
	return (first.x != second.x) || (first.y != second.y);
}
//-----------------------------------------------------------------------
// Rect2D_Int
//-----------------------------------------------------------------------
Rect2D_Int &Rect2D_Int::operator = (const Point2D_Int &other)
{
	Dimension2D_UInt size(*this);
	this->left = other.x;
	this->top = other.y;
	this->right = this->left+size.width;
	this->bottom = this->top+size.height;
	return *this;
}
Rect2D_Int &Rect2D_Int::operator += (const Point2D_Int &other)
{
	this->left+= other.x;
	this->top+= other.y;
	this->right+= other.x;
	this->bottom+= other.y;	
	return *this;
}
Rect2D_Int &Rect2D_Int::operator -= (const Point2D_Int &other)
{
	this->left-= other.x;
	this->top-= other.y;
	this->right-= other.x;
	this->bottom-= other.y;
	return *this;
}
Rect2D_Int operator + (const Rect2D_Int &rect, const Point2D_Int &point)
{
	return Rect2D_Int(rect)+= point;
}
Rect2D_Int operator - (const Rect2D_Int &rect, const Point2D_Int &point)
{
	return Rect2D_Int(rect)-= point;
}
//-----------------------------------------------------------------------
// Dimension2D_UInt
//-----------------------------------------------------------------------
Dimension2D_UInt &Dimension2D_UInt::operator += (const Dimension2D_UInt &other)
{
	this->width+= other.width;
	this->height+= other.height;
	return *this;
}
Dimension2D_UInt &Dimension2D_UInt::operator -= (const Dimension2D_UInt &other)
{
	this->width-= other.width;
	this->height-= other.height;
	return *this;
}
Dimension2D_UInt operator + (const Dimension2D_UInt &first, const Dimension2D_UInt &second)
{
	return Dimension2D_UInt(first)+= second;
}
Dimension2D_UInt operator - (const Dimension2D_UInt &first, const Dimension2D_UInt &second)
{
	return Dimension2D_UInt(first)-= second;
}
//-----------------------------------------------------------------------

}
