/*-----------------------------------------------------------------------
This source file is a part of DPEngine.

Created:		15.07.2008
LastModified:	12.10.2008

Revision history:
	* Created by Mike
	* Revised by ...

Disctription:
-----------------------------------------------------------------------*/
#ifndef _IDPE_TYPES_H_INCLUDED_
#define _IDPE_TYPES_H_INCLUDED_

#include "IRenderer.h"

#include "math\Matrix.h"
#include "math\Vector.h"
#include "math\aabb.h"
#include "math\plane.h"

#include "DEFINE_FOR_BORLANDC"

namespace DPE_Scenes
{

//! forward
struct Dimension2D_UInt;
//-----------------------------------------------------------------------
#pragma pack(push, 1)
struct Point2D_Int
{
	int x;
	int y;

	inline Point2D_Int(){x=y=0;}
	inline Point2D_Int(const Point2D_Int &other)
	{
		this->x = other.x;
		this->y = other.y;
	}
	inline Point2D_Int(const Dimension2D_UInt &other);
	inline Point2D_Int(int x, int y)
	{
		this->x = x;
		this->y = y;
	}
	inline float distanceSqr(const Point2D_Int &point) const
	{
		return (float)((x-point.x)*(x-point.x) + (y-point.y)*(y-point.y));
	}
	operator IRenderer::_2D_TEX_ROT_CENTER ()
	{
		IRenderer::_2D_TEX_ROT_CENTER center = {(float)x,(float)y};
		return center;
	}

	Point2D_Int &operator += (const Point2D_Int &other);
	Point2D_Int &operator -= (const Point2D_Int &other);
	friend Point2D_Int operator + (const Point2D_Int &first, const Point2D_Int &second);
	friend Point2D_Int operator - (const Point2D_Int &first, const Point2D_Int &second);
	Point2D_Int &operator *= (int scalar);
	Point2D_Int &operator /= (int scalar);
	friend Point2D_Int operator * (const Point2D_Int &first, int scalar);
	friend Point2D_Int operator / (const Point2D_Int &first, int scalar);
	friend Point2D_Int operator * (int scalar, const Point2D_Int &first);
	friend Point2D_Int operator / (int scalar, const Point2D_Int &first);
	friend bool operator ==(const Point2D_Int &first, const Point2D_Int &second);
	friend bool operator !=(const Point2D_Int &first, const Point2D_Int &second);
};
struct Vector2D_Float
{
	float x;
	float y;

	inline Vector2D_Float(){x=y=0.f;}
	inline Vector2D_Float(float x, float y)
	{
		this->x = x;
		this->y = y;
	}
	inline Vector2D_Float(const Vector2D_Float &other)
	{
		this->x = other.x;
		this->y = other.y;
	}
	inline Vector2D_Float(const Point2D_Int &first, const Point2D_Int &second)
	{
		this->x = (float)(second.x - first.x);
		this->y = (float)(second.y - first.y);
	}
	inline float distanceSqr(const Vector2D_Float &point) const
	{
		return ((x-point.x)*(x-point.x) + (y-point.y)*(y-point.y));
	}

	Vector2D_Float &operator += (const Vector2D_Float &other);
	Vector2D_Float &operator -= (const Vector2D_Float &other);
	friend Vector2D_Float operator + (const Vector2D_Float &first, const Vector2D_Float &second);
	friend Vector2D_Float operator - (const Vector2D_Float &first, const Vector2D_Float &second);
	friend bool operator ==(const Vector2D_Float &first, const Vector2D_Float &second);
	friend bool operator !=(const Vector2D_Float &first, const Vector2D_Float &second);
};
//-----------------------------------------------------------------------
struct Rect2D_Int
{
	int left;
	int top;
	int right;
	int bottom;

	inline Rect2D_Int(){left=top=right=bottom=0;}
	inline Rect2D_Int(const Rect2D_Int &other)
	{
		this->left = other.left;
		this->top = other.top;
		this->right = other.right;
		this->bottom = other.bottom;
	}
	inline Rect2D_Int(int left, int top, int right, int bottom)
	{
		this->left = left;
		this->top = top;
		this->right = right;
		this->bottom = bottom;
	}
	inline Rect2D_Int(const Point2D_Int &left_top, const Point2D_Int &right_bottom)
	{
		this->left = left_top.x;
		this->top = left_top.y;
		this->right = right_bottom.x;
		this->bottom = right_bottom.y;
	}
	inline Rect2D_Int(const IRenderer::_2D_VERTEX (&rect)[4]):
	left(rect[0].x),top(rect[0].y),right(rect[2].x),bottom(rect[2].y)
	{
	}

	//! set position
	Rect2D_Int &operator = (const Point2D_Int &other);
	Rect2D_Int &operator += (const Point2D_Int &other);
	Rect2D_Int &operator -= (const Point2D_Int &other);
	friend Rect2D_Int operator + (const Rect2D_Int &rect, const Point2D_Int &point);
	friend Rect2D_Int operator - (const Rect2D_Int &rect, const Point2D_Int &point);
	// for render scissor
	operator RECT ()
	{
		RECT wRect = {left, top, right+1, bottom+1};
		return wRect;
	}

	inline bool pointInside(const Point2D_Int &point)
	{
		if (point.x >= this->left && point.x <= this->right &&
		point.y >= this->top && point.y <= this->bottom) return true;
		return false;
	}
	inline bool rectInside(const Rect2D_Int &rect)
	{
		return pointInside(Point2D_Int(rect.left, rect.top)) && pointInside(Point2D_Int(rect.right, rect.bottom));
	}
	inline bool rectIntersect(const Rect2D_Int &rect)
	{
		Rect2D_Int r(rect);
		return (
			pointInside(Point2D_Int(r.left, r.top)) || 
			pointInside(Point2D_Int(r.right, r.top)) ||
			pointInside(Point2D_Int(r.right, r.bottom)) ||
			pointInside(Point2D_Int(r.left, r.bottom)) ||

			r.pointInside(Point2D_Int(left, top)) || 
			r.pointInside(Point2D_Int(right, top)) ||
			r.pointInside(Point2D_Int(right, bottom)) ||
			r.pointInside(Point2D_Int(left, bottom)) );
	}	
	inline Rect2D_Int &addPoint(const Point2D_Int &point)
	{
		if (point.x<this->left) this->left = point.x;
		if (point.y<this->top) this->top = point.y;
		if (point.x>this->right) this->right = point.x;
		if (point.y>this->bottom) this->bottom = point.y;
		return *this;
	}
	inline Point2D_Int getPosition()
	{
		return Point2D_Int(this->left, this->top);
	}
	inline Dimension2D_UInt getDimension();
};
//----------------------------------------------------------------------
struct Circle2D
{
	Point2D_Int center;
	float radius;

	inline Circle2D(){radius = 0.f;}
	inline Circle2D(float radius)
	{
		this->radius = radius;
	}
	inline Circle2D(const Point2D_Int &center, float radius)
	{
		this->center = center;
		this->radius = radius;
	}

	inline bool pointInside(const Point2D_Int &point)
	{
		Point2D_Int p = point - center; 
		return (p.x*p.x + p.y*p.y) <= radius*radius;
	}
	inline bool rectInside(const Rect2D_Int &rect)
	{
		return ( pointInside(Point2D_Int(rect.left, rect.top)) &&
				 pointInside(Point2D_Int(rect.right, rect.top)) &&
				 pointInside(Point2D_Int(rect.right, rect.bottom)) &&
				 pointInside(Point2D_Int(rect.left, rect.bottom)) );
	}
	inline bool circleInside(const Circle2D &circle)
	{
		Point2D_Int dist = center - circle.center;
		int length = (int)sqrtf((float)(dist.x*dist.x + dist.y*dist.y));
		return radius - length - circle.radius >= 0;
	}

	inline bool lineIntersect(const Point2D_Int (&line)[2])
	{
		Point2D_Int dist(line[1] - line[0]);
		float Length = sqrtf((float)(dist.x * dist.x + dist.y * dist.y));

		float Xv = dist.x / Length;
		float Yv = dist.y / Length;
		float Xd = (float)(line[0].x - center.x);
		float Yd = (float)(line[0].y - center.y);

		float b = 2 * (Xd * Xv + Yd * Yv);
		float c = Xd * Xd + Yd * Yd - radius * radius;
		float c4 = c + c; c4 += c4;
		float D = b * b - c4;
		if (D < 0) return false;

		D = sqrtf(D);
		float l1 = ( - b + D) * 0.5f;
		float l2 = ( - b - D) * 0.5f;
		bool intersects1 = ((l1 >= 0.f) && (l1 <= Length));
		bool intersects2 = ((l2 >= 0.f) && (l2 <= Length));
		
		return intersects1 || intersects2;
	}

	inline bool rectIntersect(const Rect2D_Int &rect)
	{
		Rect2D_Int r(rect);
		return r.rectIntersect(
			Rect2D_Int(	(int)(center.x - radius),
						(int)(center.y - radius),
						(int)(center.x + radius),
						(int)(center.y + radius)) );
	}
	inline bool circleIntersect(const Circle2D &circle)
	{
		Point2D_Int dist = center - circle.center;
		int length = (int)sqrtf((float)(dist.x*dist.x + dist.y*dist.y));
		return length <= radius + circle.radius;
	}	
};
//----------------------------------------------------------------------
struct Dimension2D_UInt
{
	unsigned int width;
	unsigned int height;

	inline Dimension2D_UInt(){width = height = 0;}
	inline Dimension2D_UInt(const Dimension2D_UInt &other)
	{
		width = other.width;
		height = other.height;
	}
	inline Dimension2D_UInt(unsigned int w, unsigned int h)
	{
		width = w;
		height = h;
	}
	inline Dimension2D_UInt(const Point2D_Int &point)
	{
		width = point.x;
		height = point.y;
	}
	inline Dimension2D_UInt(const Rect2D_Int &rect)
	{
		width = rect.right-rect.left;
		height = rect.bottom-rect.top;
	}

	Dimension2D_UInt &operator += (const Dimension2D_UInt &other);
	Dimension2D_UInt &operator -= (const Dimension2D_UInt &other);
	friend Dimension2D_UInt operator + (const Dimension2D_UInt &first, const Dimension2D_UInt &second);
	friend Dimension2D_UInt operator - (const Dimension2D_UInt &first, const Dimension2D_UInt &second);	
};

inline Point2D_Int::Point2D_Int(const Dimension2D_UInt &other)
{
	this->x = other.width;
	this->y = other.height;
}

inline Dimension2D_UInt Rect2D_Int::getDimension()
{
	Dimension2D_UInt dim(*this);
	return dim;
}
//-----------------------------------------------------------------------
struct ColorRGBA : public Vector
{
	inline ColorRGBA(): Vector(){}
	inline ColorRGBA(float r, float g, float b, float a) : Vector(r, g, b, a) {}

	inline ColorRGBA(unsigned int color)
	{
		this->A = (float)((color >> 24) & 0xff)/UCHAR_MAX;
		this->R = (float)((color >> 16) & 0xff)/UCHAR_MAX;
		this->G = (float)((color >> 8) & 0xff)/UCHAR_MAX;
		this->B = (float)(color & 0xff)/UCHAR_MAX;
	}
	operator unsigned int () const
	{
		return 0xffffffff & 
			((unsigned int)(A*UCHAR_MAX) << 24) | 
			((unsigned int)(R*UCHAR_MAX) << 16) | 
			((unsigned int)(G*UCHAR_MAX) << 8) | 
			((unsigned int)(B*UCHAR_MAX) << 0);
	}	
};
//-----------------------------------------------------------------------
struct Point2D_RGBA
{
	Point2D_Int pos;
	ColorRGBA color;

	inline Point2D_RGBA(){}
	inline Point2D_RGBA(const Point2D_Int &pos)
	{
		this->pos = pos;		
	}
	inline Point2D_RGBA(const Point2D_Int &pos, const ColorRGBA &color)
	{
		this->pos = pos;
		this->color = color;
	}
	inline Point2D_RGBA(int x, int y, unsigned int color)
	{
		pos.x = x, pos.y = y;
		this->color = ColorRGBA(color);
	}

	operator IRenderer::_2D_VERTEX () const
	{
		IRenderer::_2D_VERTEX vertex =
		{
			pos.x, pos.y, 
			color.R, color.G, color.B, color.A
		};
		return vertex;
	}
};
//-----------------------------------------------------------------------
struct Vertex2D_RGBA : public Point2D_RGBA
{
	float u, v;
	inline Vertex2D_RGBA() : Point2D_RGBA() {}
	inline Vertex2D_RGBA(const Point2D_Int &pos, const ColorRGBA &color, float u, float v) :
		Point2D_RGBA(pos, color)
	{
		Vertex2D_RGBA::u = u;
		Vertex2D_RGBA::v = v;
	}
	inline Vertex2D_RGBA(int x, int y, unsigned int color, float u, float v) :
		Point2D_RGBA(x, y, color)
	{
		Vertex2D_RGBA::u = u;
		Vertex2D_RGBA::v = v;
	}
	operator IRenderer::_2D_TEX_VERTEX () const
	{
		IRenderer::_2D_TEX_VERTEX vertex =
		{
			pos.x, pos.y,
			u, v,
			color.R, color.G, color.B, color.A
		};
		return vertex;
	}
};
//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
struct ColorRGB
{
	float r, g, b;
	inline ColorRGB(){r = g = b = 1.0f;}	
	inline ColorRGB(float r, float g, float b)
	{
		this->r = r;
		this->g = g;
		this->b = b;
	}
	inline ColorRGB(unsigned int color)
	{
		this->r = (float)((color >> 16) & 0xff)/UCHAR_MAX;
		this->g = (float)((color >> 8) & 0xff)/UCHAR_MAX;
		this->b = (float)(color & 0xff)/UCHAR_MAX;
	}
};
#define DPE_BLACK ColorRGB(0x00000000)
#define DPE_WHITE ColorRGB(0x00FFFFFF)
//-----------------------------------------------------------------------
struct ColorRect
{
	ColorRGB corners[4];

	ColorRect(){}	
	inline ColorRect(const ColorRGB left_top, const ColorRGB right_top, const ColorRGB right_bottom, const ColorRGB left_bottom)
	{
		this->corners[0] = left_top;
		this->corners[1] = right_top;
		this->corners[2] = right_bottom;
		this->corners[3] = left_bottom;
	}
};
//-----------------------------------------------------------------------
#pragma pack(pop)
}

#include "UNDEF_FOR_BORLANDC"

#endif
