/*-----------------------------------------------------------------------
This source file is a part of DPEngine.

Created:		14.07.2008
LastModified:	14.11.2008

Revision history:
	* Created by Mike
	* Revised by ...

Disctription:
-----------------------------------------------------------------------*/
#ifndef _IDPE_GUI_H_INCLUDED_
#define _IDPE_GUI_H_INCLUDED_

#include "IDPEObject.h"
#include "2DInterfaces.h"
#include "DPETypes.h"
#include "IEventSystem.h"

namespace DPE_Scenes
{

	namespace GUI
	{
		enum GUI_RESIZE_TYPE
		{
			RT_LEFT		= 0x01,
			RT_TOP		= 0x02,
			RT_RIGHT	= 0x04,
			RT_BOTTOM	= 0x08,
			RT_CENTER	= 0x10
		};

		enum GUI_ALIGNMENT_TYPE
		{
			AT_LEFT		= 0x01,
			AT_TOP		= 0x02,
			AT_RIGHT	= 0x04,
			AT_BOTTOM	= 0x08,
			AT_NONE		= 0x10
		};

		#define SS_ENABLED 0
		#define SS_MOUSE_OVER 1
		#define SS_MOUSE_DOWN 2
		#define SS_MOUSE_UP 3
		#define SS_CHECKED 1
		#define SS_SELECTED 2
		#define SS_DISABLED 4

		//! forwards
		interface IDPEGUISkin;


		//-----------------------------------------------------------------------
		//IDPEGUIControl//
		//-----------------------------------------------------------------------

		// {CFDEAF80-51B4-43be-B0D6-29BD16551844}
		static const GUID IID_IDPEGUIControl = 
		{ 0xcfdeaf80, 0x51b4, 0x43be, { 0xb0, 0xd6, 0x29, 0xbd, 0x16, 0x55, 0x18, 0x44 } };

		interface __declspec(uuid("{CFDEAF80-51B4-43be-B0D6-29BD16551844}")) IDPEGUIControl: public IDPEObject
		{
			// SubControls			
			virtual HRESULT CALLBACK GetParent(OUT IDPEGUIControl *&pParent) const = 0;

			virtual HRESULT CALLBACK Include(IN IDPEGUIControl *pControl) = 0;
			virtual HRESULT CALLBACK Exclude(IDPEGUIControl *pControl) = 0;
			virtual HRESULT CALLBACK ExcludeAll() = 0;
			virtual HRESULT CALLBACK ExcludeFromParent( void ) = 0;
			virtual HRESULT CALLBACK GetControlsCount( OUT unsigned int &count )const  = 0;	

			// Z order
			virtual HRESULT CALLBACK MoveToBack(IDPEGUIControl *pControl) = 0;
			virtual HRESULT CALLBACK MoveToFront(IDPEGUIControl *pControl) = 0;

			// Events subscribers
			virtual HRESULT CALLBACK AddListerner(_2D::DPE_EVENT_TYPE type, void (*subscriber)(IEvent *)) = 0;
			virtual HRESULT CALLBACK RemoveListerner(_2D::DPE_EVENT_TYPE type, void (*subscriber)(IEvent *)) = 0;

			// View
			virtual HRESULT CALLBACK SetVisible(bool visible) = 0;
			virtual HRESULT CALLBACK GetVisible(OUT bool &visible) const = 0;
			virtual HRESULT CALLBACK SetEnable(bool enable) = 0;
			virtual HRESULT CALLBACK GetEnable(OUT bool &enable) const = 0;
			virtual HRESULT CALLBACK SetAlignment(unsigned int alignment) = 0;
			virtual HRESULT CALLBACK GetAlignment(unsigned int &alignment) const = 0;
			virtual HRESULT CALLBACK SetDragged(bool dragged) = 0;
			virtual HRESULT CALLBACK GetDragged(OUT bool &dragged) const = 0;

			// Skin
			virtual HRESULT CALLBACK SetSkin(const char* name) = 0;
			virtual HRESULT CALLBACK SetSkin(IN IDPEGUISkin *&pSkin) = 0;
			virtual HRESULT CALLBACK SetFont(const char* name) = 0;

			// Placement params
			virtual HRESULT CALLBACK SetWidth(unsigned int width) = 0;
			virtual HRESULT CALLBACK SetHeight(unsigned int height) = 0;
			
			virtual HRESULT CALLBACK SetPosition(const Point2D_Int &pos) = 0;
			virtual HRESULT CALLBACK SetPosition(int x, int y) = 0;
			virtual HRESULT CALLBACK GetClientRect(OUT Rect2D_Int &rect) const = 0;
			virtual HRESULT CALLBACK GetRelativePos(OUT Point2D_Int &pos) const = 0;
			virtual HRESULT CALLBACK GetAbsolutePos(OUT Point2D_Int &pos) const = 0;

			virtual HRESULT CALLBACK SetMaxDimention(const Dimension2D_UInt &max) = 0;
			virtual HRESULT CALLBACK SetMinDimention(const Dimension2D_UInt &min) = 0;	

			// Fill mode
			virtual HRESULT CALLBACK SetFillMode(bool fill) = 0;
			virtual HRESULT CALLBACK GetFillMode(OUT bool &fill) const = 0;
			virtual HRESULT CALLBACK SetSprite(unsigned short type, const char *texture_name, unsigned int countX, unsigned int countY) = 0;
			virtual HRESULT CALLBACK GetSprite(unsigned short type, OUT _2D::ISprite *&sprite) const = 0;
			virtual HRESULT CALLBACK SetAlpha(unsigned char alpha) = 0;
			virtual HRESULT CALLBACK GetAlpha(OUT unsigned char &alpha) const = 0;
			
			virtual HRESULT CALLBACK Draw() = 0;
			virtual HRESULT CALLBACK Update(float speed) = 0;

			// System events
			virtual HRESULT CALLBACK OnKeyDown(IEvent *Event) = 0;
			virtual HRESULT CALLBACK OnKeyUp(IEvent *Event) = 0;
			virtual HRESULT CALLBACK OnKeyPress(IEvent *Event) = 0;

			virtual HRESULT CALLBACK OnMouseMove(IEvent *Event) = 0;
			virtual HRESULT CALLBACK OnMouseDown(IEvent *Event) = 0;
			virtual HRESULT CALLBACK OnMouseUp(IEvent *Event) = 0;
			virtual HRESULT CALLBACK OnMousePress(IEvent *Event) = 0;
			virtual HRESULT CALLBACK OnMouseWheel(IEvent *Event) = 0;

			virtual HRESULT CALLBACK Free() = 0;
		};

		//-----------------------------------------------------------------------
		//IDPEGUIWindow//
		//-----------------------------------------------------------------------

		// {3FBB2A6E-D13B-4985-B57A-FF129452C598}
		static const GUID IID_IDPEGUIWindow = 
		{ 0x3fbb2a6e, 0xd13b, 0x4985, { 0xb5, 0x7a, 0xff, 0x12, 0x94, 0x52, 0xc5, 0x98 } };

		/*	sprites 1:
			0: Window form

			sprite frames:
			0: enabled
			1: disabled
		*/

		interface __declspec(uuid("{3FBB2A6E-D13B-4985-B57A-FF129452C598}")) IDPEGUIWindow: public IDPEGUIControl
		{
			virtual HRESULT CALLBACK SetCaption(const char *caption) = 0;
			virtual HRESULT CALLBACK SetCaptionColor(unsigned int color) = 0;
			virtual HRESULT CALLBACK SetCaptionScale(float x, float y) = 0;
			virtual HRESULT CALLBACK SetCaptionAngle(float angle, const Point2D_Int &center) = 0;
			virtual HRESULT CALLBACK SetCaptionMargin(int margin) = 0;
			virtual HRESULT CALLBACK SetCaptionAlignment(unsigned int alignment) = 0;

			virtual HRESULT CALLBACK SetTitleRect(const Rect2D_Int &rect) = 0;
			virtual HRESULT CALLBACK SetTitleVisible(bool visible) = 0;

			virtual HRESULT CALLBACK SetCloseBtnVisible(bool visible) = 0;

			virtual HRESULT CALLBACK SetCanSize(bool changeable) = 0;
			virtual HRESULT CALLBACK SetModal(bool modal) = 0;
		};

		//-----------------------------------------------------------------------
		//IDPEGUIButton//
		//-----------------------------------------------------------------------

		// {91F06D62-57EE-40e6-A190-AB5DD0263F8D}
		static const GUID IID_IDPEGUIButton = 
		{ 0x91f06d62, 0x57ee, 0x40e6, { 0xa1, 0x90, 0xab, 0x5d, 0xd0, 0x26, 0x3f, 0x8d } };

		/*	sprites 1:
			0: Button form

			sprite frames:
			0: enabled
			1: mouse_over
			2: mouse_down
			3: mouse_up
			4: disabled
		*/

		interface __declspec(uuid("{91F06D62-57EE-40e6-A190-AB5DD0263F8D}")) IDPEGUIButton: public IDPEGUIControl
		{
			virtual HRESULT CALLBACK SetText(const char *caption) = 0;
			virtual HRESULT CALLBACK SetTextColor(unsigned int color) = 0;
			virtual HRESULT CALLBACK SetTextScale(float x, float y) = 0;
			virtual HRESULT CALLBACK SetTextAngle(float angle, const Point2D_Int &center) = 0;
			virtual HRESULT CALLBACK SetTextMargin(int margin) = 0;
			virtual HRESULT CALLBACK SetTextAlignment(unsigned int alignment) = 0;
		};

		//-----------------------------------------------------------------------
		//IDPEGUIScrollBar//
		//-----------------------------------------------------------------------

		// {E6B168D1-4253-43cc-AAE1-8DFC1D8BD24F}
		static const GUID IID_IDPEGUIScrollBar = 
		{ 0xe6b168d1, 0x4253, 0x43cc, { 0xaa, 0xe1, 0x8d, 0xfc, 0x1d, 0x8b, 0xd2, 0x4f } };

		enum DPE_CONTROL_ORIENTATION
		{
			DCO_HORISONTAL = 0,
			DCO_VERTICAL
		};

		/*	sprites 4:
			0: Scroll bar form
			1: Up button
			2: Down button
			3: Track

			sprite frames:
			0: enabled
			1: disabled
		*/

		interface __declspec(uuid("{E6B168D1-4253-43cc-AAE1-8DFC1D8BD24F}")) IDPEGUIScrollBar: public IDPEGUIControl
		{
			virtual HRESULT CALLBACK GetTrackPosition(OUT int &pos) const = 0;
			virtual HRESULT CALLBACK SetTrackMaxMin(int max, int min) = 0;
			virtual HRESULT CALLBACK SetTrackStep(int step) = 0;

			virtual HRESULT CALLBACK GetUpBtn(OUT IDPEGUIButton *&btn) const = 0;
			virtual HRESULT CALLBACK GetDownBtn(OUT IDPEGUIButton *&btn) const = 0;
			virtual HRESULT CALLBACK GetTrack(OUT IDPEGUIButton *&btn) const = 0;
		};

		//-----------------------------------------------------------------------
		//IDPEGUIListBox//
		//-----------------------------------------------------------------------

		// {6448F36D-89AB-46fe-9171-CB059D2EDA25}
		static const GUID IID_IDPEGUIListBox = 
		{ 0x6448f36d, 0x89ab, 0x46fe, { 0x91, 0x71, 0xcb, 0x5, 0x9d, 0x2e, 0xda, 0x25 } };

		/*	sprites 4:
			0: List box form
			1: Item

			Horizontal
			2: Scroll bar form
			3: Up button
			4: Down button
			5: Track

			Vertical
			6: Scroll bar form
			7: Up button
			8: Down button
			9: Track

			sprite frames for list box form:
			0: enabled
			1: disabled

			sprite frames for item:
			0: enabled
			1: mouse_over
			2: selected
			3: disabled
		*/
		interface __declspec(uuid("{6448F36D-89AB-46fe-9171-CB059D2EDA25}")) IDPEGUIListBox: public IDPEGUIControl
		{
			//! Add item. If param idx < 0 then push back.
			virtual HRESULT CALLBACK AddItem(const char *text, int idx = -1) = 0;
			//! Remove item. If param idx < 0 then remove all.
			virtual HRESULT CALLBACK RemoveItem(int idx) = 0;

			virtual HRESULT CALLBACK SetTextColor(unsigned int color) = 0;
			virtual HRESULT CALLBACK SetTextScale(float x, float y) = 0;
			virtual HRESULT CALLBACK SetTextAngle(float angle, const Point2D_Int &center) = 0;
			virtual HRESULT CALLBACK SetTextMargin(int margin) = 0;
			virtual HRESULT CALLBACK SetTextAlignment(unsigned int alignment) = 0;
			
			virtual HRESULT CALLBACK GetVScrollBar(OUT IDPEGUIScrollBar *&scroll_bar) const = 0;
			virtual HRESULT CALLBACK GetHScrollBar(OUT IDPEGUIScrollBar *&scroll_bar) const = 0;
		};

		//-----------------------------------------------------------------------
		//IDPEGUICheckBox//
		//-----------------------------------------------------------------------

		// {250B54D4-1C8F-43b1-873F-DA80F49A174A}
		static const GUID IID_IDPEGUICheckBox = 
		{ 0x250b54d4, 0x1c8f, 0x43b1, { 0x87, 0x3f, 0xda, 0x80, 0xf4, 0x9a, 0x17, 0x4a } };

		enum DPE_TEXT_LOCATION
		{
			DTL_LEFT = 0,
			DTL_RIGHT
		};

		/*	sprites 1:
			0: Check box form

			sprite frames:
			0: enabled
			1: checked
			2: disabled
		*/

		interface __declspec(uuid("{250B54D4-1C8F-43b1-873F-DA80F49A174A}")) IDPEGUICheckBox: public IDPEGUIControl
		{
			virtual HRESULT CALLBACK SetText(const char *text) = 0;
			virtual HRESULT CALLBACK SetTextColor(unsigned int color) = 0;
			virtual HRESULT CALLBACK SetTextScale(float x, float y) = 0;
			virtual HRESULT CALLBACK SetTextAngle(float angle, const Point2D_Int &center) = 0;
			virtual HRESULT CALLBACK SetTextMargin(int margin) = 0;
			virtual HRESULT CALLBACK SetTextAlignment(unsigned int alignment) = 0;

			virtual HRESULT CALLBACK SetChecked(bool checked) = 0;	
			virtual HRESULT CALLBACK GetChecked(OUT bool &checked) = 0;
			virtual HRESULT CALLBACK SetTextLocation(DPE_TEXT_LOCATION text_location) = 0;
		};

		//-----------------------------------------------------------------------
		//IDPEGUIProgressBar//
		//-----------------------------------------------------------------------

		// {A808764A-C0C3-468f-8D22-37A97B5E02A9}
		static const GUID IID_IDPEGUIProgressBar = 
		{ 0xa808764a, 0xc0c3, 0x468f, { 0x8d, 0x22, 0x37, 0xa9, 0x7b, 0x5e, 0x2, 0xa9 } };

		/*	sprites 2:
			0: Progress bar form
			1: Progress

			sprite frames:
			0: enabled
			1: disabled
		*/

		interface __declspec(uuid("{A808764A-C0C3-468f-8D22-37A97B5E02A9}")) IDPEGUIProgressBar: public IDPEGUIControl
		{
			//! From 0 to 100
			virtual HRESULT CALLBACK GetProgress(OUT int &progress) const = 0;
			virtual HRESULT CALLBACK SetProgress(int progress) = 0;
		};

		//-----------------------------------------------------------------------
		//IDPEGUIEdit//
		//-----------------------------------------------------------------------

		// {53BE0F3A-9FC4-4a69-8D14-8096AC69C120}
		static const GUID IID_IDPEGUIEdit = 
		{ 0x53be0f3a, 0x9fc4, 0x4a69, { 0x8d, 0x14, 0x80, 0x96, 0xac, 0x69, 0xc1, 0x20 } };

		/*	sprites 1:
			0: Edit form

			sprite frames:
			0: enabled
			1: disabled
		*/

		interface __declspec(uuid("{53BE0F3A-9FC4-4a69-8D14-8096AC69C120}")) IDPEGUIEdit: public IDPEGUIControl
		{
			virtual HRESULT CALLBACK SetText(const char *text) = 0;
			virtual HRESULT CALLBACK SetTextColor(unsigned int color) = 0;
			virtual HRESULT CALLBACK SetTextScale(float x, float y) = 0;
			virtual HRESULT CALLBACK SetTextAngle(float angle, const Point2D_Int &center) = 0;
			virtual HRESULT CALLBACK SetTextMargin(int margin) = 0;
		};

		//-----------------------------------------------------------------------
		//IDPEGUISkin//
		//-----------------------------------------------------------------------

		// {AE7E74D7-03AD-454d-83F1-823F4FD5C90B}
		static const GUID IID_IDPEGUISkin = 
		{ 0xae7e74d7, 0x3ad, 0x454d, { 0x83, 0xf1, 0x82, 0x3f, 0x4f, 0xd5, 0xc9, 0xb } };

		#pragma pack(push, 1)
		struct TBaseGUIDesc
		{
			ColorRect Enable, Disable;
		};
		struct TBaseExGUIDesc: public TBaseGUIDesc
		{
			ColorRect Over, Down, Up;
		};
		struct TPanelGUIDesc
		{
			TBaseGUIDesc Form, Border;
		};
		struct TPanelExGUIDesc
		{
			TBaseExGUIDesc Form, Border;
		};
		struct TGUIWindowDesc: public TPanelGUIDesc
		{
			TBaseGUIDesc Title;
		};
		struct TGUIButtonDesc: public TPanelExGUIDesc{};
		struct TGUIScrollBar: public TPanelGUIDesc{};
		struct TGUIListBox: public TPanelGUIDesc
		{
			TBaseGUIDesc ItemsBorder;
			TPanelExGUIDesc Item;
			ColorRect Select;
		};
		struct TGUICheckBox: public TPanelGUIDesc
		{
			TPanelGUIDesc CheckRect;
			TBaseGUIDesc Check;
		};
		struct TGUIProgressBar: public TPanelGUIDesc
		{
			TBaseGUIDesc Progress;
		};
		struct TGUIEdit: public TPanelGUIDesc
		{
			TPanelGUIDesc Select;
		};
		#pragma pack(pop)

		#define DEFAULT_GUI_SKIN "default"

		interface __declspec(uuid("{AE7E74D7-03AD-454d-83F1-823F4FD5C90B}")) IDPEGUISkin: public IDPEObject
		{
			virtual HRESULT CALLBACK SetWindowDesc(TGUIWindowDesc &desc) = 0;
			virtual HRESULT CALLBACK SetButtonDesc(TGUIButtonDesc &desc) = 0;
			virtual HRESULT CALLBACK SetScrollBarDesc(TGUIScrollBar &desc) = 0;
			virtual HRESULT CALLBACK SetListBoxDesc(TGUIListBox &desc) = 0;
			virtual HRESULT CALLBACK SetCheckBoxDesc(TGUICheckBox &desc) = 0;
			virtual HRESULT CALLBACK SetProgressBarDesc(TGUIProgressBar &desc) = 0;
			virtual HRESULT CALLBACK SetEditDesc(TGUIEdit &desc) = 0;
			virtual HRESULT CALLBACK SetFont(const char* name) = 0;
		};

		//-----------------------------------------------------------------------
		//IDPEGUIManager//
		//-----------------------------------------------------------------------

		// {CF9BE7CE-7946-441c-862F-692E71E63E51}
		static const GUID IID_IDPEGUIManager = 
		{ 0xcf9be7ce, 0x7946, 0x441c, { 0x86, 0x2f, 0x69, 0x2e, 0x71, 0xe6, 0x3e, 0x51 } };

		interface __declspec(uuid("{CF9BE7CE-7946-441c-862F-692E71E63E51}")) IDPEGUIManager: public IDPEObject
		{
			virtual HRESULT CALLBACK Update(float speed) = 0;
			virtual HRESULT CALLBACK Draw(void) = 0;

			virtual HRESULT CALLBACK SetDefaultFont(const char *name) = 0;

			virtual HRESULT CALLBACK CreateSkin(const char *name, OUT IDPEGUISkin *&pSkin) = 0;
			virtual HRESULT CALLBACK GetSkin(const char *name, OUT IDPEGUISkin *&pSkin) const = 0;

			virtual HRESULT CALLBACK CreateForm(const char *name, const Rect2D_Int &rect, OUT IDPEGUIWindow *&pWindow) = 0;
			virtual HRESULT CALLBACK GetForm(const char *name, OUT IDPEGUIWindow *&pWindow) const = 0;
			virtual HRESULT CALLBACK CreateButton(const char *name, const Rect2D_Int &rect, OUT IDPEGUIButton *&pBtn) = 0;
			virtual HRESULT CALLBACK GetButton(const char *name, OUT IDPEGUIButton *&pBtn) const = 0;
			virtual HRESULT CALLBACK CreateScrollBar(const char *name, const Rect2D_Int &rect, DPE_CONTROL_ORIENTATION orientation, OUT IDPEGUIScrollBar *&pScrollBar) = 0;
			virtual HRESULT CALLBACK GetScrollBar(const char *name, OUT IDPEGUIScrollBar *&pScrollBar) const = 0;
			virtual HRESULT CALLBACK CreateListBox(const char *name, const Rect2D_Int &rect, OUT IDPEGUIListBox *&pListBox) = 0;
			virtual HRESULT CALLBACK GetListBox(const char *name, OUT IDPEGUIListBox *&pListBox) const = 0;
			virtual HRESULT CALLBACK CreateCheckBox(const char *name, const Rect2D_Int &rect, OUT IDPEGUICheckBox *&pCheckBox) = 0;
			virtual HRESULT CALLBACK GetCheckBox(const char *name, OUT IDPEGUICheckBox *&pCheckBox) const = 0;
			virtual HRESULT CALLBACK CreateProgressBar(const char *name, const Rect2D_Int &rect, OUT IDPEGUIProgressBar *&pProgressBar) = 0;
			virtual HRESULT CALLBACK GetProgressBar(const char *name, OUT IDPEGUIProgressBar *&pProgressBar) const = 0;
			virtual HRESULT CALLBACK CreateEdit(const char *name, const Rect2D_Int &rect, OUT IDPEGUIEdit *&pEdit) = 0;
			virtual HRESULT CALLBACK GetEdit(const char *name, OUT IDPEGUIEdit *&pEdit) const = 0;
		};
	}
}
#endif