/*-----------------------------------------------------------------------
This source file is a part of DPEngine.

Created:		02.04.2008
LastModified:	26.08.2008

Revision history:
	* Created by Mike
	* Revised by ...

Disctription:
-----------------------------------------------------------------------*/
#ifndef _IDPE_OBJECT_H_INCLUDED_
#define _IDPE_OBJECT_H_INCLUDED_

namespace DPE_Scenes
{

enum DPE_OBJECT_TYPE
{
	DOT_DPE_OBJ = 0,
	
	// Resources
	DOT_TEXTURE,
	DOT_MATERIAL,
	DOT_MESH,
	DOT_FONT,
	DOT_VECTOR,
	DOT_EMITTER,
	DOT_RESOURCEMANAGER,

	// 2D
	DOT_2D_ITERATOR,
	DOT_2D_OBJECT_BASE,
	DOT_2D_OBJECT,
	DOT_2D_SPRITE,
	DOT_2D_SPRITE_EX,
	DOT_2D_EMITTER,
	DOT_2D_BODY,
	DOT_2D_JOINT,
	DOT_2D_DISTANCE_JOINT,
	DOT_2D_REVOLUTE_JOINT,
	DOT_2D_PRISMATIC_JOINT,
	DOT_2D_PULLEY_JOINT,
	DOT_2D_GEAR_JOINT,
	DOT_2D_MOUSE_JOINT,
	DOT_2D_CAMERA,
	DOT_2D_MANAGER,
	
	// 3D
	DOT_3D_SCENE,
	DOT_3D_NODE,
	DOT_3D_OBJECT,
	DOT_3D_MESH_OBJECT,
	DOT_3D_VECTOR,
	DOT_3D_BILLBOARD,
	DOT_3D_CAMERA,
	DOT_3D_FP_CAMERA,
	DOT_3D_MV_CAMERA,
	DOT_3D_SKY_BOX,
	DOT_3D_MANAGER,

	DOT_SCENE_NODE,
	DOT_SCENE_NODE_ITERATOR,
	DOT_EMITTER_3D_SCENE_NODE,
	DOT_VECTOR_SCENE_NODE,
	DOT_SCENE_NODE_ITER,
	DOT_MESH_SCENE_NODE,
	DOT_CAMERA_SCENE_NODE,
	DOT_FPCAMERA_SCENE_NODE,
	DOT_MVCAMERA_SCENE_NODE,
	DOT_OMNI_LIGHT_SCENE_NODE,
	DOT_DIRECTIONAL_LIGHT_SCENE_NODE,
	DOT_SCENE_MANAGER,
	DOT_SPLINE_TRACKER,
	DOT_CIRCLE_TRACKER,
	DOT_POSITION_ANIMATOR,
	DOT_LOOK_ANIMATOR,

	// GUI
	DOT_GUI_CONTROL,
	DOT_GUI_WINDOW,
	DOT_GUI_BUTTON,
	DOT_GUI_SCROLL_BAR,
	DOT_GUI_LIST_BOX,
	DOT_GUI_CHECK_BOX,
	DOT_GUI_PROGRESS_BAR,
	DOT_GUI_EDIT,
	DOT_GUI_LABEL,
	DOT_GUI_SKIN,
	DOT_GUI_MANAGER
};

// {DF5F5A2E-FD56-437d-BE20-6A5B759664AA}
static const GUID IID_IDPEObject = 
{ 0xdf5f5a2e, 0xfd56, 0x437d, { 0xbe, 0x20, 0x6a, 0x5b, 0x75, 0x96, 0x64, 0xaa } };

interface __declspec(uuid("{DF5F5A2E-FD56-437d-BE20-6A5B759664AA}")) IDPEObject: public IUnknown
{
	//virtual HRESULT CALLBACK SetName(IN const char *name) = 0;
	virtual HRESULT CALLBACK GetName(OUT const char *&name) const = 0;	
	virtual HRESULT CALLBACK GetID(OUT unsigned long &id) const = 0;
	virtual HRESULT CALLBACK GetType(OUT DPE_OBJECT_TYPE &type) const = 0;	
};

/// Indicate if object can be cloned.
interface IClone
{
	/// Clone object's property of the same type as the origin.
	/// @param pObject object to clone.
	virtual HRESULT CALLBACK Clone(IN IDPEObject *pObject) = 0;
};

/// Indicate if object can be manually freed.
interface IFree
{
	/// Free object from managers and owners.
	/// @warning If there is a reference in your program
	/// the object will be not destroied.
	virtual HRESULT CALLBACK Free() = 0;
};

}

#endif