/*-----------------------------------------------------------------------
This source file is a part of DPEngine.

Created:		04.05.2008
LastModified:	14.11.2008

Revision history:
	* Created by Mike
	* Revised by ...

Disctription:
-----------------------------------------------------------------------*/
#ifndef _NODE_ANIMATION_H_INCLUDED_
#define _NODE_ANIMATION_H_INCLUDED_

#include "math\Vector.h"

#include "IDPEObject.h"
#include "IScene.h"

namespace DPE_Scenes
{

//! Forward declaration
interface ISceneNode;

//-----------------------------------------------------------------------
//ITracker//
//-----------------------------------------------------------------------

// {83B2EAFA-D4E9-4d01-B799-F4B3A3312CCF}
static const GUID IID_ITracker = 
{ 0x83b2eafa, 0xd4e9, 0x4d01, { 0xb7, 0x99, 0xf4, 0xb3, 0xa3, 0x31, 0x2c, 0xcf } };

interface __declspec(uuid("{83B2EAFA-D4E9-4d01-B799-F4B3A3312CCF}")) ITracker: public IDPEObject
{
	virtual HRESULT CALLBACK Update(float speed) = 0;
	//! Return curent calculated point
	virtual HRESULT CALLBACK GetPoint(OUT Vector &vPoint) = 0;
	virtual HRESULT CALLBACK SetSpeed(float fSpeed) = 0;
};

//-----------------------------------------------------------------------
//ISplineTracker//
//-----------------------------------------------------------------------

// {BE740DE0-4BCE-4df1-879E-247D34095C0C}
static const GUID IID_ISplineTracker = 
{ 0xbe740de0, 0x4bce, 0x4df1, { 0x87, 0x9e, 0x24, 0x7d, 0x34, 0x9, 0x5c, 0xc } };

interface __declspec(uuid("{BE740DE0-4BCE-4df1-879E-247D34095C0C}")) ISplineTracker: public ITracker
{
	//! Add control point
	virtual HRESULT CALLBACK AddPoint(const Vector &vPoint) = 0;
	//! Remove control point
	virtual HRESULT CALLBACK RemovePoint(unsigned int iIdx) = 0;
	virtual HRESULT CALLBACK RemoveAll() = 0;
	virtual HRESULT CALLBACK SetTension(float fTension) = 0;	
	//! Return curent control point index
	virtual HRESULT CALLBACK GetPointIndex(OUT int &iIdx) = 0;	
};

//-----------------------------------------------------------------------
//ICircleTracker//
//-----------------------------------------------------------------------

// {F485AB34-759A-450a-A650-B779BA38FC60}
static const GUID IID_ICircleTracker = 
{ 0xf485ab34, 0x759a, 0x450a, { 0xa6, 0x50, 0xb7, 0x79, 0xba, 0x38, 0xfc, 0x60 } };

interface __declspec(uuid("{BE740DE0-4BCE-4df1-879E-247D34095C0C}")) ICircleTracker: public ITracker
{
	virtual HRESULT CALLBACK SetRadius(float fRadius) = 0;
	virtual HRESULT CALLBACK SetOrientation(const Vector &vCenter, const Vector &vDirection) = 0;
};

//-----------------------------------------------------------------------
//IAnimator//
//-----------------------------------------------------------------------

// {1D268D8F-86EF-447a-A9B9-84211CFB83CE}
static const GUID IID_IAnimator = 
{ 0x1d268d8f, 0x86ef, 0x447a, { 0xa9, 0xb9, 0x84, 0x21, 0x1c, 0xfb, 0x83, 0xce } };

interface __declspec(uuid("{1D268D8F-86EF-447a-A9B9-84211CFB83CE}")) IAnimator: public IDPEObject
{
	virtual HRESULT CALLBACK Animate(ISceneNode *pSceneNode, float speed) = 0;
};

//-----------------------------------------------------------------------
//IPositionAnimator//
//-----------------------------------------------------------------------

// {7440BBA3-D0B0-4d7d-A167-E876BDF18E39}
static const GUID IID_IPositionAnimator = 
{ 0x7440bba3, 0xd0b0, 0x4d7d, { 0xa1, 0x67, 0xe8, 0x76, 0xbd, 0xf1, 0x8e, 0x39 } };

interface __declspec(uuid("{7440BBA3-D0B0-4d7d-A167-E876BDF18E39}")) IPositionAnimator: public IAnimator
{
	virtual HRESULT CALLBACK AddTracker(IN ITracker *pTracker) = 0;
};

//-----------------------------------------------------------------------
//ILookAnimator//
//-----------------------------------------------------------------------

// {4B0C0EED-FBC7-4d16-9D31-4F8B03B07A38}
static const GUID IID_ILookAnimator = 
{ 0x4b0c0eed, 0xfbc7, 0x4d16, { 0x9d, 0x31, 0x4f, 0x8b, 0x3, 0xb0, 0x7a, 0x38 } };

interface __declspec(uuid("{4B0C0EED-FBC7-4d16-9D31-4F8B03B07A38}")) ILookAnimator: public IAnimator
{
	virtual HRESULT CALLBACK AddTracker(IN ITracker *pTracker) = 0;
};


}
#endif