/*-----------------------------------------------------------------------
This source file is a part of DPEngine.

Created:		15.04.2009
LastModified:	...

Revision history:
	* Created by Mike
	* Revised by ...

Disctription:
-----------------------------------------------------------------------*/
#ifndef _SCENES_H_INCLUDED_
#define _SCENES_H_INCLUDED_

#include "IDPEObject.h"

#include "math\aabb.h"
#include "math\Vector.h"
#include "math\Matrix.h"

namespace DPE_Scenes
{
	namespace _3D
	{
#pragma region IIterator, IAttach, ITransform, IVisible, IAABB, IUpdate, IDraw

		// {EAAD0A70-6D28-4b2c-9FBE-C415E6FE27AA}
		static const GUID IID_IIterator = 
		{ 0xeaad0a70, 0x6d28, 0x4b2c, { 0x9f, 0xbe, 0xc4, 0x15, 0xe6, 0xfe, 0x27, 0xaa } };

		interface __declspec(uuid("{EAAD0A70-6D28-4b2c-9FBE-C415E6FE27AA}")) IIterator: public IUnknown
		{
			virtual HRESULT CALLBACK Next() = 0;
			virtual HRESULT CALLBACK Prev() = 0;
			virtual HRESULT CALLBACK Count(OUT unsigned int &count) const = 0;
			virtual HRESULT CALLBACK Get(OUT IUnknown *&item) const = 0;
		};

		interface IAttach
		{
			virtual HRESULT CALLBACK Attach(IN IUnknown *item) = 0;
			virtual HRESULT CALLBACK Detach(IUnknown *item) = 0;
			virtual HRESULT CALLBACK DetachAll( void ) = 0;
			virtual HRESULT CALLBACK DetachFromParent( void ) = 0;
			virtual HRESULT CALLBACK GetParent(OUT IUnknown *&parent) const = 0;
			virtual HRESULT CALLBACK GetChildrenCount(OUT unsigned int &count) const = 0;
			virtual HRESULT CALLBACK GetChildren(OUT IIterator *&pIter) const = 0;
		};

		interface ITransform
		{
			virtual HRESULT CALLBACK SetScaleF(const float fScalar) = 0;
			virtual HRESULT CALLBACK SetScaleV(const Vector &vScale) = 0;
			virtual HRESULT CALLBACK GetScale(OUT Vector &vScale) const = 0;

			virtual HRESULT CALLBACK SetPositionF(float x, float y, float z) = 0;
			virtual HRESULT CALLBACK SetPositionV(const Vector &vPos) = 0;
			virtual HRESULT CALLBACK GetPosition(OUT Vector &vPos) const = 0;
			virtual HRESULT CALLBACK GetAbsolutePosition(OUT Vector &vPos) const = 0;

			virtual HRESULT CALLBACK SetRotation(const Vector &vRot) = 0;
			virtual HRESULT CALLBACK GetRotation(OUT Vector &vRot) const = 0;
			virtual HRESULT CALLBACK GetAbsoluteRotation(OUT Matrix &mRot) const = 0;

			virtual HRESULT CALLBACK GetAbsoluteTransform(OUT Matrix &m) const = 0;
		};

		interface IVisible
		{
			virtual HRESULT CALLBACK SetVisible(bool visible) = 0;
			virtual HRESULT CALLBACK GetVisible(OUT bool &visible) const = 0;
		};

		interface IAABB
		{
			virtual HRESULT CALLBACK GetAABB(OUT TAABB &aabb) const = 0;
		};

		interface IUpdate
		{
			virtual HRESULT CALLBACK Update( void ) = 0;
		};

		interface IDraw
		{
			virtual HRESULT CALLBACK Draw( void ) = 0;
		};
#pragma endregion

		// {3B3F3613-8025-4a08-878B-F95D2A60EE13}
		static const GUID IID_INode = 
		{ 0x3b3f3613, 0x8025, 0x4a08, { 0x87, 0x8b, 0xf9, 0x5d, 0x2a, 0x60, 0xee, 0x13 } };

		// Forwards
		interface IScene;
		interface IObject;
		interface ILight;

		interface __declspec(uuid("{3B3F3613-8025-4a08-878B-F95D2A60EE13}")) INode :
			public IDPEObject, public IAttach, public ITransform, public IAABB, public IVisible, 
			public IUpdate, public IFree
		{
			enum E_TYPE
			{
				e_base = 0,
				e_octree
			};

			virtual HRESULT CALLBACK GetScene(OUT IScene *&pScene) = 0;

			virtual HRESULT CALLBACK CreateNode(const char *name, INode::E_TYPE type, OUT INode *&pNode) = 0;
			virtual HRESULT CALLBACK GetNode(const char *name, OUT INode *&pNode) const = 0;

			virtual HRESULT CALLBACK SetCulling(bool frustum_culling, bool occlusion_culling) = 0;

			virtual HRESULT CALLBACK AddLight(IN ILight *pLight) = 0;
			virtual HRESULT CALLBACK RemoveLight(ILight *pLight) = 0;			

			virtual HRESULT CALLBACK Add(IN IObject *pObject) = 0;
			virtual HRESULT CALLBACK Remove(IObject *pObject) = 0;
			virtual HRESULT CALLBACK RemoveAll( void ) = 0;
			virtual HRESULT CALLBACK Count(OUT unsigned int &count) const = 0;
			virtual HRESULT CALLBACK Get(OUT IIterator *&pIter) const = 0;
		};

		// {0D884897-A7C9-4793-987B-5A3A4E9FF4A5}
		static const GUID IID_IObject = 
		{ 0xd884897, 0xa7c9, 0x4793, { 0x98, 0x7b, 0x5a, 0x3a, 0x4e, 0x9f, 0xf4, 0xa5 } };

#pragma region Objects
		interface __declspec(uuid("{0D884897-A7C9-4793-987B-5A3A4E9FF4A5}")) IObject :
			public IDPEObject, public IAttach, public ITransform, public IAABB, public IVisible,
			public IUpdate, public IFree
		{
			virtual HRESULT CALLBACK GetNode(OUT INode *&pNode) const = 0;
		};

		// {B387A204-3BC5-48b5-844E-263F67C805FA}
		static const GUID IID_IMeshObject = 
		{ 0xb387a204, 0x3bc5, 0x48b5, { 0x84, 0x4e, 0x26, 0x3f, 0x67, 0xc8, 0x5, 0xfa } };

		interface __declspec(uuid("{B387A204-3BC5-48b5-844E-263F67C805FA}")) IMeshObject :
			public IObject
		{
			static const int MAX_LODS = 4;
			struct TLOD
			{
				int Idx;	///< From 0 to 4;
			};
			virtual HRESULT CALLBACK Set(const char *mesh, const char *material, const TLOD *desc = NULL) = 0;
			virtual HRESULT CALLBACK SetMaterial(const char *material, const TLOD *desc = NULL) = 0;

			virtual HRESULT CALLBACK SetCullMode(RendererEnums::IInstance::CULL_MODE cull_mode) = 0;
			virtual HRESULT CALLBACK SetFillMode(bool wireframe) = 0;
			virtual HRESULT CALLBACK SetShadowCasting(bool cast_shadows) = 0;
		};

		// {6AE38E7A-FCB0-4141-8AA7-C5A00BEBB8A6}
		static const GUID IID_IVectorObject = 
		{ 0x6ae38e7a, 0xfcb0, 0x4141, { 0x8a, 0xa7, 0xc5, 0xa0, 0xb, 0xeb, 0xb8, 0xa6 } };

		interface __declspec(uuid("{6AE38E7A-FCB0-4141-8AA7-C5A00BEBB8A6}")) IVectorObject :
			public IObject
		{
			virtual HRESULT CALLBACK Set(const char *vector) = 0;			
			virtual HRESULT CALLBACK Unset() = 0;
			virtual HRESULT CALLBACK SetLayer(short layer) = 0;
		};

		// {92EF7774-4228-4862-9427-061A5C2B894B}
		static const GUID IID_IBillboard = 
		{ 0x92ef7774, 0x4228, 0x4862, { 0x94, 0x27, 0x6, 0x1a, 0x5c, 0x2b, 0x89, 0x4b } };

		interface __declspec(uuid("{92EF7774-4228-4862-9427-061A5C2B894B}")) IBillboard :
			public IObject
		{
			virtual HRESULT CALLBACK SetTexture(const char *texture) = 0;
			virtual HRESULT CALLBACK SetAngle(float angle) = 0;
			virtual HRESULT CALLBACK BlendMode(bool add_back) = 0;
		};
#pragma endregion

#pragma region Camera
		// {5D1BEF62-8D1D-4da5-9F5B-CA4AD02C9C68}
		static const GUID IID_ICamera = 
		{ 0x5d1bef62, 0x8d1d, 0x4da5, { 0x9f, 0x5b, 0xca, 0x4a, 0xd0, 0x2c, 0x9c, 0x68 } };

		interface __declspec(uuid("{5D1BEF62-8D1D-4da5-9F5B-CA4AD02C9C68}")) ICamera : public IDPEObject, public IUpdate
		{
			virtual HRESULT CALLBACK SetPerspectiveFoVProjectionMatrix(float fovy, float aspect, float zn, float zf) = 0;
			virtual HRESULT CALLBACK GetFoVProjectionMatrix(OUT float &fovy, OUT float &aspect, OUT float &zn, OUT float &zf) const = 0;

			virtual HRESULT CALLBACK SetPosition(const Vector &pos) = 0;			
			virtual HRESULT CALLBACK SetDirection(const Vector &look, const Vector &up = Vector(0.0f, 1.0f, 0.0f)) = 0;			
			virtual HRESULT CALLBACK SetPosAndDir(const Vector &pos, const Vector &look, const Vector &up = Vector(0.0f, 1.0f, 0.0f)) = 0;			
			
			virtual HRESULT CALLBACK GetPosition(OUT Vector &pos) const = 0;
			virtual HRESULT CALLBACK GetDirection(OUT Vector &look) const = 0;
			virtual HRESULT CALLBACK GetPosAndDir(OUT Vector &pos, OUT Vector &look) const = 0;
			virtual HRESULT CALLBACK GetUp(OUT Vector &up) const = 0;
			virtual HRESULT CALLBACK GetRight(OUT Vector &right) const = 0;

			virtual HRESULT CALLBACK GetViewMatrix(OUT Matrix &viewMatrix) const = 0;
			virtual HRESULT CALLBACK GetProjectionMatrix(OUT Matrix &projMatrix) const = 0;

			virtual HRESULT CALLBACK Unproject(const Point2D_Int &pos, OUT Vector &origin, OUT Vector &direction) = 0;

			virtual HRESULT CALLBACK DrawLine(size_t vcount, const Vector vertices[], const Vector &rgb, bool AA, bool closed) = 0;
			virtual HRESULT CALLBACK DrawLine(size_t vcount, const Vector vertices[], const Vector rgb[], bool AA, bool closed) = 0;

			virtual HRESULT CALLBACK PointInFrustum(const Vector &vPoint, OUT bool &result) = 0;
			virtual HRESULT CALLBACK SphereInFrustum(const Vector &vCenter, float fRaius, OUT bool &result) = 0;
			virtual HRESULT CALLBACK AABBInFrustum(const TAABB &aabb, OUT bool &result) = 0;
		};

		// {CACFA811-76A5-48fd-8FC8-D9A15E09D7C5}
		static const GUID IID_IFPCamera = 
		{ 0xcacfa811, 0x76a5, 0x48fd, { 0x8f, 0xc8, 0xd9, 0xa1, 0x5e, 0x9, 0xd7, 0xc5 } };

		interface __declspec(uuid("{CACFA811-76A5-48fd-8FC8-D9A15E09D7C5}")) IFPCamera : public ICamera
		{
			virtual HRESULT CALLBACK LookAt(const Vector &v, const Vector &up = Vector(0.0f, 1.0f, 0.0f)) = 0;

			virtual HRESULT CALLBACK MoveLeftRight(float units) = 0;
			virtual HRESULT CALLBACK MoveUpDown(float units) = 0;
			virtual HRESULT CALLBACK MoveForwBack(float units) = 0;
			virtual HRESULT CALLBACK RotateRight(float angle) = 0;
			virtual HRESULT CALLBACK RotateUp(float angle) = 0;
			virtual HRESULT CALLBACK RotateLook(float angle) = 0;			
		};

		// {FEB7679A-64F0-477a-A674-0C5D4A9D80C0}
		static const GUID IID_IMVCamera = 
		{ 0xfeb7679a, 0x64f0, 0x477a, { 0xa6, 0x74, 0xc, 0x5d, 0x4a, 0x9d, 0x80, 0xc0 } };

		interface _declspec(uuid("{FEB7679A-64F0-477a-A674-0C5D4A9D80C0}")) IMVCamera : public IFPCamera
		{
		};

#pragma endregion

#pragma region Light
		// {071FBE7D-F3F0-4148-883C-AC72623774D8}
		static const GUID IID_ILight = 
		{ 0x71fbe7d, 0xf3f0, 0x4148, { 0x88, 0x3c, 0xac, 0x72, 0x62, 0x37, 0x74, 0xd8 } };

		interface __declspec(uuid("{071FBE7D-F3F0-4148-883C-AC72623774D8}")) ILight : public IDPEObject
		{
			virtual HRESULT CALLBACK SetDiffuse(const Vector &diffuse) = 0;
			virtual HRESULT CALLBACK SetSpecular(const Vector &specular) = 0;
			virtual HRESULT CALLBACK SetEnable(bool enable) = 0;
			virtual HRESULT CALLBACK SetShadowCasting(const INode *pNode, bool cast_shadows) = 0;
		};

		// {B3EFB7FF-905C-4e2d-BBA6-C5F3E4686396}
		static const GUID IID_IPointLight = 
		{ 0xb3efb7ff, 0x905c, 0x4e2d, { 0xbb, 0xa6, 0xc5, 0xf3, 0xe4, 0x68, 0x63, 0x96 } };

		interface __declspec(uuid("{B3EFB7FF-905C-4e2d-BBA6-C5F3E4686396}")) IPointLight :
			public ILight, public IFree
		{
			virtual HRESULT CALLBACK SetPosition(const Vector &pos) = 0;

			virtual HRESULT CALLBACK SetRange(float fRange) = 0;
			virtual HRESULT CALLBACK SetConstantAttenuationFactor(float factor) = 0;
			virtual HRESULT CALLBACK SetLinearAttenuationFactor(float factor) = 0;
			virtual HRESULT CALLBACK SetQuadraticAttenuationFactor(float factor) = 0;
		};

		// {C5E7D771-9447-4789-BF29-C145187A517E}
		static const GUID IID_IDirectionLight = 
		{ 0xc5e7d771, 0x9447, 0x4789, { 0xbf, 0x29, 0xc1, 0x45, 0x18, 0x7a, 0x51, 0x7e } };

		interface __declspec(uuid("{C5E7D771-9447-4789-BF29-C145187A517E}")) IDirectionLight :
			public ILight, public IFree
		{
			virtual HRESULT CALLBACK SetDirection(const Vector &direction) = 0;
		};
#pragma endregion

		// {9FD01DF9-A73D-424d-BE4C-55A97A88EC08}
		static const GUID IID_ISkyBox = 
		{ 0x9fd01df9, 0xa73d, 0x424d, { 0xbe, 0x4c, 0x55, 0xa9, 0x7a, 0x88, 0xec, 0x8 } };

		interface __declspec(uuid("{9FD01DF9-A73D-424d-BE4C-55A97A88EC08}")) ISkyBox :
			public IDPEObject, public IVisible
		{
			virtual HRESULT CALLBACK SetTexture(const char *name, int side) = 0;
		};		

		// {CA8198ED-6632-432c-86CA-64087CA04BE3}
		static const GUID IID_IScene = 
		{ 0xca8198ed, 0x6632, 0x432c, { 0x86, 0xca, 0x64, 0x8, 0x7c, 0xa0, 0x4b, 0xe3 } };

		interface __declspec(uuid("{CA8198ED-6632-432c-86CA-64087CA04BE3}")) IScene: public IDPEObject, public IDraw, public IUpdate, public IFree
		{
			virtual HRESULT CALLBACK GetRootNode(OUT INode *&pNode) const = 0;
			virtual HRESULT CALLBACK GetNode(const char *name, OUT INode *&pNode) const = 0;
			virtual HRESULT CALLBACK SetCamera(IN ICamera *pCamera) = 0;
			virtual HRESULT CALLBACK SetSkyBox(IN ISkyBox *pSkyBox) = 0;
		};

		// {7CCAB817-CBBB-4180-BE31-729DF581C2AF}
		static const GUID IID_IManager3D = 
		{ 0x7ccab817, 0xcbbb, 0x4180, { 0xbe, 0x31, 0x72, 0x9d, 0xf5, 0x81, 0xc2, 0xaf } };

		interface __declspec(uuid("{7CCAB817-CBBB-4180-BE31-729DF581C2AF}")) IManager3D: public IDPEObject
		{
			virtual HRESULT CALLBACK SetBackground(const Vector &color) = 0;

			virtual HRESULT CALLBACK CreateScene(const char *name, OUT IScene *&pScene) = 0;
			virtual HRESULT CALLBACK GetScene(const char *name, OUT IScene *&pScene) = 0;
			
			virtual HRESULT CALLBACK CreateCamera(const char *name, OUT ICamera *&pCamera) = 0;
			virtual HRESULT CALLBACK CreateFPCamera(const char *name, OUT IFPCamera *&pCamera) = 0;
			virtual HRESULT CALLBACK CreateMVCamera(const char *name, OUT IMVCamera *&pCamera) = 0;

			virtual HRESULT CALLBACK CreateSkyBox(OUT ISkyBox *&pSkyBox) = 0;
			
			//virtual HRESULT CALLBACK GetNode(const char *name, OUT INode *&pNode) const = 0;

			virtual HRESULT CALLBACK CreateMeshObject(const char *name, OUT IMeshObject *&pObject) = 0;
			virtual HRESULT CALLBACK GetMeshObject(const char *name, OUT IMeshObject *&pObject) const = 0;

			virtual HRESULT CALLBACK CreateVectorObject(const char *name, OUT IVectorObject *&pObject) = 0;
			virtual HRESULT CALLBACK GetVectorObject(const char *name, OUT IVectorObject *&pObject) const = 0;

			virtual HRESULT CALLBACK CreateBillboard(const char *name, OUT IBillboard *&pObject) = 0;
			virtual HRESULT CALLBACK GetBillboard(const char *name, OUT IBillboard *&pObject) const = 0;

			virtual HRESULT CALLBACK CreatePointLight(const char *name, OUT IPointLight *&pLight) = 0;
			virtual HRESULT CALLBACK GetPointLight(const char *name, OUT IPointLight *&pLight) const = 0;

			virtual HRESULT CALLBACK CreateDirectionLight(const char *name, OUT IDirectionLight *&pLight) = 0;
			virtual HRESULT CALLBACK GetDirectionLight(const char *name, OUT IDirectionLight *&pLight) const = 0;
		};
	};
}
#endif// _SCENES_H_INCLUDED_