#ifndef EVENT_SYSTEM_H_INCLUDED
#define EVENT_SYSTEM_H_INCLUDED

#include <list>
#include <windows.h>
#include "IEventSystem.h"
#include "Delegate.h"

class SEvent
{
private:
	char*						_name;
	ULONG						_hash;
	CDelegate1<void,IEvent*>	_listeners;
public:
	SEvent(const char* name);
	~SEvent();
	void AddEventListener(IDelegate1<void,IEvent*> *Listener);
    void RemoveEventListener(IDelegate1<void,IEvent*> *Listener);
	HRESULT SendEvent(IEvent *Event);
	const char* Name();
	ULONG Hash();
};

class CEventSystem: public IEventSystem
{
	typedef std::list<SEvent*> EventList;
private:	
	EventList		_events;
	IEventSystem	*_mainEventSystem;

	static CEventSystem *pInstance;
	ULONG ref_count;
public:
	static CEventSystem* Instance()
	{
		if(pInstance == NULL) pInstance = new CEventSystem();
		return pInstance;
	}

	static void Free()
	{
		delete pInstance;
    }

	void AddEventListener(const char* EventName, IDelegate1<void,IEvent*> *Listener);
	void RemoveEventListener(const char* EventName, IDelegate1<void,IEvent*> *Listener);

	HRESULT CALLBACK QueryInterface(REFIID riid, void **ppvObject);
	ULONG CALLBACK AddRef();	            
	ULONG CALLBACK Release();

	HRESULT CALLBACK SendEvent(IEvent *Event);
private:
	CEventSystem();
	~CEventSystem();

	HRESULT CALLBACK SetMainEventSystem(IEventSystem *MainSystem);
	HRESULT CALLBACK DispatchEvent(IEvent *Event);	
};

class CEvent: public IEvent
{
private:
    char*	_name;
	ULONG	_hash;
	ULONG	ref_count;
public:
	CEvent(const char* name);
	virtual ~CEvent();
	virtual HRESULT CALLBACK Send();
    virtual HRESULT CALLBACK Hash(ULONG *hash);
    virtual HRESULT CALLBACK Name(char** name);

	virtual HRESULT CALLBACK QueryInterface(REFIID riid, void **ppvObject);		
	virtual ULONG CALLBACK AddRef();	            
	virtual ULONG CALLBACK Release();
};

#ifdef CORE
class CMainEventSystem: public IEventSystem
{
	typedef std::list<IEventSystem*> EventSystemList;
	typedef std::list<SEvent*> EventList;
private:
	EventSystemList _eventSubSystems;
	EventList		_events;

	static CMainEventSystem *pInstance;
	ULONG ref_count;
public:
	static CMainEventSystem* Instance()
	{
		if(pInstance == NULL) pInstance = new CMainEventSystem();
		return pInstance;
	}

	void AddEventListener(const char* EventName, IDelegate1<void,IEvent*> *Listener);
	void RemoveEventListener(const char* EventName, IDelegate1<void,IEvent*> *Listener);

	HRESULT CALLBACK AddEventSubSystem(IEventSystem *SubSystem);
	HRESULT CALLBACK RemoveEventSubSystem(IEventSystem *SubSystem);

	HRESULT CALLBACK QueryInterface(REFIID riid, void **ppvObject);		
	ULONG CALLBACK AddRef();	            
	ULONG CALLBACK Release();

	HRESULT CALLBACK SendEvent(IEvent *Event);
private:
	CMainEventSystem();
	~CMainEventSystem();

	HRESULT CALLBACK SetMainEventSystem(IEventSystem *MainSystem);
	HRESULT CALLBACK DispatchEvent(IEvent *Event);
};
#endif

#endif //EVENT_SYSTEM_H_INCLUDED