#ifndef IVENTS_IMPL_H_INCLUDED
#define IVENTS_IMPL_H_INCLUDED

#include "EventSystem.h"
#include "Events.h"

//-----------------------------------------------------------------------
// ISenderEvent
//-----------------------------------------------------------------------
class SenderEvent: public CEvent, public ISenderEvent
{
	IUnknown		*m_pSender;
public:
	SenderEvent(const char* name, IUnknown *pSender):
	  CEvent(name)	
	{
		m_pSender = pSender;
		if (m_pSender) m_pSender->AddRef();
	}
	virtual ~SenderEvent()
	{
		if (m_pSender) m_pSender->Release();
	}
public:
	
	virtual HRESULT STDMETHODCALLTYPE Send()
	{
		IEvent *pEvent = static_cast<IEvent*>(static_cast<ISenderEvent*>(this));
		#ifdef CORE
		return CMainEventSystem::Instance()->SendEvent(pEvent);
		#else
		return CEventSystem::Instance()->SendEvent(pEvent);
		#endif
	}

	virtual HRESULT STDMETHODCALLTYPE Hash(ULONG *hash){return CEvent::Hash(hash);}
	virtual HRESULT STDMETHODCALLTYPE Name(char** name){return CEvent::Name(name);}

	virtual HRESULT STDMETHODCALLTYPE SetSender(IN IUnknown *pSender)
	{
		if (m_pSender) m_pSender->Release();
		m_pSender = pSender;
		if (m_pSender) m_pSender->AddRef();
		return S_OK;
	}
	virtual HRESULT STDMETHODCALLTYPE GetSender(OUT IUnknown *&pSender)
	{
		pSender = m_pSender;
		if (m_pSender) m_pSender->AddRef();
		return S_OK;
	}

	virtual HRESULT STDMETHODCALLTYPE QueryInterface(REFIID riid, void **ppvObject)
	{		
		*ppvObject = NULL;

		if(::memcmp(&riid,&IID_IUnknown,sizeof(GUID)) == 0) 
			*ppvObject = static_cast<IUnknown *>(static_cast<IEvent*>(static_cast<ISenderEvent*>(this)));

		else if(::memcmp(&riid,&IID_IEvent,sizeof(GUID)) == 0)
			*ppvObject = static_cast<IEvent*>(static_cast<ISenderEvent*>(this));

		else if(::memcmp(&riid,&IID_ISenderEvent,sizeof(GUID)) == 0)
			*ppvObject = static_cast<ISenderEvent*>(this);
		
		else return E_NOINTERFACE;

		AddRef();
		return S_OK;
	}
	virtual ULONG STDMETHODCALLTYPE AddRef(void){return CEvent::AddRef();}
	virtual ULONG STDMETHODCALLTYPE Release(void){return CEvent::Release();}
};

//-----------------------------------------------------------------------
// IInputKeyEvent
//-----------------------------------------------------------------------
class InputKeyEvent: public SenderEvent, public IInputKeyEvent
{	
	unsigned int	m_key;
public:
	InputKeyEvent(const char* name, unsigned int key):
	  SenderEvent(name, NULL)
	{
		m_key = key;
	}
	virtual ~InputKeyEvent(){}
public:
	virtual HRESULT STDMETHODCALLTYPE GetParam(OUT unsigned int &key)
	{
		key = m_key;
		return S_OK;
	}

	virtual HRESULT STDMETHODCALLTYPE Send()
	{
		IEvent *pEvent = static_cast<IEvent*>(static_cast<ISenderEvent*>(static_cast<IInputKeyEvent *>(this)));
		#ifdef CORE
		return CMainEventSystem::Instance()->SendEvent(pEvent);
		#else	
		return CEventSystem::Instance()->SendEvent(pEvent);
		#endif
	}
	virtual HRESULT STDMETHODCALLTYPE Hash(ULONG *hash){return SenderEvent::Hash(hash);}
    virtual HRESULT STDMETHODCALLTYPE Name(char** name){return SenderEvent::Name(name);}
	virtual HRESULT STDMETHODCALLTYPE SetSender(IN IUnknown *pSender){return SenderEvent::SetSender(pSender);}
	virtual HRESULT STDMETHODCALLTYPE GetSender(OUT IUnknown *&pSender){return SenderEvent::GetSender(pSender);}

	virtual HRESULT STDMETHODCALLTYPE QueryInterface(REFIID riid, void **ppvObject)
	{		
		*ppvObject = NULL;

		if(::memcmp(&riid,&IID_IUnknown,sizeof(GUID)) == 0) 
			*ppvObject = static_cast<IUnknown *>(static_cast<IEvent*>(static_cast<ISenderEvent*>(static_cast<IInputKeyEvent*>(this))));

		else if(::memcmp(&riid,&IID_IEvent,sizeof(GUID)) == 0)
			*ppvObject = static_cast<IEvent*>(static_cast<ISenderEvent*>(static_cast<IInputKeyEvent*>(this)));

		else if(::memcmp(&riid,&IID_ISenderEvent,sizeof(GUID)) == 0)
			*ppvObject = static_cast<ISenderEvent*>(static_cast<IInputKeyEvent*>(this));

		else if(::memcmp(&riid,&IID_IInputKeyEvent,sizeof(GUID)) == 0)
			*ppvObject = static_cast<IInputKeyEvent*>(this);

		else return E_NOINTERFACE;

		AddRef();
		return S_OK;
	}
	virtual ULONG STDMETHODCALLTYPE AddRef(void){return SenderEvent::AddRef();}	        
	virtual ULONG STDMETHODCALLTYPE Release(void){return SenderEvent::Release();}
};

//-----------------------------------------------------------------------
// IInputMouseEvent
//-----------------------------------------------------------------------
class InputMouseEvent: public SenderEvent, public IInputMouseEvent
{	
	sMouseStates		m_mouse;
	DPE_MOUSE_BUTTONS	m_button;
public:
	InputMouseEvent(const char* name, sMouseStates mouse, DPE_MOUSE_BUTTONS button):
	  SenderEvent(name, NULL)
	{
		m_mouse = mouse;
		m_button = button;
	}
	virtual ~InputMouseEvent(){}
public:
	virtual HRESULT STDMETHODCALLTYPE GetParam(OUT sMouseStates &mouse, OUT DPE_MOUSE_BUTTONS &button)
	{
		mouse = m_mouse;
		button = m_button;
		return S_OK;
	}

	virtual HRESULT STDMETHODCALLTYPE Send()
	{
		IEvent *pEvent = static_cast<IEvent*>(static_cast<ISenderEvent*>(static_cast<IInputMouseEvent *>(this)));
		#ifdef CORE
		return CMainEventSystem::Instance()->SendEvent(pEvent);
		#else
		return CEventSystem::Instance()->SendEvent(pEvent);
		#endif
	}

	virtual HRESULT STDMETHODCALLTYPE Hash(ULONG *hash){return SenderEvent::Hash(hash);}
    virtual HRESULT STDMETHODCALLTYPE Name(char** name){return SenderEvent::Name(name);}
	virtual HRESULT STDMETHODCALLTYPE SetSender(IN IUnknown *pSender){return SenderEvent::SetSender(pSender);}
	virtual HRESULT STDMETHODCALLTYPE GetSender(OUT IUnknown *&pSender){return SenderEvent::GetSender(pSender);}

	virtual HRESULT STDMETHODCALLTYPE QueryInterface(REFIID riid, void **ppvObject)
	{		
		*ppvObject = NULL;

		if(::memcmp(&riid,&IID_IUnknown,sizeof(GUID)) == 0) 
			*ppvObject = static_cast<IUnknown *>(static_cast<IEvent*>(static_cast<ISenderEvent*>(static_cast<IInputMouseEvent*>(this))));

		else if(::memcmp(&riid,&IID_IEvent,sizeof(GUID)) == 0)
			*ppvObject = static_cast<IEvent*>(static_cast<ISenderEvent*>(static_cast<IInputMouseEvent*>(this)));

		else if(::memcmp(&riid,&IID_ISenderEvent,sizeof(GUID)) == 0)
			*ppvObject = static_cast<ISenderEvent*>(static_cast<IInputMouseEvent*>(this));

		else if(::memcmp(&riid,&IID_IInputMouseEvent,sizeof(GUID)) == 0)
			*ppvObject = static_cast<IInputMouseEvent*>(this);

		else return E_NOINTERFACE;

		AddRef();
		return S_OK;
	}
	virtual ULONG STDMETHODCALLTYPE AddRef(void){return SenderEvent::AddRef();}
	virtual ULONG STDMETHODCALLTYPE Release(void){return SenderEvent::Release();}
};

//-----------------------------------------------------------------------
// IWinEvent
//-----------------------------------------------------------------------
class WinEvent: public CEvent, public IWinEvent
{	
	unsigned int _message;
	WPARAM _wParam;
	LPARAM _lParam;
public:
	
	WinEvent(const char* name, unsigned int message, WPARAM wParam, LPARAM lParam): CEvent(name)
	{
		_message = message;
		_wParam = wParam;
		_lParam = lParam;		
	}
	
	virtual HRESULT STDMETHODCALLTYPE GetParam(OUT unsigned int *message, OUT WPARAM *wParam, OUT LPARAM *lParam)
	{
		*message = _message;
		*wParam = _wParam;
		*lParam = _lParam;

		return S_OK;
	}
	
	virtual HRESULT STDMETHODCALLTYPE Send()
	{
		IEvent *pEvent = static_cast<IEvent*>(static_cast<IWinEvent*>(this));
		#ifdef CORE
		return CMainEventSystem::Instance()->SendEvent(pEvent);
		#else
		return CEventSystem::Instance()->SendEvent(pEvent);
		#endif
	}
	virtual HRESULT STDMETHODCALLTYPE Hash(ULONG *hash){return CEvent::Hash(hash);}
    virtual HRESULT STDMETHODCALLTYPE Name(char** name){return CEvent::Name(name);}

	virtual HRESULT STDMETHODCALLTYPE QueryInterface(REFIID riid, void **ppvObject)
	{		
		*ppvObject = NULL;

		if(::memcmp(&riid,&IID_IUnknown,sizeof(GUID)) == 0) 
			*ppvObject = static_cast<IUnknown *>(static_cast<IEvent*>(static_cast<IWinEvent*>(this)));

		else if(::memcmp(&riid,&IID_IEvent,sizeof(GUID)) == 0)
			*ppvObject = static_cast<IEvent*>(static_cast<IWinEvent*>(this));

		else if(::memcmp(&riid,&IID_IWinEvent,sizeof(GUID)) == 0)
			*ppvObject = static_cast<IWinEvent*>(this);

		else return E_NOINTERFACE;

		AddRef();
		return S_OK;
	}
	virtual ULONG STDMETHODCALLTYPE AddRef(void){return CEvent::AddRef();}
	virtual ULONG STDMETHODCALLTYPE Release(void){return CEvent::Release();}
};
//-----------------------------------------------------------------------
// IRenderStatusEvent
//-----------------------------------------------------------------------
class CRenderStatusEvent: public CEvent, public IRenderStatusEvent
{
	bool			do_render;
public:
	CRenderStatusEvent(bool do_render):
	  CEvent(EV_RENDER_STATUS)
	{
		CRenderStatusEvent::do_render = do_render;
	}
	virtual ~CRenderStatusEvent(){}
public:
	virtual HRESULT STDMETHODCALLTYPE GetParam(OUT volatile bool &do_render)
	{
		do_render = CRenderStatusEvent::do_render;
		return S_OK;
	}
	virtual HRESULT STDMETHODCALLTYPE Send()
	{
		IEvent *pEvent = static_cast<IEvent*>(static_cast<IRenderStatusEvent*>(this));
		#ifdef CORE
		return CMainEventSystem::Instance()->SendEvent(pEvent);
		#else
		return CEventSystem::Instance()->SendEvent(pEvent);
		#endif
	}
    virtual HRESULT STDMETHODCALLTYPE Hash(ULONG *hash){return CEvent::Hash(hash);}
    virtual HRESULT STDMETHODCALLTYPE Name(char** name){return CEvent::Name(name);}

	virtual HRESULT STDMETHODCALLTYPE QueryInterface(REFIID riid, void **ppvObject)
	{		
		*ppvObject = NULL;

		if(::memcmp(&riid,&IID_IUnknown,sizeof(GUID)) == 0) 
			*ppvObject = static_cast<IUnknown *>(static_cast<IEvent*>(static_cast<IRenderStatusEvent*>(this)));

		else if(::memcmp(&riid,&IID_IEvent,sizeof(GUID)) == 0)
			*ppvObject = static_cast<IEvent*>(static_cast<IRenderStatusEvent*>(this));

		else if(::memcmp(&riid,&IID_IRenderStatusEvent,sizeof(GUID)) == 0)
			*ppvObject = static_cast<IRenderStatusEvent*>(this);

		else return E_NOINTERFACE;

		AddRef();
		return S_OK;
	}
	virtual ULONG STDMETHODCALLTYPE AddRef(void){return CEvent::AddRef();}
	virtual ULONG STDMETHODCALLTYPE Release(void){return CEvent::Release();}
};
#endif