/*-----------------------------------------------------------------------
This source file is a part of DPEngine.

Created:		?
LastModified:	1.08.2008

Revision history:
	* Created by GAS
	* Revised by Mike
	* Revised by GAS

Disctription:
-----------------------------------------------------------------------*/
#pragma once

#include <windows.h>


// {95B4981E-3F27-45f9-8118-7D4DA6A02E32}
static const GUID IID_IFMesh = 
{ 0x95b4981e, 0x3f27, 0x45f9, { 0x81, 0x18, 0x7d, 0x4d, 0xa6, 0xa0, 0x2e, 0x32 } };

class __declspec(uuid("{95B4981E-3F27-45f9-8118-7D4DA6A02E32}")) IFMesh: public IUnknown
{
public:
	struct MESH_DATA
	{
		//! Faces, Normals, Textures indeces
		struct INDEX_BUFFER
		{
			unsigned long	*IndexBufferArray[3];
			unsigned int	IBLength;

			inline INDEX_BUFFER()
			{
				IndexBufferArray[0] = IndexBufferArray[1] = IndexBufferArray[2] = NULL;
				IBLength = 0;
			}

			inline void Destroy()
			{
				if (IndexBufferArray[0])
				{
					delete[] IndexBufferArray[0];
					IndexBufferArray[0] = NULL;
				}
				if (IndexBufferArray[1])
				{
					delete[] IndexBufferArray[1];
					IndexBufferArray[1] = NULL;
				}
				if (IndexBufferArray[2])
				{
					delete[] IndexBufferArray[2];
					IndexBufferArray[2] = NULL;
				}
				IBLength = 0;
			}
		};

		struct VERTEX_BUFFER
		{
			float (*VertexCoordsVB)[3];
			float (*NormalVB)[3];
			float (*TexCoordsVB)[2];
			float (*TangentVB)[3];
			float (*BitangentVB)[3];
			unsigned int VBLengthArray[3];
			unsigned int FaceCount;

			inline VERTEX_BUFFER()
			{
				VertexCoordsVB = NormalVB = TangentVB = BitangentVB = NULL, TexCoordsVB = NULL;
				VBLengthArray[0] = VBLengthArray[1] = VBLengthArray[2] = 0;
				FaceCount = 0;
			}

			inline void Destroy()
			{
				if (VertexCoordsVB)
				{
					delete[] VertexCoordsVB;
					VertexCoordsVB = NULL;
				}
				if (NormalVB)
				{
					delete[] NormalVB;
					NormalVB = NULL;
				}
				if (TexCoordsVB)
				{
					delete[] TexCoordsVB;
					TexCoordsVB = NULL;
				}

				if (TangentVB)
				{
					delete[] TangentVB;
					TangentVB = NULL;
				}
				if (BitangentVB)
				{
					delete[] BitangentVB;
					BitangentVB = NULL;
				}
				VBLengthArray[0] = VBLengthArray[1] = VBLengthArray[2] = 0;
				FaceCount = 0;
			}
		};

		INDEX_BUFFER	IndexBuffer;
		VERTEX_BUFFER	VertexBuffer;
		inline void Destroy()
		{
			IndexBuffer.Destroy();
			VertexBuffer.Destroy();
		}
	};
	virtual HRESULT STDMETHODCALLTYPE GetMeshData(OUT MESH_DATA &mesh) = 0;
};