#ifndef I_FS_CUSTOM_FS
#define I_FS_CUSTOM_FS

enum FS_FIND_FLAGS
{
	FS_FIND_RECURSIVE=1,
};

enum FS_OPEN_FLAGS
{
	FS_READ		= 0x01,	//read
	FS_WRITE	= 0x02,	//write
	FS_TRUNK	= 0x04,	//clear file
	FS_BINARY	= 0x08	//read/write binary format
};

enum FS_SEEK
{
	FS_BEGIN	= 0,
	FS_CUR		= 1,
	FS_END		= 2
};

// {AF6AC090-7EAD-44be-BB00-51D4CF2D57A4}
static const GUID IID_IFILE = 
{ 0xaf6ac090, 0x7ead, 0x44be, { 0xbb, 0x0, 0x51, 0xd4, 0xcf, 0x2d, 0x57, 0xa4 } };

interface __declspec(uuid("{AF6AC090-7EAD-44be-BB00-51D4CF2D57A4}")) IFile: public IUnknown
{
	virtual HRESULT CALLBACK Read(void *Buffer, unsigned int Count, unsigned int &read) = 0;
	virtual HRESULT CALLBACK Write(const void* Buffer, unsigned int Count, unsigned int &written) = 0;
	virtual HRESULT CALLBACK Seek(unsigned int Offset, FS_SEEK Way, unsigned int &Position) = 0;
	virtual HRESULT CALLBACK GetSize(unsigned int &Size) = 0;
	virtual HRESULT CALLBACK IsOpen() = 0;
	virtual HRESULT CALLBACK GetFName(char* name /*max 256 chars*/) = 0; //relate file name (not full name!)
};

// {2C37B0D9-F36A-471e-B652-2BA5560847A6}
static const GUID IID_IFILE_ITERATOR = 
{ 0x2c37b0d9, 0xf36a, 0x471e, { 0xb6, 0x52, 0x2b, 0xa5, 0x56, 0x8, 0x47, 0xa6 } };

interface __declspec(uuid("{2C37B0D9-F36A-471e-B652-2BA5560847A6}")) IFileIterator: public IUnknown
{
	virtual HRESULT CALLBACK FileName(const char* &name) = 0;
	virtual HRESULT CALLBACK Next() = 0;
};

// {F0B5EA3A-697E-4715-A884-0E297F24AA44}
static const GUID IID_CUSTOM_FILE_SYSTEM = 
{ 0xf0b5ea3a, 0x697e, 0x4715, { 0xa8, 0x84, 0xe, 0x29, 0x7f, 0x24, 0xaa, 0x44 } };

interface __declspec(uuid("{F0B5EA3A-697E-4715-A884-0E297F24AA44}")) IFS_Custom_FileSystem: public IUnknown
{
	virtual HRESULT CALLBACK OpenFile(const char* Name, FS_OPEN_FLAGS Flags, IFile **File) = 0;	
	virtual HRESULT CALLBACK DeleteFile(const char* Name) = 0;
	virtual HRESULT CALLBACK FileExists(const char* Name) = 0;
	virtual HRESULT CALLBACK Find(const char* Mask, FS_FIND_FLAGS Flags, IFileIterator **Iterator) = 0;

};


#endif