#ifndef IFVECTORIMG_H_INCLUDED
#define IFVECTORIMG_H_INCLUDED

#include <Unknwn.h>

class __declspec(uuid("{B1F6C281-1AAC-4afc-BCE4-D594B4CF3194}")) IFVectorImg: public IUnknown
{
public:

	typedef struct
	{
		float x, y;
	} COORD;

	//primitive type defines
	enum
	{
		POINT	= 0,
		LINE	= 1,
		POLYGON = 2,
		TEXT	= 3,
		BEZIER	= 4
	};

	typedef struct
	{
		char name[255];
		int  size;
	} FONT_RECORD;

	typedef struct
	{
		unsigned int id;
		char name[128];
		int width, height;
		int *data;
	} TEXTURE_RECORD;

	typedef struct base_object
	{
		char			type;
		unsigned int	id;
		unsigned long	color;
		float			size;
		unsigned int	texture_id;

		unsigned int	count;

		COORD	*points;
	}PRIMITIVE_RECORD, *P_PRIMITIVE_RECORD;

	struct point : base_object
	{
		char			billboard;
		char			relative_size;
	};

	struct line : base_object
	{
		char			relative_size;
	};
	
	struct polygon : base_object
	{
		int				inner_color;
	};

	struct bezier : base_object
	{
		float			*w; //array of W size same as COORD array
	};

	struct text : base_object
	{
		int				angle;
		char			billboard;
		char 			relative_size;
		float			scale;
		int				text_size;
		char			*text_string;
	};


	virtual HRESULT CALLBACK GetFontCount( size_t &count ) const = 0;
	virtual HRESULT CALLBACK GetFontData( const FONT_RECORD *&data ) const = 0;

	virtual HRESULT CALLBACK GetTextureCount( size_t &count ) const = 0;
	virtual HRESULT CALLBACK GetTextureData( const TEXTURE_RECORD *&data ) const = 0;

	virtual HRESULT CALLBACK GetPrimitiveCount( size_t &count ) const = 0;
	virtual HRESULT CALLBACK GetPrimitiveData( const P_PRIMITIVE_RECORD *&data ) const = 0;

};
#endif