#ifndef I_FILE_SYSTEM
#define I_FILE_SYSTEM

#include <Windows.h>
#include "IFS_Custom_FileSystem.h"

// {1534E4AE-E227-4497-8111-98ADAB190D3E}
static const GUID IID_IFileSystem = 
{ 0x1534e4ae, 0xe227, 0x4497, { 0x81, 0x11, 0x98, 0xad, 0xab, 0x19, 0xd, 0x3e } };

// {1F5E429D-9A68-4d9d-A576-EDF30269C2DA}
static const GUID IID_IFileTransfer = 
{ 0x1f5e429d, 0x9a68, 0x4d9d, { 0xa5, 0x76, 0xed, 0xf3, 0x2, 0x69, 0xc2, 0xda } };

enum ext_type
{
	et_UNKNOWN = 0,
	et_MIXED_FILE_FORMATS, 
	et_CUSTOM_FILE_SYSTEM,
	et_TEXTURE, 
	et_STATIC_MESH,
	et_BITMAP_FONT, 
	et_STANDART_MATERIAL, 
	et_SOUNDWAVE,
	et_VECTOR_IMAGE
};

typedef struct
{	
	ext_type	type;
	IUnknown*	file;
	int			idx;
	char		extra[32];

} TFileData;

interface __declspec(uuid("{1F5E429D-9A68-4d9d-A576-EDF30269C2DA}")) IFileTransfer: public IUnknown
{
	virtual HRESULT STDMETHODCALLTYPE GetFileData(OUT TFileData **) = 0;
	virtual HRESULT STDMETHODCALLTYPE GetFileDataCount(OUT size_t &count) = 0;
	virtual HRESULT STDMETHODCALLTYPE GetFileDataCount(IN ext_type ext, OUT size_t &count) = 0;
	virtual HRESULT STDMETHODCALLTYPE GetFileExtension(OUT char* ext) = 0;
};

interface __declspec(uuid("{1534E4AE-E227-4497-8111-98ADAB190D3E}")) IFileSystem: public IUnknown
{

	virtual HRESULT STDMETHODCALLTYPE IsExtensionRegistered( const char* Ext, OUT bool &exists, OUT ext_type &type ) = 0;
	virtual HRESULT STDMETHODCALLTYPE GetRegisteredExtensions( OUT char* exts ) = 0;
	virtual HRESULT STDMETHODCALLTYPE FileExists( const char* FileName ) = 0;
	virtual HRESULT STDMETHODCALLTYPE GetFileSystem( const char* Ext, OUT IFS_Custom_FileSystem** fs ) = 0; // if Ext == "" then basic HDD File System will be returned
	virtual HRESULT STDMETHODCALLTYPE GetFile( const char* FileName, OUT IFile** file, IN FS_OPEN_FLAGS Flags = (FS_OPEN_FLAGS)(FS_READ | FS_BINARY) ) = 0;
	virtual HRESULT STDMETHODCALLTYPE LoadFile( const char* FileName, OUT IFileTransfer** file ) = 0; 
	virtual HRESULT STDMETHODCALLTYPE RegCustom( const char* Name, const char* Extension, ext_type c_type, IN IUnknown* Custom ) = 0;

};


#endif