#ifndef I_LOG_H_INCLUDED
#define I_LOG_H_INCLUDED

#include <windows.h>

enum E_LOG_LEVEL
{
	//! Normally used for debugging
	ELL_DEBUG = 0,
	//! Informational messages 
	ELL_INFO,
	//! Any warnings
	ELL_WARNING,
	//! Any errors
	ELL_ERROR,
	//! Critical conditions like hardware problems
	ELL_CRITICAL,
	//! Any condition that demand immediate attention
	ELL_ALERT
};

//! Interface for logging messages, warnings and errors

// {3525E915-CE23-461d-9E54-45E2C01BA9F8}
static const GUID IID_ILOG = 
{ 0x3525e915, 0xce23, 0x461d, { 0x9e, 0x54, 0x45, 0xe2, 0xc0, 0x1b, 0xa9, 0xf8 } };

interface __declspec(uuid("{3525E915-CE23-461d-9E54-45E2C01BA9F8}")) ILog: public IUnknown
{
	virtual HRESULT STDMETHODCALLTYPE Log(const char* text, E_LOG_LEVEL level = ELL_INFO) = 0;
	virtual HRESULT STDMETHODCALLTYPE BeginList(const char* head, E_LOG_LEVEL level = ELL_INFO) = 0;
	virtual HRESULT STDMETHODCALLTYPE EndList(void) = 0;
	virtual HRESULT STDMETHODCALLTYPE Debug(const char* text) = 0;
	virtual HRESULT STDMETHODCALLTYPE Info(const char* text) = 0;
	virtual HRESULT STDMETHODCALLTYPE Warning(const char* text) = 0;
	virtual HRESULT STDMETHODCALLTYPE Error(const char* text) = 0;
	virtual HRESULT STDMETHODCALLTYPE Critical(const char* text) = 0;
	virtual HRESULT STDMETHODCALLTYPE Alert(const char* text) = 0;
};

#endif//I_LOG_H_INCLUDED

