/*-----------------------------------------------------------------------
This source file is a part of DPEngine.

Created:		04.04.2008
LastModified:	04.04.2008

Revision history:
	* Created by Gas
	* Revised by Mike

Disctription:
-----------------------------------------------------------------------*/
#include "ScalarTypes.h"
#include "vector.h"
#include "matrix.h"

#include "DEFINE_FOR_BORLANDC"

inline float ABS(float x) {
	return x>=0?x:-x;
}

Matrix::Matrix() {}

inline Matrix::Matrix(const float array1D[16]) {
	set_value(array1D);
}

inline Matrix::Matrix(const float array2D[4][4]) {
	set_value(array2D);
}

inline Matrix::Matrix(const float array0[4], const float array1[4], const float array2[4], const float array3[4]) {
	set_value(array0, array1, array2, array3);
}

inline Matrix::Matrix(const Vector &vec0, const Vector &vec1, const Vector &vec2, const Vector &vec3) {
	set_value(vec0, vec1, vec2, vec3);
}

Matrix::Matrix(float _0, float _1, float _2, float _3,
			   float _4, float _5, float _6, float _7,
			   float _8, float _9, float _A, float _B,
			   float _C, float _D, float _E, float _F) {
	set_value(
		_0, _1, _2, _3,
		_4, _5, _6, _7,
		_8, _9, _A, _B,
		_C, _D, _E, _F);
}

void Matrix::get_affine_transform(float (&affine_transform)[4][3]) const {
	for (unsigned int i=0; i < 4; i++)
		for (unsigned int j = 0; j < 3; j++)
			affine_transform[i][j] = array2D[i][j];
}

inline Matrix &Matrix::set_value(const float array1D[16]) {
	for (unsigned int i=0; i < 16; i++)
		Matrix::array1D[i]=array1D[i];
	return *this;
}

inline Matrix &Matrix::set_value(const float array2D[4][4]) {
	for (unsigned int i=0; i < 16; i++)
		Matrix::array2D[0][i]=array2D[0][i];
	return *this;
}

inline Matrix &Matrix::set_value(const float array0[4], const float array1[4], const float array2[4], const float array3[4]) {
	unsigned int i;
	for (i=0; i < 4; i++)
		array2D[0][i]=array0[i];
	for (i=0; i < 4; i++)
		array2D[1][i]=array1[i];
	for (i=0; i < 4; i++)
		array2D[2][i]=array2[i];
	for (i=0; i < 4; i++)
		array2D[3][i]=array3[i];
	return *this;
}

inline Matrix &Matrix::set_value(const Vector &vec0, const Vector &vec1, const Vector &vec2, const Vector &vec3) {
	unsigned int i;
	for (i=0; i < 4; i++)
		array2D[0][i]=vec0[i];
	for (i=0; i < 4; i++)
		array2D[1][i]=vec1[i];
	for (i=0; i < 4; i++)
		array2D[2][i]=vec2[i];
	for (i=0; i < 4; i++)
		array2D[3][i]=vec3[i];
	return *this;
}

inline Matrix &Matrix::set_value(float _0, float _1, float _2, float _3,
						 float _4, float _5, float _6, float _7,
						 float _8, float _9, float _A, float _B,
						 float _C, float _D, float _E, float _F) {
	Matrix::_0=_0, Matrix::_1=_1, Matrix::_2=_2, Matrix::_3=_3;
	Matrix::_4=_4, Matrix::_5=_5, Matrix::_6=_6, Matrix::_7=_7;
	Matrix::_8=_8, Matrix::_9=_9, Matrix::_A=_A, Matrix::_B=_B;
	Matrix::_C=_C, Matrix::_D=_D, Matrix::_E=_E, Matrix::_F=_F;
	return *this;
}

Matrix Matrix::inverse() const {
	float mat[4][8]={
		array2D[0][0], array2D[0][1], array2D[0][2], array2D[0][3], 1.0, 0.0, 0.0, 0.0,
		array2D[1][0], array2D[1][1], array2D[1][2], array2D[1][3], 0.0, 1.0, 0.0, 0.0,
		array2D[2][0], array2D[2][1], array2D[2][2], array2D[2][3], 0.0, 0.0, 1.0, 0.0,
		array2D[3][0], array2D[3][1], array2D[3][2], array2D[3][3], 0.0, 0.0, 0.0, 1.0
	};
	float *rows[4]={ mat[0], mat[1], mat[2], mat[3] };
	unsigned int i;
	unsigned int r;
	for (i=0; i < 4; i++) {
		unsigned int row_num=i;
		float major=ABS(rows[i][i]);
		for (r=i+1; r < 4; r++) {
			float cur_ABS=ABS(rows[r][i]);
			if (cur_ABS > major) {
				major=cur_ABS;
				row_num=r;
			}
		}
		if (row_num != i) {
			(INT_PTR &)rows[i]^=(INT_PTR)rows[row_num];//?
			(INT_PTR &)rows[row_num]^=(INT_PTR)rows[i];
			(INT_PTR &)rows[i]^=(INT_PTR)rows[row_num];
		}
		for (r=i+1; r < 4; r++) {
			float factor=rows[r][i]/rows[i][i];
			for (unsigned int c=i; c < 8; c++)
				rows[r][c]-=factor*rows[i][c];
		}
	}
	for (i=3; i > 0; i--)
		for ( r=0; r < i; r++) {
			float factor=rows[r][i]/rows[i][i];
			for (unsigned int c=4; c < 8; c++)
				rows[r][c]-=factor*rows[i][c];
		}
	return Matrix(
		rows[0][4]/rows[0][0], rows[0][5]/rows[0][0], rows[0][6]/rows[0][0], rows[0][7]/rows[0][0],
		rows[1][4]/rows[1][1], rows[1][5]/rows[1][1], rows[1][6]/rows[1][1], rows[1][7]/rows[1][1],
		rows[2][4]/rows[2][2], rows[2][5]/rows[2][2], rows[2][6]/rows[2][2], rows[2][7]/rows[2][2],
		rows[3][4]/rows[3][3], rows[3][5]/rows[3][3], rows[3][6]/rows[3][3], rows[3][7]/rows[3][3]);
}

Matrix Matrix::transpose() const {
	return Matrix(
		array2D[0][0], array2D[1][0], array2D[2][0], array2D[3][0],
		array2D[0][1], array2D[1][1], array2D[2][1], array2D[3][1],
		array2D[0][2], array2D[1][2], array2D[2][2], array2D[3][2],
		array2D[0][3], array2D[1][3], array2D[2][3], array2D[3][3]);
}

Matrix Matrix::operator - () const {
	return Matrix(
		-_0, -_1, -_2, -_3,
		-_4, -_5, -_6, -_7,
		-_8, -_9, -_A, -_B,
		-_C, -_D, -_E, -_F);
}

Vector &Matrix::operator [] (unsigned int i) {
	return (Vector &)array2D[i][0];
}

const Vector &Matrix::operator [] (unsigned int i) const {
	return (Vector &)array2D[i][0];
}

float &Matrix::operator () (unsigned int i) {
	return array1D[i];
}

const float &Matrix::operator () (unsigned int i) const {
	return array1D[i];
}

float &Matrix::operator () (unsigned int i, unsigned int j) {
	return array2D[i][j];
}

const float &Matrix::operator () (unsigned int i, unsigned int j) const {
	return array2D[i][j];
}

Matrix &Matrix::operator += (float _) {
	for (unsigned int i=0; i < 16; i++)
		this->operator () (i) += _;
	return *this;
}

Matrix &Matrix::operator -= (float _) {
	for (unsigned int i=0; i < 16; i++)
		this->operator () (i) -= _;
	return *this;
}

Matrix &Matrix::operator *= (float _) {
	for (unsigned int i=0; i < 16; i++)
		this->operator () (i) *= _;
	return *this;
}

Matrix &Matrix::operator /= (float _) {
	for (unsigned int i=0; i < 16; i++)
		this->operator () (i) /= _;
	return *this;
}

Matrix &Matrix::operator += (const Matrix &mat) {
	for (unsigned int i=0; i < 16; i++)
		this->operator () (i) += mat(i);
	return *this;
}

Matrix &Matrix::operator -= (const Matrix &mat) {
	for (unsigned int i=0; i < 16; i++)
		this->operator () (i) -= mat(i);
	return *this;
}

Matrix &Matrix::operator *= (const Matrix &mat) {
	return *this=*this*mat;//?
}

Matrix operator + (const Matrix &mat, float _) {
	return Matrix(mat)+=_;
}

Matrix operator - (const Matrix &mat, float _) {
	return Matrix(mat)-=_;
}

Matrix operator * (const Matrix &mat, float _) {
	return Matrix(mat)*=_;
}

Matrix operator / (const Matrix &mat, float _) {
	return Matrix(mat)/=_;
}

Matrix operator + (const Matrix &mat1, const Matrix &mat2) {
	return Matrix(mat1)+=mat2;
}

Matrix operator - (const Matrix &mat1, const Matrix &mat2) {
	return Matrix(mat1)-=mat2;
}

Matrix operator * (const Matrix &mat1, const Matrix &mat2) {
	Matrix product(
		0.0, 0.0, 0.0, 0.0,
		0.0, 0.0, 0.0, 0.0,
		0.0, 0.0, 0.0, 0.0,
		0.0, 0.0, 0.0, 0.0);
	for (unsigned int i=0; i < 4; i++)
		for (unsigned int j=0; j < 4; j++)
			for (unsigned int k=0; k < 4; k++)
				product(i, j)+=mat1(i, k)*mat2(k, j);
	return product;
}

Matrix identity() {
	return scale(1.0);
}

Matrix scale(float _) {
	return scale(Vector(_, _, _));
}

Matrix scale(const Vector &vec) {
	return Matrix(
		vec.X, 0.0  , 0.0  , 0.0  ,
		0.0  , vec.Y, 0.0  , 0.0  ,
		0.0  , 0.0  , vec.Z, 0.0  ,
		0.0  , 0.0  , 0.0  , vec.W);
}

Matrix translate(const Vector &vec) {
	return Matrix(
		1.0  , 0.0  , 0.0  , 0.0,
		0.0  , 1.0  , 0.0  , 0.0,
		0.0  , 0.0  , 1.0  , 0.0,
		vec.X, vec.Y, vec.Z, 1.0);
}

Matrix rotate(Vector axis, float angle) {
	axis.normalize3();
	float
		&x=axis.X,
		&y=axis.Y,
		&z=axis.Z,
		sin_angle=sin(angle),
		cos_angle=cos(angle);
	return Matrix(
		(1.f-x*x)*cos_angle+x*x         , z*sin_angle+x*y*(1.f-cos_angle) , x*z*(1.f-cos_angle)-y*sin_angle , 0.f,
		x*y*(1.f-cos_angle)-z*sin_angle , (1.0f-y*y)*cos_angle+y*y        , y*z*(1.f-cos_angle)+x*sin_angle , 0.f,
		x*z*(1.f-cos_angle)+y*sin_angle , y*z*(1.f-cos_angle)-x*sin_angle , (1.f-z*z)*cos_angle+z*z         , 0.f,
		0.f                             , 0.f                             , 0.f                             , 1.f);
}
Matrix rotate(Vector rotation){
	float
		cr = cos( rotation.X ),
		sr = sin( rotation.X ),
		cp = cos( rotation.Y ),
		sp = sin( rotation.Y ),
		cy = cos( rotation.Z ),
		sy = sin( rotation.Z ),
		srsp = sr*sp,
		crsp = cr*sp;

	return Matrix(
		cp*cy, cp*sy, -sp, 0.0f,
		srsp*cy-cr*sy, srsp*sy+cr*cy, sr*cp, 0.0f,
		crsp*cy+sr*sy, crsp*sy-sr*cy, cr*cp, 0.0f,
		0.0f, 0.0f, 0.0f, 1.0f);
}

Vector Matrix::get_rotate() const{
	float Y = -asinf(_02);
	float C = cosf(Y);

	float rotx, roty, X, Z;

	if (fabsf(C) > 0.0005f)
	{
		rotx = _22 / C;
		roty = _12 / C;
		X = atan2f( roty, rotx );
		rotx = _00 / C;
		roty = _01 / C;
		Z = atan2f( roty, rotx );
	}
	else
	{
		X = 0.f;
		rotx = _11;
		roty = -_10;
		Z = atan2f( roty, rotx );
	}
	
	while (X < 0.f) X+= 2*(float)PI;
	while (Y < 0.f) Y+= 2*(float)PI;
	while (Z < 0.f) Z+= 2*(float)PI;
	if (X > 2*(float)PI) X = X - 2*(float)PI*(int)( X/(2*(float)PI) );
	if (Y > 2*(float)PI) Y = Y - 2*(float)PI*(int)( Y/(2*(float)PI) );
	if (Z > 2*(float)PI) Z = Z - 2*(float)PI*(int)( Z/(2*(float)PI) );

	return Vector(X,Y,Z);
}
#include "UNDEF_FOR_BORLANDC"
