/*-----------------------------------------------------------------------
This source file is a part of DPEngine.

Created:		04.04.2008
LastModified:	17.11.2008

Revision history:
	* Created by Gas
	* Revised by Gas

Disctription:
-----------------------------------------------------------------------*/
#ifndef MATRIX_H
#define MATRIX_H

#include <windows.h> // for INT_PTR
#include <Math.h>
#include "ScalarTypes.h"
#include "vector.h"

struct Matrix {
	union{
		float array1D[16];
		float array2D[4][4];
		struct {
			float
				_0, _1, _2, _3,
				_4, _5, _6, _7,
				_8, _9, _A, _B,
				_C, _D, _E, _F;
		};
		struct {
			float
				_00, _01, _02, _03,
				_10, _11, _12, _13,
				_20, _21, _22, _23,
				_30, _31, _32, _33;
		};
		struct {
			float
				a, b, c, p,
				d, e, f, q,
				g, h, i, r,
				x, y, z, w;
		};
	};
	Matrix();
	inline Matrix(const float array1D[16]);//? inline
	inline Matrix(const float array2D[4][4]);
	inline Matrix(const float array0[4], const float array1[4], const float array2[4], const float array3[4]);
	inline Matrix(const Vector &vec0, const Vector &vec1, const Vector &vec2, const Vector &vec3);
	Matrix(
		float _0, float _1, float _2, float _3,
		float _4, float _5, float _6, float _7,
		float _8, float _9, float _A, float _B,
		float _C, float _D, float _E, float _F);
	void get_affine_transform(float (&affine_transform)[4][3]) const;
	inline Matrix &set_value(const float array1D[16]);
	inline Matrix &set_value(const float array2D[4][4]);
	inline Matrix &set_value(const float array0[4], const float array1[4], const float array2[4], const float array3[4]);
	inline Matrix &set_value(const Vector &vec0, const Vector &vec1, const Vector &vec2, const Vector &vec3);
	inline Matrix &set_value(
		float _0, float _1, float _2, float _3,
		float _4, float _5, float _6, float _7,
		float _8, float _9, float _A, float _B,
		float _C, float _D, float _E, float _F);
	Matrix inverse() const;
	Matrix transpose() const;
	Vector get_rotate() const;
	Matrix operator - () const;
	Vector &operator [] (unsigned int i);
	const Vector &operator [] (unsigned int i) const;
	float &operator () (unsigned int i);
	const float &operator () (unsigned int i) const;
	float &operator () (unsigned int i, unsigned int j);
	const float &operator () (unsigned int i, unsigned int j) const;
	Matrix &operator += (float _);
	Matrix &operator -= (float _);
	Matrix &operator *= (float _);
	Matrix &operator /= (float _);
	Matrix &operator += (const Matrix &mat);
	Matrix &operator -= (const Matrix &mat);
	Matrix &operator *= (const Matrix &mat);
	friend Matrix operator + (const Matrix &mat, float _);
	friend Matrix operator - (const Matrix &mat, float _);
	friend Matrix operator * (const Matrix &mat, float _);
	friend Matrix operator / (const Matrix &mat, float _);
	friend Matrix operator + (const Matrix &mat1, const Matrix &mat2);
	friend Matrix operator - (const Matrix &mat1, const Matrix &mat2);
	friend Matrix operator * (const Matrix &mat1, const Matrix &mat2);
	friend Matrix identity();
	friend Matrix scale(float _);
	friend Matrix scale(const Vector &vec);
	friend Matrix translate(const Vector &vec);
	friend Matrix rotate(Vector axis, float angle);
	friend Matrix rotate(Vector rotation);	
};

#endif