/*-----------------------------------------------------------------------
This source file is a part of DPEngine.

Created:		16.06.2008
LastModified:	16.06.2008

Revision history:
	* Created by Mike
	* Revised by

Disctription:
-----------------------------------------------------------------------*/
#include "plane.h"

namespace DPE_Scenes
{

TPlane::TPlane()
{
	Normal = Vector(0.f, 0.f, 0.f, 1.f);
	Distance = 0.0f;
}
TPlane::TPlane(const Vector &vNormal, const Vector &vPoint)
{
	Normal = vNormal;
	Normal.normalize3();
	Distance = - Normal.dot3(vPoint);	
}
TPlane::TPlane(const Vector &vA, const Vector &vB, const Vector &vC)
{
	Normal = (vA - vB).cross(vC - vB);	
	Normal.normalize3();
	Distance = - Normal.dot3(vB);	
}
bool TPlane::LineIntersected(const Vector &vA, const Vector &vB)
{
	if ( (Normal.dot3(vA) + Distance) * (Normal.dot3(vB) + Distance) >= 0) return false;            
	return true;           
}
Vector TPlane::IntersectionPoint(const Vector &vA, const Vector &vB)
{
	Vector vPoint, vLineDir;  
	float Numerator = 0.0f, Denominator = 0.0f, dist = 0.0f;

	vLineDir = (vB - vA).normalize3();

	Numerator = - Normal.dot3(vA) + Distance;
	Denominator = Normal.dot3(vLineDir);

	if( Denominator == 0.0f) return vA;          
	dist = Numerator / Denominator;          

	vPoint.X = (vA.X + (vLineDir.X * dist));
	vPoint.Y = (vA.Y + (vLineDir.Y * dist));
	vPoint.Z = (vA.Z + (vLineDir.Z * dist));

	return vPoint;                
}
float TPlane::DistanceToPoint(const Vector &vPoint)
{
	return Normal.dot3(vPoint)+Distance;
}

}