#include <math.h>
#include "math\point.h"

//bool PointInsideLine(const float (&point)[2], const float (&line_begin)[2], const float (&line_end)[2], float half_width_sq)
//{
//	// line direction
//	const float line_dir[2] = { line_end[0] - line_begin[0], line_end[1] - line_begin[1] };
//	const float line_length_sq = line_dir[0] * line_dir[0] + line_dir[1] * line_dir[1];
//	const float v[2] = { point[0] - line_begin[0], point[1] - line_begin[1] };
//
//	// project point onto line
//	const float a = (v[0] * line_dir[0] + v[1] * line_dir[1]) / line_length_sq;
//	if (a < 0 || a > 1)
//		return false;
//
//	// compute ortho component
//	const float ortho[2] =
//	{
//		v[0] - line_dir[0] * a,
//		v[1] - line_dir[1] * a
//	};
//
//	// inside if length(ortho) <= line_width / 2
//	return ortho[0] * ortho[0] + ortho[1] * ortho[1] <= half_width_sq;
//}
bool PointInsideLine(const float (&point)[2], const float (&line_begin)[2], const float (&line_end)[2], float half_width_sq)
{
	// line direction and normal
	const float line_dir[2] = { line_end[0] - line_begin[0], line_end[1] - line_begin[1] };
	const float line_normal[2] = { -line_dir[1], +line_dir[0] };
	const float v[2] = { point[0] - line_begin[0], point[1] - line_begin[1] };

	// project point onto line normal
	const float normal_length_sq = line_normal[0] * line_normal[0] + line_normal[1] * line_normal[1];
	float a = (v[0] * line_normal[0] + v[1] * line_normal[1]);
	if (a * a / normal_length_sq > half_width_sq)
		return false;

	// project point onto line
	const float line_length_sq = line_dir[0] * line_dir[0] + line_dir[1] * line_dir[1];
	a = (v[0] * line_dir[0] + v[1] * line_dir[1]) / line_length_sq;
	return a >= 0 && a <= 1;
}

template<unsigned int stride>
static inline const float (&GetVertex(const float (*vertices)[2], unsigned long idx))[2]
{
	const unsigned char (&raw_data)[stride] = reinterpret_cast<const unsigned char (*)[stride]>(vertices)[idx];
	return reinterpret_cast<const float (&)[2]>(raw_data);
}

template<unsigned int stride>
bool PointInsidePolygon(const float (&point)[2], const float (*vertices)[2], unsigned long vcount)
{
	const float (*&polygon_vertices)[2] = vertices;
	float prev_cross = 0;	// (prev_cross * cross < 0) == false first time
	const float (*v0)[2] = &GetVertex<stride>(vertices, 0), (*v1)[2];
	for (unsigned long i = 1; i < vcount; i++, v0 = v1)
	{
		v1 = &GetVertex<stride>(vertices, i);
		const float
			edge[2] = { (*v1)[0] - (*v0)[0], (*v1)[1] - (*v0)[1] },
			point_dir[2] = { point[0] - (*v0)[0], point[1] - (*v0)[1] },
			cross = point_dir[0] * edge[1] - point_dir[1] * edge[0];
		if (prev_cross * cross < 0)
			return false;
		prev_cross = cross;
	}
	return true;
}

template bool PointInsidePolygon<12>(const float (&point)[2], const float (*vertices)[2], unsigned long vcount);