/*-----------------------------------------------------------------------
This source file is a part of DPEngine.

Created:		04.04.2008
LastModified:	04.04.2008

Revision history:
	* Created by Gas
	* Revised by Mike

Disctription:
-----------------------------------------------------------------------*/
#include "ScalarTypes.h"
#include "vector.h"
#include "matrix.h"

Vector::Vector() {}

Vector::Vector(const Vector &vec) {
	_0=vec._0;
	_1=vec._1;
	_2=vec._2;
	_3=vec._3;
}

Vector::Vector(const float array[4]):
_0(array[0]), _1(array[1]), _2(array[2]), _3(array[3]) {}

Vector::Vector(float _):
_0(_), _1(_), _2(_), _3(1.0) {}

Vector::Vector(float _0, float _1, float _2):
_0(_0), _1(_1), _2(_2), _3(1.0) {}

Vector::Vector(float _0, float _1, float _2, float _3):
_0(_0), _1(_1), _2(_2), _3(_3) {}

float Vector::sqr_norm3() const {
	return _0*_0+_1*_1+_2*_2;
}

float Vector::norm3() const {
	return sqrt(this->sqr_norm3());
}

Vector &Vector::normalize3() {
	return *this/=this->norm3();//?
}

Vector &Vector::homogenize3() {
	this->X/=this->W;
	this->Y/=this->W;
	this->Z/=this->W;
	return *this;
}
Vector &Vector::homogenize4() {
	return *this/=this->W;//?
}

float &Vector::operator [] (unsigned int i) {
	return array[i];
}

const float &Vector::operator [] (unsigned int i) const {
	return array[i];
}

Vector Vector::operator - () const {
	return Vector(-_0, -_1, -_2, -_3);
}

Vector &Vector::operator += (float _) {
	_0+=_;
	_1+=_;
	_2+=_;
	_3+=_;
	return *this;
}

Vector &Vector::operator -= (float _) {
	_0-=_;
	_1-=_;
	_2-=_;
	_3-=_;
	return *this;
}

Vector &Vector::operator *= (float _) {
	_0*=_;
	_1*=_;
	_2*=_;
	_3*=_;
	return *this;
}

Vector &Vector::operator /= (float _) {
	_0/=_;
	_1/=_;
	_2/=_;
	_3/=_;
	return *this;
}

Vector &Vector::operator += (const Vector &vec) {
	_0+=vec._0;
	_1+=vec._1;
	_2+=vec._2;
	_3+=vec._3;
	return *this;
}

Vector &Vector::operator -= (const Vector &vec) {
	_0-=vec._0;
	_1-=vec._1;
	_2-=vec._2;
	_3-=vec._3;
	return *this;
}

Vector &Vector::operator *= (const Vector &vec) {
	_0*=vec._0;
	_1*=vec._1;
	_2*=vec._2;
	_3*=vec._3;
	return *this;
}

Vector &Vector::operator /= (const Vector &vec) {
	_0/=vec._0;
	_1/=vec._1;
	_2/=vec._2;
	_3/=vec._3;
	return *this;
}

Vector &Vector::operator *= (const Matrix &mat) {
	return *this=*this*mat;//?
}

Vector operator + (const Vector &vec, float _) {
	return Vector(vec)+=_;
}

Vector operator - (const Vector &vec, float _) {
	return Vector(vec)-=_;
}

Vector operator * (const Vector &vec, float _) {
	return Vector(vec)*=_;
}

Vector operator / (const Vector &vec, float _) {
	return Vector(vec)/=_;
}

Vector operator + (const Vector &vec1, const Vector &vec2) {
	return Vector(vec1)+=vec2;
}

Vector operator - (const Vector &vec1, const Vector &vec2) {
	return Vector(vec1)-=vec2;
}

Vector operator * (const Vector &vec1, const Vector &vec2) {
	return Vector(vec1)*=vec2;
}

Vector operator / (const Vector &vec1, const Vector &vec2) {
	return Vector(vec1)/=vec2;
}

Vector operator * (const Vector &vec, const Matrix &mat) {
	Vector product(0.0, 0.0, 0.0, 0.0);
	for (unsigned int i=0; i < 4; i++)
		for (unsigned int j=0; j < 4; j++)
			product[i]+=vec[j]*mat(j, i);
	return product;
}

bool operator == (const Vector &vec1, const Vector &vec2)
{
	if (vec1.X == vec2.X &&
		vec1.Y == vec2.Y &&
		vec1.Z == vec2.Z &&
		vec1.W == vec2.W) return true;
	return false;
}
bool operator != (const Vector &vec1, const Vector &vec2)
{
	return !(operator ==(vec1, vec2));
}

float Vector::dot3(const Vector &vec) const {
	return this->X*vec.X+this->Y*vec.Y+this->Z*vec.Z;
}

float Vector::dot4(const Vector &vec) const {
	return this->X*vec.X+this->Y*vec.Y+this->Z*vec.Z+this->W*vec.W;
}

Vector Vector::cross(const Vector &vec) const {
	Vector product;
	product.X=this->Y*vec.Z-vec.Y*this->Z;
	product.Y=-(this->X*vec.Z-vec.X*this->Z);
	product.Z=this->X*vec.Y-vec.X*this->Y;
	product.W=1;
	return product;
}