/*-----------------------------------------------------------------------
This source file is a part of DPEngine.

Created:		04.04.2008
LastModified:	04.04.2008

Revision history:
	* Created by Gas
	* Revised by Mike

Disctription:
-----------------------------------------------------------------------*/
#ifndef VECTOR_H
#define VECTOR_H

#include <Math.h>
#include "ScalarTypes.h"

struct Matrix;
struct Vector {
	union{
		float array[4];
		struct {
			float _0, _1, _2, _3;
		};
		struct {
			float X, Y, Z, W;
		};
		struct {
			float R, G, B, A;
		};
	};
	Vector();// ? inline
	Vector(const Vector &vec);
	Vector(const float array[4]);
	Vector(float _);
	Vector(float _0, float _1, float _2);
	Vector(float _0, float _1, float _2, float _3);
	float sqr_norm3() const;
	float norm3() const;
	Vector &normalize3();
	Vector &homogenize3();
	Vector &homogenize4();
	float &operator [] (unsigned int i);
	const float &operator [] (unsigned int i) const;
	Vector operator - () const;
	Vector &operator += (float _);
	Vector &operator -= (float _);
	Vector &operator *= (float _);
	Vector &operator /= (float _);
	Vector &operator += (const Vector &vec);
	Vector &operator -= (const Vector &vec);
	Vector &operator *= (const Vector &vec);
	Vector &operator /= (const Vector &vec);
	Vector &operator *= (const Matrix &mat);
	friend Vector operator + (const Vector &vec, float _);
	friend Vector operator - (const Vector &vec, float _);
	friend Vector operator * (const Vector &vec, float _);
	friend Vector operator / (const Vector &vec, float _);
	friend Vector operator + (const Vector &vec1, const Vector &vec2);
	friend Vector operator - (const Vector &vec1, const Vector &vec2);
	friend Vector operator * (const Vector &vec1, const Vector &vec2);
	friend Vector operator / (const Vector &vec1, const Vector &vec2);
	friend Vector operator * (const Vector &vec, const Matrix &mat);
	friend bool operator == (const Vector &vec1, const Vector &vec2);
	friend bool operator != (const Vector &vec1, const Vector &vec2);
	float dot3(const Vector &vec) const;
	float dot4(const Vector &vec) const;
	Vector cross(const Vector &vec) const;
};


#endif