#include "GameMap.h"

vector<TDB_Obj>	GameMap::obj_base;

void GameMap::_loadobjectsdb()
{
	if(obj_base.size()>0) return;

	obj_base.clear();

	IFileSystem *_pFS;
	pEngineCore->FileSystem(&_pFS);

	 if (FAILED(_pFS->FileExists(DATA_PATH("mobjects.db"))))
		return;

	 IFile *file;
	 _pFS->GetFile(DATA_PATH("mobjects.db"),&file);

	 unsigned int read;
	 int count;

	 file->Read((void*)&count,sizeof(int),read);

	 for (int i = 0; i < count; i++)
	 {
		TDB_Obj _t;
		file->Read((void*)&_t,sizeof(TDB_Obj),read);
		if(_t.special==-1)
		pMainResManager->Load(DATA_PATH(("map_objects\\"+string(_t.tex_name)).c_str()),_t.tex_name);
		obj_base.push_back(_t);
	 }


	 file->Release();
	 _pFS->Release();

	pEngineCore->Con_Write("Obj_base loaded.");
}

vector<TRespawnData>* GameMap::GetRespawns()
{
	return &respawns;
}

void GameMap::_freemap()
{
	if(sprites.size()>0)
		for(int i = 0; i<sprites.size(); i++)
		{
			sprites[i].sprite->Free();
			sprites[i].sprite->Release();

			if(sprites[i].body)
			{
				sprites[i].body->Free();
				sprites[i].body->Release();
			}
		}
		sprites.clear();
}

void GameMap::_loadmap(const char *name)
{
	IFileSystem *_pFS;
	pEngineCore->FileSystem(&_pFS);

	IFile *file;

	_pFS->GetFile(DATA_PATH("maps\\"+string(name)),&file,(FS_OPEN_FLAGS)(FS_READ | FS_BINARY));

	unsigned int read;

	if(FAILED(file->Read((void*)&_header,sizeof(TMapHeader),read)))
		return;
	
	respawns.clear();

	for (int i = 0; i < _header.count; i++)
	{
		TMapObj t;
		file->Read((void*)&t,sizeof(TMapObj),read);
		
		if(obj_base[t.idx].special!=-1)
		{
			TRespawnData tr;
			tr.type = obj_base[t.idx].special;
			tr.x = t.XCoord[0];
			tr.y = t.YCoord[0];
			respawns.push_back(tr);
			continue;
		}
		
		ISprite *sprite;
		pMan2D->CreateSprite(string("id:"+IntToStr(i)).c_str(),sprite);

		if(t.idx==53 )
		{
			sprite->SetZOrder(50);
		}

		sprite->SetTexture(obj_base[t.idx].tex_name);

		sprite->SetDimension(t.w,t.h);

		float scalex = (float)t.w/obj_base[t.idx].width, scaley = (float)t.h/obj_base[t.idx].height;

		ICollisionShape::TPolygonDesc pShapeDesc;
		
		pShapeDesc.vertexCount = 0;

		pShapeDesc.density = t.density * 20;

		pShapeDesc.friction = 1.f;

		for(int j=0; j<8; j++)
		{
			if(obj_base[t.idx].borderX[j]!=-1&&obj_base[t.idx].borderY[j]!=-1)
			{
				pShapeDesc.vertices[j].x = obj_base[t.idx].borderX[j]*scalex;
				pShapeDesc.vertices[j].y = obj_base[t.idx].borderY[j]*scaley;
				pShapeDesc.vertexCount++;
			}
		}

		sprite->SetShape(&pShapeDesc);

		if(t.ground==0) 
		{
			static int gz = 0;
			sprite->SetZOrder(gz++);
			sprite->SetCameraOffsetScale(Vector2D_Float(0.5,0.5));
		}
		else
		if(t.ground==1) 
		{
			static int gone = 500;
			sprite->SetZOrder(gone++);
		}
		else
		if(t.ground==2) 
		{		
			static int gtwo = 1000;
			sprite->SetZOrder(gtwo++);
			sprite->SetCameraOffsetScale(Vector2D_Float(2.f,2.f));
		}

		IBody *body = NULL;

		if(t.ispassable)
		{
			sprite->SetPosition(t.XCoord[0],t.YCoord[0]);
		}
		else
		{
			sprite->SetPosition(0,0);

			IBody::TBodyDesc body_desc;

			body_desc.position.x = t.XCoord[0];
			body_desc.position.y = t.YCoord[0];

			pMan2D->CreateCollidableBody(string("id:"+IntToStr(i)).c_str(),&body_desc,body);

			body->AppendShapes(sprite);

			if(!t.isstatic)
				body->SetMassFromShapes();
		}

		sprites.push_back(TInagemePhysSprite(sprite,body));
	}

	file->Release();

	_pFS->Release();

	pEngineCore->Con_Write(("Map \""+string(name)+"\" loaded.").c_str());
}

void GameMap::DrawBG()
{
	Point2D_RGBA pnts[4] = {
		Point2D_RGBA(Point2D_Int(0,0),ColorRGBA((float)255/GetRValue(_header.color1),(float)255/GetGValue(_header.color1),(float)255/GetBValue(_header.color1),1.f)),
		Point2D_RGBA(Point2D_Int(800,0),ColorRGBA((float)255/GetRValue(_header.color1),(float)255/GetGValue(_header.color1),(float)255/GetBValue(_header.color1),1.f)),
		Point2D_RGBA(Point2D_Int(800,600),ColorRGBA((float)255/GetRValue(_header.color2),(float)255/GetGValue(_header.color2),(float)255/GetBValue(_header.color2),1.f)),
		Point2D_RGBA(Point2D_Int(0,600),ColorRGBA((float)255/GetRValue(_header.color2),(float)255/GetGValue(_header.color2),(float)255/GetBValue(_header.color2),1.f))};

	pMan2D->DrawRectangle(pnts,true);
}

GameMap::GameMap(IEngineCore *eng, const char *name)
{
	pEngineCore = eng;
	
	_header.color1 = 0;
	_header.color2 = 0;
	_header.count  = 0;

	pEngineCore->PluginByType(reinterpret_cast<ICommonPlugin **>(&pSceneManFact),Pl_SCENES);
	pSceneManFact->GetResourceManager(pMainResManager);
	pSceneManFact->GetManager2D(pMan2D);

	_loadobjectsdb();
	_loadmap(name);

}

GameMap::~GameMap()
{
	_freemap();
	pMan2D->Release();
	pMainResManager->Release();
	pSceneManFact->Release();
}
