#ifndef _BIPED_DESC_H_INCLUDED_
#define _BIPED_DESC_H_INCLUDED_

#if defined(_MSC_VER) && (_MSC_VER > 1000)
#pragma once
#endif	/* _MSC_VER > 1000 */

#include <DPE_Application_Include.h>

#include <assert.h>

class CBipedDesc
{
public:
	CBipedDesc();
	virtual ~CBipedDesc();

	void setMotorTorque(float);
	void setMotorSpeed(float);
	void setDensity(float);
	void setFriction(float);
	void setRestitution(float);
	void setLinearDamping(float);
	void setAngularDamping(float);
	void enableLimit();
	void disableLimit();
	void setLimit(bool);
	void enableMotor();
	void disableMotor();
	void setMotor(bool);
	void setGroupIndex(signed short);
	void setPosition(int, int);
	void setPosition(const Point2D_Int &);
	void setBullet(bool);
	void setFixedRotation(bool);

	// For group index
	static signed short NEXT_GROUP;

	IBody::TBodyDesc					HeadDesc, ChestDesc, LThighDesc, RThighDesc, LCalfDesc, RCalfDesc, LFootDesc, RFootDesc,
										LUpperArmDesc, RUpperArmDesc, LForearmDesc, RForearmDesc;
	
	ICollisionShape::TPolygonDesc		ChestPoly,
										LThighPoly, RThighPoly, LCalfPoly, RCalfPoly, LFootPoly, RFootPoly,										
										LUpperArmPoly, RUpperArmPoly, LForearmPoly, RForearmPoly;

	ICollisionShape::TCircleDesc		HeadCirc;

	_2D::IObject						*HeadShape, *ChestShape,
										*LThighShape, *RThighShape, *LCalfShape, *RCalfShape, *LFootShape, *RFootShape,
										*LUpperArmShape, *RUpperArmShape, *LForearmShape, *RForearmShape;

	IRevoluteJoint::TRevoluteJointDesc	NeckDesc, LHipDesc, RHipDesc,
										LAnkleDesc, RAnkleDesc, LKneeDesc, RKneeDesc,
										LShoulderDesc, RShoulderDesc, LElbowDesc, RElbowDesc;

	void humanVertices();
	void humanPositions();
	void humanJoints();

	void alienVertices();
	void alienPositions();
	void alienJoints();

	void Free();

	void setHead(IN _2D::IObject *obj);
	void setChest(IN _2D::IObject *obj);
	void setLThigh(IN _2D::IObject *obj);
	void setRThigh(IN _2D::IObject *obj);
	void setLCalf(IN _2D::IObject *obj);
	void setRCalf(IN _2D::IObject *obj);
	void setLFoot(IN _2D::IObject *obj);
	void setRFoot(IN _2D::IObject *obj);
	void setLUpperArm(IN _2D::IObject *obj);
	void setRUpperArm(IN _2D::IObject *obj);
	void setLForearm(IN _2D::IObject *obj);
	void setRForearm(IN _2D::IObject *obj);
};


#endif	/* _BIPED_DESC_H_INCLUDED_ */