#include "biped.h"

CBiped::CBiped(IManager2D *man, CBipedDesc *desc, const Point2D_Int &position) :
	LFoot(NULL), RFoot(NULL), LCalf(NULL), RCalf(NULL), LThigh(NULL), RThigh(NULL),
	Chest(NULL), Head(NULL),
	LUpperArm(NULL), RUpperArm(NULL), LForearm(NULL), RForearm(NULL),

	LAnkle(NULL), RAnkle(NULL), LKnee(NULL), RKnee(NULL), LHip(NULL), RHip(NULL), 
	Neck(NULL),
	LShoulder(NULL), RShoulder(NULL), LElbow(NULL), RElbow(NULL)
{
	IBody::TBodyDesc bd;

	descr = desc;

	char res[16];
	sprintf(res, "%d", descr->NEXT_GROUP);

	mName = std::string("biped_") + res;

	// create body parts
	// head
	{
		bd = descr->HeadDesc;
		bd.position += position;
		man->CreateCollidableBody((mName+"_head").c_str(), &bd, Head);
		assert(Head);
		Head->AppendShapes(descr->HeadShape);
	}
	// chest
	{
		bd = descr->ChestDesc;
		bd.position += position;
		man->CreateCollidableBody((mName+"_chest").c_str(), &bd, Chest);
		assert(Chest);
		Chest->AppendShapes(descr->ChestShape);
	}
	// left thigh
	{
		bd = descr->LThighDesc;
		bd.position += position;
		man->CreateCollidableBody((mName+"_left_thigh").c_str(), &bd, LThigh);
		assert(LThigh);
		LThigh->AppendShapes(descr->LThighShape);
	}
	// right thigh
	{
		bd = descr->RThighDesc;
		bd.position += position;
		man->CreateCollidableBody((mName+"_right_thigh").c_str(), &bd, RThigh);
		assert(RThigh);
		RThigh->AppendShapes(descr->RThighShape);
	}
	// left calf
	{
		bd = descr->LCalfDesc;
		bd.position += position;
		man->CreateCollidableBody((mName+"_left_calf").c_str(), &bd, LCalf);
		assert(LCalf);
		LCalf->AppendShapes(descr->LCalfShape);
	}
	// right calf
	{
		bd = descr->RCalfDesc;
		bd.position += position;
		man->CreateCollidableBody((mName+"_right_calf").c_str(), &bd, RCalf);
		assert(RCalf);
		RCalf->AppendShapes(descr->RCalfShape);
	}
	// left foot
	{
		bd = descr->LFootDesc;
		bd.position += position;
		man->CreateCollidableBody((mName+"_left_foot").c_str(), &bd, LFoot);
		assert(LFoot);
		LFoot->AppendShapes(descr->LFootShape);
	}
	// right foot
	{
		bd = descr->RFootDesc;
		bd.position += position;
		man->CreateCollidableBody((mName+"_right_foot").c_str(), &bd, RFoot);
		assert(RFoot);
		RFoot->AppendShapes(descr->RFootShape);
	}
	// left upperarm
	{
		bd = descr->LUpperArmDesc;
		bd.position += position;
		man->CreateCollidableBody((mName+"_left_upperarm").c_str(), &bd, LUpperArm);
		assert(LUpperArm);
		LUpperArm->AppendShapes(descr->LUpperArmShape);
	}
	// right upperarm
	{
		bd = descr->RUpperArmDesc;
		bd.position += position;
		man->CreateCollidableBody((mName+"_right_upperarm").c_str(), &bd, RUpperArm);
		assert(RUpperArm);
		RUpperArm->AppendShapes(descr->RUpperArmShape);
	}
	// left forearm
	{
		bd = descr->LForearmDesc;
		bd.position += position;
		man->CreateCollidableBody((mName+"_left_forearm").c_str(), &bd, LForearm);
		assert(LForearm);
		LForearm->AppendShapes(descr->LForearmShape);
	}
	// right upperarm
	{
		bd = descr->RForearmDesc;
		bd.position += position;
		man->CreateCollidableBody((mName+"_right_forearm").c_str(), &bd, RForearm);
		assert(RForearm);
		RForearm->AppendShapes(descr->RForearmShape);
	}
	
	// link body parts
	descr->NeckDesc.pBody1	= Head;
	descr->NeckDesc.pBody2	= Chest;
	descr->LHipDesc.pBody1	= Chest;
	descr->LHipDesc.pBody2	= LThigh;
	descr->RHipDesc.pBody1	= Chest;
	descr->RHipDesc.pBody2	= RThigh;
	descr->LKneeDesc.pBody1	= LThigh;
	descr->LKneeDesc.pBody2	= LCalf;
	descr->RKneeDesc.pBody1	= RThigh;
	descr->RKneeDesc.pBody2	= RCalf;
	descr->LAnkleDesc.pBody1	= LCalf;
	descr->LAnkleDesc.pBody2	= LFoot;
	descr->RAnkleDesc.pBody1	= RCalf;
	descr->RAnkleDesc.pBody2	= RFoot;
	descr->RKneeDesc.pBody2	= RCalf;
	descr->LShoulderDesc.pBody1	= Chest;
	descr->LShoulderDesc.pBody2	= LUpperArm;
	descr->RShoulderDesc.pBody1	= Chest;
	descr->RShoulderDesc.pBody2	= RUpperArm;
	descr->LElbowDesc.pBody1	= LUpperArm;
	descr->LElbowDesc.pBody2	= LForearm;
	descr->RElbowDesc.pBody1	= RUpperArm;
	descr->RElbowDesc.pBody2	= RForearm;

	// create joints
	// Neck
	man->CreateRevoluteJoint((mName+"_neck").c_str(), &desc->NeckDesc, Neck);
	assert(Neck);
	// left hip
	man->CreateRevoluteJoint((mName+"_left_hip").c_str(), &desc->LHipDesc, LHip);
	assert(LHip);
	// right hip
	man->CreateRevoluteJoint((mName+"_right_hip").c_str(), &desc->RHipDesc, RHip);
	assert(RHip);
	// left knee
	man->CreateRevoluteJoint((mName+"_left_knee").c_str(), &desc->LKneeDesc, LKnee);
	assert(LKnee);
	// right knee
	man->CreateRevoluteJoint((mName+"_right_knee").c_str(), &desc->RKneeDesc, RKnee);
	assert(RKnee);
	// left ankle
	man->CreateRevoluteJoint((mName+"_left_ankle").c_str(), &desc->LAnkleDesc, LAnkle);
	assert(LAnkle);
	// right ankle
	man->CreateRevoluteJoint((mName+"_right_ankle").c_str(), &desc->RAnkleDesc, RAnkle);
	assert(RAnkle);
	// left shoulder
	man->CreateRevoluteJoint((mName+"_left_shoulder").c_str(), &desc->LShoulderDesc, LShoulder);
	assert(LShoulder);
	// right shoulder
	man->CreateRevoluteJoint((mName+"_right_shoulder").c_str(), &desc->RShoulderDesc, RShoulder);
	assert(RShoulder);
	// left elbow
	man->CreateRevoluteJoint((mName+"_left_elbow").c_str(), &desc->LElbowDesc, LElbow);
	assert(LElbow);
	// right elbow
	man->CreateRevoluteJoint((mName+"_right_elbow").c_str(), &desc->RElbowDesc, RElbow);
	assert(RElbow);
}
CBiped::~CBiped()
{
	// joints
	SAFE_FREE(Neck);
	SAFE_FREE(LHip);
	SAFE_FREE(RHip);
	SAFE_FREE(LKnee);
	SAFE_FREE(RKnee);
	SAFE_FREE(LAnkle);
	SAFE_FREE(RAnkle);
	SAFE_FREE(LShoulder);
	SAFE_FREE(RShoulder);
	SAFE_FREE(LElbow);
	SAFE_FREE(RElbow);
	
	// bodies
	SAFE_FREE(Head);
	SAFE_FREE(Chest);
	SAFE_FREE(LThigh);
	SAFE_FREE(RThigh);
	SAFE_FREE(LCalf);
	SAFE_FREE(RCalf);
	SAFE_FREE(LFoot);
	SAFE_FREE(RFoot);	
	SAFE_FREE(LUpperArm);
	SAFE_FREE(RUpperArm);
	SAFE_FREE(LForearm);
	SAFE_FREE(RForearm);

	descr->Free();
	delete descr;
}


void CBiped::setPosition(const Point2D_Int &pos)
{
	Chest->SetPosition(pos);
}

void CBiped::getPosition(Point2D_Int &pos)
{
	Chest->GetPosition(pos);
}

void CBiped::ApplyImp(float imp)
{
	Point2D_Int p;
	Chest->GetPosition(p);
	Chest->ApplyImpulse(Vector2D_Float(imp, 500.0f),p);
}

void CBiped::setMassFromShapes()
{
	Head->SetMassFromShapes();
	Chest->SetMassFromShapes();
	LThigh->SetMassFromShapes();
	RThigh->SetMassFromShapes();
	LFoot->SetMassFromShapes();
	RFoot->SetMassFromShapes();
	LCalf->SetMassFromShapes();
	RCalf->SetMassFromShapes();
	LUpperArm->SetMassFromShapes();
	RUpperArm->SetMassFromShapes();
	LForearm->SetMassFromShapes();
	RForearm->SetMassFromShapes();
}