#include "player.h"
#include "Game.h"
#include <algorithm>

int CPlayer::_counter = 0;

CPlayer::CPlayer(const char *name, const char *texdat, IEngineCore *core) :
	mCore(NULL),	
	mMan2d(NULL),
	mResMan(NULL),
	mListner(NULL),
	mBody(NULL),
	mCtrl_up(NULL),
	mCtrl_down(NULL),
	mCurGun(NULL),
	mArmLeft(NULL),
	mArmRight(NULL),
	mbPlayAnimations(false),
	mbInAir(false),
	mbSeating(false),
	mbWalk(false),
	mbLoaded(false),
	id(0),
	_health(100)
{
	
	id = _counter++;

	mCore = core;
	assert(mCore);
	mCore->AddRef();

	_oldpos = Point2D_Int(0,0);

	IManagersFactory *pSceneManFact = NULL;
	core->PluginByType(reinterpret_cast<ICommonPlugin **>(&pSceneManFact),Pl_SCENES);
	assert(pSceneManFact);
	pSceneManFact->GetResourceManager(mResMan);
	assert(mResMan);
	pSceneManFact->GetManager2D(mMan2d);
	assert(mMan2d);

	mMan2d->GetContactListener(mListner);
	assert(mListner);

	mName = name;
	tex_data = texdat;
}

CPlayer::~CPlayer()
{
	if (mBody)
	 {
	  mBody->Free();
	  mBody->Release();
	 }
	 if(mCtrl_up)
	 {
	  mCtrl_up->DetachAll();
	  mCtrl_up->Free();
	  mCtrl_up->Release();
	 }
	 if(mCtrl_down)
	 {
	  mCtrl_down->DetachAll();
	  mCtrl_down->Free();
	  mCtrl_down->Release();
	 }
	 if (mCurGun)
	 {
	  mCurGun->Release();
	 }

	 for (size_t i = 0; i < mSprites.size(); i++)
	 {
	  mSprites[i]->SetZOrder(750);
	  mMan2d->Attach(mSprites[i]);
	 }

	 // system
	 if (mCore) mCore->Release();
	 if (mMan2d) mMan2d->Release();
	 if (mResMan) mResMan->Release();
	 if (mListner) mListner->Release();
}

int  CPlayer::gethealth()
{
	return _health;
}

void CPlayer::createBody(const ICollisionShape::TPolygonDesc &sd_up, const ICollisionShape::TPolygonDesc &sd_down, IBody::TBodyDesc &bd)
{
	memcpy((void *)&mSDUp, (const void *)&sd_up, sizeof(ICollisionShape::TPolygonDesc));
	memcpy((void *)&mSDDown, (const void *)&sd_down, sizeof(ICollisionShape::TPolygonDesc));

	bd.fixedRotation = true;

	mMan2d->CreateCollidableBody(mName.c_str(), &bd, mBody);
	assert(mBody);

	mMan2d->CreateEmptyObject((mName+"_up").c_str(), mCtrl_up);
	assert(mCtrl_up);
	mCtrl_up->SetShape(&mSDUp);

	mMan2d->CreateEmptyObject((mName+"_down").c_str(), mCtrl_down);
	assert(mCtrl_down);
	mCtrl_down->SetShape(&mSDDown);

	setSprites();
	
	mBody->AppendShapes(mCtrl_up);
	mBody->AppendShapes(mCtrl_down);
	mBody->SetMassFromShapes();

	mBody->GetWorldCenter(oldp);
}

void CPlayer::setSprites()
{
	if (!mBody || !mCtrl_up)
	{
		mCore->Con_Write(("Failed to set sprites for "+mName).c_str());
		return;
	}

	mCtrl_up->SetZOrder(750);

	for (size_t i = 0; i < mSprites.size(); i++)
	{
		mCtrl_up->Attach(mSprites[i]);
	}
	mCore->Con_Write(("Sprites is set for "+mName).c_str());
}

ISprite *CPlayer::loadSprite(const char *file, const char *name)
{
	ISprite *sprite = NULL;
	if ( FAILED(mMan2d->CreateSprite(String(String(name)+IntToStr(id)).c_str(), sprite)) || NULL == sprite )
			return NULL;
	sprite->SetTexture(name);
	mSprites.push_back(sprite);
	sprite->Release();
	return sprite;
}

bool CPlayer::loadSpritesDB(std::istrstream &istr)
{
	for (TSpritesList::iterator i_sprite = mSprites.begin(); i_sprite != mSprites.end(); ++i_sprite)
	{
		(*i_sprite)->Free();
		(*i_sprite)->Release();
	}
	mSprites.clear();

	try
	{
		if (istr.bad()) throw std::exception("Bad string stream.");

		char rbuf[255];

		// sprites begin
		istr >> rbuf;
		istr.ignore(1,'\t');
		// count
		int count = 0;
		istr >> count;
		if (count <= 0) throw std::exception("Sprites DB is empty.");
		istr.ignore(1,'\n');

		// pass sprites	
		mSprites.reserve(count);	
		std::string name, tex_file;
		for (int i = 0; i < count; i++)
		{
			// name
			istr >> name;
			istr.ignore(1,'\t');
			// texture
			istr >> tex_file;
			istr.ignore(1,'\t');

			ISprite *sprite = loadSprite(DATA_PATH(std::string(tex_data+"\\")+tex_file), name.c_str());
			if (sprite)
			{
				Dimension2D_UInt dim;
				Point2D_Int rot, pos;
				float angle = 0.0f;
				bool flipx = false, flipy = false;
				int framesx = 1, framesy = 1;

				// width
				istr >> dim.width;
				istr.ignore(1,'\t');
				// height
				istr >> dim.height;
				istr.ignore(1,'\t');
				// x
				istr >> pos.x;
				istr.ignore(1,'\t');
				// y
				istr >> pos.y;
				istr.ignore(1,'\t');
				// rot x
				istr >> rot.x;
				istr.ignore(1,'\t');
				// rot y
				istr >> rot.y;
				istr.ignore(1,'\t');
				// angle
				istr >> angle;
				istr.ignore(1, '\t');
				// flipx
				istr >> flipx;
				istr.ignore(1, '\t');
				// flipy
				istr >> flipy;
				istr.ignore(1, '\t');
				// framesx
				istr >> framesx;
				istr.ignore(1, '\t');
				// framesy
				istr >> framesy;
				istr.ignore(1, '\t');

				sprite->SetDimension(dim.width, dim.height);
				sprite->SetPosition(pos);
				sprite->SetAngle(angle, rot);
				if (flipx) sprite->FlipX();
				if (flipy) sprite->FlipY();
				sprite->SetFramesCount(framesx, framesy);
			} else
			{
				mCore->Con_Write((std::string("Failed to load sprite ")+name+".").c_str());
				istr.getline(&rbuf[0], 255, '\n');
			}
		}
	}
	catch(std::exception &ex)
	{
		mCore->Con_Write("Failed to load sprites db.");
		mCore->Con_Write(ex.what());
		return false;
	}
	catch(...)
	{
		mCore->Con_Write("Failed to load sprites db.");
		return false;
	}

	return true;		
}
bool CPlayer::loadAnimDB(std::istrstream &istr)
{
	mAnimations.clear();

	try
	{
		if (istr.bad()) throw std::exception("Bad string stream.");

		char rbuf[255];

		// pass animation
		// animations begin
		istr >> rbuf;
		istr.ignore(1,'\t');
		// count
		int count = 0;
		istr >> count;
		if (count <= 0) throw std::exception("Animations DB is empty.");
		istr.ignore(1,'\n');

		// pass animatios
		std::string anim_name;
		for (int i = 0; i < count; i++)
		{
			// frames
			istr >> anim_name;
			istr.ignore(1,'\t');
			mAnimations[anim_name] = TAnimation(anim_name.c_str());
			// count
			int frames_count = 0;
			istr >> frames_count;
			istr.ignore(1,'\n');

			for (int j = 0; j < frames_count; j++)
			{
				// sprites begin
				istr >> rbuf;
				istr.ignore(1,'\t');
				// count
				int sprites_count = 0;
				istr >> sprites_count;
				istr.ignore(1,'\n');

				TFrame frame;
				frame.Sprites.reserve(sprites_count);
				for (int k = 0; k < sprites_count; k++)
				{				
					// name
					istr >> rbuf;
					strcpy(rbuf,String(String(rbuf)+IntToStr(id)).c_str()); 
					istr.ignore(1,'\t');
					TSpritesList::iterator i_sprite = std::find_if(mSprites.begin(), mSprites.end(), equal_sprites(rbuf));
					if (i_sprite != mSprites.end())
					{					
						Point2D_Int pos;
						// x
						istr >> pos.x;
						istr.ignore(1,'\t');
						// y
						istr >> pos.y;
						istr.ignore(1,'\t');

						// angle
						float angle = 0.0f;
						istr >> angle;
						istr.ignore(1, '\t');

						// flip
						// x
						bool flipx = false;
						istr >> flipx;
						istr.ignore(1, '\t');
						
						// y
						bool flipy = false;
						istr >> flipy;
						istr.ignore(1, '\t');

						// frame
						int cur_frame = 0;
						istr >> cur_frame;
						istr.ignore(1, '\n');

						frame.Sprites.push_back(TSpriteFrame(*i_sprite, pos, angle, flipx, flipy, cur_frame));
					}
					else
					{
						mCore->Con_Write((std::string("Unknown sprite in animation ")+anim_name+".").c_str());
						istr.ignore(1, '\n');
					}
				}

				mAnimations[anim_name].Frames.push_back(frame);
			}
		}
	}
	catch(std::exception &ex)
	{
		mCore->Con_Write("Failed to load animation db.");
		mCore->Con_Write(ex.what());
		return false;
	}
	catch(...)
	{
		mCore->Con_Write("Failed to load animation db.");
		return false;
	}

	return true;
}

bool CPlayer::loadDB()
{
	if (mbLoaded) return true;

	char *buf = NULL;
	unsigned int file_size = 0;
	try
	{
		{
			IFileSystem *_pFS;
			mCore->FileSystem(&_pFS);

			//if (FAILED( _pFS->FileExists(DATA_PATH((std::string(tex_data)+"\\hero.anim").c_str())) ))  throw std::exception("File doesn't exist.");

			IFile *pFile;
			_pFS->GetFile(DATA_PATH((std::string(tex_data)+"\\hero.anim").c_str()),&pFile);

			unsigned int read = 0;
			pFile->GetSize(file_size);
			buf = new char[file_size];

			pFile->Read((void*)buf,sizeof(char)*file_size,read);
			if (read != file_size) throw std::exception("Not all data readed.");

			pFile->Release();
			_pFS->Release();
		}
	
		istrstream istr(buf, file_size);	
		if (istr.bad()) throw std::exception("Bad string stream.");

		loadSpritesDB(istr);
		loadAnimDB(istr);

		if (buf) delete[] buf;
	}
	catch(std::exception &ex)
	{
		if (buf) delete[] buf;
		mCore->Con_Write(("Failed to load "+mName).c_str());
		mCore->Con_Write(ex.what());
		return false;
	}
	catch(...)
	{
		mCore->Con_Write(("Failed to load "+mName).c_str());
		return false;
	}

	mbLoaded = true;
	mCore->Con_Write((mName+" is loaded.").c_str());
	return true;
}

bool CPlayer::enableAnimation(const char *name, bool enable, bool rev)
{
	TAnimationsList::iterator i_anim = mAnimations.find(name);
	if (i_anim != mAnimations.end())
	{
		i_anim->second.Enabled = enable;
		i_anim->second.Reverse = rev;
		return true;
	}

	return false;
}

bool CPlayer::isEnabledAnimation(const char *name)
{
	TAnimationsList::iterator i_anim = mAnimations.find(name);
	if (i_anim != mAnimations.end()) return i_anim->second.Enabled;
}

int clamp(int idx, int count)
{
	return ( idx<0 ? count+idx : ( idx>=count ? idx-count: idx ) );
}

void CPlayer::updateAnimations(float speed)
{
	// play animation
	if (mAnimations.size() > 0 && mbPlayAnimations)
	{
		TAnimationsList::iterator i_anim = mAnimations.begin();
		while(i_anim != mAnimations.end())
		{
			if (i_anim->second.Enabled)
			{
				const float koef_speed = 0.01f;
				i_anim->second.Start+= speed * koef_speed;
				if (i_anim->second.Start > (float)i_anim->second.Frames.size())
					i_anim->second.Start = (float)( (int)floorf(i_anim->second.Start) % i_anim->second.Frames.size() );
	
				if(i_anim->second.Reverse)
				{
				TAnimation::TFramesList::reverse_iterator i_frame =i_anim->second.Frames.rbegin();
				advance(i_frame, clamp((int)i_anim->second.Start, i_anim->second.Frames.size()));
				for (std::vector<TSpriteFrame>::iterator i_sprite = i_frame->Sprites.begin(); i_sprite != i_frame->Sprites.end(); ++i_sprite)
				{
					i_sprite->Sprite->SetPosition(i_sprite->Position);
					i_sprite->Sprite->SetAngle(i_sprite->Angle);
					bool flipx, flipy;
					i_sprite->Sprite->IsFlipX(flipx);
					i_sprite->Sprite->IsFlipY(flipy);
					if (i_sprite->FlipX != flipx) i_sprite->Sprite->FlipX();
					if (i_sprite->FlipY != flipy) i_sprite->Sprite->FlipY();
					i_sprite->Sprite->SetCurrentFrame(i_sprite->Frame);
				}

				}
				else
				{				
				TAnimation::TFramesList::iterator i_frame =i_anim->second.Frames.begin();
				advance(i_frame, clamp((int)i_anim->second.Start, i_anim->second.Frames.size()));
				for (std::vector<TSpriteFrame>::iterator i_sprite = i_frame->Sprites.begin(); i_sprite != i_frame->Sprites.end(); ++i_sprite)
				{
					i_sprite->Sprite->SetPosition(i_sprite->Position);
					i_sprite->Sprite->SetAngle(i_sprite->Angle);
					bool flipx, flipy;
					i_sprite->Sprite->IsFlipX(flipx);
					i_sprite->Sprite->IsFlipY(flipy);
					if (i_sprite->FlipX != flipx) i_sprite->Sprite->FlipX();
					if (i_sprite->FlipY != flipy) i_sprite->Sprite->FlipY();
					i_sprite->Sprite->SetCurrentFrame(i_sprite->Frame);
				}
				}
			}
			else i_anim->second.Start = 0.0f;
			++i_anim;
		}
	}
}

unsigned long CPlayer::getid(bool down)
{
	unsigned long id = 0;
	if (!down) mCtrl_up->GetID(id);
    else
    if(mCtrl_down!=NULL) mCtrl_down->GetID(id);
	return id;
}

void CPlayer::dechealth(int dec)
{
	_health-=dec;
}

void CPlayer::updateInput(IInput *input)
{
	if(!input || !mBody) return;

	// walk	
	{
		static const float imp_value = 1400.0f/10.f;
		Vector2D_Float impulse, velocity;
		Point2D_Int pos;
		mBody->GetWorldCenter(pos);

		// reset player
		mBody->GetLinearVelocity(velocity);
		if (abs(velocity.x) < 30.0f && (int)velocity.y == 0 && !mbSeating && !mbInAir)
		{
			enableAnimation("stand", true);
			enableAnimation("down", false);
			enableAnimation("walk", false);
			enableAnimation("jump", false);
		}

		static bool pressed = false;

		input->GetKey(KEY_DOWN,pressed);
		if (pressed)
		{
				mbSeating = true;
				enableAnimation("stand", false);
				enableAnimation("walk", false);
				enableAnimation("jump", false);
				enableAnimation("down", true);
				// delete down shape
				if (mCtrl_down)
				{
					mCtrl_down->Free();
					mCtrl_down->Release();
					mCtrl_down = NULL;
				}
				impulse+= Vector2D_Float(0.0f, imp_value/2);
		}
		else
		{
			// return down shape
			if (!mCtrl_down && !mbInAir)
			{
				enableAnimation("stand", true);
				mbSeating = false;

				Point2D_Int pos;
				mBody->GetPosition(pos);
				pos.y-=52;
				mBody->SetPosition(pos);

				mMan2d->CreateEmptyObject((mName+"_down").c_str(), mCtrl_down);
				assert(mCtrl_down);
				mCtrl_down->SetShape(&mSDDown);

				mBody->AppendShapes(mCtrl_down);
				mBody->SetMassFromShapes();
			}
		}
		
		input->GetKey(KEY_LEFT,pressed);
		if (pressed && !mbSeating)
		{
			if (!mbInAir)
			{
				enableAnimation("stand", false);
				enableAnimation("down", false);
				enableAnimation("jump", false);
				enableAnimation("walk", true, true);
				mbSeating = false;
				mbWalk = true;
			}
			impulse+= Vector2D_Float(-imp_value/1.5, 0.0f);
		}

		input->GetKey(KEY_RIGHT,pressed);
		if (pressed && !mbSeating)
		{
			if (!mbInAir)
			{
				enableAnimation("stand", false);
				enableAnimation("down", false);
				enableAnimation("jump", false);
				enableAnimation("walk", true);
				mbSeating = false;
				mbWalk = true;
			}
			impulse+= Vector2D_Float(imp_value/2, 0.0f);
		}

		input->GetKey(KEY_UP,pressed);
		if (pressed)
		{			
			if (!mbInAir)
			{
				enableAnimation("stand", false);
				enableAnimation("down", false);
				enableAnimation("walk", false);
				enableAnimation("jump", true);
				impulse+= Vector2D_Float(0.0f, -13*imp_value);
				mbInAir = true;
				mbSeating = false;
				mbWalk = false;
			}
		}		

		if (velocity.x >= 150.0f) impulse.x = 0.0f;
		if (velocity.y <= -200.0f) impulse.y = 0.0f;
		if (velocity.x <= -50.0f) impulse.x = 0.0f;
		mBody->ApplyImpulse(impulse, pos);
	}
}

void CPlayer::setGun(const char *gun, bool visible)
{
	if (mCurGun)
	{		
		mCurGun->Release();
	}
	mMan2d->GetSprite((String(gun)+IntToStr(id)).c_str(),mCurGun);
	if (mCurGun)
	{
		mCurGun->SetVisible(visible);
	}
}

void CPlayer::setPosition(const Point2D_Int &pos)
{
	if (mBody)
	{
		mBody->SetPosition(pos);
	}
}

void CPlayer::getPosition(Point2D_Int &pos)
{
	if (mBody)
	{
		mBody->GetPosition(pos);
	}
}

void CPlayer::update(IInput *input, float speed)
{
 
 Point2D_Int p1,p2;

 if(_oldpos.x+_oldpos.y!=0)
 {
	 ICamera2D *cam;
	 mMan2d->GetCamera(cam);
	 mBody->GetPosition(p1);
	 cam->GetPosition(p2);
	 if(p1.x<p2.x-400) 
	 {
		p1.x = _oldpos.x;
		mBody->SetPosition(p1);
	 }
	  cam->Release();
 }
 mbInAir = true;

 // process contact posints
 if (mCtrl_up && mBody)
 {
  int count = 0;
  mListner->GetCount(count);
  IContactListener::TContactPoint cpoint;
  // check inair player, other game logic is not supported while.
  if (count > 0)
  {
   unsigned long id_shape = 0, id1 = 0, id2 = 0;
   if (mbSeating) mCtrl_up->GetID(id_shape);
   else
	   if(mCtrl_down!=NULL) mCtrl_down->GetID(id_shape);
   for (int i = 0; i < count; i++)
   {
    cpoint.pShape1 = NULL;
    cpoint.pShape2 = NULL;
    mListner->NextPoint(cpoint);
    cpoint.pShape1->GetID(id1);
    cpoint.pShape2->GetID(id2);
    // player shape is founded
    // check if collision with foot
    Vector2D_Float vy(0.0f, 1.0f), normal = cpoint.normal;
    bool collision = false;
    if (id_shape == id1) //   !!!!!
    {
     collision = true;
    }
    else if (id_shape == id2)
    {
     collision = true;
     vy.y*= -1.0f;
    }

    if (collision)
    {
     // angle between oy     
     float length = sqrt(normal.x*normal.x+normal.y*normal.y);
     normal = Vector2D_Float(normal.x/length, normal.y/length);
     float angle = acosf( normal.x*vy.x+normal.y*vy.y );

     if (abs(angle) < PI/4.0f)
     {
      mbInAir = false;
      break;
     }
    }
   }
  }
 }

 
 mBody->GetPosition(p1);
 _oldpos = p1;

 updateInput(input);
 updateAnimations(speed);
}