#ifndef _PLAYER_H_INCLUDED_
#define _PLAYER_H_INCLUDED_

#if defined(_MSC_VER) && (_MSC_VER > 1000)
#pragma once
#endif	/* _MSC_VER > 1000 */

#include <DPE_Application_Include.h>

#include <vector>
#include <map>
#include <string>
#include <assert.h>
#include <strstream>
#include <StringUtils.h>

using namespace DPE_Scenes;
using namespace DPE_Scenes::_2D;

template <typename T>
class CEqualByName
{
	const char *name;
public:
	CEqualByName(const char *name)
	{
		CEqualByName::name = name;
	}
	bool operator () (const T &param)
	{
		const char *name = NULL;
		param->GetName(name);
		return strcmp(name, CEqualByName::name) == 0;
	}
};

typedef CEqualByName<ISprite *> equal_sprites;

struct TSpriteFrame
{
	ISprite *Sprite;
	Point2D_Int Position;
	float Angle;
	bool FlipX, FlipY;
	int Frame;

	TSpriteFrame(ISprite *sprite, const Point2D_Int &pos, float angle, bool flipX = false, bool flipY = false, int frame = 0)
	{
		Sprite = sprite;
		Position = pos;
		Angle = angle;
		FlipX = flipX;
		FlipY = flipY;
		Frame = frame;
	}
};

struct TFrame
{
	std::vector<TSpriteFrame> Sprites;
};

struct TAnimation
{
	static const int NAME_LENGTH = 64;
	char Name[NAME_LENGTH];
	typedef std::vector<TFrame> TFramesList;
	TFramesList Frames;
	bool Enabled;
	bool Reverse;
	float Start;

	TAnimation() : Enabled(false), Start(0.0f)
	{
		Name[0] = '\n';
	}

	TAnimation(const char *name) : Enabled(false), Start(0.0f)
	{
		memcpy(Name, name, sizeof(char)*NAME_LENGTH);
	}
};


class CPlayer
{
public:
	typedef std::vector<ISprite *> TSpritesList;
	typedef std::map<std::string, TAnimation> TAnimationsList;

	CPlayer(const char *name, const char *texdat, IEngineCore *core);
	virtual ~CPlayer();

	void createBody(const ICollisionShape::TPolygonDesc &sd_up, const ICollisionShape::TPolygonDesc &sd_down, IBody::TBodyDesc &bd);
	bool loadDB();
	//bool setArms(const char *left, const char *right);

	bool enableAnimation(const char *name, bool enable, bool rev = false);
	bool isEnabledAnimation(const char *name);

	void playAnimations() {mbPlayAnimations = true;}
	void stopAnimations() {mbPlayAnimations = false;}
	bool isPlaingAnimations() const {return mbPlayAnimations;}

	void setGun(const char *gun, bool visible);
	
	void setPosition(const Point2D_Int &pos);
	void getPosition(Point2D_Int &pos);

	virtual void update(IInput *input, float speed);

	int  gethealth();

	unsigned long getid(bool down);

	void dechealth(int dec);

	ISprite* GetSprite(char *name)
	{
		const char *n = 0;
		for(int i = 0; i<mSprites.size(); i++)
		{
			mSprites[i]->GetName(n);
			if(strcmp(String(String(name)+IntToStr(id)).c_str(),n)==0)
			{
				return mSprites[i];
			}
		}
	}

protected:
	Point2D_Int oldp;
	static int _counter;

	void updateInput(IInput *input);
	void updateAnimations(float speed);
	
	void setSprites();
	
	bool loadSpritesDB(std::istrstream &istr);
	bool loadAnimDB(std::istrstream &istr);
	virtual ISprite *loadSprite(const char *file, const char *name);

	std::string				mName;
	std::string				tex_data;

	int						_health;

	IBody					*mBody;
	ICollisionShape::TPolygonDesc mSDUp, mSDDown;
	_2D::IObject			*mCtrl_up, *mCtrl_down;
	ISprite					*mCurGun;
	ISprite					*mArmLeft, *mArmRight;// without hand animation!

	// replace on states
	enum E_STATE
	{
		e_none	= 1,
		e_air	= 2,
		e_seat	= 4
	};
	int						mState;

	int						id;

	Point2D_Int				_oldpos;
	bool					mbInAir;
	bool					mbSeating;
	bool					mbWalk;
	
	TSpritesList			mSprites;
	TAnimationsList			mAnimations;
	bool					mbLoaded;
	bool					mbPlayAnimations;

	Point2D_Int				mMousePos;

	// system
	IEngineCore				*mCore;
	IManager2D				*mMan2d;
	IResourceManager		*mResMan;
	IContactListener		*mListner;
};

#endif	/* _PLAYER_H_INCLUDED_ */