#include "AppCore.h"

#include <DPE_CPP_Include.h>

void (AppCore::*AppCore::p_render)();
void (AppCore::*AppCore::p_process)();
void (AppCore::*AppCore::p_init)();
void (AppCore::*AppCore::p_free)();

AppCore* AppCore::_pInstance = NULL;

AppCore::AppCore(HWND rendertarget, bool developer)
{

   _gamemap = NULL;

   #ifndef _DEBUG
   _dev		= developer;
   #else
   _dev		= true;
   #endif
   _handle 	= rendertarget;
   _log 	= NULL;
   _pInstance = this;

   p_init 		= &AppCore::_init;
   p_free 		= &AppCore::_free;
   p_render 	= &AppCore::_render;
   p_process 	= &AppCore::_process;

   _hServer = NULL;

   int (CALLBACK *pCreateEngine)(IEngineCore **ppv) = NULL;

   _hServer = LoadLibrary("..\\release\\DeeProEngine.dll");

   if(_hServer == NULL) return;

	if(pCreateEngine == NULL)
	{
		pCreateEngine = reinterpret_cast<int (CALLBACK *)(IEngineCore **)>
			(GetProcAddress(_hServer,("GetEngine")));

		if(pCreateEngine == NULL) return;
	}

	(*pCreateEngine)(&_pEngineCore);

   _pEngineCore->AddProcedure(PROC_RENDER,&AppCore::Render);
   _pEngineCore->AddProcedure(PROC_PROCESS,&AppCore::Process);
   _pEngineCore->AddProcedure(PROC_INIT,&AppCore::Init);
   _pEngineCore->AddProcedure(PROC_FREE,&AppCore::Free);

	RECT rect;
	GetWindowRect(rendertarget,&rect);
	_wind.Width 	= (rect.right-rect.left);
	_wind.Height 	= (rect.bottom-rect.top);
	_wind.BitDepth 	=  32;
	_wind.VSync		= false;
	_wind.FullScreen= false;
	_wind.sample_count=1;

	_pEngineCore->InitializeEngine(rendertarget,"Map Craft",_wind,50,"",ELT_HTML);

	_pEngineCore->GetLog(&_log);

	IEventSystem *pES;
	CEventSystem::Instance()->QueryInterface(__uuidof(IEventSystem),(void**)&pES);
	_pEngineCore->AddSubEventSystem(pES);
	pES->Release();
	CEventSystem::Instance()->AddEventListener(EV_WIN_MESSAGE, NewDelegate(this, &AppCore::wndproc));

	_pEngineCore->PluginByType((ICommonPlugin**)&_pCoreRenderer,Pl_RENDER);

	_pEngineCore->PluginByType((ICommonPlugin**)&_pFactory,Pl_SCENES);

	_log->Info("Application \"Map Craft\" initialized succesfully.");

	_pEngineCore->AllowAutoPause(true);

	_pEngineCore->Con_Visible(_dev);

	_pEngineCore->StartEngine();

}

AppCore::~AppCore()
{
	if(_gamemap)
	delete _gamemap;

	if (_hServer!=NULL)
	 {
		_log->Info("Application \"Map Craft\" finalized.");
		if(_log) _log->Release();
		_pCoreRenderer->Release();
		_pEngineCore->QuitEngine();
		CEventSystem::Free();
		_pEngineCore->Release();
		FreeLibrary(_hServer);
	 }

}

imdmap* AppCore::map()
{
  return _gamemap;
}

void AppCore::resetmap()
{
  if(_gamemap) delete _gamemap;
  _gamemap = new imdmap(_pEngineCore);
}

void AppCore::wndproc(IEvent *pEvent)
{
	IWinEvent *pWinEvent;

	pEvent->QueryInterface(IID_IWinEvent,(void**)&pWinEvent);

	unsigned int message;
	WPARAM wParam;
	LPARAM lParam;

	pWinEvent->GetParam(&message,&wParam,&lParam);

	switch(message)
	{
	}

}

void AppCore::_render()
{

	if(_gamemap) _gamemap->DrawMap();

	if (_dev)
	{
		int fps;
		_pEngineCore->GetFPS(fps);
		_pCoreRenderer->DrawTextA(2,2,1.0,1.0,1.0,0.7,String("FPS:"+IntToStr(fps)).c_str());
	}
}

void AppCore::_process()
{

}

void AppCore::_init()
{
	_pFactory->GetResourceManager(_pResMan);
}

void AppCore::_free()
{
	_pResMan->Release();
	_pFactory->Release();
}

//Engine registered procs
void AppCore::Render()
{
   if(_pInstance)(_pInstance->*p_render)();
}

void AppCore::Process()
{
	if(_pInstance)(_pInstance->*p_process)();
}

void AppCore::Init()
{
	if(_pInstance)(_pInstance->*p_init)();
}

void AppCore::Free()
{
	if(_pInstance)(_pInstance->*p_free)();
}

