unit DFont;

interface

uses
  DUtils,
  DMath,
  DGraph;

type
TFontMan = class;

TFont = class(TStreamable)
private
  FMan: TFontMan;
  FID: String;
  FFileName: String;
  FTex: TTexture;
  FSpacing: Integer;
  FWidth: Integer;
  FHeight: Integer;
public
  constructor Create(Man: TFontMan; FileName: String; Spacing: Integer = 0);
  destructor Destroy; override;
  procedure Draw(Pos: TVec2f; Text: String);
  function GetWidth(Text: String): Integer;
  function GetHeight(Text: String): Integer;
  property ID: String read FID;
  property FileName: String read FFileName;
  property Spacing: Integer read FSpacing;
  property Width: Integer read FWidth;
  property Height: Integer read FHeight;
end;

TFontMan = class
private
  FGraph: TGraph;
public
  constructor Create(Graph: TGraph);
  function CreateFont(FileName: String; Spacing: Integer = 0): TFont;
  property Graph: TGraph read FGraph;
end;

implementation

//{{{ TFont
constructor TFont.Create;
begin
  FMan := Man;
  FFileName := FileName;
  FID := ClassName + '[' + FFileName + '][' + IntToStr(Spacing) + ']';
  FTex := FMan.Graph.LoadTex(FFileName);
  FWidth := FMan.Graph.TexWidth(FTex) div 16;
  FHeight := FMan.Graph.TexHeight(FTex) div 16;
  FSpacing := Spacing;
end;

destructor TFont.Destroy; 
begin
end;

procedure TFont.Draw(Pos: TVec2f; Text: String);
  var
    I: Integer;
    T: TVec2f;
begin
  FMan.Graph.SetTexture(FTex);
  for I := 1 to Length(Text) do begin
    T.X := (Ord(Text[I]) mod 16)/16;
    T.Y := (Ord(Text[I]) div 16)/16;
    FMan.Graph.DrawTexture(Rect(Pos, Pos + Vec2f(FWidth, FHeight)),
                           Rect(T, T + Vec2f(1, 1)/16));
    Pos.X += FWidth + Spacing;
  end;
  FMan.Graph.SetTexture(-1);
end;

function TFont.GetWidth(Text: String): Integer;
begin
  Result := Length(Text) * (FWidth + FSpacing);
end;

function TFont.GetHeight(Text: String): Integer;
begin
  Result := Height;
end;
//}}}
// TFontMan//{{{
constructor TFontMan.Create(Graph: TGraph);
begin
  FGraph := Graph;
end;

function TFontMan.CreateFont;
begin
  Result := TFont.Create(Self, FileName, Spacing);
end;
//}}}

end.
