{$MACRO ON}
unit DGame;

interface

uses
  //GL,
  windows,

  DCore,
  DUtils,
  DDebug,
  DMath,
  DEvents,
  DWinApi,
  DSound,
  DGraph,
  DFont,
  DCamera2D,

  DMap,
  DLevelData;

const
  CROUCH_VALUE = 10;

type
TControlID = (
                ciLeft, 
                ciRight, 
                ciJump, 
                ciLeftFire, 
                ciRightFire, 
                ciCrouch
             );

TControl = array[TControlID] of Boolean;

TPlayer = class;
//{{{ TActive
TActive = class(TStreamable)
private
  FPos: TVec2f;
  FFrc: TVec2f;
  FRect: TRect;
  FLevel: Integer;
  FControl: TControl;
  FColor: TVec4f;
protected
  FDamagable: Boolean; //   
  FHP: Integer;
  FVel: TVec2f;
  FHalf: TVec2f; //  
  FOnFloor: Boolean;
  FCrouch: Boolean;
  FGarbage: Boolean;
  procedure Stream; override;
public
  constructor Create;
  procedure Kill;
  procedure Render; virtual;
  procedure Update; virtual;
  procedure SetPos(NewPos: TVec2f); virtual;
  procedure MovePos(Delta: TVec2f);
  procedure ApplyForce(Delta: TVec2f);
  procedure ActiveControl(ID: TControlID);
  procedure ResetControls;
  procedure Damage(Value: Integer); virtual;
  property Color: TVec4f read FColor write FColor;
  property Pos: TVec2f read FPos write SetPos;
  property Vel: TVec2f read FVel write FVel;
  property Frc: TVec2f read FFrc;
  property Rect: TRect read FRect;
  property Level: Integer read FLevel write FLevel;
  property OnFloor: Boolean read FOnFloor write FOnFloor;
  property Garbage: Boolean read FGarbage;
  property Damagable: Boolean read FDamagable;
  property HP: Integer read FHP;
end;
//}}}
//{{{ TEnemy
TEnemy = class(TActive)
protected
  FNeedCollided: Boolean; 
  procedure Stream; override;
public
  constructor Create;
  procedure Damage(Value: Integer); override;
  procedure Collide(Player: TPlayer); virtual;
  property NeedCollided: Boolean read FNeedCollided write FNeedCollided;
end;
//}}}
//TEnemyTexture//{{{
TEnemyTexture = class(TEnemy)
private
  FTexture: TTexture;
protected
  procedure Stream; override;
public
  constructor Create(Pos: TVec2f; Tex: String; Color: TVec4f);
  procedure Render; override;
  procedure Update; override;
end;
//}}}
TWheel = class(TEnemyTexture)//{{{
private
  FTime: Integer;
  FVel: TVec2f;
protected
  procedure Stream; override;
public
  constructor Create(Pos: TVec2f; Level: Integer; Vel: TVec2f);
  procedure Update; override;
end;
//}}}
//{{{ TSimpleEnemy
TSimpleEnemy = class(TEnemy)
private
  FSeed: Single;
protected
  procedure Stream; override;
public
  constructor Create;
  procedure Render; override;
  procedure Update; override;
end;
//}}}
//{{{ TLineEnemy
TLineEnemy = class(TEnemy)
private
  FSeed: Single;
protected
  procedure Stream; override;
public
  constructor Create;
  procedure Render; override;
  procedure Update; override;
end;
//}}}
//{{{ TPotron
TPotron = class(TActive)
  constructor Create;
  procedure OnCollide(Map: TMap; Block: Integer); virtual; overload;
  procedure OnCollide(Enemy: TEnemy); virtual; overload;
end;
//}}}
//{{{ TSimplePotron
TSimplePotron = class(TPotron)
private
  FStart: TVec2f;
  FColor: TVec4f;
  FDist: Single;
protected
  procedure Stream; override;
public
  constructor Create(Pos, Vel: TVec2f; Size, Dist: Single; Color: TVec4f);
  destructor Destroy; override;
  procedure Update; override;
  procedure Render; override;
end;
//}}}
//{{{ TWeapon
TWeapon = class(TStreamable)
private
  FPos: TVec2f;
  FLastFire: Integer;
protected
  FRechargeTime: Integer;
  procedure Stream; override;
  procedure DoFire(Aim: TVec2f; Level: Integer); virtual;
public
  constructor Create; virtual; overload;
  constructor Create(RechargeTime: Integer); overload;
  //   -    16
  procedure Render; virtual;
  procedure Update; virtual;
  //  : Pos -   ,   
  class procedure RenderItem(Pos: TVec2f); virtual;
  procedure Fire(Aim: TVec2f; Level: Integer);
  function GetReady: Boolean;
  property Pos: TVec2f read FPos write FPos;
end;
CWeapon = class of TWeapon;
//}}}
//{{{ TSimpleWeapon
TSimpleWeapon = class(TWeapon)
protected
  procedure DoFire(Aim: TVec2f; Level: Integer); override;
public
  constructor Create; override;
  procedure Render; override;
  class procedure RenderItem(Pos: TVec2f); override;
end;
//}}}
//{{{ TPlayer
TPlayer = class(TActive)
private
  FWeapons: array[0..1] of TWeapon;
protected
  procedure Stream; override;
public
  constructor Create;
  destructor Destroy; override;
  procedure SetPos(NewPos: TVec2f); override;
  procedure Render; override;
  procedure Update; override;
  procedure AddWeapon(Weapon: TWeapon);
end;
//}}}
//{{{ TItem
TItem = class(TActive)
  procedure OnPlayer(Player: TPlayer); virtual;
  procedure Update; override;
end;
//}}}
//{{{ TWeaponItem
TWeaponItem = class(TItem)
private
  FWeapon: CWeapon;
protected
  procedure Stream; override;
public
  constructor Create(Weapon: CWeapon);
  procedure OnPlayer(Player: TPlayer); override;
  procedure Render; override;
end;
//}}}
//{{{ TEnemyLine
TEnemyLine = object
  FPos: TVec2f;
  FCount: Byte;
  FEnemies: array of TEnemy;
  procedure Create;
  procedure Render;
  procedure Update;
  function Garbage: Boolean;
  procedure ChangeColor;
end;
//}}}
//{{{ TScene
TScene = record
  Name: String;
  Start: TProcOfObj;
  Finish: TProcOfObj;
  Render: TProcOfObj;
  Update: TProcOfObj;
end;
//}}}
//{{{ TSummary
TSummary = record
  Killed: Integer;
  Dead: Integer;
  Passed: Integer;
  Potrons: Integer;
end;
//}}}
//{{{ TGame
TGame = class(TWinApp)
private
  FMap: TMap;
  {FPlayer: TRect;
  FLevel: Integer;}
  FTime: Integer;
  FPlayer: TPlayer;
  FPotrons: array of TPotron;
  FEnemies: array of TEnemy;
  FEnemiesLine: array of TEnemyLine;
  FItems: array of TItem;
  FCamera: TCamera2D;
  FScenes: array of TScene;
  FCurScene: Integer;
  FNextScene: Integer;
  FLastFPS: Integer;
  FData: TData;
  FFontMan: TFontMan;
  FFont: TFont;
  FFixed: TFont;
  FLevels: array of TLevelData;
  FCurLevel: Integer;
  FNextLevel: String;
  FSummary: TSummary;
  FLastSaved: String;
  FColor: TVec4f;
  function GetTime: Integer;
  procedure AddScene(Name: String; Start, Finish, Render, Update: TProcOfObj);
  procedure GameStart;
  procedure GameFinish;
  procedure GameUpdate;
  procedure GameRender;
  procedure WinStart;
  procedure WinFinish;
  procedure WinUpdate;
  procedure WinRender;
  procedure TitleStart;
  procedure TitleFinish;
  procedure TitleUpdate;
  procedure TitleRender;
  procedure LoseStart;
  procedure LoseFinish;
  procedure LoseUpdate;
  procedure LoseRender;
  procedure FreeObjects;
  procedure UpdateEnemies;
public
  procedure GoToScene(Name: String);
  constructor Create;
  destructor Destroy; override;
  procedure AddLevel(Level: TLevelData);
  procedure LoadLevel(Name: String);
  procedure SaveGame(Name: String);
  procedure LoadGame(Name: String);
  procedure AddPotron(Potron: TPotron);
  procedure AddEnemy(Enemy: TEnemy);
  procedure AddEnemyLine(Enemy: TEnemyLine);
  procedure AddItem(Item: TItem);
  procedure Render; override;
  procedure Update; override;
  procedure OnEvent(const Event: TEvent); override;
  procedure KeyPressed(Key: Byte);
  function GetEnemyCount: Integer;
  function GetEnemy(Index: Integer): TEnemy;
  function GetLevel: TLevelData;
  procedure DrawText(X, Y: Single; AX, AY: Single; Text: String);
  property Map: TMap read FMap;
  property Player: TPlayer read FPlayer;
  property Time: Integer read GetTime;
  property Camera: TCamera2D read FCamera;
  property CurLevel: Integer read FCurLevel;
  property EnemyCount: Integer read GetEnemyCount;
  property Enemies[Index: Integer]: TEnemy read GetEnemy;
  property Summary: TSummary read FSummary;
end;
//}}}
TGameStream = class(TStreamable)//{{{
protected
  procedure Stream; override;
end;
//}}}

var
  Game: TGame;

implementation

uses
  DWeapon,
  DBossOnOff;

//{{{ TEnemyLine 
procedure TEnemyLine.Create;
var
  Numb: Byte;
  R: TVec4f;
  Rand: Integer;
  Color: array[0..1] of TVec4f;
begin
  //R := Vec4f(Random*2*pi, Random*2*pi, Random*2*pi, 1);
  //Color1 := Vec4f(Cos(R.X), Sin(R.X)*Cos(R.Y), Sin(R.X)*Sin(R.Y), 1);
  //Color2 := Vec4f(Sin(R.X), Cos(R.X)*Sin(R.Y), Cos(R.X)*Cos(R.Y), 1);
  Rand := Random(1000);
  for Numb := 0 to 1 do begin
    Color[Numb] := Vec4f(((Rand + Numb + 1) * 50 mod 177)/177,
                         ((Rand + Numb + 2) * 50 mod 177)/177,
                         ((Rand + Numb + 3) * 50 mod 177)/177, 1);
  end;
  for Numb := 0 to FCount do
  begin
    SetLength(FEnemies, Length(FEnemies) + 1);
    FEnemies[Numb] := TLineEnemy.Create; //    ,      (TLineenemy)
    FEnemies[Numb].Pos := FPos;
    FEnemies[Numb].FVel := Vec2f(0, 5);
    Fpos += Vec2f(+30, +10); //  y   )        ) 
    FEnemies[Numb].Color := (Numb*Color[0] + (FCount - Numb)*Color[1])*(1/FCount);
  end;
end;

procedure TEnemyLine.Render;
var
  I: Byte;
begin
  for I := 0 to High(FEnemies) do
    FEnemies[I].Render;
end;

procedure TEnemyLine.Update;
var
  I: Byte = 0;
begin
  While I < Length(FEnemies) do begin
    FEnemies[I].Update;
    if FEnemies[I].Garbage then begin
      FEnemies[I].Free;
      FEnemies[I] := FEnemies[High(FEnemies)];
      SetLength(FEnemies, Length(FEnemies) - 1);
    end else
      Inc(I);
  end;
end;

function TEnemyLine.Garbage: Boolean;
begin
  Result := Length(FEnemies) = 0;
end;

procedure TEnemyLine.ChangeColor;
var
  I: Byte;
begin
  for I := 0 to High(FEnemies) do
  begin
    FEnemies[I + 1].Color := FEnemies[I].Color;  
  end;
  FEnemies[0].Color := FEnemies[High(FEnemies)].Color;
end;

procedure TLineEnemy.Stream;
begin
  inherited;
  Stream(@FSeed, SizeOf(FSeed));
end;

constructor TLineEnemy.Create;
  var
    R1, R2: Single;
begin
  inherited;
  R1 := Random;
  R2 := Random;
  FHalf := Vec2f(9, 9); 
  Color := Vec4f(Cos(R1*pi/2), Sin(R1*pi/2)*Cos(R2*pi/2), 
                 Sin(R1*pi/2)*Sin(R2*pi/2),
                 1.0);
  FSeed := Random * pi;
end;

procedure TLineEnemy.Render;
begin
  with Core.Graph do begin
    Color4f(Color);
    DrawRect(Pos - Vec2f(8, 8)/2, Vec2f(8, 8) + Vec2f(0.5, 0), True);
    SetLineWidth(1);
    DrawRect(Pos - Vec2f(14, 14)/2, Vec2f(14, 14), False);
    DrawRect(Pos - Vec2f(15, 15)/2, Vec2f(15, 15), False);
    DrawRect(Pos - Vec2f(16, 16)/2, Vec2f(16, 16), False);
    DrawRect(Pos - Vec2f(17, 17)/2, Vec2f(17, 17), False);
    DrawRect(Pos - Vec2f(18, 18)/2, Vec2f(18, 18), False);
    SetLineWidth(1);
  end;
end;

procedure TLineEnemy.Update;
begin
  Pos := Pos + FVel;
end;
//}}}
//{{{ TActive
procedure TActive.Stream;
begin
  inherited;
  Stream(@FPos, SizeOf(FPos));
  Stream(@FFrc, SizeOf(FFrc));
  Stream(@FRect, SizeOf(FRect));
  Stream(@FLevel, SizeOf(FLevel));
  Stream(@FControl, SizeOf(FControl));
  Stream(@FVel, SizeOf(FVel));
  Stream(@FColor, SizeOf(FColor));
  Stream(@FDamagable, SizeOf(FDamagable));
  Stream(@FHP, SizeOf(FHP));  
  Stream(@FHalf, SizeOf(FHalf));
  Stream(@FOnFloor, SizeOf(FOnFloor));
  Stream(@FCrouch, SizeOf(FCrouch));
  Stream(@FGarbage, SizeOf(FGarbage));
end;

constructor TActive.Create;
begin
  FVel := Vec2f(0, 0);
  FFrc := Vec2f(0, 0);
  FGarbage := False;
  FDamagable := False;
end;

procedure TActive.Kill;
begin
  FGarbage := True;
end;

procedure TActive.Render;
begin
end;

// ,  ... ;)
procedure TActive.Update;
begin
  FCrouch := FControl[ciCrouch] and (FOnFloor or FCrouch);
  if FControl[ciJump] and FOnFloor and not FCrouch then begin
    ApplyForce(Vec2f(0, -98*2));
    FVel := Vec2f(0, 0);
  end else if not FOnFloor then
    ApplyForce(Vec2f(0, 98*0.22));
  if FControl[ciLeft] and not FCrouch then begin
    Pos := Pos + Vec2f(-15, 0);
    FVel.X := -15;
  end else if FControl[ciRight] and not FCrouch then begin
    Pos := Pos + Vec2f(15, 0);
    FVel.X := 15;
  end else if FOnFloor then
    FVel.X *= 0.1;
  FVel.Y *= 0.99;
  FVel := FVel + FFrc;
  if FOnFloor then
    FVel.X *= 0.95;
  Pos := Pos + Vec2f(0.0, FVel.Y/10);

  FFrc := Vec2f(0, 0);
  ResetControls;
end;

procedure TActive.SetPos(NewPos: TVec2f);
begin
  FPos := NewPos;
  FRect := DMath.Rect(FPos - FHalf, FPos + FHalf);
end;

procedure TActive.MovePos(Delta: TVec2f);
begin
  SetPos(FPos + Delta);
end;

procedure TActive.ApplyForce(Delta: TVec2f);
begin
  FFrc += Delta;
end;

procedure TActive.ActiveControl(ID: TControlID);
begin
  FControl[ID] := True;
end;

procedure TActive.ResetControls;
begin
  FillChar(FControl, SizeOf(FControl), False);
end;

procedure TActive.Damage(Value: Integer);
begin
  if FDamagable then
    FHP -= Value;
  if FHP < 0 then
    Kill;
end;
//}}}
//{{{ TEnemy
procedure TEnemy.Stream;
begin
  inherited;
  Stream(@FNeedCollided, SizeOf(FNeedCollided));
end;

constructor TEnemy.Create;
begin
  inherited;
  FDamagable := True;
  Level := 100000;
  FNeedCollided := False;
end;

procedure TEnemy.Damage(Value: Integer);
begin
  inherited;
  Game.Summary.Killed += 1;
end;

procedure TEnemy.Collide(Player: TPlayer);
begin
  if Player.Garbage then
    Exit;
  Game.GoToScene('lose');
end;
// }}}
//{{{ TEnemyTexture
procedure TEnemyTexture.Stream;
begin
  inherited;
  if Loading then
    FTexture := Core.Graph.LoadTex(Data.ReadStr)
  else
    Data.WriteStr(Core.Graph.GetTexFileName(FTexture));
end;

constructor TEnemyTexture.Create;
begin
  inherited Create;
  FTexture := Core.Graph.LoadTex(Tex);
  FHalf := Vec2f(Core.Graph.TexWidth(FTexture), 
                 Core.Graph.TexHeight(FTexture)) / 2;
  Self.Pos := Pos - Vec2f(0, FHalf.Y);
  Self.Color := Color;
  FDamagable := False;
end;

procedure TEnemyTexture.Render; 
begin
  Core.Graph.SetBlendMode(bmAlpha);
  Core.Graph.SetTexture(FTexture);
  Core.Graph.Color4f(Color);
  Core.Graph.DrawTexture(Rect);
  Core.Graph.SetTexture(-1);
  Core.Graph.SetBlendMode(bmNo);
  //Core.Graph.DrawRect(Rect);
end;

procedure TEnemyTexture.Update;
begin
end;
//}}}
// TWheel//{{{
procedure TWheel.Stream;
begin
  inherited;
  Stream(@FTime, SizeOf(FTime));
  Stream(@FVel, SizeOf(FVel));
end;

constructor TWheel.Create(Pos: TVec2f; Level: Integer; Vel: TVec2f);
begin
  inherited Create(Pos, 'DATA\textures\wheel.bit', Vec4f(0.8, 0.2, 0.2, 1.0));
  Self.Level := Level;
  FHP := 5;
  FDamagable := True;
  FTime := Game.Time;
  FNeedCollided := True;
  FVel := Vel;
  //SDebug.Log('Start: ' + IntToStr(Level));
end;

procedure TWheel.Update;
begin
  if Game.Time - FTime > 20000 then
    Kill;
  if OnFloor then
    Pos := Pos + Vec2f(FVel.X, 0);
  Pos := Pos + Vec2f(0, FVel.Y);
end;
//}}}
// {{{ TSimpleEnemy
procedure TSimpleEnemy.Stream;
begin
  inherited;
  Stream(@FSeed, SizeOf(FSeed));
end;

constructor TSimpleEnemy.Create;
  var
    R1, R2: Single;
begin
  inherited;
  R1 := Random;
  R2 := Random;
  FHalf := Vec2f(11, 11); 
  Color := Vec4f(Cos(R1*pi/2), Sin(R1*pi/2)*Cos(R2*pi/2), 
                 Sin(R1*pi/2)*Sin(R2*pi/2),
                 1.0);
  FSeed := Random * pi;
end;

procedure TSimpleEnemy.Render;
begin
  with Core.Graph do begin
    Color4f(Color);
    DrawRect(Pos - Vec2f(8, 8)/2, Vec2f(8, 8) + Vec2f(0.5, 0), True);
    SetLineWidth(1);
    DrawRect(Pos - Vec2f(14, 14)/2, Vec2f(14, 14), False);
    DrawRect(Pos - Vec2f(15, 15)/2, Vec2f(15, 15), False);
    DrawRect(Pos - Vec2f(16, 16)/2, Vec2f(16, 16), False);
    DrawRect(Pos - Vec2f(17, 17)/2, Vec2f(17, 17), False);
    DrawRect(Pos - Vec2f(18, 18)/2, Vec2f(18, 18), False);
    SetLineWidth(1);
  end;
end;

procedure TSimpleEnemy.Update;
begin
  Pos := Pos + Vec2f(Cos(GetTimer/400 + FSeed)*10, 2.5);
end;
//}}}
//{{{ TPotron
constructor TPotron.Create;
begin
  //Game.Summary.Potrons += 1;
end;

procedure TPotron.OnCollide(Map: TMap; Block: Integer); 
begin
  Kill; //   -   
end;

procedure TPotron.OnCollide(Enemy: TEnemy); 
begin
  Game.Summary.Passed += 1;
  Enemy.Damage(1);
  //Enemy.Kill; //    
  Kill;
end;
// }}}
// {{{ TSimplePotron 
procedure TSimplePotron.Stream;
begin
  inherited;
  Stream(@FStart, SizeOf(FStart));
  Stream(@FColor, SizeOf(FColor));
  Stream(@FDist, SizeOf(FDist));
end;

constructor TSimplePotron.Create;
begin
  inherited Create;
  Self.Pos := Pos;
  FStart := Pos;
  FHalf := Vec2f(Size/2, Size/2);
  FVel := Vel;
  FDist := Dist;
  FColor := Color;
  //FSound := Core.Sound.LoadSound('DATA\sounds\Glass1.wav');
  //Core.Sound.PlaySound(FSound);
end;

destructor TSimplePotron.Destroy;
begin
  //Core.Sound.DelSound(FSound);
end;

procedure TSimplePotron.Update; 
begin
  Pos := Pos + FVel;
  if Len2f(FStart - Pos) > FDist then
    FGarbage := True;
end;

procedure TSimplePotron.Render; 
begin
  Core.Graph.Color4f(FColor);
  Core.Graph.DrawCircle(Pos, FHalf.X, True);
end;
//}}}
//{{{ TWeapon
constructor TWeapon.Create;
begin
end;

constructor TWeapon.Create(RechargeTime: Integer);
begin
  FRechargeTime := RechargeTime;
  FLastFire := 0;
end;

procedure TWeapon.Stream; 
begin
  Stream(@FPos, SizeOf(FPos));
  Stream(@FLastFire, SizeOf(FLastFire));
  Stream(@FRechargeTime, SizeOf(FRechargeTime));
end;

procedure TWeapon.DoFire;
begin
  Game.Summary.Potrons += 1;
end;

procedure TWeapon.Render;
begin
end;

procedure TWeapon.Update;
begin
end;

class procedure TWeapon.RenderItem(Pos: TVec2f);
begin
end;

procedure TWeapon.Fire;
begin
  if FLastFire + FRechargeTime < GetTimer then begin
    DoFire(Aim, Level);
    FLastFire := GetTimer;
  end;
end;

function TWeapon.GetReady: Boolean;
begin
  Result := FLastFire + FRechargeTime < Game.Time;
end;
//}}}
//{{{ TSimpleWeapon
procedure DrawGlassHalf(Pos: TVec2f);
  var
    I: Integer;
    Vertices: array of TVec2f;
begin
  SetLength(Vertices, 10);
  for I := 0 to High(Vertices) do
    Vertices[I] := Vec2f(Pos.X + 8*Cos(I/High(Vertices)*pi),
                         Pos.Y + 8*Sin(I/High(Vertices)*pi));
  with Core.Graph do begin
    DrawPolygon(Vertices);
  end;
end;

constructor TSimpleWeapon.Create;
begin
  inherited Create(100);
end;

procedure TSimpleWeapon.DoFire;
  var
    Potron: TPotron;
begin
  inherited;
  Potron := TSimplePotron.Create(Pos, (Aim-Pos)/Len2f(Aim-Pos)*10, 
                                      4, 2000, Vec4f(0, 0.4, 0.0, 1.0));
  Potron.Level := Level;
  Game.AddPotron(Potron);
  Core.Sound.PlaySound('DATA\sounds\SimpleWeapon.wav');
end;

procedure TSimpleWeapon.Render;
begin
  Core.Graph.SetColor(0.0, 0.25, 0.0, 0.8);
  DrawGlassHalf(Pos);
end;

class procedure TSimpleWeapon.RenderItem(Pos: TVec2f); 
begin
  Core.Graph.SetColor(0.0, 0.25, 0.0, 0.8);
  DrawGlassHalf(Pos);
end;
//}}}
//{{{ TPlayer
procedure TPlayer.Stream;
begin
  inherited;
  Stream(TStreamable(FWeapons[0]));
  Stream(TStreamable(FWeapons[1]));
end;

constructor TPlayer.Create;
begin
  inherited Create;
  FHalf := Vec2f(24, 16);
  FWeapons[0] := TSimpleWeapon.Create;
  FWeapons[1] := TSimpleWeapon.Create;
end;

destructor TPlayer.Destroy; 
begin
  FWeapons[0].Free;
  FWeapons[1].Free;
  inherited Destroy;
end;

procedure TPlayer.SetPos;
begin
  //inherited SetPos(NewPos + Vec2f(0, Ord(FCrouch) * (CROUCH_VALUE)));
  inherited;
  FWeapons[0].Pos := Pos + Vec2f(  (4 + 8), -10 {+ Ord(FCrouch)*CROUCH_VALUE});
  FWeapons[1].Pos := Pos + Vec2f(- (4 + 8), -10 {+ Ord(FCrouch)*CROUCH_VALUE});
  FHalf := Vec2f(24, 16 - Ord(FCrouch) * (CROUCH_VALUE));
end;

procedure TPlayer.Render;
  var
    Offset: Single;
begin
  //Pos := Pos;
  with Core.Graph do begin
    SetColor(0.367, 0.226, 0.191, 1);
    SetLineWidth(3);
    if not FCrouch then begin
      Offset := - Ord(OnFloor) * FVel.X/15*4 * Ord(Abs(FVel.X) > 1);
      DrawLine(Pos + Vec2f(-4, -8), Pos + Vec2f(4, -8));
      DrawLine(Pos + Vec2f(-20, -8), Pos + Vec2f(-24, -8));
      DrawLine(Pos + Vec2f(20, -8), Pos + Vec2f(24, -8));
      DrawLine(Pos + Vec2f(-24, -8), Pos + Vec2f(-24 + Offset, 10));
      DrawLine(Pos + Vec2f(24, -8), Pos + Vec2f(24 + Offset, 10));
      DrawLine(Pos + Vec2f(-24 + Offset, 10), Pos + Vec2f(-20 + Offset, 16));
      DrawLine(Pos + Vec2f(24 + Offset, 10), Pos + Vec2f(20 + Offset, 16));
    end else begin
      Offset := CROUCH_VALUE;      
      Offset := 0;      
      DrawLine(Pos + Vec2f(-4,  -8 + Offset), Pos + Vec2f(4,   -8 + Offset));
      DrawLine(Pos + Vec2f(-20, -8 + Offset), Pos + Vec2f(-24, -8 + Offset));
      DrawLine(Pos + Vec2f(20,  -8 + Offset), Pos + Vec2f(24,  -8 + Offset));
      DrawLine(Pos + Vec2f(-24, -8 + Offset), Pos + Vec2f(-24, -4 + Offset));
      DrawLine(Pos + Vec2f(24,  -8 + Offset), Pos + Vec2f(24,  -4 + Offset));
      DrawLine(Pos + Vec2f(-24, -4 + Offset), Pos + Vec2f(20,   4 + Offset));
      DrawLine(Pos + Vec2f(-20,  4 + Offset), Pos + Vec2f(24,  -4 + Offset));
      DrawLine(Pos + Vec2f(20,   4 + Offset), Pos + Vec2f(24,   6 + Offset));
      DrawLine(Pos + Vec2f(-20,  4 + Offset), Pos + Vec2f(-24,  6 + Offset));
    end;

    FWeapons[0].Render;
    FWeapons[1].Render;

    //DrawRect(Rect, False);
  end;
end;

procedure TPlayer.Update;
begin
  Pos := Pos;
  if FControl[ciRightFire] then
    FWeapons[0].Fire(Game.Camera.Pos + Core.Window.Mouse, Level);
  if FControl[ciLeftFire] then
    FWeapons[1].Fire(Game.Camera.Pos + Core.Window.Mouse, Level);
  inherited;
end;

procedure TPlayer.AddWeapon(Weapon: TWeapon);
begin
  FWeapons[1].Free;
  FWeapons[1] := FWeapons[0];
  FWeapons[0] := Weapon;
end;
//}}}
// TItem//{{{
procedure TItem.OnPlayer(Player: TPlayer);
begin
  Core.Sound.PlaySound('DATA\sounds\Item.wav');
end;

procedure TItem.Update; 
begin
end;
//}}}
// TWeaponItem//{{{
procedure TWeaponItem.Stream;
  var
    S: String;
begin
  inherited;
  {if Loading then begin
    Stream(S);
    FWeapon := CWeapon(Factory.GetClass(S));
  end else begin
    S := FWeapon.ClassName;
    Stream(S);
  end;}
  Stream(CStreamable(FWeapon));
end;

constructor TWeaponItem.Create(Weapon: CWeapon);
begin
  FWeapon := Weapon;
end;

procedure TWeaponItem.OnPlayer(Player: TPlayer);
begin
  inherited;
  Player.AddWeapon(FWeapon.Create);
end;

procedure TWeaponItem.Render;
begin
  FWeapon.RenderItem(Pos);
  Core.Graph.SetColor(0, 0, 0.8, 1);
  Core.Graph.SetLineWidth(2);
  Core.Graph.DrawCircle(Pos, 14, False);
end;
//}}}
//{{{ TGame
function TGame.GetTime: Integer;//{{{
begin
  Result := GetTimer - FTime;
end;
//}}}
procedure TGame.AddScene;//{{{
begin
  SetLength(FScenes, Length(FScenes) + 1);
  FScenes[High(FScenes)].Name := Name;
  FScenes[High(FScenes)].Start := Start;
  FScenes[High(FScenes)].Finish := Finish;
  FScenes[High(FScenes)].Render := Render;
  FScenes[High(FScenes)].Update := Update;
end;
//}}}
procedure TGame.GoToScene(Name: String);//{{{
  var
    I: Integer;
begin
  for I := 0 to High(FScenes) do
    if FScenes[I].Name = Name then begin
      FNextScene := I;
      Exit;
    end;
  SDebug.Log('! Scene not found: ' + Name);
end;

//}}}
procedure TGame.GameStart;//{{{
begin
  Core.Sound.PlayMusic(FLevels[FCurLevel].Music);
  Core.Sound.PlaySoundMMSystem('');
  //SDebug.Log('Start');
end;
//}}}
procedure TGame.GameFinish;//{{{
begin
end;
//}}}
procedure TGame.GameUpdate;//{{{
  var
    Delta: TVec2f;
    I, J, C: Integer;
    Enemy: TSimpleEnemy;
    EnemyLine: TEnemyLine;
begin
  FMap.Update;
  FMap.UpdateEntries(FPlayer.Rect);
  if Random(1000) = 0 then begin
    with EnemyLine do
    begin
      FCount := 9; // - 
      FPos := Vec2f(FPlayer.Pos.X + Random(100) - 50, 
                    FPlayer.Pos.Y - GHeight - 40); 
      // ,    
      Create;
    end;
    AddEnemyLine(EnemyLine);
  end;
  Delta := Vec2f(0, 0);
  with Core do begin
    if Key(Ord('A')) then
      FPlayer.ActiveControl(ciLeft);
    if Key(Ord('D')) then
      FPlayer.ActiveControl(ciRight);
    if Key(Ord('S')) then
      FPlayer.ActiveControl(ciCrouch);
    if Mouse(miLeft) then
      FPlayer.ActiveControl(ciLeftFire);
    if Mouse(miRight) then
      FPlayer.ActiveControl(ciRightFire);
  end;

  FPlayer.Update;
  FPlayer.OnFloor := False;
  for I := 0 to FMap.BlocksCount - 1 do 
    if FMap.Blocks[I].Speed < 0.1 then begin
      if (I < FPlayer.Level) and FMap.Higher(FPlayer.Rect, I) then
        FPlayer.Level := I;
      if I >= FPlayer.Level then begin
        if FMap.Collide(FPlayer.Rect, I, Delta) = mcUp then begin
          FPlayer.Level := I;
          FPlayer.OnFloor := True;
        end;
        FPlayer.MovePos(Delta);
      end;
    end else 
      if (FPlayer.Vel.Y >= 0) and
         (FMap.Blocks[I].Rect.A.Y + 50 > FPlayer.Rect.B.Y) and
         (FMap.Collide(FPlayer.Rect, I, Delta) = mcUp) then begin
        FPlayer.MovePos(Delta);
        FPlayer.OnFloor := True;
      end;
  
  // FAKE:
  if FLevels[FCurLevel].Name = 'level02' then
    FPlayer.Level := FMap.GetLevel(FPlayer.Rect);
  
  //Writeln('Level: ', FPlayer.Level, ' ', FPlayer.OnFloor);

  UpdateEnemies;

  I := 0;
  While I < Length(FPotrons) do begin
    FPotrons[I].Update;
    for J := 0 to FMap.BlocksCount - 1 do 
      if FPotrons[I].Level <= J then
        if FMap.Collide(FPotrons[I].Rect, J, Delta) <> mcNo then
          FPotrons[I].OnCollide(FMap, J);
    for J := 0 to High(FEnemies) do
      if Intersect(FPotrons[I].Rect, FEnemies[J].Rect) then
        FPotrons[I].OnCollide(FEnemies[J]);
    if FPotrons[I].Garbage then begin
      FPotrons[I].Free;
      FPotrons[I] := FPotrons[High(FPotrons)];
      SetLength(FPotrons, Length(FPotrons) - 1);
    end else
      Inc(I);
  end;

  I := 0;
  while I < Length(FItems) do begin
    FItems[I].Update;
    if Intersect(FItems[I].Rect, FPlayer.Rect) or FItems[I].Garbage then begin
      FItems[I].OnPlayer(FPlayer);
      FItems[I].Free;
      FItems[I] := FItems[High(FItems)];
      SetLength(FItems, Length(FItems) - 1);
    end else
      Inc(I);
  end;

  FCamera.Update(FPlayer.Pos);
end;

//}}}
procedure TGame.GameRender;//{{{
  var
    I: Integer;
begin
  with Core.Graph do begin
    FCamera.Bind;
    SetBlendMode(bmAlpha);
    DrawRect(Rect(Vec2f(0, 0), Vec2f(GWidth, GHeight)),
             FLevels[FCurLevel].Sky1,//Vec4f(214/255, 168/255, 135/255, 1.0),
             FLevels[FCurLevel].Sky1,//Vec4f(214/255, 168/255, 135/255, 1.0),
             FLevels[FCurLevel].Sky2,//Vec4f(205/255, 41/255, 77/255, 1.0),
             FLevels[FCurLevel].Sky2//Vec4f(205/255, 41/255, 77/255, 1.0)
            );
    FCamera.Focus;
    SetBlendMode(bmAlpha);
    //PushMatrix;
    FMap.Render;
    SetBlendMode(bmAlpha);
    
    for I := 0 to High(FItems) do
      FItems[I].Render; 
    
    SetColor(0.25, 0.0, 0.0, 0.1);
    //DrawRect(FPlayer);
    FPlayer.Render;

    //for I := 0 to High(FEnemiesLine) do
    //  FEnemiesLine[I].Render;

    for I := 0 to High(FEnemies) do
      FEnemies[I].Render;
    for I := 0 to High(FPotrons) do
      FPotrons[I].Render;
    FCamera.Unfocus;

    if Core.Key(Ord('3')) then begin
      SetBlendMode(bmAlpha);
      SetColor(0.9, 0.9, 0.9, 1.0);
      FFixed.Draw(Vec2f(20, 40),  '[1]      ');
      FFixed.Draw(Vec2f(20, 60),  '[2]      ');
      FFixed.Draw(Vec2f(20, 80),  '[3]     ');
      FFixed.Draw(Vec2f(20, 120), '[a]     ');
      FFixed.Draw(Vec2f(20, 140), '[d]     ');
      FFixed.Draw(Vec2f(20, 160), '[w]     ');
      FFixed.Draw(Vec2f(20, 180), '[s]     ');
      FFixed.Draw(Vec2f(20, 220), '[]  ');
      FFixed.Draw(Vec2f(20, 240), '[   ]  ');
      FFixed.Draw(Vec2f(20, 260), '[  ]  ');
      FFixed.Draw(Vec2f(20, 300), '[Enter] ');
      FFixed.Draw(Vec2f(20, 320), '[Esc] ');
    end;
  end;
end;

//}}}
procedure TGame.WinStart;//{{{
begin
  Core.Sound.PlayMusic('DATA\music\win0.xm');
  FPlayer.Pos := Vec2f(-100, 300);
  FNextLevel := FLevels[FCurLevel].NextLevel;
end;
//}}}
procedure TGame.WinFinish;//{{{
begin
end;
//}}}
procedure TGame.WinUpdate;//{{{
begin
  FPlayer.ActiveControl(ciRight);
  FPlayer.Pos := FPlayer.Pos + Vec2f(5, 0);
  FPlayer.Vel := Vec2f(15, 0);
end;

//}}}
procedure TGame.WinRender;//{{{
  var
    Met: Integer;
begin
  Core.Graph.Clear(214/255, 168/255, 135/255);
  FPlayer.Render;
  Core.Graph.SetColor(0.2, 0.8, 0.2, 1.0);
  Core.Graph.SetBlendMode(bmAlpha);
  FFont.Draw(Vec2f(330 - FFont.GetWidth(':'), 100), ':'); 
  FFont.Draw(Vec2f(370, 100), IntToStr(FSummary.Killed) + ' ' + GetRaz(FSummary.Killed));
  FFont.Draw(Vec2f(330 - FFont.GetWidth(':'), 150), ': '); 
  FFont.Draw(Vec2f(370, 150), IntToStr(FSummary.Dead) + ' ' + GetRaz(FSummary.Dead));
  //     ,   ]:->
  FFont.Draw(Vec2f(330 - FFont.GetWidth(':'), 200), ': '); 
  Met := Min(97, Max(3, Round(100*FSummary.Passed/Max(1, FSummary.Potrons))));
  FFont.Draw(Vec2f(370, 200), IntToStr(Met) + '%');
  FFont.Draw(Vec2f(GWidth/2 - FFont.GetWidth(' Enter')/2, 400), ' Enter');
end;

//}}}
procedure TGame.TitleStart;//{{{
begin
  FColor := Vec4f(1, 1, 1, 0);
end;
//}}}
procedure TGame.TitleFinish;//{{{
begin
end;
//}}}
procedure TGame.TitleUpdate;//{{{
begin
  FColor.W += 0.01;
end;

//}}}
procedure TGame.TitleRender;//{{{
  var
    Met: Integer;
begin
  GameRender;
  Core.Graph.SetBlendMode(bmAlpha);
  Core.Graph.Color4f(FColor);
  Core.Graph.DrawRect(Rect(Vec2f(0, 0), Vec2f(GWidth, GHeight)));
  Core.Graph.SetColor(0.2, 0.8, 0.2, 1.0);
  Core.Graph.SetBlendMode(bmAlpha);
  DrawText(GWidth/2, GHeight-30, 0.5, 12.0, '');
  DrawText(GWidth/2, GHeight-30, 0.5, 11.0, '(The Benevollow)');
  DrawText(GWidth/2, GHeight-30, 0.5, 10.0, '');
  DrawText(GWidth/2, GHeight-30, 0.5, 9.0, '');
  DrawText(GWidth/2, GHeight-30, 0.5, 8.0, ', OnOff');
  DrawText(GWidth/2, GHeight-30, 0.5, 7.0, '');
  DrawText(GWidth/2, GHeight-30, 0.5, 6.0, ' ');
  DrawText(GWidth/2, GHeight-30, 0.5, 5.0, ' 47  igdc');
  DrawText(GWidth/2, GHeight-30, 0.5, 4.0, 'http://igdc.ru');
  DrawText(GWidth/2, GHeight-30, 0.5, 3.0, '');
  DrawText(GWidth/2, GHeight-30, 0.5, 2.0, '  !');
  DrawText(GWidth/2, GHeight-30, 0.5, 1.0, ' Enter');
end;

//}}}
procedure TGame.LoseStart;//{{{
begin
  Core.Sound.PlayMusic('');
  Core.Sound.PlaySoundMMSystem('DATA\sounds\DieGamer.wav');
  FColor := Vec4f(1, 1, 1, 0);
  Inc(Summary.Dead);
end;
//}}}
procedure TGame.LoseFinish;//{{{
begin
end;
//}}}
procedure TGame.LoseUpdate;//{{{
begin
  FColor.W += 0.01;
end;
//}}}
procedure TGame.LoseRender;//{{{
begin
  GameRender;
  Core.Graph.SetBlendMode(bmAlpha);
  Core.Graph.Color4f(FColor);
  Core.Graph.DrawRect(Rect(Vec2f(0, 0), Vec2f(GWidth, GHeight)));
  Core.Graph.SetColor(0.8, 0.0, 0.2, 1.0);
  FFont.Draw(Vec2f(100, 200), ' !'); 
  FFont.Draw(Vec2f(140, 250), '  Enter.');  
end;
//}}}
procedure TGame.FreeObjects;//{{{
  var
    I: Integer;
begin
  for I := 0 to High(FPotrons) do
    FPotrons[I].Free;
  SetLength(FPotrons, 0);
  for I := 0 to High(FEnemies) do
    FEnemies[I].Free;
  SetLength(FEnemies, 0);
  for I := 0 to High(FItems) do
    FItems[I].Free;
  SetLength(FItems, 0);
end;
//}}}
procedure TGame.UpdateEnemies;//{{{
  var
    I, J: Integer;
    Delta: TVec2f;
begin
  J := 0;
  While J < Length(FEnemies) do begin
    FEnemies[J].Update;
    FEnemies[J].OnFloor := False;
    if FEnemies[J].NeedCollided then begin
      for I := 0 to FMap.BlocksCount - 1 do 
        if FMap.Blocks[I].Speed < 0.1 then begin
          //if (I < FEnemies[J].Level) and FMap.Higher(FEnemies[J].Rect, I) then
          //  FEnemies[J].Level := I;
          if I >= FEnemies[J].Level then begin
            if FMap.Collide(FEnemies[J].Rect, I, Delta) = mcUp then begin
              FEnemies[J].Level := I;
              FEnemies[J].OnFloor := True;
            end;
            FEnemies[J].MovePos(Delta);
          end;
        end;
      //FEnemies[J].Level := FMap.GetLevel(FEnemies[J].Rect);
      //SDebug.Log(IntToStr(FEnemies[J].Level));
    end;
    if FEnemies[J].Garbage then begin
      FEnemies[J].Free;
      FEnemies[J] := FEnemies[High(FEnemies)];
      SetLength(FEnemies, Length(FEnemies) - 1);
    end else
      Inc(J);
  end;
  
  I := 0;
  While I < Length(FEnemiesLine) do
  begin
    if FEnemiesLine[I].Garbage then begin
      FEnemiesLine[I] := FEnemiesLine[High(FEnemiesLine)];
      SetLength(FEnemiesLine, Length(FEnemiesLine) - 1);
    end else
    begin
      Inc(I);
    end;
  end;

  for I := 0 to High(FEnemies) do
    if Intersect(FPlayer.Rect, FEnemies[I].Rect) then
      FEnemies[I].Collide(FPlayer); 
end;
//}}}
constructor TGame.Create;//{{{
  const
    Offset = 200;
  var
    Data: TData;
begin
  FCamera := TCamera2D.Create;
  FCamera.Center := Rect(Vec2f(Offset, 150), Vec2f(GWidth - Offset, 400));

  AddScene('game', GameStart, GameFinish, GameRender, GameUpdate);
  AddScene('win', WinStart, WinFinish, WinRender, WinUpdate);
  AddScene('lose', LoseStart, LoseFinish, LoseRender, LoseUpdate);
  AddScene('title', TitleStart, TitleFinish, TitleRender, TitleUpdate);
  GoToScene('win');
  FCurScene := 0;
  FNextScene := -1;

  Data := TData.Create('DATA\maps\map0.dmp');
  FMap := TMap.Create(Data);
  Data.Free;

  FPlayer := TPlayer.Create;
  FPlayer.Pos := Vec2f(60, 300);

  SetLength(FPotrons, 0);
  SetLength(FEnemies, 0);

  //Core.Sound.PlayMusic('DATA\music\music0.xm');
  Core.Sound.SoundVolume := 1;
  Core.Sound.MusicVolume := 0.93;

  FFontMan := TFontMan.Create(Core.Graph);
  FFont := FFontMan.CreateFont('DATA\fonts\FrankC.bit', -3);
  FFixed := FFontMan.CreateFont('DATA\fonts\FixedSys.bit', -3);

  FData := TData.Create;
end;
//}}}
destructor TGame.Destroy;//{{{
begin
  FFont.Free;
  FFontMan.Free;
  FData.Free;
  FPlayer.Free;
  FMap.Free;
  FCamera.Free;
end;
//}}}
procedure TGame.AddLevel(Level: TLevelData);//{{{
begin
  SetLength(FLevels, Length(FLevels) + 1);
  FLevels[High(FLevels)] := Level;
end;

//}}}
procedure TGame.LoadLevel(Name: String);//{{{
  var
    Item: TItem;
    I: Integer;
begin
  FreeObjects;
  for I := 0 to High(FLevels) do
    if FLevels[I].Name = Name then
      FCurLevel := I;
  FPlayer.Pos := FLevels[FCurLevel].Player;
  FMap.LoadBlocks(FLevels[FCurLevel].Blocks);
  FMap.LoadEntries;
  FLevels[FCurLevel].Load;
  {Item := TWeaponItem.Create(TShotGun);
  Item.Pos := FPlayer.Pos + Vec2f(200, -30);
  AddItem(Item);
  Item := TWeaponItem.Create(TLaser);
  Item.Pos := FPlayer.Pos + Vec2f(250, -30);
  AddItem(Item);
  Item := TWeaponItem.Create(TPinkWeapon);
  Item.Pos := FPlayer.Pos + Vec2f(300, -30);
  AddItem(Item);}
  FCamera.Center := FLevels[FCurLevel].Center;
  FLastSaved := '';
  //AddEnemy(TBossOnOff.Create( FPlayer.Pos + Vec2f(300, -100)));
  SaveGame('DATA\saves\' + GetLevel.Name + '.sav');
  FNextLevel := '';
  GoToScene('game');
end;
//}}}
procedure TGame.SaveGame(Name: String);//{{{
  var
    GameStream: TGameStream;
begin
  if FPlayer.Garbage then
    Exit;
  GameStream := TGameStream.Create;
  GameStream.Save(Name);
  GameStream.Free;
  FLastSaved := Name;
end;
//}}}
procedure TGame.LoadGame(Name: String);//{{{
  var
    GameStream: TGameStream;
begin
  GameStream := TGameStream.Create;
  GameStream.Load(Name);
  GameStream.Free;
  Core.Sound.PlaySoundMMSystem('');
  Core.Sound.PlayMusic(GetLevel.Music);
  GoToScene('game');
end;
//}}}
procedure TGame.AddPotron(Potron: TPotron);//{{{
begin
  SetLength(FPotrons, Length(FPotrons) + 1);
  FPotrons[High(FPotrons)] := Potron;
end;
//}}}
procedure TGame.AddEnemy(Enemy: TEnemy);//{{{
begin
  SetLength(FEnemies, Length(FEnemies) + 1);
  FEnemies[High(FEnemies)] := Enemy;
end;
//}}}
procedure TGame.AddEnemyLine(Enemy: TEnemyLine);//{{{
  var
    I: Integer;
begin
  SetLength(FEnemiesLine, Length(FEnemiesLine) + 1);
  FEnemiesLine[High(FEnemiesLIne)] := Enemy;
  for I := 0 to High(Enemy.FEnemies) do
    AddEnemy(Enemy.FEnemies[I]);
end;
//}}}
procedure TGame.AddItem(Item: TItem);//{{{
begin
  SetLength(FItems, Length(FItems) + 1);
  FItems[High(FItems)] := Item;
end;

//}}}
procedure TGame.Render;//{{{
begin
  with Core.Graph do begin
    //Clear(214/255, 168/255, 135/255);
    BeginRender2D(GWidth, GHeight);
    FScenes[FCurScene].Render;
    EndRender;
  end;
end;
//}}}
procedure TGame.Update;//{{{
begin
  Core.Update;

  if FNextScene <> -1 then begin
    FScenes[FCurScene].Finish;
    FScenes[FNextScene].Start;
    FCurScene := FNextScene;
    FNextScene := -1;
  end;
  
  FScenes[FCurScene].Update;

  if FLastFPS <> Core.WinApi.FPS then
    Core.Window.Caption := GName + ' (FPS: ' + IntToStr(Core.WinApi.FPS) + ')';
  FLastFPS := Core.WinApi.FPS;
end;
//}}}
procedure TGame.OnEvent(const Event: TEvent);//{{{
begin
  if Event.ID = eiKey then
    if not Core.Key(Event.Key) then 
      KeyPressed(Event.Key);
  if Event.ID = eiClick then
    case Event.Button of
      miLeft:  FPlayer.ActiveControl(ciLeftFire);
      miRight: FPlayer.ActiveControl(ciRightFire);
    end;
  {with Event do 
      case ID of
        eiKey: if Key = Ord('s') then
                 Delta := Vec2f(0, -0.1)
               else if Key = Ord('w') then
                 Delta := Vec2f(0, 0.1)
               else if Key = Ord('a') then
                 Delta := Vec2f(-0.1, 0)
               else if Key = Ord('d') then
                 Delta := Vec2f(0.1, 0)
               else if Key = 27 then
                 Core.WinApi.Shutdown;
        else}
          Core.OnEvent(Event);
      //end;
end;

//}}}
procedure TGame.KeyPressed(Key: Byte);//{{{
begin
  if Key = Ord('W') then
    FPlayer.ActiveControl(ciJump);
  if Key = Ord('1') then
    Core.Sound.MusicVolume := Core.Sound.MusicVolume - 0.04;
  if Key = Ord('2') then
    Core.Sound.MusicVolume := Core.Sound.MusicVolume + 0.04;
  if Key = 13 then
    if FScenes[FCurScene].Name = 'title' then
      Core.WinApi.Shutdown
    else if FNextLevel <> '' then
      LoadLevel(FNextLevel)
    else if FLastSaved <> '' then
      LoadGame(FLastSaved);

  //  ;)

  {// / TPlayer'
  if Key = Ord('3') then begin
    FData.Clear;
    FData.WriteStr(FPlayer.ClassName);
    FPlayer.Save(FData);
  end;
  if Key = Ord('4') then begin
    FPlayer.Free;
    FData.Rewind;
    TStreamable(FPlayer) := Factory.CreateStreamable(FData.ReadStr, FData);
  end;
  //     
  if Key = Ord('5') then
    Core.Sound.MusicPattern := 162;
  if Key = Ord('6') then
    GoToScene('win');

  if Key = Ord('9') then
    SaveGame('DATA\saves\level01.sav');
  if Key = Ord('0') then
    if FLastSaved <> '' then
      LoadGame(FLastSaved);}
end;

//}}}
function TGame.GetEnemyCount: Integer;//{{{
begin
  Result := Length(FEnemies);
end;

//}}}
function TGame.GetEnemy(Index: Integer): TEnemy;//{{{
begin
  Result := FEnemies[Index];
end;

//}}}
function TGame.GetLevel: TLevelData;//{{{
begin
  Result := FLevels[FCurLevel];
end;
//}}}
procedure TGame.DrawText(X, Y: Single; AX, AY: Single; Text: String);//{{{
  var
    Pos: TVec2f;
begin
  Pos := Vec2f(X - AX*FFont.GetWidth(Text), Y - AY*FFont.GetHeight(Text));
  FFont.Draw(Pos, Text);
end;
//}}}
//}}}
//{{{ TGameStream
procedure TGameStream.Stream;
  var
    I, Count: Integer;
    Delta: Integer;
begin
  inherited;
  with Game do begin
    Stream(TStreamable(Game.FMap));
    Stream(TStreamable(FPlayer));

    Delta := GetTimer - FTime;
    Stream(@Delta, SizeOf(Delta));
    FTime := GetTimer - Delta;

    Count := Length(FPotrons);
    Stream(@Count, SizeOf(Count));
    SetLength(FPotrons, Count);
    for I := 0 to Count - 1 do
      Stream(TStreamable(FPotrons[I]));
    
    Count := Length(FEnemies);
    Stream(@Count, SizeOf(Count));
    SetLength(FEnemies, Count);
    for I := 0 to Count - 1 do
      Stream(TStreamable(FEnemies[I]));
    SetLength(FEnemiesLine, 0);

    Count := Length(FItems);
    Stream(@Count, SizeOf(Count));
    SetLength(FItems, Count);
    for I := 0 to Count - 1 do
      Stream(TStreamable(FItems[I]));
    
    Stream(TStreamable(FCamera));

    Stream(@FCurScene, SizeOf(FCurScene));
    Stream(@FNextScene, SizeOf(FNextScene));
    
    // TODO: !
    Stream(@FCurLevel, SizeOf(FCurLevel));
    
    //Stream(@FSummary, SizeOf(FSummary));
  end;
end;
//}}}

initialization//{{{
  Factory.Regist(TActive);
  Factory.Regist(TEnemy);
  Factory.Regist(TSimpleEnemy);
  Factory.Regist(TEnemyTexture);
  Factory.Regist(TWheel);
  Factory.Regist(TLineEnemy);
  Factory.Regist(TPotron);
  Factory.Regist(TSimplePotron);
  Factory.Regist(TWeapon);
  Factory.Regist(TSimpleWeapon);
  Factory.Regist(TPlayer);
  Factory.Regist(TItem);
  Factory.Regist(TWeaponItem);
  Factory.Regist(TGameStream);
//}}}
end.
