//{$MACRO on}
unit DUtils;

interface

uses
  windows;

Type
Float = Single;
TData = class;

TStreamable = class;
CStreamable = class of TStreamable;

TProcOfObj = procedure of object;

TUtils = class
  class function GetTimer: Integer;
end;

// {{{ TStreamable
// ,       
//
// 1.        Stream,   .
//          ,      
//      , 
//      
//      Stream(FName);
//      Stream(FID);
//      Stream(@FPos, SizeOf(FPos));
//      Stream(TStreamable(FWeapon));
//    ,     / ;)
//
// 2.    ,   ,   
//       Factory.   :
//
//      Factory.Regist(TWeapon);
//
//            
//
//      !!! [TStreamableFactory.CreateStreamable] Class not found: TWeapon
//
//         ,    
//       . :)
TStreamable = class(TObject)
private
  FLoading: Boolean;
  FData: TData;
protected
  procedure Stream(P: Pointer; Size: Integer); overload;
  procedure Stream(var I: Integer); overload;
  procedure Stream(var F: Float); overload;
  procedure Stream(var S: String); overload;
  procedure Stream(var Obj: TStreamable); overload;
  procedure Stream(var Obj: CStreamable); overload;
  procedure Stream; virtual; overload;
  property Loading: Boolean read FLoading;
  property Data: TData read FData;
public
  constructor CreateFromStream(Data: TData);
  procedure Save(Data: TData); overload;
  procedure Load(Data: TData); overload;
  procedure Save(FileName: String); overload;
  procedure Load(FileName: String); overload;
  procedure Stream(Loading: Boolean; Data: TData); overload;
end;
//}}}

generic TList<TItem> = class(TObject)
var private
  FItems: Array Of TItem;
public
  procedure SetLength(Len: Integer); inline;
  function High: Integer; inline;
  function Length: Integer; inline;
  procedure Add(Item: TItem);
  procedure Del(Item: TItem; All: Boolean = True);
  function Find(Item: TItem): Integer;
  function GetItem(Index: Integer): TItem; inline;
  procedure SetItem(Index: Integer; Item: TItem); inline;
  property Items[Index: Integer]: TItem read GetItem write SetItem; default;
end;
TListOfInteger = specialize TList<Integer>;
TListOfString = specialize TList<String>;
TListOfObject = specialize TList<TObject>;

TListOfCStreamable = specialize TList<CStreamable>;
TStreamableFactory = class
private
  FClasses: TListOfCStreamable;
public
  constructor Create;
  destructor Destroy; override;
  procedure Regist(Streamable: CStreamable); inline;
  function GetClass(Name: String): CStreamable;
  function CreateStreamable(Name: String; Data: TData): TStreamable;
end;

TData = class
private
  FBytes: array of Byte;
  FPos: Integer;
public
  constructor Create; overload;
  constructor Create(FileName: string); overload;
  procedure Clear;
  procedure ReadFromFile(FileName: string);
  procedure WriteToFile(FileName: string);
  procedure WriteVar(P: Pointer; Size: Integer);
  procedure WriteInt(I: Integer);
  procedure WriteByte(B: Byte);
  procedure WriteFloat(F: Float);
  procedure WriteStr(S: String);
  procedure Rewind;
  procedure SeekAbs(NewPos: Integer);
  procedure ReadVar(P: Pointer; Size: Integer);
  function ReadInt: Integer;
  function ReadUint: Cardinal;
  function ReadByte: Byte;
  function ReadStr: String;
  function GetPtr: Pointer;
  function GetData: Pointer;
  function GetSize: Integer;
end;

Function IntToStr(Value: integer): string;
Function StrToInt(Value: string): integer;
Function StrToIntDef(Value: String; Def: integer): integer;
Function StrToFloat(S: String): Single;
Function FloatToStr(F: Single): String;
function CharArrToStr(const C : Array of Char) : String;
Function HexToInt(S: String): Integer;
Function IntToHex(Value: Integer; Digits: Integer = 0): String;
Function FileExist(Filename: string): boolean;
Procedure CreateFile(FileName: string);
Procedure DeleteFile(Filename: string);
Function GetTimer: integer;
//     
Function AnsiLowerCase(const S: string): string;

function ToStr(V: Array Of Const): String;

var
  Factory: TStreamableFactory;

function GetRaz(I: Integer): String;

implementation

uses
  DDebug;

class function TUtils.GetTimer: Integer;
var
  T, F: LARGE_INTEGER;
begin
  QueryPerformanceFrequency(int64(F));
  QueryPerformanceCounter(int64(T));
  Result:=Trunc(1000*T.QuadPart/F.QuadPart);
end;

//{{{ TStreamable
procedure TStreamable.Stream(P: Pointer; Size: Integer);//{{{
begin
  if FLoading then
    FData.ReadVar(P, Size)
  else
    FData.WriteVar(P, Size);
end;

//}}}
procedure TStreamable.Stream(var I: Integer);//{{{
begin
  Stream(@I, SizeOf(I));
end;

//}}}
procedure TStreamable.Stream(var F: Float);//{{{
begin
  Stream(@F, SizeOf(F));
end;

//}}}
procedure TStreamable.Stream(var S: String);//{{{
begin
  if FLoading then
    S := FData.ReadStr
  else
    FData.WriteStr(S);
end; 

//}}}
procedure TStreamable.Stream(var Obj: TStreamable);//{{{
begin
  if FLoading then
    Obj := Factory.CreateStreamable(FData.ReadStr, FData)
  else begin
    FData.WriteStr(Obj.ClassName);
    Obj.Stream(FLoading, FData);
  end;
end;

//}}}
procedure TStreamable.Stream(var Obj: CStreamable);//{{{
begin
  if FLoading then
    Obj := Factory.GetClass(FData.ReadStr)
  else begin
    FData.WriteStr(Obj.ClassName);
  end;
end;
//}}}
procedure TStreamable.Stream;//{{{
begin
end;

//}}}
constructor TStreamable.CreateFromStream(Data: TData);//{{{
begin
  inherited;
  Load(Data);
end;

//}}}
procedure TStreamable.Save(Data: TData); //{{{
begin
  Stream(False, Data);
end;

//}}}
procedure TStreamable.Load(Data: TData);//{{{
begin
  Stream(True, Data);
end;

//}}}
procedure TStreamable.Save(FileName: String);//{{{
  var
    Data: TData;
begin
  Data := TData.Create;
  Save(Data);
  Data.WriteToFile(FileName);
  Data.Free;
end;
//}}}
procedure TStreamable.Load(FileName: String);//{{{
  var
    Data: TData;
begin
  Data := TData.Create(FileName);
  Load(Data);
  Data.Free;
end;
//}}}
procedure TStreamable.Stream(Loading: Boolean; Data: TData); //{{{
begin
  FLoading := Loading;
  FData := Data;
  Stream;
end;

//}}}

//}}}

//  generic TList//{{{
//
procedure TList.SetLength(Len: Integer);
begin
  System.SetLength(FItems, Len);
end;

function TList.High: Integer;
begin
  Result := System.High(FItems);
end;

function TList.Length: Integer;
begin
  Result := System.Length(FItems);
end;

//{$DEFINE Length:=L}
//{$DEFINE High:=H}
//{$DEFINE SetLength:=S}
procedure TList.Add(Item: TItem);
begin
  SetLength(Length + 1);
  FItems[High] := Item;
end;

procedure TList.Del(Item: TItem; All: Boolean = True);
  var
    I: Integer;
begin
  I := 0;
  while I < High do
    if FItems[I] = Item then begin
      FItems[I] := FItems[High];
      SetLength(Length - 1);
      if not All then
        Exit;
    end else
      Inc(I);
end;

function TList.Find(Item: TItem): Integer;
  var
    I: Integer;
begin
  for I := 0 to High do
    if FItems[I] = Item then begin
      Result := I;
      Exit;
    end;
  Result := -1;
end;

function TList.GetItem(Index: Integer): TItem;
begin
  Result := FItems[Index];
end;

procedure TList.SetItem(Index: Integer; Item: TItem);
begin
  FItems[Index] := Item;
end;

//{$UNDEF Length}
//{$UNDEF High}
//{$UNDEF SetLength}
//}}}
// TStreamableFactory//{{{
constructor TStreamableFactory.Create;
begin
  FClasses := TListOfCStreamable.Create;
end;

destructor TStreamableFactory.Destroy;
begin
  FClasses.Free;
end;

procedure TStreamableFactory.Regist; 
begin
  FClasses.Add(Streamable);
end;

function TStreamableFactory.GetClass;
  var
    I: Integer;
begin
  for I := 0 to FClasses.High do
    if FClasses[I].ClassName = Name then begin
      Result := FClasses[I];
      Exit;
    end;
  SDebug.Log('!!! [TStreamableFactory.GetClass] Class not found: ' + Name);
  Result := nil;
end;

function TStreamableFactory.CreateStreamable;
  var
    I: Integer;
begin
  for I := 0 to FClasses.High do
    if FClasses[I].ClassName = Name then begin
      Result := FClasses[I].CreateFromStream(Data);
      Exit;
    end;
  SDebug.Log('!!! [TStreamableFactory.CreateStreamable] Class not found: ' + Name);
  Result := nil;
end;

//}}}
//{{{ TData
constructor TData.Create;
begin
  Clear;
  Rewind;
end;

constructor TData.Create(FileName: string);
begin
  Create;
  ReadFromFile(FileName);
end;

procedure TData.Clear;
begin
  SetLength(FBytes, 0);
end;

procedure TData.ReadFromFile(FileName: string);
  var
    F: file of Byte;
begin
  Assign(F, FileName);
  {$I-}
  Reset(F);
  {$I+}
  if IOResult <> 0 then
    Exit;
  SetLength(FBytes, Length(FBytes) + FileSize(F));
  BlockRead(F, FBytes[Length(FBytes) - FileSize(F)], FileSize(F));
  Close(F);
end;

procedure TData.WriteToFile(FileName: string);
  var
    F: file of Byte;
begin
  Assign(F, FileName);
  {$I-}
  Rewrite(F);
  {$I+}
  if IOResult <> 0 then
    Exit;
  BlockWrite(F, FBytes[0], Length(FBytes));
  Close(F);
end;

procedure TData.WriteVar(P: Pointer; Size: Integer);
begin
  SetLength(FBytes, Length(FBytes) + Size);
  Move(P^, FBytes[Length(FBytes) - Size], Size);
end;

procedure TData.WriteInt(I: Integer);
begin
  WriteVar(@I, SizeOf(Integer));
end;

procedure TData.WriteByte(B: Byte);
begin
  WriteVar(@B, SizeOf(Byte));
end;

procedure TData.WriteFloat(F: Float);
begin
  WriteVar(@F, SizeOf(Float));
end;

procedure TData.WriteStr(S: String);
begin
  WriteByte(Length(S));
  WriteVar(@(S[1]), Length(S));
end;

procedure TData.Rewind;
begin
  FPos := 0;
end;

procedure TData.SeekAbs(NewPos: Integer);
begin
  FPos := NewPos;
end;

procedure TData.ReadVar(P: Pointer; Size: Integer);
begin
  if FPos + Size <= Length(FBytes) then begin
    Move(FBytes[FPos], P^, Size);
    FPos += Size;
  end;
end;

function TData.ReadInt: Integer;
begin
  ReadVar(@Result, SizeOf(Integer));
end;

function TData.ReadUint: Cardinal;
begin
  ReadVar(@Result, SizeOf(Cardinal));
end;

function TData.ReadByte: Byte;
begin
  ReadVar(@Result, SizeOf(Byte))
end;

function TData.ReadStr: String;
  var
    Len: Byte;
begin
  Len := ReadByte;
  SetLength(Result, Len);
  ReadVar(@Result[1], Len);
end;

function TData.GetPtr: Pointer;
begin
  Result := @FBytes[FPos];
end;

function TData.GetData: Pointer;
begin
  Result := @FBytes[0];
end;

function TData.GetSize: Integer;
begin
  Result := Length(FBytes);
end;
//}}}

Function InttoStr;//{{{
begin
  Str(Value, Result);
end;
//}}}
Function StrtoInt;//{{{
  Var
    Code: integer;
begin
  Val(Value, Result, Code);
end;
//}}}
Function StrToIntDef(Value: String; Def: integer): integer;//{{{
  Var
    Code: integer;
begin
  Val(Value, Result, Code);
    If Code <> 0 then
      Result := Def;
end;
//}}}
Function StrToFloat(S: String): Single;//{{{
  Var
    Code: Integer;
begin
    If Pos('.', S) <> 0 then begin
      //Insert(',', S, Pos('.', S));
      //Delete(S, Pos('.', S), 1);
    end;
  Val(S, Result, Code);
end;
//}}}
Function FloatToStr(F: Single): String;//{{{
begin
  Str(F, Result);
    If Pos(',', Result) <> 0 then begin
      Insert('.', Result, Pos(',', Result));
      Delete(Result, Pos(',', Result), 1);
    end;
end;
//}}}
Function HexDigitToInt(C: Char): Byte;//{{{
begin
  if C in ['A'..'F'] then
    Result := Ord(C) - Ord('A') + 10
  else if C in ['a'..'f'] then
    Result := Ord(C) - Ord('a') + 10
  else if C in ['0'..'9'] then
    Result := Ord(C) - Ord('0')
  else
    Result := 0;
end;
//}}}
Function HexToInt(S: String): Integer;//{{{
begin
  Result := 0;
  while Length(S) > 0 do begin
    Result := Result*16 + HexDigitToInt(S[1]);
    Delete(S, 1, 1);
  end;
end;
//}}}
const
   HexDigits: array[0..15] of char = '0123456789ABCDEF';

function IntToHex;//{{{
  var 
    i: integer;
begin
 SetLength(result, digits);
 for i := 0 to digits - 1 do
  begin
   result[digits - i] := HexDigits[value and 15];
   value := value shr 4;
  end ;
 while value <> 0 do begin
   result := HexDigits[value and 15] + result;
   value := value shr 4;
 end;
end;

//}}}
function CharArrToStr(const C : Array of Char) : String;//{{{
var I : Integer;
begin
  // convert the array of characters to a String
  I :=0;
  result :='';
  while C[i] <> #0 do
  begin
    result := result + C[I];
    Inc(I);
  end;
end;
//}}}
Function FileExist;//{{{
  Var
    F: File;
begin
  FileMode := 64;
  AssignFile(F, Filename);
  {$I-}
  Reset(F);
  {$I+}
  Result := IOResult = 0;
    If Result then
      CloseFile(F);
end;
//}}}
Procedure CreateFile;//{{{
  Var
    F: TextFile;
begin
    If FileExist(FileName) then
      Exit;
  Assign(F, FileName);
  {$I-}
  Rewrite(F);
  {$I+}
    If IOResult = 0 then
      Close(F);
end;
//}}}
Procedure DeleteFile;//{{{
begin
    If not FileExist(Filename) then
      Exit;
  Windows.DeleteFile(PAnsiChar(Filename));
end;
//}}}
Function GetTimer;//{{{
var
  T, F: LARGE_INTEGER;
begin
  QueryPerformanceFrequency(int64(F));
  QueryPerformanceCounter(int64(T));
  Result:=Trunc(1000*T.QuadPart/F.QuadPart);
end;
//}}}
Function AnsiLowerCase(Const S: String): String;//{{{
  Var
    Len: Integer;
begin
  Len := Length(S);
  SetString(Result, PChar(S), Len);
    If Len > 0 then
      CharLowerBuff(Pointer(Result), Len);
end;
//}}}
Function ToStr(V: Array Of Const): String;//{{{
  Var
    I: Integer;
begin
  Result := '';
  for I := 0 to High(V) do
    case V[I].vType of 
      vtInteger: Result += IntToStr(V[I].vInteger);
      vtBoolean: Result += IntToStr(Ord(V[I].vBoolean));
      vtChar: Result += V[I].vChar;
      vtString: Result += V[I].vString^;
      vtPointer: Result += '@' + IntToStr(LongInt(V[I].vPointer));
      vtPChar: Result += V[I].vPChar;
      vtObject:
        begin
          Result += '<' + V[I].vObject.ClassName + '>';
        end;
      vtClass: Result += '<' + V[I].vClass.ClassName + '>';
      vtAnsiString: Result += AnsiString(V[I].vAnsiString);
    else
      Result += '?';
    end;
end;
//}}}

function GetRaz(I: Integer): String;
begin
  if (I div 10 <> 1) and ((I mod 10) in [2, 3, 4]) then
    Result := ''
  else
    Result := '';
end;

initialization
  Factory := TStreamableFactory.Create;
finalization
  Factory.Free;
end.
