unit DDebug;

interface

{$I Config.inc}

Uses
  DUtils;

Procedure PrintToLogFile(Mess: String);

Type
TLog = class
  UseConsole: Boolean;
  UseLogFile: Boolean;
  Proc: Procedure;
  Lines: Array Of String;
  SingletonMessages: Array Of String;
  FCritical: TCriticalSection;
  Constructor Create;
  Destructor Destroy; override;
  Procedure Print(Mess: String);
  Procedure PrintLn(Mess: String = '');
  Procedure PrintLnSingleton(Mess: String);
  Procedure Clear;
  Function GetLine(ID: Integer): String;
  Property Items[Index: Integer]: String read GetLine; default;
end;

Var
  Log: TLog;

Procedure Assert(Condition: Boolean; Mess: String = '');

implementation

Var
  F: TextFile;

Procedure PrintToLogFile(Mess: String);
begin
  Assign(F, 'Log.txt');
  {$I-}
  Append(F);
  {$I+}
    If IOResult <> 0 then
      Exit;
  Writeln(F, '[' + IntToStr(GetTimer) + '] ' + Mess);
  Close(F);
end;

Constructor TLog.Create;
begin
  FCritical := TCriticalSection.Create;
  Inherited Create;
  UseLogFile := True;
  UseConsole := True;
  SetLength(Lines, 1);
  SetLength(SingletonMessages, 0);
  Println('TLog.Create');
end;

Destructor TLog.Destroy;
begin
  Clear;
  Println('TLog.Destroy');
  SetLength(Lines, 0);
  Inherited Destroy;
  FCritical.Free;
end;

Procedure TLog.Print(Mess: String);
begin
  FCritical.Enter;
  Lines[High(Lines)] := Lines[High(Lines)] + Mess;
  FCritical.Leave;
end;

Procedure TLog.PrintLn(Mess: String = '');
begin
  FCritical.Enter;
  Print(Mess);
  {$IFDEF DOJ_CONSOLE}
    If UseConsole then
      Writeln(Lines[High(Lines)]);
  {$ENDIF}
    If UseLogFile then
      PrintToLogFile(Lines[High(Lines)]);
    If @Proc <> Nil then
      Proc;
  SetLength(Lines, Length(Lines) + 1);
  Lines[High(Lines)] := '';
  FCritical.Leave;
end;

Procedure TLog.PrintLnSingleton(Mess: String);
  Var
    I: Integer;
begin
  FCritical.Enter;
    For I := 0 to High(SingletonMessages) do
      If SingletonMessages[I] = Mess then begin
        Exit;
      end;
  SetLength(SingletonMessages, Length(SingletonMessages) + 1);
  SingletonMessages[High(SingletonMessages)] := Mess;
  PrintLn(Mess);
  FCritical.Leave;
end;

Procedure TLog.Clear;
begin
  FCritical.Enter;
    If Lines[High(Lines)] <> '' then
      PrintLn;
  SetLength(Lines, 1);
  Lines[0] := '';
  FCritical.Leave;
end;

Function TLog.GetLine;
begin
  FCritical.Enter;
    If ID > High(Lines) then
      Result := ''
    else
      Result := Lines[High(Lines) - ID];
  FCritical.Leave;
end;

Procedure Assert;
begin
  {$IFDEF DOJ_ASSERT}
    If Not Condition then begin
      Log.PrintLn('Assert error: ' + Mess);
    end;
  {$ENDIF}
end;

initialization
  Assign(F, 'Log.txt');
  Rewrite(F);
  Close(F);
  Log := TLog.Create;
finalization
  Log.Free;
end.
