unit DTypeClasses;

interface

Uses
  DDebug,
  DUtils,
  DCore;

Type
TValueType = (vtStr, vtInt, vtDbl);
TValue = packed record
  S: String;
  {_Type: TValueType;
  I: Integer;
  D: Double;}
  case _Type: TValueType of
    vtInt: (I: Integer);
    vtDbl: (D: Double);
end; 

TUniVar = class(TTypeClass)
private
  FValue: TValue;
public
  Constructor Create(Owner: TEntity); override; overload;
  Destructor Destroy; override;
  Function ToString: String; override;
  Function ToInteger: Integer; override;
  Function ToDouble: Double; override;
  Procedure SetString(S: String); override;
  Procedure SetInteger(I: Integer); override;
  Procedure SetDouble(D: Double); override;  
end;

TIntegerClass = class(TTypeClass)
  Constructor Create(Owner: TEntity); override; overload;
  //Constructor Create(Owner: TEntity; P: PInteger); overload;
  Destructor Destroy; override;
  Function ToString: String; override;
  Function ToInteger: Integer; override;
  Function ToDouble: Double; override;
  Procedure SetString(S: String); override;
  Procedure SetInteger(I: Integer); override;
  Procedure SetDouble(D: Double); override;  
end;

TStringClass = class(TTypeClass)
  Constructor Create(Owner: TEntity); override; overload;
  //Constructor Create(Owner: TEntity; P: PString); overload;
  Destructor Destroy; override;
  Function ToString: String; override;
  Function ToInteger: Integer; override;
  Function ToDouble: Double; override;
  Procedure SetString(S: String); override;
  Procedure SetInteger(I: Integer); override;
  Procedure SetDouble(D: Double); override;  
end;

implementation

//
//  TUniVar
//
Constructor TUniVar.Create(Owner: TEntity); 
begin
  Inherited Create(Owner);
  FValue._Type := vtInt;
  FValue.I := 0;
end;

Destructor TUniVar.Destroy; 
begin
  Inherited Destroy;
end;

Function TUniVar.ToString; 
begin
  Case FValue._Type of
    vtInt: Result := IntToStr(FValue.I);
    vtStr: Result := FValue.S;
    vtDbl: Result := FloatToStr(FValue.D);
  end;
end;

Function TUniVar.ToInteger;
begin
  Case FValue._Type of
    vtInt: Result := FValue.I;
    vtStr: Result := Round(StrToFloat(FValue.S));
    vtDbl: Result := Round(FValue.D);
  end;
end;

Function TUniVar.ToDouble;
begin
  Case FValue._Type of
    vtInt: Result := FValue.I;
    vtStr: Result := StrToFloat(FValue.S);
    vtDbl: Result := FValue.D;
  end;
end;

Procedure TUniVar.SetString;
begin
  FValue._Type := vtStr;
  FValue.S := S;
end;

Procedure TUniVar.SetInteger;
begin
  FValue._Type := vtInt;
  FValue.I := I;
end;

Procedure TUniVar.SetDouble; 
begin
  FValue._Type := vtDbl;
  FValue.D := D;
end;

//
//  TIntegerClass
//
Constructor TIntegerClass.Create(Owner: TEntity); 
begin
  Inherited Create(Owner);
  FValue := New(PInteger);
  FNeedFree := True;
end;
{
Constructor TIntegerClass.Create(Owner: TEntity; P: PInteger);
begin
  Inherited Create(Owner);
  FValue := P;
  FNeedFree := False;
end;
}
Destructor TIntegerClass.Destroy;
begin
    If FNeedFree then
      Dispose(PInteger(FValue));
  Inherited Destroy;
end;

Function TIntegerClass.ToString;
begin
  Result := IntToStr(PInteger(FValue)^);
end;

Function TIntegerClass.ToInteger: Integer;
begin
  Result := PInteger(FValue)^;
end;

Function TIntegerClass.ToDouble: Double;
begin
  Result := PInteger(FValue)^;
end;

Procedure TIntegerClass.SetString;
begin
  PInteger(FValue)^ := StrToInt(S);
end;

Procedure TIntegerClass.SetInteger;
begin
  PInteger(FValue)^ := I;
end;

Procedure TIntegerClass.SetDouble(D: Double);   
begin
  PInteger(FValue)^ := Round(D);
end;

//
//  TStringClass
//
Constructor TStringClass.Create(Owner: TEntity);
begin
  Inherited Create(Owner);
  FValue := New(PString);
  FNeedFree := True;
end;
{
Constructor TStringClass.Create(Owner: TEntity; P: PString);
begin
  Inherited Create(Owner);
  FValue := P;
  FNeedFree := False;
end;
}
Destructor TStringClass.Destroy;
begin
    If FNeedFree then
      Dispose(PString(FValue));
end;

Function TStringClass.ToString;
begin
  Result := PString(FValue)^;
end;

Function TStringClass.ToInteger;
begin
  Result := StrToInt(PString(FValue)^);
end;

Function TStringClass.ToDouble;
begin
  Result := StrToFloat(PString(FValue)^);
end;

Procedure TStringClass.SetString;
begin
  PString(FValue)^ := S;
end;

Procedure TStringClass.SetInteger(I: Integer);
begin
  PString(FValue)^ := IntToStr(I);
  Log.PrintLn('Set variable: ' + IntToStr(I));
end;

Procedure TStringClass.SetDouble;
begin
  PString(FValue)^ := FloatToStr(D);
end;

initialization
  //Factory.RegistClass(TIntegerClass);
  //Factory.RegistClass(TStringClass);
end.
