﻿package elm.core {
	import elm.core.*;
	import elm.utils.*;
	import flash.display.*;
	import flash.events.*;
	import flash.net.*;
	import flash.utils.*;
	
	public class Game {
		public static const MODE_FRONT:int = 0;
		public static const MODE_BACK:int = 1;
		public static const DELTA_MIN:Number = 0.3;
		
		public static var clip:Layer; // kongregate hack -> old: private
		public static var SYS:Layer; // GlowNoid hack -> old: private
		
		private static var states:Object;
		private static var state_curr:String;
		private static var state_old:String;
		private static var vars:Object;
		private static var last_time:Number;
		private static var last_fps_time:Number;
		private static var fps_num:uint;
		private static var delta_orig:Number;
		
		public static var delta:Number;
		public static var fps:uint;
		public static var mouseX:Number;
		public static var mouseY:Number;
		
		private static var m_is_debug:Boolean = true;
		//private static var m_is_sfx:Boolean = true;
		
		public static function init(clip:Layer) {
			//Game.root = PreloaderClass.clip;
			Game.clip = clip;
			states = { };
			state_curr = null;
			state_old = null;
			vars = [];
			
			clip.addEventListener(Event.ENTER_FRAME, Game.frame);
			
			SYS = Gfx.createLayer(null, "sys", 999, false);
			Gfx.createText(SYS, "fps", 0, 0, Gfx.Width, 20, { font:"_sans", embed:false, size:12, align:"center", color:0xCCCCCC, bold:false }, []);
			
			resetFps();
		}
		public static function resetFps():void {
			last_time = last_fps_time = getTimer();
		}
		public static function addState(name:String, state:State):void {
			states[name] = state;
		}
		public static function setState(name:String, mode:int = MODE_FRONT, attr:* = null):State {
			if(states[name] == undefined) {
				Log.Error("State '"+name+"' not found.");
				return null;
			}
			/*if (state_old != null) {
				Log.Error("Old state '"+state_old+"' not hided. New state '"+name+"'.");
			}*/
			trace("setState("+name+")");
			if (state_curr == name) {
				states[state_curr].free();
				Gfx.freeLayers(state_curr);
				//if (data != null) states[state_curr].setData(data);
				states[state_curr].init(attr);
				return states[state_curr];
			}
			if(state_curr != null) {
				clip.stage.removeEventListener(MouseEvent.CLICK, states[state_curr].onMouseClick);
				clip.stage.removeEventListener(MouseEvent.DOUBLE_CLICK, states[state_curr].onMouseDoubleClick);
				clip.stage.removeEventListener(MouseEvent.MOUSE_DOWN, states[state_curr].onMouseDown);
				clip.stage.removeEventListener(MouseEvent.MOUSE_UP, states[state_curr].onMouseUp);
				clip.stage.removeEventListener(MouseEvent.MOUSE_MOVE, states[state_curr].onMouseMove);
				clip.stage.removeEventListener(KeyboardEvent.KEY_DOWN, states[state_curr].onKeyDown);
				clip.stage.removeEventListener(KeyboardEvent.KEY_UP, states[state_curr].onKeyUp);
				
				state_old = state_curr;
			}
			
			state_curr = name;
			
			if(mode == MODE_FRONT) {
				var s:State = clip.addChild(states[name]) as State;
				clip.swapChildren(SYS, s);
			} else if (mode == MODE_BACK) {
				clip.addChildAt(states[name], 0);
			}
			
			//if (data != null) states[name].setData(data);
			
			if (!states[name].init(attr)) {
				clip.removeChild(states[name]);
				//state_old = null;
				//state_curr = null;
				Log.Error("State '" + name + "' not inited.");
				return null;
			}
			
			if (state_old == null) state_old = ""; // _initState(state_curr);
			
			clip.stage.focus = null;
			
			last_time = last_fps_time = getTimer();
			delta_orig = delta = 0;
			
			return states[state_curr];
		}
		private static function _initState(name:String):void {
			clip.stage.doubleClickEnabled = true;
			clip.stage.addEventListener(MouseEvent.CLICK, states[name].onMouseClick);
			//clip.stage.addEventListener(MouseEvent.DOUBLE_CLICK, states[state].onMouseDoubleClick);
			clip.stage.addEventListener(MouseEvent.MOUSE_DOWN, states[name].onMouseDown);
			clip.stage.addEventListener(MouseEvent.MOUSE_UP, states[name].onMouseUp);
			clip.stage.addEventListener(MouseEvent.MOUSE_MOVE, states[name].onMouseMove);
			clip.stage.addEventListener(KeyboardEvent.KEY_DOWN, states[name].onKeyDown);
			clip.stage.addEventListener(KeyboardEvent.KEY_UP, states[name].onKeyUp);
			last_time = last_fps_time = getTimer();
			delta_orig = delta = 0;
		}
		public static function getState(name:String = null):State {
			if (name == null) name = state_curr;
			if (name != null && states[name] != undefined) return states[name];
			else return null;
		}
		public static function getStateName():String {
			return state_curr;
		}
		public static function frame(e:Event) {
			delta_orig = (getTimer() - last_time) / 1000;
			if (delta_orig > DELTA_MIN) delta = DELTA_MIN;
			last_time = getTimer();
			fps_num++;
			if(getTimer() - last_fps_time >= 1000) {
				fps = fps_num;
				//setText("sys", "fps", fpsNum + " fps, "+delta+" delta");
				fps_num = 0;
				last_fps_time = getTimer();
			}
			delta = delta_orig;
			
			mouseX = clip.mouseX;
			mouseY = clip.mouseY;
			
			if (state_old != null) {
				var b1:Boolean = state_old == "" || states[state_old].isHide();
				var b2:Boolean = states[state_curr].isShow();
				if(b1 && b2) {
					if(state_old && state_old != "") {
						states[state_old].free();
						Gfx.freeLayers(state_old);
						clip.removeChild(states[state_old]);
					}
					_initState(state_curr);
					state_old = null;
				}
			} else if (state_curr != null) {
				states[state_curr].frame();
			}
			
			Sfx.frame();
			
			Gfx.setText(SYS, "fps", fps + " fps, "+delta+" delta, x:"+mouseX+", y:"+mouseY);
		}
		static public function getDelta():Number { return delta_orig; }
		public static function setVar(name:String, value:*):void {
			vars[name] = Data.clone(value);
			Log.Message("setVar: "+name+" = "+value);
		}
		public static function getVar(name:String):* {
			return Data.clone(vars[name]);
		}
		public static function clearVars():void {
			vars = [];
		}
		public static function saveVars(name:String):void {
			var so:SharedObject = SharedObject.getLocal(name);
			so.setProperty("vars", Data.clone(vars));
			//so.data.vars = Data.clone(vars);
			so.flush();
		}
		public static function loadVars(name:String):void {
			vars = null;
			var so:SharedObject = SharedObject.getLocal(name);
			//vars = Data.clone(so.data.vars);
			vars = so.data.vars;
		}
		public static function clearSavesVars(name:String):void {
			var so:SharedObject = SharedObject.getLocal(name);
			so.clear();
		}
		public static function isSavedVars(name:String):Boolean {
			var so:SharedObject = SharedObject.getLocal(name);
			return (so.data.vars != undefined);
		}
		public static function setConfig(name:String, value:*):void {
			Log.Message("setConfig: "+name+" = "+value);
			var so:SharedObject = SharedObject.getLocal("config");
			so.data[name] = Data.clone(value);
			so.flush();
		}
		public static function getConfig(name:String, def:* = ""):* {
			Log.Message("getConfig: "+name);
			var so:SharedObject = SharedObject.getLocal("config");
			if (so.data[name] != undefined) return so.data[name]; else return def;
		}
		public static function get Width():Number { return Gfx.Width; }
		public static function get Height():Number { return Gfx.Height; }
		public static function set Debug(val:Boolean):void {
			m_is_debug = val;
			var m:DisplayObject = Gfx.getMovie(SYS, "fps");
			if (m) {
				if (!m_is_debug) {
					m.y = -10;
				} else {
					m.y = 10;
				}
			}
		}
		public static function get Debug():Boolean {
			return m_is_debug;
		}
		public static function set is_debug(val:Boolean):void {
			m_is_debug = val;
			var m:DisplayObject = Gfx.getMovie(SYS, "fps");
			if (m) {
				if (!m_is_debug) {
					m.y = -10;
				} else {
					m.y = 10;
				}
			}
		}
		public static function get is_debug():Boolean { return m_is_debug; }
		public static function isUrl(urls:Array):Boolean {
			var url:String = clip.stage.loaderInfo.loaderURL;
			var urlStart:Number = url.indexOf("://")+3;
			var urlEnd:Number = url.indexOf("/", urlStart);
			var domain:String = url.substring(urlStart, urlEnd);
			var LastDot:Number = domain.lastIndexOf(".")-1;
			var domEnd:Number = domain.lastIndexOf(".", LastDot)+1;
			domain = domain.substring(domEnd, domain.length);
			
			for (var i:int = 0; i < urls.length; i++) {
				if (domain == urls[i]) {
					return true;
				}
			}
			return false;
		}
		public static function openUrl(url:String):void {
			var request:URLRequest = new URLRequest(url);
			try {
				navigateToURL(request, '_blank');
			} catch (e:Error) {
				Log.Error("Game.openUrl");
				Log.Message(e.message);
			}
		}
		public static function addEventListener(type:String, listener:Function, useCapture:Boolean = false, priority:int = 0, useWeakReference:Boolean = false):void {
			clip.stage.addEventListener(type, listener, useCapture, priority, useWeakReference);
		}
		public static function removeEventListener(type:String, listener:Function, useCapture:Boolean = false):void {
			clip.stage.removeEventListener(type, listener, useCapture);
		}
		public static function getLoaderInfo():LoaderInfo {
			return clip.stage.loaderInfo;
		}
		public static function getClassByName(name:String):Class {
			var def:Object = null;
			try {
				def = getDefinitionByName(name);
			} catch (error:Error) {
				trace(error.message);
				return null;
			}
			if (def is Class) return def as Class;
			trace(name+" is not a Class.");
			return null;
		}
	}	
	// end package
}
