﻿package elm.gfx {
	import elm.core.*;
	import elm.gfx.*;
	import flash.display.*;

	public dynamic class Animation extends MovieClip {
		public static var FORVARD:uint = 0;
		public static var BACK:uint = 1;
		public static var PINPONG:uint = 2;
	
		private var ani:Object = null;
		public var userData:* = null;
		
		public function Animation() {
			ani = new Object();
		}
		public function Set(name:String, fps:Number, begin:uint, end:uint, type:uint = 0, is_loop:Boolean = true):void {
			ani[name] = new Object();
			ani[name].timer = 0;
			if (fps != 0) ani[name].fps = 1.0 / fps;
			else ani[name].fps = 0;
			ani[name].begin = begin;
			ani[name].curr = begin;
			ani[name].end = end;
			ani[name].type = type;
			ani[name].is_loop = is_loop;
			ani[name].percent = -1;
			if(int(type) == 1) ani[name].step = -1;
			else {
				ani[name].step = 1;
				if(int(type) == 2) ani[name].is_loop = true;
			}
			// triggers
			ani[name].trigger = [];
		}
		public function SetEnd(name:String, end:uint):void {
			if (ani[name] == undefined) return;
			ani[name].end = end;
		}
		public function IsStop(name:String):Boolean {
			if (ani[name] == undefined) return true;
			if (ani[name].is_loop) return false; // !!!
			if (ani[name].type == FORVARD && ani[name].curr == ani[name].end) return true;
			if (ani[name].type == BACK && ani[name].curr == ani[name].begin) return true;
			return false;
		}
		public function IsParam(name:String, param:String):* {
			if (ani[name] == undefined || ani[name][param] == undefined) return false;
			return ani[name][param];
		}
		public function Reset(name:String):void {
			if (ani[name].type == BACK) ani[name].curr = ani[name].end;
			else ani[name].curr = ani[name].begin;
		}
		public function AddTrigger(name:String, frame:int, func:Function):void {
			if (ani[name] == undefined) return;
			ani[name].trigger.push({frame:frame, func:func});
		}
		public function Update(name:String, delta:Number):void {
			if(ani[name] == undefined) {
				Log.Error(this.name+". Animation '"+name+"' not found.");
				return;
			}
			ani[name].timer += delta;
			var i:Number = 0;
			while(ani[name].timer >= ani[name].fps && ani[name].fps > 0) {
				ani[name].timer -= ani[name].fps;
				if (ani[name].fps == 0) ani[name].timer = 0;
				ani[name].curr += ani[name].step;
				ani[name].percent = -1;
				if(ani[name].type == Animation.FORVARD) {
					if(ani[name].curr > ani[name].end){
						if(ani[name].is_loop)
							ani[name].curr = ani[name].begin;
						else
							ani[name].curr = ani[name].end;
					}
				} else if(ani[name].type == Animation.BACK) {
					if(ani[name].curr < ani[name].begin) {
						if(ani[name].is_loop)
							ani[name].curr = ani[name].end;
						else
							ani[name].curr = ani[name].begin;
					}
				} else {
					if(ani[name].curr == ani[name].begin || ani[name].curr == ani[name].end) ani[name].step = -ani[name].step;
				}
				for (var t:int = 0; t < ani[name].trigger.length; t++) {
					if (ani[name].curr == ani[name].trigger[t].frame && ani[name].trigger[t].func) ani[name].trigger[t].func(userData);
				}
				i++;
				if(i > 10) {
					ani[name].timer = 0;
					trace("Хрень!");
					break;
				}
			}
			gotoAndStop(ani[name].curr);
		}
		public function SetPercent(name:String, percent:uint):void {
			if(!ani[name]) {
				Log.Error(this.name+". Animation '"+name+"' not found.");
				return;
			}
			if(percent < 0) percent = 0;
			if(percent > 100) percent = 100;
			if(ani[name].percent == percent) return;
			var n:Number = ani[name].end - ani[name].begin + 1;
			ani[name].curr = int(ani[name].begin + n * 0.01 * percent)-1;
			gotoAndStop(ani[name].curr);
			ani[name].percent = percent;
		}
		public function GetPercent(name:String):uint {
			if(!ani[name]) {
				Log.Error(this.name+". Animation '"+name+"' not found.");
				return 0;
			}
			var n:Number = (ani[name].end - ani[name].begin + 1) * 0.01;
			//trace("count = "+n);
			var per:uint = int((ani[name].curr - ani[name].begin + 1) / n);
			if(per > 100) per = 100;
			return per;
		}
		public function copyCurr(from:String, to:String):void {
			if(!ani[from] || !ani[to]) {
				Log.Error(this.name+". Animation '"+from+"' or '"+to+"' not found.");
				return;
			}
			ani[to].curr = ani[from].curr;
			ani[to].timer = 0;
		}
	}
}