package game.actor {
	import flash.events.EventDispatcher;
	import game.*;
	import game.actor.player.PlayerActor;

	public class ActorManager extends EventDispatcher {
		use namespace ManagerAccess;
		
		private var list:Array;
		private var list_dead:Array;
		private var player:PlayerActor = null;
	
		public function ActorManager() {
			list = [];
			list_dead = [];
		}
		
		public function add(actorIn:Actor):Actor {
			if (!actorIn) {
				trace("Actor is null.");
				return null;
			}
			actorIn.setActorManager(this);
			list.push(actorIn);
			if (actorIn is PlayerActor) {
				player = actorIn as PlayerActor;
			}
			actorIn.addEventListener(ActorEvent.DEATH, onDeath);
			return actorIn;
		}
		public function remove(actorIn:Actor):void {
			if (actorIn is PlayerActor) player = null;
			var id:int = list.indexOf(actorIn);
			if(id >= 0) {
				list.splice(id, 1);
			}
		}
		public function removeAll():void {
			list = [];
			list_dead = [];
		}
		public function update(delta:Number):void {
			for(var i:int = 0; i < list.length; i++) {
				if(list[i].is_alive) list[i].update(delta);
			}
			
			if(list_dead.length > 0) {
				for (var i:int = 0; i < list_dead.length; i++) {
					if (list_dead[i].is_die_quietly) list_dead[i].onDeath();
					list_dead[i].kill();
					//dispatchEvent(new EntityEvent(list_dead[i], EntityEvent.KILLED));
				}
				list_dead = [];
			}
		}
		public function render():void {
			for(var i:int = 0; i < list.length; i++) {
				list[i].render();
			}
		}
		public function onDeath(e:ActorEvent):void {
			if (!e.actor.is_alive) return;
			list_dead.push(e.actor);
			e.actor.is_alive = false;
		}
		
		public function findPlayer():PlayerActor {
			return player;
		}
		public function findActorByClass(actorClass:Class):Actor {
			for (var i:int = 0; i < list.length; i++) {
				if (list[i] is actorClass) return list[i];
			}
			return null;
		}
		public function getActors():Array {
			return list;
		}
	}
}