﻿package game.actor {
	import Box2D.Collision.b2ContactPoint;
	import Box2D.Collision.Shapes.b2Shape;
	import Box2D.Dynamics.Contacts.b2ContactResult;
	import game.actor.boxes.Box;
	import game.actor.enemies.EnemyPost;
	import game.room.entity.*;

	public class WalkingActor extends Actor {
		public static const DOWN_ANGLE:Number = 90.0 * (Math.PI / 180.0);
		public static const DOWN_ANGLE_RANGE:Number = 55.0 * (Math.PI / 180.0);
		
		public var dir:int = 0;
		public var dir_vertical:int = 0;
		public var speed_vertical:Number = 30;
		public var speed_horizontal:Number = 30;
		public var speed_jump:Number = 80;
		
		public var is_ladder:Boolean = false;
		public var is_jumped:Boolean = true;
		public var is_sitting:Boolean = false;
		public var jump_time:Number = 0;
		public var is_wall_left:Boolean = false;
		public var is_wall_right:Boolean = false;
		
		public function WalkingActor(attrIn:Object) {
			super(attrIn);
		}
		public override function free():void {
			super.free();
		}
		public override function update(delta:Number):void {
			super.update(delta);
			
			//if ((is_wall_left && dir == -1) || (is_wall_right && dir == 1)) walk(0);
			if (!is_ground && !is_ladder) {
				is_jumped = true;
			}
			if (is_jumped) {
				is_sitting = false;
				if (is_ground || is_ladder) {
					is_jumped = false;
					if (is_ladder) jump_time = 0;
				} else {
					jump_time += delta;
				}
			} else {
				jump_time = 0;
			}
			
			if (is_ladder) {
				is_sitting = false;
				is_fall_through_platforms = true;
			} else {
				is_fall_through_platforms = false;
			}
			
			is_ground = false;
			is_wall_left = false;
			is_wall_right = false;
		}
		
		public function walk(dirIn:int):void {
			if (is_sitting) return;
			dir = dirIn;
			if(!(is_wall_left && dir == -1) && !(is_wall_right && dir == 1)) {
				if (body) {
					// walk left/right
					body.m_linearVelocity.x = dir * speed_horizontal;
				}
			}
		}
		public function walkUp(dirIn:int):void {
			if (dir_vertical > 0 && dirIn < 0) {
				is_fall_through_platforms = true;
				is_jumped = true;
			}
			if (is_ladder) {
				if (body) {
					// walk up/down
					body.m_linearVelocity.y = dirIn * speed_vertical;
				}
			} else if (is_ground) {
				if (dirIn < 0 && !is_jumped) {
					is_sitting = false;
					if (body) {
						// jump
						body.m_linearVelocity.y = -speed_jump;
					}
				} // else
				if (/*dir_vertical == 0 &&*/ dirIn > 0 && !is_ladder && !is_jumped) {
					// sit
					is_sitting = true;
				} else {
					is_sitting = false;
				}
			}
			dir_vertical = dirIn;
		}
		public function isCollised(actor_shape:b2Shape):Boolean {
			return true;
		}
		
		public override function onContactUpdate(shape:b2Shape, reverse:int, contactPoint:b2ContactPoint):void {
			super.onContactUpdate(shape, reverse, contactPoint);
			
			if (!shape.GetUserData() || shape.GetUserData() is Platform || shape.GetUserData() is WalkingActor || shape.GetUserData() is Box) {
				var normalAngle:Number = Math.atan2(contactPoint.normal.y * reverse, contactPoint.normal.x * reverse);
				if (Math.abs(normalAngle - DOWN_ANGLE) < DOWN_ANGLE_RANGE) {
					//leftmostFloorAngle  = Math.min( leftmostFloorAngle,  normalAngle );
					//rightmostFloorAngle = Math.max( rightmostFloorAngle, normalAngle );
					is_ground = true;
					//if (shape is Ladder) trace("ladder ground");
				}
				else if (Math.abs(normalAngle - -DOWN_ANGLE) < DOWN_ANGLE_RANGE) {
					//trace("ceiling collision");
				}
				else if (Math.abs(normalAngle) < Math.PI / 2.0) {
					if(!(shape.GetUserData() is EnemyPost)) is_wall_right = true;
				}
				else {
					if(!(shape.GetUserData() is EnemyPost)) is_wall_left = true;
				}
			}
			
			// ladder
			if (shape.GetUserData() is Ladder) {
				is_ladder = true;
				is_gravity = false;
			}
		}
		public override function onContactRemove(shape:b2Shape, reverse:int, contactPoint:b2ContactPoint):void {
			super.onContactRemove(shape, reverse, contactPoint);
			
			// ladder
			if (shape.GetUserData() is Ladder) {
				is_ladder = false;
				if (body) body.m_linearVelocity.y = 0;
				is_ground = true;
				is_gravity = true;
			}
		}
		public override function onContactResult(shape:b2Shape, reverse:int, contactResult:b2ContactResult):void {
			super.onContactResult(shape, reverse, contactResult);
			
			// ladder
			if (shape.GetUserData() is Ladder) {
				is_ladder = true;
				is_gravity = false;
			}
		}
	}
}