﻿package game.actor.bullets {
	import Box2D.Collision.Shapes.b2Shape;
	import elm.core.*;
	import elm.gfx.Animation;
	import elm.types.*;
	import Box2D.Common.Math.b2Vec2;
	import Box2D.Dynamics.b2Body;
	import Box2D.Collision.b2ContactPoint;
	import flash.display.MovieClip;
	import game.actor.DamagedActor;
	import game.actor.player.PlayerActor;
	import game.room.entity.Platform;
	
	public class GrenadeBullet extends Bullet {
		public static const SPEED:Number = 100;
		public static const EPLOSION_DISTANCE:Number = 100;
		public static const EXPLOSION_FORCE:Number = 600;
		public static const EXPLOSION_DAMAGE:Number = 50;
		
		public function GrenadeBullet(attrIn:Object) {
			super(attrIn);
			
			o = Gfx.makeMovie("gfx.GrenadeBullet", "bullet" + id, { x:attr.x, y:attr.y } ) as Animation;
			
			is_gravity = true;
			timer = 3;
		}
		public override function free():void {
			o = null;
		}
		
		public override function die(quality:Boolean = true):void {
			if (!isAlive()) return;
			super.die(quality);
			
			var r:int = int(Math.random()*int.MAX_VALUE);
			
			if (actor_manager) {
				var v:Vec2 = new Vec2();
				v.SetV(body.GetPosition());
				var list:Array = actor_manager.getActors();
				for (var i:int = 0; i < list.length; i++) {
					if (list[i] != this) {
						var u:Vec2 = new Vec2();
						var b:b2Body = list[i].getBody();
						u.SetV(b.GetPosition());
						var dist:Number = v.DistanceTo(u);
						if (dist < EPLOSION_DISTANCE) {
							var power:Number = (EPLOSION_DISTANCE - dist) / EPLOSION_DISTANCE;
							var force:Vec2 = new Vec2();
							force.Move(power * EXPLOSION_FORCE, v.AngleTo(u));
							b.ApplyImpulse(force, u);
							if (list[i] is DamagedActor) {
								(list[i] as DamagedActor).damage({damage:(power * EXPLOSION_DAMAGE)});
							}
						}
					}
				}
			}
		}
		
		public override function damageActor(actor:DamagedActor, contactPoint:b2ContactPoint):void {
			die();
		}
		
		public override function setBody(bodyIn:b2Body):void {
			super.setBody(bodyIn);
			
			if (body) {
				var v:Vec2 = new Vec2();
				v.Move(SPEED, attr.angle);
				body.m_linearVelocity.SetV(v);
			}
		}
	}
	
}