﻿package game.actor.enemies {
	import Box2D.Collision.Shapes.b2PolygonDef;
	import elm.core.*;
	import elm.game.CooldownTimer;
	import elm.gfx.Animation;
	import elm.types.Vec2;
	import elm.utils.*;
	import Box2D.Collision.Shapes.b2CircleDef;
	import Box2D.Dynamics.b2Body;
	import Box2D.Dynamics.b2BodyDef;
	import flash.geom.Point;
	import state.*;
	import game.actor.*;
	import game.actor.player.*;
	import game.actor.bullets.*;

	public class Boss extends Enemy {
		public static const STATE_WAIT:int = 0;
		public static const STATE_ATTACK:int = 1;
		public static const TIMES:Array = [0.5, 2];
		public static const ATTACK_DISTANCE:Number = 550;
		public static const ATTACK_ANGLE_DIFFERENCE:Number = 10;
		public static const SPEED_ANGLE:Number = 30;
		
		private var health_max:Number;
		
		// ai data
		private var ai_state:int;
		private var player_pos:Vec2 = new Vec2();
		private var pos:Vec2 = new Vec2();
		private var angle_to_player0:Number;
		private var angle_to_player1:Number;
		
		// gun
		var guns:Array = [null, null, null, null, null];
		var gun_angles:Array = [0, 0, 0, 0, 0];
		
		// cooldowns
		private var aiCooldown:CooldownTimer;
		
		public function Boss(attrIn:Object) {
			super(attrIn);
			
			if (o) {
				// animations
				o.Set("stand", 0, 1, 1, 0, true);
				
				// gun
				for (var i:int = 0; i < 5; i++) o["point"+i]["gun"].gotoAndStop(1);
			}
			
			for (var i:int = 0; i < 5; i++) {
				guns[i] = new ActorGun( { owner:this, movie:o["point"+i] } );
				guns[i].setGun( { name:"Gun", frame:1, time:(2+1.7*i), bullet:"BossBullet", dist:150 } );
			}
			
			is_gravity = false;
			speed_horizontal = 0;
			speed_vertical = 0;
			speed_jump = 0;
			health_max = health = 500 + 100 * int(Game.getVar("map"));
			
			ai_state = STATE_WAIT;
			aiCooldown = new CooldownTimer(TIMES[ai_state]);
			aiCooldown.start();
		}
		public override function free():void {
			aiCooldown = null;
			player_pos = null;
			pos = null;
			
			super.free();
		}
		
		public override function update(delta:Number):void {
			if (aiCooldown.isComplete()) {
				switch(ai_state) {
					case STATE_WAIT:
						if (!actor_manager) {
							die();
							return;
						}
						player = actor_manager.findPlayer();
						if (!player) {
							die();
							return;
						}
						for (var i:int = 0; i < guns.length; i++) {
							guns[i].setRoom(room);
							guns[i].start();
						}
						player_pos.SetV(player.getBody().GetPosition());
						//if (player_pos.y > pos.y && pos.DistanceTo(player_pos) <= ATTACK_DISTANCE * 2) {
							ai_state = STATE_ATTACK;
							aiCooldown.start(TIMES[ai_state]);
						/*} else {
							aiCooldown.start();
						}*/
					break;
					case STATE_ATTACK:
						if (pos.DistanceTo(player_pos) > ATTACK_DISTANCE) {
							ai_state = STATE_WAIT;
						}
						aiCooldown.start(TIMES[ai_state]);
					break;
				}
			}
			if (ai_state == STATE_ATTACK) {
				if (player) {
					player_pos.SetV(player.getBody().GetPosition());
					
					for (var i:int = 0; i < guns.length; i++) {
						gun_angles[i] = guns[i].getAngle(player_pos);
						guns[i].update(delta);
						if (guns[i].isComplete()) {
							guns[i].shoot(gun_angles[i], null);
						}
					}
				} else {
					trace("Not player.");
					ai_state = STATE_WAIT;
				}
			}
			
			if (guns.length == 4 && (health / health_max) < 0.8) {
				o["point4"].alpha = 0;
				guns.splice(4, 1);
			}
			if (guns.length == 3 && (health / health_max) < 0.6) {
				o["point4"].alpha = 0;
				guns.splice(3, 1);
			}
			if (guns.length == 4 && (health / health_max) < 0.4) {
				o["point4"].alpha = 0;
				guns.splice(2, 1);
			}
			if (guns.length == 1 && (health / health_max) < 0.2) {
				o["point4"].alpha = 0;
				guns.splice(1, 1);
			}
			
			aiCooldown.update(delta);
			
			// animations
			var ani_new:String = "stand";
			if (is_ground) {
				if (dir == 0) ani_new = "stand";
				else ani_new = "walk";
			}
			if (ani != ani_new) {
				ani = ani_new;
			}
			
			super.update(delta);
		}
		
		public override function render():void {
			super.render();
			if (o && body) {
				for (var i:int = 0; i < guns.length; i++) {
					o["point"+i]["gun"].rotation = gun_angles[i];
				}
			}
		}
		
		public override function damage(param:Object):void {
			super.damage(param);
		}
		
		public override function getBodyDef():b2BodyDef {
			var def:b2BodyDef = new b2BodyDef();
			def.userData = this;
			def.fixedRotation = true;
			def.position.Set(attr.x, attr.y);
			
			return def;
		}
		public override function setBody(bodyIn:b2Body):void {
			super.setBody(bodyIn);
			
			var def:b2PolygonDef = new b2PolygonDef();
			def.userData = this;
			def.SetAsBox(100, 300);
			def.density = 0;
			def.friction = 1;
			def.restitution = 0;
			
			body.CreateShape(def);
			body.SetMassFromShapes();
			
			pos.SetV(body.GetPosition());
		}
	}
}