﻿package game.actor.enemies {
	import Box2D.Collision.Shapes.b2PolygonDef;
	import elm.core.*;
	import elm.game.CooldownTimer;
	import elm.gfx.Animation;
	import elm.types.Vec2;
	import elm.utils.*;
	import Box2D.Collision.Shapes.b2CircleDef;
	import Box2D.Dynamics.b2Body;
	import Box2D.Dynamics.b2BodyDef;
	import flash.geom.Point;
	import state.*;
	import game.actor.*;
	import game.actor.player.*;
	import game.actor.bullets.*;

	public class Grenadir extends Enemy {
		public static const STATE_WAIT:int = 0;
		public static const STATE_ATTACK:int = 1;
		public static const TIMES:Array = [1, 1.5];
		public static const ATTACK_DISTANCE:Number = 500;
		public static const SPEED_ANGLE:Number = 80;
		
		// ai data
		private var ai_state:int;
		private var player_pos:Vec2 = new Vec2();
		private var pos:Vec2 = new Vec2();
		private var angle_to_player:Number;
		
		// gun
		var gun_angle:Number = 0;
		var gun_angle_d:Number = 0;
		var show_dir:Number = 1;
		var show_angle:Number = 90;
		
		// cooldowns
		private var aiCooldown:CooldownTimer;
		
		public function Grenadir(attrIn:Object) {
			super(attrIn);
			
			if (o) {
				o.stop();
				// animations
				o.Set("stand", 0, 1, 1, 0, true);
				ani = "stand";
				
				// gun
				o["point"]["gun"].gotoAndStop(3);
			}
			
			is_gravity = true;
			speed_horizontal = 0;
			speed_vertical = 0;
			speed_jump = 0;
			health = 70;
			
			ai_state = STATE_WAIT;
			aiCooldown = new CooldownTimer(TIMES[ai_state]);
			aiCooldown.start();
		}
		public override function free():void {
			aiCooldown = null;
			player_pos = null;
			pos = null;
			
			super.free();
		}
		
		public override function update(delta:Number):void {
			if (aiCooldown.isComplete()) {
				switch(ai_state) {
					case STATE_WAIT:
						if (!actor_manager) {
							die();
							return;
						}
						player = actor_manager.findPlayer();
						if (!player) {
							die();
							return;
						}
						player_pos.SetV(player.getBody().GetPosition());
						if (player_pos.y > pos.y && pos.DistanceTo(player_pos) <= ATTACK_DISTANCE) {
							ai_state = STATE_ATTACK;
							aiCooldown.start(TIMES[ai_state]);
						} else {
							aiCooldown.start();
						}
					break;
					case STATE_ATTACK:
						aiCooldown.start(TIMES[ai_state]);
						//if (pos.DistanceTo(player_pos) <= ATTACK_DISTANCE) {
							var p:Point = o["point"]["gun"]["bullet"].localToGlobal(new Point());
							p = Gameplay.stageToWorld(p);
							var v:Vec2 = new Vec2(p.x, p.y);
							room.add(new GrenadeBullet( { owner:this, x:v.x, y:v.y, angle:gun_angle, point:null } ));
						//} else {
							//ai_state = STATE_WAIT;
							//aiCooldown.start(TIMES[ai_state]);
						//}
					break;
				}
			}
			if (ai_state == STATE_ATTACK) {
				if (player) {
					//pos.SetV(body.GetPosition());
					player_pos.SetV(player.getBody().GetPosition());
					angle_to_player = gun_angle_d = Angle.normal(pos.AngleTo(player_pos));
					if (gun_angle_d < 180) gun_angle_d -= 20;
					else gun_angle_d += 20;
					gun_angle = Angle.approach(delta * SPEED_ANGLE, gun_angle, gun_angle_d);
					if (gun_angle < 180) {
						show_dir = 1;
						show_angle = gun_angle;
					} else {
						show_dir = -1;
						show_angle = 360 - gun_angle;
					}
				}
			}
			
			aiCooldown.update(delta);
			
			super.update(delta);
		}
		
		public override function render():void {
			super.render();
			if (o && body) {
				o.scaleX = show_dir;
				o["point"]["gun"].rotation = show_angle;
			}
		}
		
		public override function damage(param:Object):void {
			super.damage(param);
		}
		
		public override function getBodyDef():b2BodyDef {
			var def:b2BodyDef = new b2BodyDef();
			def.userData = this;
			def.fixedRotation = true;
			def.position.Set(attr.x, attr.y);
			
			return def;
		}
		public override function setBody(bodyIn:b2Body):void {
			super.setBody(bodyIn);
			
			var def:b2PolygonDef = new b2PolygonDef();
			def.userData = this;
			def.SetAsBox(8, 30);
			for (var i:int = 0; i < def.vertexCount; i++) {
				def.vertices[i].y -= 14;
			}
			def.density = 1;
			def.friction = 1;
			def.restitution = 0;
			
			body.CreateShape(def);
			body.SetMassFromShapes();
			
			pos.SetV(body.GetPosition());
		}
	}
}