﻿package game.actor.enemies {
	import Box2D.Collision.Shapes.b2CircleDef;
	import Box2D.Dynamics.b2Body;
	import Box2D.Dynamics.b2BodyDef;
	import elm.game.CooldownTimer;
	import elm.gfx.Animation;
	import elm.types.Vec2;
	import elm.utils.*;
	import flash.geom.Point;
	import state.*;
	import game.actor.*;
	import game.actor.player.*;
	import game.actor.bullets.*;

	public class Larva extends Enemy {
		public static const STATE_WAIT:int = 0;
		public static const STATE_ATTACK:int = 1;
		public static const TIMES:Array = [2, 1];
		public static const ATTACK_DISTANCE:Number = 450;
		public static const ATTACK_ANGLE_DIFFERENCE:Number = 10;
		public static const SPEED_ANGLE:Number = 120;
		
		// ai data
		private var ai_state:int;
		private var player_pos:Vec2 = new Vec2();
		private var pos:Vec2 = new Vec2();
		private var angle_to_player:Number;
		
		// gun
		var gun_angle:Number = 0;
		var gun_angle_d:Number = 0;
		
		// cooldowns
		private var aiCooldown:CooldownTimer;
		
		public function Larva(attrIn:Object) {
			super(attrIn);
			
			health = 50;
			ai_state = STATE_WAIT;
			aiCooldown = new CooldownTimer(TIMES[ai_state]);
			aiCooldown.start();
		}
		public override function free():void {
			player_pos = null;
			pos = null;
			aiCooldown = null;
			
			super.free();
		}
		
		public override function update(delta:Number):void {
			if (aiCooldown.isComplete()) {
				switch(ai_state) {
					case STATE_WAIT:
						if (!actor_manager) {
							die();
							return;
						}
						player = actor_manager.findPlayer();
						if (!player) {
							die();
							return;
						}
						player_pos.SetV(player.getBody().GetPosition());
						if (player_pos.y > pos.y && pos.DistanceTo(player_pos) <= ATTACK_DISTANCE) {
							ai_state = STATE_ATTACK;
							aiCooldown.start(TIMES[ai_state]);
						} else {
							aiCooldown.start();
						}
					break;
					case STATE_ATTACK:
						aiCooldown.start(TIMES[ai_state]);
						if (player_pos.y > pos.y && pos.DistanceTo(player_pos) <= ATTACK_DISTANCE) {
							var a2:Number = Angle.delta(angle_to_player, gun_angle);
							if (a2 < ATTACK_ANGLE_DIFFERENCE) {
								var p:Point = o["point"]["gun"]["bullet"].localToGlobal(new Point());
								p = Gameplay.stageToWorld(p);
								var v:Vec2 = new Vec2(p.x, p.y);
								room.add(new LarvaBullet( { owner:this, x:v.x, y:v.y, angle:gun_angle, point:null} ));
							} else {
								
							}
						}
					break;
				}
			}
			if (ai_state == STATE_ATTACK) {
				player_pos.SetV(player.getBody().GetPosition());
				angle_to_player = gun_angle_d = Angle.normal(pos.AngleTo(player_pos));
				if (gun_angle_d < 95) gun_angle_d = 95;
				if (gun_angle_d > 265) gun_angle_d = 265;
				gun_angle = Angle.approach(delta * SPEED_ANGLE, gun_angle, gun_angle_d);
			}
			
			aiCooldown.update(delta);
			super.update(delta);
		}
		
		public override function render():void {
			super.render();
			if (o && body) {
				o["point"]["gun"].rotation = gun_angle;
			}
		}
		
		public override function damage(param:Object):void {
			super.damage(param);
		}
		
		public override function getBodyDef():b2BodyDef {
			var def:b2BodyDef = new b2BodyDef();
			def.userData = this;
			def.fixedRotation = true;
			def.position.Set(attr.x, attr.y);
			
			return def;
		}
		public override function setBody(bodyIn:b2Body):void {
			super.setBody(bodyIn);
			
			var def:b2CircleDef = new b2CircleDef();
			def.userData = this;
			def.radius = 22;
			def.density = 0;
			def.friction = 1;
			def.restitution = 0;
			def.isSensor = true;
			
			body.CreateShape(def);
			body.SetMassFromShapes();
			
			pos.SetV(body.GetPosition());
		}
	}
}