﻿package game.actor.enemies {
	import Box2D.Collision.Shapes.b2PolygonDef;
	import elm.core.*;
	import elm.game.CooldownTimer;
	import elm.gfx.Animation;
	import elm.types.Vec2;
	import elm.utils.*;
	import Box2D.Collision.Shapes.b2CircleDef;
	import Box2D.Dynamics.b2Body;
	import Box2D.Dynamics.b2BodyDef;
	import flash.geom.Point;
	import state.*;
	import game.actor.*;
	import game.actor.player.*;
	import game.actor.bullets.*;

	public class Soldier extends Enemy {
		public static const STATE_WAIT:int = 0;
		public static const STATE_ATTACK:int = 1;
		public static const TIMES:Array = [1, 0.5];
		public static const ATTACK_DISTANCE:Number = 300;
		public static const ATTACK_ANGLE_DIFFERENCE:Number = 10;
		public static const SPEED_ANGLE:Number = 120;
		
		// ai data
		private var ai_state:int;
		private var player_pos:Vec2 = new Vec2();
		private var pos:Vec2 = new Vec2();
		private var angle_to_player:Number;
		
		// gun
		var gun_angle:Number = 0;
		var gun_angle_d:Number = 0;
		var show_dir:Number = 1;
		var show_angle:Number = 90;
		var ai_dir:int = 0;
		var ai_vdir:int = 0;
		
		// cooldowns
		private var aiCooldown:CooldownTimer;
		
		public function Soldier(attrIn:Object) {
			super(attrIn);
			
			o = Gfx.makeMovie("gfx.Soldier", "enemy" + id, { x:attr.x, y:attr.y } ) as Animation;
			if (o) {
				o.stop();
				// animations
				o.Set("stand", 0, 1, 1, 0, true);
				o.Set("sit", 0, 5, 5, 0, true);
				o.Set("walk", 5, 2, 4, Animation.PINPONG, true);
				o.Set("jump_up", 0, 6, 6, 0, true);
				o.Set("jump_down", 0, 7, 7, 0, true);
				o.Set("ladder", 0, 8, 8, 0, true);
				o.Set("ladder_up", 5, 8, 9, 0, true);
				o.Set("ladder_down", 5, 8, 9, 0, true);
				
				// gun
				o["point"]["gun"].gotoAndStop(1);
			}
			
			is_gravity = true;
			speed_horizontal = 15;
			speed_vertical = 10;
			speed_jump = 50;
			health = 30;
			
			ai_state = STATE_WAIT;
			aiCooldown = new CooldownTimer(TIMES[ai_state]);
			aiCooldown.start();
		}
		public override function free():void {
			o = null;
			aiCooldown = null;
			player_pos = null;
			pos = null;
			
			super.free();
		}
		
		public override function update(delta:Number):void {
			if (aiCooldown.isComplete()) {
				switch(ai_state) {
					case STATE_WAIT:
						if (!actor_manager) {
							die();
							return;
						}
						player = actor_manager.findPlayer();
						if (!player) {
							die();
							return;
						}
						player_pos.SetV(player.getBody().GetPosition());
						if (player_pos.y > pos.y && pos.DistanceTo(player_pos) <= ATTACK_DISTANCE * 2) {
							ai_state = STATE_ATTACK;
							aiCooldown.start(TIMES[ai_state]);
						} else {
							aiCooldown.start();
						}
					break;
					case STATE_ATTACK:
						aiCooldown.start(TIMES[ai_state]);
						if (pos.DistanceTo(player_pos) <= ATTACK_DISTANCE) {
							var a2:Number = Math.abs(Angle.delta(angle_to_player, gun_angle));
							var p:Point = o["point"]["gun"]["bullet"].localToGlobal(new Point());
							p = Gameplay.stageToWorld(p);
							var v:Vec2 = new Vec2(p.x, p.y);
							room.add(new SoldierBullet( { owner:this, x:v.x, y:v.y, angle:gun_angle, point:null} ));
						} else {
							if (player_pos.x < pos.x) ai_dir = -1;
							else ai_dir = 1;
							if (player_pos.y <= pos.y-5) ai_vdir = -1;
							else if (is_ladder) ai_vdir = 1;
						}
					break;
				}
			}
			if (ai_state == STATE_ATTACK) {
				if (player) {
					pos.SetV(body.GetPosition());
					player_pos.SetV(player.getBody().GetPosition());
					angle_to_player = gun_angle_d = Angle.normal(pos.AngleTo(player_pos));
					gun_angle = Angle.approach(delta * SPEED_ANGLE, gun_angle, gun_angle_d);
					if (gun_angle < 180) {
						show_dir = 1;
						show_angle = gun_angle;
					} else {
						show_dir = -1;
						show_angle = 360 - gun_angle;
					}
					walk(ai_dir);
					walkUp(ai_vdir);
					ai_vdir = 0;
				}
			}
			
			aiCooldown.update(delta);
			
			// animations
			var ani_new:String = "stand";
			if (!is_ground && is_ladder) {
				if (dir_vertical < 0) ani_new = "ladder_up";
				if (dir_vertical > 0) ani_new = "ladder_down";
				else ani_new = "ladder";
			}
			if (is_ground) {
				if (dir == 0) ani_new = "stand";
				else ani_new = "walk";
			}
			if (ani != ani_new) {
				ani = ani_new;
			}
			
			super.update(delta);
		}
		
		public override function render():void {
			super.render();
			if (o && body) {
				o.scaleX = show_dir;
				o["point"]["gun"].rotation = show_angle;
			}
		}
		
		public override function damage(param:Object):void {
			super.damage(param);
		}
		
		public override function getBodyDef():b2BodyDef {
			var def:b2BodyDef = new b2BodyDef();
			def.userData = this;
			def.fixedRotation = true;
			def.position.Set(attr.x, attr.y);
			
			return def;
		}
		public override function setBody(bodyIn:b2Body):void {
			super.setBody(bodyIn);
			
			var def:b2PolygonDef = new b2PolygonDef();
			def.userData = this;
			def.SetAsBox(8, 30);
			for (var i:int = 0; i < def.vertexCount; i++) {
				def.vertices[i].y -= 14;
			}
			def.density = 1;
			def.friction = 1;
			def.restitution = 0;
			
			body.CreateShape(def);
			body.SetMassFromShapes();
			
			pos.SetV(body.GetPosition());
		}
	}
}