﻿package game.actor.enemies {
	import Box2D.Collision.Shapes.b2PolygonDef;
	import elm.core.*;
	import elm.game.CooldownTimer;
	import elm.gfx.Animation;
	import elm.types.Vec2;
	import elm.utils.*;
	import Box2D.Collision.Shapes.b2CircleDef;
	import Box2D.Dynamics.b2Body;
	import Box2D.Dynamics.b2BodyDef;
	import flash.geom.Point;
	import state.*;
	import game.actor.*;
	import game.actor.player.*;
	import game.actor.bullets.*;

	public class Tank extends Enemy {
		public static const STATE_WAIT:int = 0;
		public static const STATE_ATTACK:int = 1;
		public static const TIMES:Array = [0.5, 2];
		public static const ATTACK_DISTANCE:Number = 200;
		public static const ATTACK_ANGLE_DIFFERENCE:Number = 10;
		public static const SPEED_ANGLE:Number = 50;
		
		// ai data
		private var ai_state:int;
		private var player_pos:Vec2 = new Vec2();
		private var pos:Vec2 = new Vec2();
		private var angle_to_player0:Number;
		private var angle_to_player1:Number;
		
		// gun
		var guns:Array = [null, null];
		var gun_angles:Array = [0, 0];
		var ai_dir:int = 0;
		
		// cooldowns
		private var aiCooldown:CooldownTimer;
		
		public function Tank(attrIn:Object) {
			super(attrIn);
			
			if (o) {
				// animations
				o.Set("stand", 0, 1, 1, 0, true);
				o.Set("walk", 0, 1, 1, 0, true);
				
				// gun
				o["point0"]["gun"].gotoAndStop(1);
				o["point1"]["gun"].gotoAndStop(1);
			}
			
			guns[0] = new ActorGun( { owner:this, movie:o["point0"] } );
			guns[1] = new ActorGun( { owner:this, movie:o["point1"] } );
			guns[0].setGun( { name:"Gun", frame:1, time:2.5, bullet:"TankBullet", dist:150 } );
			guns[1].setGun( { name:"Gun", frame:1, time:4.2, bullet:"TankBullet", dist:150 } );
			
			is_gravity = true;
			speed_horizontal = 10;
			speed_vertical = 0;
			speed_jump = 0;
			health = 300;
			
			ai_state = STATE_WAIT;
			aiCooldown = new CooldownTimer(TIMES[ai_state]);
			aiCooldown.start();
		}
		public override function free():void {
			aiCooldown = null;
			player_pos = null;
			pos = null;
			
			super.free();
		}
		
		public override function update(delta:Number):void {
			if (aiCooldown.isComplete()) {
				switch(ai_state) {
					case STATE_WAIT:
						if (!actor_manager) {
							die();
							return;
						}
						player = actor_manager.findPlayer();
						if (!player) {
							die();
							return;
						}
						for (var i:int = 0; i < guns.length; i++) {
							guns[i].setRoom(room);
							guns[i].start();
						}
						player_pos.SetV(player.getBody().GetPosition());
						//if (player_pos.y > pos.y && pos.DistanceTo(player_pos) <= ATTACK_DISTANCE * 2) {
							ai_state = STATE_ATTACK;
							aiCooldown.start(TIMES[ai_state]);
						/*} else {
							aiCooldown.start();
						}*/
					break;
					case STATE_ATTACK:
						aiCooldown.start(TIMES[ai_state]);
						if (pos.DistanceTo(player_pos) > ATTACK_DISTANCE) {
							if (player_pos.x < pos.x) ai_dir = -1;
							else ai_dir = 1;
						} else {
							ai_dir = 0;
						}
					break;
				}
			}
			if (ai_state == STATE_ATTACK) {
				if (player) {
					player_pos.SetV(player.getBody().GetPosition());
					/*pos.SetV(body.GetPosition());
					angle_to_player0 = angle_to_player1 = gun_angle_d0 = gun_angle_d1 = Angle.normal(pos.AngleTo(player_pos));
					gun_angle0 = gun_angle1 = Angle.approach(delta * SPEED_ANGLE, gun_angle0, gun_angle_d0);
					if (gun_angle0 < 180) {
						show_angle0 = show_angle1 = gun_angle0;
					} else {
						show_angle0 = show_angle1 = 360 - gun_angle0;
					}*/
					walk(ai_dir);
					
					for (var i:int = 0; i < guns.length; i++) {
						gun_angles[i] = guns[i].getAngle(player_pos);
						guns[i].update(delta);
						if (guns[i].isComplete()) {
							guns[i].shoot(gun_angles[i], null);
						}
					}
				} else {
					trace("Not player.");
					ai_state = STATE_WAIT;
				}
			}
			
			aiCooldown.update(delta);
			
			// animations
			var ani_new:String = "stand";
			if (is_ground) {
				if (dir == 0) ani_new = "stand";
				else ani_new = "walk";
			}
			if (ani != ani_new) {
				ani = ani_new;
			}
			
			super.update(delta);
		}
		
		public override function render():void {
			super.render();
			if (o && body) {
				o["point0"]["gun"].rotation = gun_angles[0];
				o["point1"]["gun"].rotation = gun_angles[1];
			}
		}
		
		public override function damage(param:Object):void {
			super.damage(param);
		}
		
		public override function getBodyDef():b2BodyDef {
			var def:b2BodyDef = new b2BodyDef();
			def.userData = this;
			def.fixedRotation = true;
			def.position.Set(attr.x, attr.y);
			
			return def;
		}
		public override function setBody(bodyIn:b2Body):void {
			super.setBody(bodyIn);
			
			var def:b2PolygonDef = new b2PolygonDef();
			def.userData = this;
			def.SetAsBox(30, 30);
			def.density = 1;
			def.friction = 1;
			def.restitution = 0;
			
			body.CreateShape(def);
			body.SetMassFromShapes();
			
			pos.SetV(body.GetPosition());
		}
	}
}