package game.actor.player {
	import elm.core.*;
	import elm.game.*;
	import elm.gfx.Animation;
	import elm.types.Vec2;
	import elm.utils.*;
	import Box2D.Collision.Shapes.b2PolygonDef;
	import Box2D.Collision.b2ContactPoint;
	import Box2D.Collision.Shapes.b2CircleDef;
	import Box2D.Collision.Shapes.b2Shape;
	import Box2D.Dynamics.*;
	import Box2D.Dynamics.Contacts.b2ContactResult;
	import flash.display.*;
	import flash.geom.Point;
	import game.actor.triggers.PortalTrigger;
	import state.*;
	import game.*;
	import game.actor.*;
	import game.room.entity.*;
	import physic.*;

	public class PlayerActor extends WalkingActor implements DamagedActor {
		public static const JUMP_TIME_NORMAL:Number = 2;
		public static const GUN_CHANGE_TIME:Number = 2;
		
		public var health_max:Number = 200;
		
		public var aim_angle:Number;
		public var show_dir:Number = 1;
		public var show_gun_angle:Number = 0;
		
		private var shapes:Array = [];
		
		// gun
		var gun:Object;
		var actor_gun:ActorGun;
		
		// cooldowns
		private var gunChangeCooldown:CooldownTimer;
		
		// portal
		private var is_portal:Boolean = false;
		
		public function PlayerActor(attrIn:Object) {
			super(attrIn);
			
			if(o) {
				// animations
				o.Set("stand", 0, 1, 1, 0, true);
				o.Set("sit", 0, 5, 5, 0, true);
				o.Set("walk", 5, 2, 4, Animation.PINPONG, true);
				o.Set("jump_up", 0, 6, 6, 0, true);
				o.Set("jump_down", 0, 7, 7, 0, true);
				o.Set("ladder", 0, 8, 8, 0, true);
				o.Set("ladder_up", 5, 8, 9, 0, true);
				o.Set("ladder_down", 5, 8, 9, 0, true);
				
				// gun
				o["point"]["gun"].gotoAndStop(1);
			}
			
			// speed
			speed_horizontal = 20;
			speed_vertical = 15;
			speed_jump = 60;
			
			// health
			if (attr.health_max) health_max = attr.health_max;
			if (attr.health) health = attr.health; else health = health_max;
			
			// cooldowns
			gunChangeCooldown = new CooldownTimer(GUN_CHANGE_TIME);
			
			// gun !!!
			actor_gun = new ActorGun({owner:this, movie:o["point"]});
			gun = { guns:[], active: -1 };
			if (attr.guns) {
				for (var i:int = 0; i < attr.guns.length; i++) {
					addGun(attr.guns[i]);
				}
				activateGun(gun.guns[0].name);
			}
		}
		public override function free():void {
			gunChangeCooldown = null;
			actor_gun = null;
			gun = null;
			
			super.free();
		}
		public override function update(delta:Number):void {
			if (!is_jumped && jump_time > JUMP_TIME_NORMAL) {
				health -= jump_time * 10;
				trace("Health: "+int(health));
			}
			
			// gun
			if (gunChangeCooldown.isComplete()) {
				gunChangeCooldown.stop();
				actor_gun.setGun(gun.guns[gun.active]);
				actor_gun.setRoom(room);
			}
			
			// health
			if (health <= 0) {
				die();
			}
			
			// cooldowns
			gunChangeCooldown.update(delta);
			actor_gun.update(delta);
			
			// animations
			var ani_new:String = "stand";
			if (!is_ground && is_ladder) {
				if (dir_vertical < 0) ani_new = "ladder_up";
				if (dir_vertical > 0) ani_new = "ladder_down";
				//else ani_new = "ladder";
			} else if (is_ground) {
				if (dir == 0) ani_new = "stand";
				else ani_new = "walk";
			} else {
				if (body.m_linearVelocity.y <= 0) ani_new = "jump_up";
				else ani_new = "jump_down";
			}
			if (is_sitting && !is_ladder && !is_jumped) ani_new = "sit";
			if (ani != ani_new) {
				ani = ani_new;
			}
			
			super.update(delta);
		}
		public override function render():void {
			super.render();
			if (o) {
				o.scaleX = show_dir;
				o["point"]["gun"].rotation = show_gun_angle;
			}
		}
		public override function die(quietly:Boolean = true):void {
			super.die(quietly);
		}
		
		public function aim(x:Number, y:Number):void {
			if (!body) return;
			/*var v:Vec2 = new Vec2(x, y);
			var u:Vec2 = new Vec2();
			u.SetV(body.GetPosition());
			if (o) {
				u.x += o["point"].x;
				u.y += o["point"].y;
			}
			
			aim_angle = Angle.normal(u.AngleTo(v));*/
			aim_angle = actor_gun.getAngle(new Vec2(x, y));
			if (aim_angle < 180) {
				//show_dir = 1;
				show_gun_angle = aim_angle;
			} else {
				//show_dir = -1;
				show_gun_angle = 360 - aim_angle;
			}
			if (x < body.GetPosition().x) show_dir = -1; else show_dir = 1;
		}
		public function shoot(point:Vec2):void {
			/*if (gunShootCooldown.isComplete()) {
				gunShootCooldown.start();
				if (room && body && o) {
					var defClass:Class = EntityClassMap.getEntityClass(gun.guns[gun.active].bullet);
					if (defClass) {
						var p:Point = o["point"]["gun"]["bullet"].localToGlobal(new Point());
						p = Gameplay.stageToWorld(p);
						var v:Vec2 = new Vec2(p.x, p.y);
						room.add(new defClass( { owner:this, x:v.x, y:v.y, angle:aim_angle, point:point} ));
						trace("shoot");
					}
				}
			}*/
			if (actor_gun.isComplete()) {
				actor_gun.shoot(aim_angle, point);
			}
 		}
		public function addGun(gunInfoIn:Object) {
			if (!gunInfoIn) return;
			for (var i:int = 0; i < gun.guns.length; i++) {
				if (gun.guns[i].name == gunInfoIn.name) {
					return;
				}
			}
			var id:int = gun.guns.push(gunInfoIn) - 1;
			if(gun.active <= 0) activateGun(gun.guns[id].name);
		}
		public function activateGun(name:String):Boolean {
			o["point"]["gun"].gotoAndStop(1);
			for (var i:int = 0; i < gun.guns.length; i++) {
				if (gun.guns[i].name == name) {
					gun.active = i;
					actor_gun.stop();
					gunChangeCooldown.start();
					return true;
				}
			}
			return false;
		}
		public function getGunDist():Number {
			if (gun.active >= 0) {
				if (gun.guns[gun.active].dist) return gun.guns[gun.active].dist;
			}
			return 80;
		}
		public function gotoPortal(portal:PortalTrigger):void {
			is_portal = true;
		}
		public function isPortal():Boolean {
			return is_portal;
		}
		
		public override function onDeath():void {
		}
		
		public function damage(param:Object):void {
			health -= param.damage;
		}
		
		public override function isCollised(actor_shape:b2Shape):Boolean {
			if ((!is_sitting && actor_shape == shapes[0]) || (is_sitting && actor_shape == shapes[1])) return true;
			return false;
		}
		
		public override function getBodyDef():b2BodyDef {
			var def:b2BodyDef = new b2BodyDef();
			def.userData = this;
			def.fixedRotation = true;
			def.position.Set(attr.x, attr.y);
			
			return def;
		}
		public override function setBody(bodyIn:b2Body):void {
			super.setBody(bodyIn);
			
			// stand
			var def:b2PolygonDef = new b2PolygonDef();
			def.userData = this;
			def.SetAsBox(8, 30);
			for (var i:int = 0; i < def.vertexCount; i++) {
				def.vertices[i].y -= 12;
			}
			def.density = 1;
			def.friction = 1;
			def.restitution = 0;
			shapes.push(body.CreateShape(def));
			
			// sit
			def = new b2PolygonDef();
			def.userData = this;
			def.SetAsBox(28, 8);
			for (var i:int = 0; i < def.vertexCount; i++) {
				def.vertices[i].y += 16;
			}
			def.density = 1;
			def.friction = 1;
			def.restitution = 0;
			shapes.push(body.CreateShape(def));
			
			body.SetMassFromShapes();
		}
	}
}