﻿package state {
	import elm.core.*;
	import elm.game.Camera2D;
	import elm.input.*;
	import Box2D.Dynamics.*;
	import Box2D.Collision.*;
	import Box2D.Collision.Shapes.*;
	import Box2D.Common.Math.*;
	import Box2D.Dynamics.*;
	import elm.types.Vec2;
	import flash.display.*;
	import flash.events.*;
	import flash.geom.Point;
	import flash.ui.Keyboard;
	import game.*;
	import game.actor.*;
	import game.actor.bullets.Bullet;
	import game.actor.enemies.Soldier;
	import game.actor.player.*;
	import game.room.*;
	import physic.*;
	
	public class Gameplay extends State {
		public static const PHYSIC_GRAVITY:Number = 100;
		public static const GUNS:Array = ["Pistol", "Automatic", "Grenade", "Rocket"];
		
		public var attr:Object;
		// layer
		public var BACK:Layer;
		public var GAME:Layer;
			public var MAP:Layer;
			public var BULLET:Layer;
			public var ENEMY:Layer;
			public var DEBUG:Layer;
		public var GUI:Layer;
		
		// flags
		public var is_victory:Boolean;
		public var is_gameover:Boolean;
		public var is_pause:Boolean;
		
		// stat
		public var stat:Object;
		
		// game data
		public var room:Room;
		public var actor_manager:ActorManager;
		public var player:PlayerActor;
		public var player_gun:int;
		public var camera:Camera2D;
		public var mouseCoords:Vec2 = new Vec2();
		
		// physic
		public var world:b2World;
		public var collision:Collision;
		public var contact_filter:ContactFilter;
		
		// gui
		public var gui:MovieClip;
		
		// control
		public var control:InputDispatcher;
		private var key_left:Boolean;
		private var key_right:Boolean;
		private var key_up:Boolean;
		private var key_down:Boolean;
		private var key_shoot:Boolean;
		
		public function Gameplay() {
		}
		public override function init(attrIn:* = null):Boolean {
			attr = attrIn;
			
			// layers
			BACK = Gfx.createLayer(null, "back", 0, true);
			GAME = Gfx.createLayer(null, "game", 50, true);
				MAP = Gfx.createLayer(GAME, "map", 10, true);
				BULLET = Gfx.createLayer(GAME, "bullet", 20, true);
				ENEMY = Gfx.createLayer(GAME, "enemy", 30, true);
				DEBUG = Gfx.createLayer(GAME, "debug", 100, true);
			GUI = Gfx.createLayer(null, "gui", 100, true);
			
			// flags
			is_victory = false;
			is_gameover = false;
			is_pause = false;
			
			// stat
			stat = {};
			
			// game data
			actor_manager = new ActorManager();
			room = new Room(attr);
			room.addEventListener(EntityEvent.ADD, onEntityAdd);
			room.addEventListener(EntityEvent.REMOVE, onEntityRemove);
			camera = new Camera2D(GAME, Room.MAP_WIDTH, Room.MAP_HEIGHT, 1000);
			player_gun = -1;
			
			// gui
			gui = Gfx.createMovie(GUI, "gui.Gameplay", "gui") as MovieClip;
			gui["gun"].stop();
			gui["life"].stop();
			
			// control
			control = new InputDispatcher();
			control.add("exit", [27]);
			control.add("left", [Keyboard.LEFT, 65]);
			control.add("right", [Keyboard.RIGHT, 68]);
			control.add("up", [Keyboard.UP, 87]);
			control.add("down", [Keyboard.DOWN, 83]);
			control.add("weapon", [69, 71]);
			control.add("shoot", null);
			control.addEventListener(InputEvent.INPUT, onInput);
			control.init();
			key_left = false;
			key_right = false;
			key_up = false;
			key_down = false;
			key_shoot = false;
			
			initPhysic();
			gen();
			
			Log.Message("Gameplay init.");
			return true;
		}
		public override function free():void {			
			// game data
			actor_manager.removeAll();
			actor_manager = null;
			
			// control
			control.removeEventListener(InputEvent.INPUT, onInput);
			control.free();
			control = null;
			
			// physic
			contact_filter = null;
			collision = null;
			world = null;
			
			// gui
			gui = null;
			
			Log.Message("Gameplay free.");
		}
		private function initPhysic():void {
			var aabb:b2AABB = new b2AABB();
			aabb.lowerBound.Set(-10, -10);
			aabb.upperBound.Set(Room.MAP_WIDTH+10, Room.MAP_HEIGHT+10);

			var gravity:b2Vec2 = new b2Vec2(0, 0);
			var do_sleep:Boolean = false;
			
			world = new b2World(aabb, gravity, do_sleep);
			collision = new Collision();
			world.SetContactListener(collision);
			contact_filter = new ContactFilter();
			world.SetContactFilter(contact_filter);
			
			// debug draw
			/*var dd:b2DebugDraw = new b2DebugDraw();
			dd.m_sprite = DEBUG;
			dd.SetFlags(b2DebugDraw.e_shapeBit | b2DebugDraw.e_centerOfMassBit);
			world.SetDebugDraw(dd);*/
			
			var body:b2Body;
			var bodyDef:b2BodyDef;
			var boxDef:b2PolygonDef;
			var circleDef:b2CircleDef;
			
			// down
			bodyDef = new b2BodyDef();
			bodyDef.position.Set(Room.MAP_WIDTH/2, Room.MAP_HEIGHT);
			bodyDef.angle = 0;
			boxDef = new b2PolygonDef();
			boxDef.SetAsBox(Room.MAP_WIDTH/2, 2);
			boxDef.friction = 1;
			boxDef.density = 0;
			body = world.CreateBody(bodyDef);
			body.CreateShape(boxDef);
			body.SetMassFromShapes();
			// up
			bodyDef = new b2BodyDef();
			bodyDef.position.Set(Room.MAP_WIDTH/2, 0);
			bodyDef.angle = 0;
			boxDef = new b2PolygonDef();
			boxDef.SetAsBox(Room.MAP_WIDTH/2, 2);
			boxDef.friction = 1;
			boxDef.density = 0;
			body = world.CreateBody(bodyDef);
			body.CreateShape(boxDef);
			body.SetMassFromShapes();
			//left
			bodyDef = new b2BodyDef();
			bodyDef.position.Set(0, Room.MAP_HEIGHT/2);
			bodyDef.angle = 0;
			boxDef = new b2PolygonDef();
			boxDef.SetAsBox(2, Room.MAP_HEIGHT/2);
			boxDef.friction = 1;
			boxDef.density = 0;
			body = world.CreateBody(bodyDef);
			body.CreateShape(boxDef);
			body.SetMassFromShapes();
			//right
			bodyDef = new b2BodyDef();
			bodyDef.position.Set(Room.MAP_WIDTH, Room.MAP_HEIGHT/2);
			bodyDef.angle = 0;
			boxDef = new b2PolygonDef();
			boxDef.SetAsBox(2, Room.MAP_HEIGHT/2);
			boxDef.friction = 1;
			boxDef.density = 0;
			body = world.CreateBody(bodyDef);
			body.CreateShape(boxDef);
			body.SetMassFromShapes();
		}
		private function gen():void {
			var map:MovieClip = Gfx.createMovie(MAP, "ideMap" + int(Game.getVar("map")), "map") as MovieClip;
			if (!map) {
				Game.setState("replay");
				return;
			}
			attr.scene = map;
			// load
			room.build();
			camera.Update(0);
			
			// stat
			stat.scorenow = 0;
			stat.map = int(Game.getVar("map"));
			
			//player
		}
		public override function frame():void {
			if (is_victory) return;
			if (is_gameover) return;
			if (is_pause) return;
			
			var delta:Number = Game.delta;
			
			var p:Point = new Point(Game.mouseX, Game.mouseY);
			p = stageToWorld(p);
			mouseCoords.SetP(p);
			
			// input
			if (player) {
				player.aim(mouseCoords.x, mouseCoords.y);
				
				if (key_shoot) player.shoot(mouseCoords);
				if (key_left) player.walk( -1);
				else if (key_right) player.walk(1);
				else player.walk(0);
				if (key_down) player.walkUp(1);
				else if (key_up) player.walkUp(-1);
				else player.walkUp(0);
			}
			
			// gravity
			for (var body:b2Body = world.GetBodyList(); body !== null; body = body.GetNext()) {
				if (body.IsSleeping() || body.GetMass() == 0) {
					continue;
				}
				
				var applyGravity:Boolean = true;
				var bodyUserData:* = body.GetUserData();
				if (bodyUserData is Actor) {
					applyGravity = (bodyUserData as Actor).is_gravity;
				}
				if (applyGravity) {
					body.m_linearVelocity.y += delta * PHYSIC_GRAVITY;
				}
			}
			
			// actor update
			actor_manager.update(delta);
			
			// physic
			world.Step(delta * 4, 10);
			
			// actor render
			actor_manager.render();
			
			// portal
			if (player && player.isPortal()) {
				setVictory();
			}
			
			// camera
			if(player && player.getBody()) {
				var v:Vec2 = new Vec2();
				v.SetV(player.getBody().GetPosition());
				var u:Vec2 = new Vec2(mouseCoords.x, mouseCoords.y);
				u.Subtract(v);
				var dist:Number = Math.min(u.Length(), player.getGunDist());
				u.Normalize();
				u.Multiply(dist * 0.5);
				//u.Multiply(0.5);
				u.Add(v);
				camera.MoveV(u);
				camera.Update(delta);
			}
			
			// gui
			if (player) {
				gui["gun"].gotoAndStop(player_gun + 2);
				var frame:int = 20 - int(player.health / player.health_max * 20);
				gui["life"].gotoAndStop(frame);
			}
		}
		//public override function isMouseEvent():Boolean { return false; }
		public override function onKeyDown(e:KeyboardEvent):void {
		}
		public function onInput(e:InputEvent):void {
			switch(e.action) {
				case "exit":
					Game.setState("mainmenu");
				break;
				case "left":
					if (e.isDown) {
						key_left = true;
						//if (player) player.walk( -1);
					} else {
						key_left = false;
						//if (player) player.walk(0);
					}
				break;
				case "right":
					if (e.isDown) {
						key_right = true;
						//if (player) player.walk(1);
					} else {
						key_right = false;
						//if (player) player.walk(0);
					}
				break;
				case "up":
					if (e.isDown) {
						key_up = true;
						//if (player) player.walkUp( -1);
					} else {
						key_up = false;
						//if (player) player.walkUp(0);
					}
				break;
				case "down":
					if (e.isDown) {
						key_down = true;
						//if (player) player.walk(1);
					} else {
						key_down = false;
						//if (player) player.walk(0);
					}
				break;
				case "weapon":
					if (e.isDown) {
						if (player) {
							do {
								player_gun++;
								if (player_gun >= GUNS.length) player_gun = 0;
							} while (!player.activateGun(GUNS[player_gun]));
						}
					}
				break;
				case "shoot":
					if(e.isDown) {
						key_shoot = true;
					} else {
						key_shoot = false;
					}
				break;
			}
		}
		public function onEntityAdd(e:EntityEvent):void {
			if (e.entity is PhysicRecipient) {
				var body:b2Body = world.CreateBody((e.entity as PhysicRecipient).getBodyDef());
				(e.entity as PhysicRecipient).setBody(body);
			}
			if (e.entity is Actor) actor_manager.add(e.entity as Actor);
			if (e.entity is PlayerActor) {
				player = e.entity as PlayerActor;
				camera.SetV(player.getBody().GetPosition());
			}
			if (e.entity is Bullet) {
				BULLET.addChild(e.entity.getVisualObject());
			}
			if (e.entity is Soldier) {
				ENEMY.addChild(e.entity.getVisualObject());
			}
		}
		public function onEntityRemove(e:EntityEvent):void {
			if (e.entity is PhysicRecipient) {
				world.DestroyBody((e.entity as PhysicRecipient).getBody());
			}
			if (e.entity is Actor) {
				actor_manager.remove(e.entity as Actor);
			}
			if (e.entity is Bullet) {
				BULLET.removeChild(e.entity.getVisualObject());
			}
			if (e.entity is Soldier) {
				ENEMY.removeChild(e.entity.getVisualObject());
			}
			if (e.entity is PlayerActor/*&& e.entity == player*/) {
				player = null;
				setGameover();
			}
			e.entity.free();
		}
		public static function newGame():void {
			Game.setVar("map", 0);
			play("map0");
		}
		public static function play(name:String):void {
			//Game.setState("xmlloader", 0, {name:name, state:"gameplay"});
			Game.setState("fader", Game.MODE_FRONT, {color:0x000000, state:"gameplay", attr:{name:"", xml_str:""}});
		}
		public function setVictory():void {
			trace("VICTORY");
			is_victory = true;
			
			Game.setVar("map", stat.map + 1);
			play("map" + Game.getVar("map"));
		}
		public function setGameover():void {
			trace("GAME OVER");
			is_gameover = true;
			Game.setState("replay");
		}
		
		// UTILS
		
		public static function stageToWorld(p:Point, statename:String = "gameplay"):Point {
			var gameplay:Gameplay = Game.getState(statename) as Gameplay;
			return gameplay.GAME.globalToLocal(p);
		}
	}
}