//=============================================================================
//           : Tower Defence
//   : e_Console.pas
//        :    
//            : Darthman and Soulghai
//    : 17.10.2006
//       :    ,  
//                      .
//
//=============================================================================
unit e_Console;

interface

uses
  windows, OpenGL, e_about, e_utils, e_types, e_font, e_models, e_textures, e_console_commands;

type
  TConsoleParam = (cpString, cpInteger, cpBoolean, cpProcVoid, cpProcBool, cpProcInteger, cpProcString);

  TConsoleParams = array[1..8] of string;

  TConsoleCMD = packed record
    CMD   : string;                   //  
    Param : Pointer;                  //  
    ParamType : TConsoleParam;        //   
  end;

  TConsole = class
  public
    StringCount,CurrStr  : Dword;
    Text : array of String;                          //      
    CMDCount : Integer;
    CMDs : array of TConsoleCMD;
    alpha : single;
    LastCommand : string;
    Active : boolean;
    log_filename : string;
  public
    constructor Create(_log_filename : string);
    procedure   AddCommand(aCMD : string; aParam : Pointer; aParamType : TConsoleParam );
    function    Execute(str : string) : string;
    procedure   SupString(str : string);
    procedure   Update(count : single);
    procedure   Render;
    function    AddChar(c : char) : boolean;
    function    AddKey(c : char) : boolean;
    procedure   Write(st : string);
    function    GetParams(s : string) : TConsoleParams;
  end;

  TConsoleProcVoid = procedure;
  TConsoleProcBool = procedure(param : Boolean);
  TConsoleProcInteger = procedure(param : Integer);
  TConsoleProcString = procedure(param : string);

var
  console : TConsole;

const
  save_log = true;

implementation

//=============================================================================
// convert string in upper case
//=============================================================================
function UpperCase(const S : string): string;
var
  Ch: Char;
  L: Integer;
  Source, Dest: PChar;
begin
  L:= Length(S);
  SetLength(Result, L);
  Source:= Pointer(S);
  Dest:= Pointer(Result);
  while L <> 0 do
  begin
    Ch:= Source^;
    if (Ch >= 'a') and (Ch <= 'z') then Dec(Ch, 32);
    Dest^:= Ch;
    Inc(Source);
    Inc(Dest);
    Dec(L);
  end;
end;

//=============================================================================
//
//=============================================================================
constructor TConsole.Create(_log_filename : string);
var
f : textfile;
begin
  inherited Create;
  CMDCount:= 0;
  SetLength(CMDs, CMDCount);
  StringCount := 1;
  CurrStr := 0;
  SetLength(Text,1);
  alpha := -0.1;
  Active := false;
  log_filename := _log_filename;

  if save_log then
    begin
      AssignFile(f, log_filename);
      Rewrite(f);
      Writeln(f, program_name + ', ' + developer_name + ' v0.' + inttostr(program_version) + program_version_letter);
      Writeln(f, '');
      CloseFile(f);
    end;
  Write('Console');
end;

//=============================================================================
//
//=============================================================================
procedure TConsole.AddCommand(aCMD : string; aParam : Pointer; aParamType : TConsoleParam );
begin
  inc(CMDCount);
  SetLength(CMDs, CMDCount);

  with CMDs[CMDCount-1] do
  begin
    CMD:= uppercase(aCMD);
    Param:= aParam;
    ParamType:= aParamType;
  end;
end;

//=============================================================================
//
//=============================================================================
function TConsole.Execute(str : string) : string;
var
  ProcVoid    : TConsoleProcVoid;
  ProcBoolean : TConsoleProcBool;
  ProcInteger : TConsoleProcInteger;
  ProcString  : TConsoleProcString;
  command : string;
  parameter : string;
  i : Integer;
  found : Boolean;
  paramexist : Boolean;
  foundindex : Integer;
begin
  result := '';
  if length(str) = 0 then exit;
  if str[1] = ' ' then exit;

  //  
  command:= str;
  i:= 1;
  while command[i] <> ' ' do
  inc(i);
  Delete(command, i, length(command) - i + 1);
  command:= UpperCase(command);

  //  
  parameter:= str;
  Delete(parameter, 1, i);
  paramexist:= false;
  if Length(parameter) > 0 then
  paramexist:= true;

  //      :)
  i:= 0;
  found:= false;
  repeat
    if command = CMDs[i].CMD then
    begin
      found:= true;
      foundindex:= i;
    end;
    inc(i);
  until (found) or (i = CMDCount);

  if not found then
  begin
    Result:= 'Unknown command : ' + command;
    exit;
  end;

  //         .
  if CMDs[foundindex].ParamType = cpProcVoid then
  begin
    ProcVoid:= CMDs[foundindex].Param;
    ProcVoid;
  end;

  case CMDs[foundindex].ParamType of
    cpString : begin
                 if paramexist then String(CMDs[foundindex].Param^):= parameter;
                 Result:= command + ' value is "' + String(CMDs[foundindex].Param^) + '"';
               end;
    cpInteger : begin
                  if paramexist then integer(CMDs[foundindex].Param^):= strtoint(parameter);
                  Result:= command + ' value is "' + inttostr(integer(CMDs[foundindex].Param^)) + '"';
                end;
    cpboolean : begin
                  if paramexist then
                  begin
                    if parameter = '1' then Boolean(CMDs[foundindex].Param^):= true;
                    if parameter = '0' then Boolean(CMDs[foundindex].Param^):= false;
                  end;
                  if boolean(CMDs[foundindex].Param^) = false then
                  Result:= command + ' value is "0"' else
                  Result:= command + ' value is "1"';
                end;
    cpProcBool : begin
                   if not paramexist then
                   begin
                     Result:= 'Parameter must be "1" or "0"';
                     exit;
                   end;
                   
                   ProcBoolean:= CMDs[foundindex].Param;
                   if parameter = '1' then ProcBoolean(true);
                   if parameter = '0' then ProcBoolean(false);
                 end;
    cpProcInteger : begin
                      if not paramexist then
                      begin
                        Result:= 'Parameter must be Integer';
                        exit;
                      end;

                      ProcInteger:= CMDs[foundindex].Param;
                      ProcInteger(strtoint(parameter));
                    end;
    cpProcString : begin
                     if not paramexist then
                     begin
                       Result:= 'Parameter must be any string value';
                       exit;
                     end;

                      ProcString:= CMDs[foundindex].Param;
                      ProcString(parameter);
                    end;

  end;
end;

procedure TConsole.SupString(str : string);
var
  i : word;
  cnt : byte;
  sCount : byte;
  s : array of string;
  c : byte;
begin
  sCount := 0;
  SetLength(s,0);
  cnt := Length(str);
  if cnt = 0 then exit;
  for i := 0 to CMDCount - 1 do begin
    if UpperCase(str) = copy(CMDs[i].CMD,0,cnt) then begin
      inc(sCount);
      SetLength(s,sCount);
      s[sCount-1] := '    \' + CMDs[i].CMD;
      c:=i;
    end;
  end;
  if sCount = 0 then exit;
  if sCount = 1 then Text[CurrStr] := CMDs[c].CMD + ' ' else begin
    Write('\'+str);
    for i := 0 to sCount - 1 do Write(s[i]);
  Text[CurrStr] := str;
  end;
end;

procedure TConsole.Update(count : single);
var
  t : single;
  s : string;
begin
t := -0.1;
if Active then t := 1;
Aspire(alpha, t, count * 0.003);
end;

function TConsole.AddChar(c : char) : boolean;
var
s : string;
begin
result := false;

// TAB \\
if c = chr(vk_tab) then
  begin
    result := true;
    Active := not Active;
    exit;
  end;

if not Active then exit;

result := true;

//Baskspace\\
if c = chr(vk_back) then
  begin
    Delete(Text[CurrStr], Length(Text[CurrStr]), 1);
    exit;
  end;

//Enter\\
if c = chr(vk_return) then
  if Text[CurrStr] <> '' then
    begin
      LastCommand := Text[CurrStr];
      Write(Text[CurrStr]);
      s := Execute(Text[CurrStr-1]);
      if s <> '' then Write(s);
      exit;
    end;

if c = chr(vk_up) then
  begin
    Text[CurrStr] := LastCommand;
    exit;
  end;

{// Page Up \\
if c = chr(vk_next) then
  if TextPos > 0 then
    TextPos := TextPos - FntMan.Items[fnt].HShift;
    Input.Keys[$21] := false;
  end;

    // Page Down \\
     if Input.Keys[$22] then begin
        if Top-20+TextPos-(StringCount)*FntMan.Items[fnt].HShift < 0 then
        TextPos := TextPos + FntMan.Items[fnt].HShift;
      Input.Keys[$22] := false;
    end;
    // Button Up \\
     if Input.Keys[$26] then begin
        Text[CurrStr] := LastCommand;
      Input.Keys[$26] := false;
    end;
  end else if Top > 0 then Top := Top - Speed else
           if Top < 0 then begin
             Top := 0;
             Text[CurrStr] := '';
           end;     }

Text[CurrStr] := Text[CurrStr] + c;
end;

function TConsole.AddKey(c : char) : boolean;
var
s : string;
begin
result := false;

if not Active then exit;

result := true;

if c = chr(vk_up) then
  begin
    Text[CurrStr] := LastCommand;
    exit;
  end;
end;

procedure TConsole.Render;
var
  i : dword;
  p : integer;
begin
if alpha <= 0 then exit;

glMatrixMode(GL_PROJECTION);
glLoadIdentity();
glOrtho(0, 1, 1, 0, -1, 1);
glMatrixMode(GL_MODELVIEW);
glLoadIdentity();

glDisable(GL_DEPTH_TEST);
glEnable(GL_TEXTURE_2D);
glEnable(GL_BLEND);
glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

glColor4f(0.3, 0.3, 0.3, alpha);
glDisable(GL_TEXTURE_2D);

glPushMatrix;
glTranslatef(0.5, 0, 0);
glCallList(model_quad);
glPopMatrix;

font.flip := true;

glColor4f(0.8, 0.9, 1, alpha);
for i := StringCount-2 downto 0 do
  font.TextOut(0.12, 0.04, 0.47 + (i - StringCount * 1.0) * 0.03, Text[i]);

font.TextOut(0.12, 0.04, 0.47 - 0.03, Text[StringCount-1] + '|');


glEnable(GL_TEXTURE_2D);
font.flip := false;
end;

procedure TConsole.Write(st : string);
var
  f : TextFile;
begin
  inc(StringCount);
  SetLength(Text,StringCount);
  Text[CurrStr] := st;
  CurrStr := StringCount-1;

  if not save_log then exit;
  
  AssignFile(f, log_filename);
  Reset(f);
  Append(f);
  Writeln(f, st);
  CloseFile(f);
end;

function TConsole.GetParams(s : string) : TConsoleParams;
var
i, c : integer;
t, e : string;
begin
for i := 1 to high(result) do
  result[i] := '0';
  
c := 1; s := ' ' + s + ' ';
for i := 0 to length(s) do
  begin
    e := copy(s,i,1);
    if (e <> ' ') and (e <> '	') then t := t + e
    else
      if (t <> ' ') and (t <> '') and (t <> '	') then
        begin
          result[c] := t;
          inc(c);
          if c > high(result) then
            exit;
          t := '';
        end;
  end;
end;

end.
