unit e_enemies;

interface

uses OpenGL, e_types, e_textures, e_models, e_particles, e_sound, e_utils,
     e_global, e_console, e_camera;

type
  TEnemy = class(TCreature)
    map            : integer;
    size           : single;

    health         : single;
    target         : TPoint;
    speed          : TPoint;

    particles1     : TParticlesEmitter;
    particles2     : TParticlesEmitter;
  public
    constructor Create(_pos : single);                       virtual;
    procedure   GetDamage(damage : single);                  virtual;
    procedure   Render;                                      virtual;
    procedure   Update(count : single);                      virtual;
  end;

  TEnemy1 = class(TEnemy)
  public
    constructor Create(_pos : single);                       override;
    procedure   Render;                                      override;
    procedure   Update(count : single);                      override;
  end;

  TEnemy2 = class(TEnemy)
    ready : boolean;
    timer_ready : single;
  public
    constructor Create(_pos : single);                       override;
    procedure   Render;                                      override;
    procedure   Update(count : single);                      override;
  end;

  TEnemy3 = class(TEnemy)
    height        : single;
    timer_height  : single;
  public
    constructor Create(_pos : single);                       override;
    procedure   Render;                                      override;
    procedure   Update(count : single);                      override;
  end;



  TEnemyBoss1 = class(TEnemy)
    underground : boolean;
    speed_depth : single;
    timer_born  : single;
  public
    constructor Create(_pos : single);                       override;
    procedure   GetDamage(damage : single);                  override;
    procedure   Render;                                      override;
    procedure   Update(count : single);                      override;
  end;

implementation

uses e_game;

constructor TEnemy.Create(_pos : single);
begin
inherited Create;

health       := 10;
size         := 2;
pos          := GetPoint(_pos, 0, 0);
target       := game.player.pos;    
end;

procedure TEnemy.GetDamage(damage : single);
var
c : TColor;
n : integer;
s : single;
begin
if health <= 0 then exit;

health := health - damage;
particles.AddEmitter(particles_blood, pos, 20, game.player.color);
SoundPlay(sound_damage);

game.player.GetScore(1);
end;

procedure TEnemy.Render;
begin

end;

procedure TEnemy.Update(count : single);
begin
if pos.x < camera.eye.x - game_width then
  health := -1;

//player collisions
if CollideCircles(pos, game.player.pos, size / 2, game.player.radius) then
  game.player.GetDamage();

if particles1 <> nil then
  begin
    SqrAspire(particles1.color.r, game.color_dots.r, count * 0.0002);
    SqrAspire(particles1.color.g, game.color_dots.g, count * 0.0002);
    SqrAspire(particles1.color.b, game.color_dots.b, count * 0.0002);
  end;

if particles2 <> nil then
  begin
    SqrAspire(particles2.color.r, game.color.r, count * 0.0002);
    SqrAspire(particles2.color.g, game.color.g, count * 0.0002);
    SqrAspire(particles2.color.b, game.color.b, count * 0.0002);
  end;
end;

///////////////////

constructor TEnemy1.Create;
var
i : integer;
begin
inherited Create(_pos);

health := 55;
side   := -1;
size   := 2;

particles1 := particles.AddEmitter(particles_enemy1, pos, 20, game.color_dots);
particles1.size := 2.5;
particles2 := particles.AddEmitter(particles_enemy1, pos, 20, game.color);
particles2.size := 2;

particles1.parent := self;
particles2.parent := self;
end;
    
procedure TEnemy1.Render;
begin
{SetTexture(texture_enemy1_back);
glColor4f(1, 1, 1, 1);

glPushMatrix;
with pos do glTranslatef(x, y, 0);
glScalef(size, size, 1);
glCallList(model_quad);
glPopMatrix;   }
end;

procedure TEnemy1.Update(count : single);
var
s : single;
begin
inherited Update(count);

pos.x := pos.x + side * count * 0.004;
map := game.GetMap(pos.x);

if keys[ord('T')] then
  speed.y := 20;

if speed.y < -19 then
  if random > 0.98 then
    speed.y := 10 + random * 10;

speed.x := 0;
SqrAspire(speed.y, -20, count * 0.001);
pos.y := pos.y + speed.y * count * 0.001;
s := game.GetHeight(map, pos.x);
if pos.y < s then
  pos.y := s;

SqrAspire(angle, -PointsAngle(game.map[map + 1], game.map[map]), count * 0.005);

particles1.pos := pos;
particles2.pos := pos;
end;

///////////////////

constructor TEnemy2.Create;
var
i : integer;
begin
inherited Create(_pos);

speed.x := 0;
speed.y := 0;

size := 3;

health := 70;

particles1 := particles.AddEmitter(particles_enemy2, pos, 20, game.color_dots);
particles2 := particles.AddEmitter(particles_enemy2, pos, 30, game.color);
particles2.size := 2.3;
//particles2.multiply := true;

ready := false;
timer_ready := 5000 + random * 2000;

particles1.parent := self;
particles2.parent := self;
end;
    
procedure TEnemy2.Render;
begin
end;

procedure TEnemy2.Update(count : single);
var
s : single;
t : TEnemy;
begin
inherited Update(count);

pos.x := pos.x + speed.x * count * 0.007;
pos.y := pos.y + speed.y * count * 0.007;

map := game.GetMap(pos.x);
s := game.GetHeight(map, pos.x) + 0.5;
if pos.y < s then
  begin
    pos.y := s;
    speed.x := 0;
    ready := true;
    angle := - PointsAngle(game.map[map + 1], game.map[map]);
  end
else
  begin
    ready := false;
    speed.y := speed.y - count * 0.003;
    if speed.x > 0 then
      angle := 90 + PointAngle(speed)
    else
      angle := 270 - PointAngle(speed)
  end;

if ready then
  if timer_ready > 0 then
    timer_ready := timer_ready - count
  else
    begin
      timer_ready := 5000 + random * 2000;
      t := game.AddEnemy(2, pos.x);
      t.pos := pos;
      t.speed := GetPoint((random + 1) * 0.7, (random + 1) * 1.5, 0);
      if random(2) = 0 then
        t.speed.x := -t.speed.x;
    end;

side := sign(speed.x);

particles1.pos := pos;
if ready then
  particles1.pos.y := game.GetHeight(map, pos.x, true) + 0.5;
  
particles2.pos := particles1.pos;
end;

///////////////////

constructor TEnemy3.Create;
var
i : integer;
begin
inherited Create(_pos);

pos.y := game.GetHeight(game.GetMap(pos.x), pos.x) + 30;

size := 2;
health := 30;
height := 1 + random * 10;
timer_height := 4000 + 3000 * random;
particles1 := particles.AddEmitter(particles_enemy1, pos, 20, game.color_dots);
particles1.size := 2;
particles2 := particles.AddEmitter(particles_enemy1, pos, 20, game.color);
particles2.size := 1.5;

particles1.parent := self;
particles2.parent := self;
end;
    
procedure TEnemy3.Render;
begin
end;

procedure TEnemy3.Update(count : single);
var
s : single;
t : TEnemy;
begin
inherited Update(count);

pos.x := pos.x - count * 0.004;

map := game.GetMap(pos.x);
s := game.GetHeight(map, pos.x);

SqrAspire(pos.y, s + height, count * 0.003);

if pos.y < s then
  pos.y := s;

if timer_height > 0 then
  timer_height := timer_height - count
else
  begin
    timer_height := 4000 + 3000 * random;
    height := height + GetRandom * 3;
    if height < 1 then height := 1;
  end;

side := -1;

particles1.pos := pos;
particles2.pos := pos;
end;

///////////////////

constructor TEnemyBoss1.Create;
var
i : integer;
begin
inherited Create(_pos);

speed.x := 0;
speed.y := 0;

side := -1;
size := 3;

health := 1000;  
particles1 := particles.AddEmitter(particles_enemy1, pos, 80, game.player.color_base);
particles1.size := 3;
particles2 := particles.AddEmitter(particles_enemy1, pos, 80, game.player.color);
particles2.size := 2.5;
particles2.multiply := true;

underground := false;
speed_depth := 1;
timer_born  := 0;

particles1.parent := self;
particles2.parent := self;

game.player.map_max := pos.x + 15;
end;

procedure TEnemyBoss1.GetDamage(damage : single);
begin
if underground or (particles1 = nil) then
  inherited GetDamage(damage);

if health <= 0 then
  game.player.map_max := -1;
end;

procedure TEnemyBoss1.Render;
begin
end;

procedure TEnemyBoss1.Update(count : single);
var
s : single;
t : TEnemy;
begin
inherited Update(count);

particles1.color := game.player.color_base;
particles2.color := game.player.color;

speed.y := speed.y - count * 0.003;
pos.x := pos.x + speed.x * count * 0.007;
pos.y := pos.y + speed.y * count * 0.007;

map := game.GetMap(pos.x);
s := game.GetHeight(map, pos.x);
if pos.y < s then
  begin
    underground := true;
    pos.y := s;
    speed.x := 0;
  end;

if underground then
  if speed_depth > 0 then
    if particles2.pos.y >= s then
      begin
        underground := false;
        side  := -side;
        speed := GetPoint(side * 2, 3, 0);
        speed_depth := -7;
      end;

if not underground then
  begin
    timer_born := timer_born - count;
    if timer_born < 0 then
      begin
        timer_born := 400;
        t := game.AddEnemy(1, pos.x);
        t.side := side;
        t.pos  := pos;
      end;
  end;

particles1.pos := pos;
particles2.pos.x := pos.x;
if underground then
  begin
    speed_depth := speed_depth + count * 0.0035;
    particles2.pos.y := particles2.pos.y + speed_depth * count * 0.002;
  end
else
  particles2.pos.y := pos.y;
end;

///////////////////

end.
