unit e_font;

interface

uses Windows, OpenGL, e_textures, e_types, e_global, e_utils, e_screen;

type
  TFontAlign = (faLeft, faCenter, faRight);

  TFont = class
    list     : gluint;
    texture  : gluint;
    size     : integer;
    kerning  : single;
    gui_mode : boolean;
    flip     : boolean;
  public
    constructor Create;
    procedure   TextOut(s, x, y : single; text : string; align : TFontAlign = faLeft; alpha : single = 1);
    procedure   TextOutAngle(s, x, y : single; text : string; angle : single);
  end;

const
  font_text   : string[64] = ' 0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ.,:+-=()[]{}|"!?#/\%^*''';
  font_colors : array[0..9] of TColor = (
    (r : 1; g : 1; b : 1; a : 1),

    (r : 1; g : 0.1; b : 0.1; a : 1),
    (r : 1; g : 0; b : 0; a : 1),
    (r : 1; g : 0; b : 0; a : 1),

    (r : 1; g : 0.8; b : 0.1; a : 1),
    (r : 1; g : 0; b : 0; a : 1),
    (r : 1; g : 0; b : 0; a : 1),
    (r : 1; g : 0; b : 0; a : 1),
    (r : 1; g : 0; b : 0; a : 1),
    (r : 1; g : 0; b : 0; a : 1)
    );

var
  font : TFont;

implementation

constructor TFont.Create;
var
  i, x, y : integer;
  w : single;
const
  s = 0.0;
begin
kerning := 1.22;
kerning := 1.7;
if render_font_high then
  texture := LoadPNGTexture('textures/font_high.png')
else
  texture := LoadPNGTexture('textures/font.png');

list := glGenLists(64);

w := 16 / 128;
w := 1 / 8;

for i := 0 to 63 do
  begin
    x := (i - 1) mod 8;
    y := 7 - (i - 1) div 8;

    glNewList(list + i, GL_COMPILE);
    glBegin(GL_QUADS);
      glTexCoord2f(w *  x + s,      w * y + s);        glVertex2f(-1, -1);
      glTexCoord2f(w * (x + 1) - s, w * y + s);        glVertex2f( 1, -1);
      glTexCoord2f(w * (x + 1) - s, w * (y + 1) - s);  glVertex2f( 1, +1);
      glTexCoord2f(w *  x + s,      w * (y + 1) - s);  glVertex2f(-1, +1);
    glEnd();   
    glEndList;
  end;
end;

procedure TFont.TextOut(s, x, y : single; text : string; align : TFontAlign = faLeft; alpha : single = 1);
var
  i, p, a : integer;
begin
text := UpperCase(text);
glEnable(GL_BLEND);
glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
SetTexture(texture);          
glEnable(GL_TEXTURE_2D);

glPushMatrix;
glTranslatef(x, y, 0);
s := s * 0.1;
glScalef(s, s, 1);
if align = faRight then
  glTranslatef(-length(text) + 1, 0, 0);
if align = faCenter then
  glTranslatef((-length(text) + 1) * 0.5, 0, 0);

a := 0;
for i := 1 to Length(text) do
  if text[i] = '^' then
    begin
      if text[i+1] = 'B' then
        begin
          glTranslatef(-kerning * a, 2.5, 0);
          a := 0;
        end
      else
        with font_colors[StrToInt(text[i+1])] do glColor4f(r, g, b, alpha);
      delete(text, i, 1);
    end
  else
    begin
      inc(a);
      if pos(text[i], font_text) < 1 then text[i] := '@';
      p := pos(text[i], font_text);

     { if gui_mode then
        begin
          glPushMatrix;
          glScalef(1, -screen.aspect, 1);
        end;
      if p > 0 then glCallList(font.list + p);
      if gui_mode then glPopMatrix;   }

      glPushMatrix;
      if gui_mode then glScalef(1/screen.aspect, screen.aspect, 1);
      if flip then glScalef(1, -1, 1);

      if p > 0 then glCallList(font.list + p);
      glPopMatrix;

      glTranslatef(1, 0, 0);
    end;

glPopMatrix;
end;

procedure TFont.TextOutAngle(s, x, y : single; text : string; angle : single);
var
  i, p, a : integer;
begin
text := UpperCase(text);
glEnable(GL_BLEND);
glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
SetTexture(texture);          
glEnable(GL_TEXTURE_2D);

glPushMatrix;
glTranslatef(x, y, 0);
s := s * 0.1;
glScalef(s, s, 1);

a := 0;
for i := 1 to Length(text) do
  if text[i] = '^' then
    begin
      if text[i+1] = 'B' then
        begin
          glTranslatef(-kerning * a, 2.5, 0);
          a := 0;
        end
      else
        with font_colors[StrToInt(text[i+1])] do glColor4f(r, g, b, 1);
      delete(text, i, 1);
    end
  else
    begin
      inc(a);
      if pos(text[i], font_text) < 1 then text[i] := '@';
      p := pos(text[i], font_text);

      glPushMatrix;
      if gui_mode then glScalef(1/screen.aspect, screen.aspect, 1);
      if flip then glScalef(1, -1, 1);

      glRotatef(angle, 0, 0, 1);
      if p > 0 then glCallList(font.list + p);
      glPopMatrix;

      glTranslatef(1, 0, 0);
    end;

glPopMatrix;
end;

end.

