unit e_game;

interface

uses OpenGL, windows, e_models, e_textures, e_types, e_global, e_utils,
     e_particles, e_sound, e_camera, e_font, e_weapons,
     e_player, e_about, inifiles,
     e_console, e_enemies, e_level1, e_level2;

type
  TLetter = class
    active : boolean;
    c      : char;
    pos    : TPoint;
    angle  : single;
    map    : integer;
    target : single;
    size   : single;
  public
    constructor Create(_c : char; _pos : TPoint; _size : single);
    procedure   Render;
    procedure   Update(count : single);
  end;

  TBonus = class
    mode   : byte;
    pos    : TPoint;
    dead   : boolean;
  public
    constructor Create(_mode : byte; _pos : single);
    procedure   Render;
    procedure   Update(count : single);
  end;

  TLabel = class
    text  : string;
    alpha : single;
    alpha_target : single;
  public
    constructor Create(_text : string);
    procedure   Render;
    procedure   Update(count : single);
  end;

  TStageObject = record
    mode   : byte;
    pos    : single;
  end;

  TGame = class
    color          : TColor;
    color_target   : TColor;
    color_dots     : TColor;
    color_dots_target : TColor;

    player         : TPlayer;
    map            : array of TPoint;
    text           : string;
    letters        : array of TLetter;
    bonuses        : array of TBonus;
    bullets        : array of TChuck;
    enemies        : array of TEnemy;

    stage          : array of TStageObject;
    stage_number   : integer;
    stage_progress : integer;

    timer_wait     : single;
    script_waiting : integer;
    script         : integer;

    map_dots_size       : TPoint;
    map_dots_amplitude  : single;
    map_dots_speed      : single;
    map_dots_length     : single;

    game_over      : boolean;

    play_time      : single;

    label_stage1   : TLabel;
    label_stage2   : TLabel;
  public
    constructor Create(_stage_number : integer);

    procedure   StageClear;
    procedure   StageParse(_stage : integer);
    procedure   StageUpdate();

    function    GetMap(_pos : single) : integer;
    function    GetHeight(_map : integer; _pos : single; add_wave : boolean = false) : single;
    function    GetHeightWave(_map : integer; _pos : single; var _wave : single) : single;
    function    GetDelta(_map : integer) : TPoint;
    function    GetMapColor(_map : integer) : TColor;

    function    AddLetter(_c : char; _pos : TPoint; _size : single) : TLetter;
    function    AddBonus(mode : byte; pos : single) : TBonus;
    function    AddBullet(mode : byte; parent : TCreature) : TChuck;
    function    AddEnemy(mode : byte; pos : single) : TEnemy;

    procedure   RenderMap;
    procedure   Render;
    function    Wait(seconds : integer) : boolean;
    procedure   Update(count : single);
    procedure   Save;
    procedure   GameOver(win : boolean);
  end;

  TProfiles = class
    procedure Load;
    procedure Save;
  end;

var
  game : TGame;
  profiles : TProfiles;

implementation

uses e_main, e_gui, e_gui_init;

constructor TLetter.Create(_c : char; _pos : TPoint; _size : single);
begin
active := false;
c     := _c;
pos   := _pos;
size  := _size * 0.1;
map   := -1;
angle := 0;
end;

procedure TLetter.Render;
begin
{glDisable(GL_TEXTURE_2D);

glColor4f(1, 1, 1, 1);
glPushMatrix;
with pos do glTranslatef(x, y, 0);
glScalef(size, size, 1);
glTranslatef(0, 0.5, 0);
glCallList(model_o);
glPopMatrix;

glEnable(GL_TEXTURE_2D); }
SetColor(game.color_dots);
font.TextOutAngle(size * 25, pos.x, pos.y + size * 1.7, c, angle);
end;

procedure TLetter.Update(count : single);
var
i : integer;
begin
if map < 0 then
  begin
    for i := 0 to high(game.map) do
      if game.map[i].x < pos.x then
        if map < i then
          map := i;
  end;
target := game.GetHeight(map, pos.x, true);

if game.player.pos.x + 20 > pos.x then
  active := true;

if not active then
  exit;

if pos.y < 100 then
  begin
    SqrAspire(pos.y, target, count * 0.004);
    Aspire(pos.y, target, count * 0.003);
  end
else
  Aspire(pos.y, target, count * 0.003);

if pos.y < target + 0.3 then
  SqrAspire(angle, PointsAngle(game.map[map + 1], game.map[map]), count * 0.005);

{if pos.y <= target then
  pos.y := target
else
  begin
    speed := speed - count * 0.001;
    pos.y := pos.y + speed * count * 0.001;
  end;  }
end;

constructor TBonus.Create(_mode : byte; _pos : single);
begin
mode := _mode;
pos  := GetPoint(_pos, game.GetHeight(game.GetMap(_pos), _pos) + 5, 0);
dead := false;
end;

procedure TBonus.Render;
var
s : string;
begin
SetTexture(texture_circle);

glPushMatrix;
with pos do glTranslatef(x, y, 0);

glScalef(2.5, 2.5, 1);

SetColor(game.color_dots);
glCallList(model_quad);

SetColor(game.color);
glScalef(0.8, 0.8, 1);
glCallList(model_quad);
glTranslatef(0.2, 0, 0);
glCallList(model_quad);
glPopMatrix;

  case mode of
    1 : s := 'blades';
    2 : s := 'meteors';
    3 : s := 'fireworks';
    4 : s := '+10';
    5 : s := '+50';
    6 : s := 'random';
    7 : s := 'blue';
    8 : s := 'red';
    9 : s := 'laim';
  end;
//font.TextOut(8, pos.x - 0.05, pos.y + 0.2, s);
//font.TextOut(9, pos.x - 0.1, pos.y + 0.2, s);

SetColor(game.color_dots);
font.TextOut(9, pos.x, pos.y + 0.2, s);
end;

procedure TBonus.Update(count : single);
begin
if not CollideCircles(game.player.pos, pos, game.player.radius, 1) then
  exit;

if mode <> 6 then
  begin
    dead := true;
    SoundPlay(sound_bonus);
    particles.AddEmitter(particles_blood, pos, 30, game.color_dots);
  end
else
  SoundPlay(sound_bonus, true);

// 1, 2, 3: weapon
// 4, 5:    score
// color
with game.player do
  case mode of
    1..3   : weapon := mode;
    4      : game.player.GetScore(10);
    5      : game.player.GetScore(50);

    6      : begin
               repeat
               color := GetColor(random, random, random, 1);
               until (color.r > 0.5) or (color.g > 0.5) or (color.b > 0.5);
             end;

    7      : color := GetColor(0.0, 0.5, 0.8, 1);
    8      : color := GetColor(0.5, 0, 0, 1);
    9      : color := GetColor(0.3, 0.7, 0.2, 1);
  end;
end;

constructor TLabel.Create(_text : string);
begin
text := _text;
alpha := -0.1;
alpha_target := -0.1;
end;

procedure TLabel.Render;
begin
if alpha < 0 then
  exit;
  
glColor4f(1, 1, 1, alpha);
font.TextOut(20, camera.eye.x, camera.eye.y, text, faCenter);
end;

procedure TLabel.Update(count : single);
begin
SqrAspire(alpha, alpha_target, count * 0.001);
end;

constructor TGame.Create(_stage_number : integer);
begin
player := TPlayer.Create;

timer_wait  := 0;
script_waiting := -1;
script  := 0;

map_dots_size      := GetPoint(4, 1, 1);
map_dots_amplitude := 0;
map_dots_speed     := 0.5;
map_dots_length    := 1;

label_stage1 := TLabel.Create('stage one');
label_stage2 := TLabel.Create('stage two');
stage_number := _stage_number;
end;

procedure TGame.StageClear();
var
i : integer;
begin
stage_progress := 0;
script := 0;
SetLength(map, 0);

for i := 0 to high(letters) do
  if letters[i] <> nil then
    begin
      letters[i].Free;
      letters[i] := nil;
    end;
SetLength(letters, 0);

for i := 0 to high(enemies) do
  if enemies[i] <> nil then
    begin
      enemies[i].Free;
      enemies[i] := nil;
    end;
SetLength(enemies, 0);

for i := 0 to high(enemies) do
  if enemies[i] <> nil then
    begin
      enemies[i].Free;
      enemies[i] := nil;
    end;
SetLength(enemies, 0);

for i := 0 to high(bullets) do
  if bullets[i] <> nil then
    begin
      bullets[i].Free;
      bullets[i] := nil;
    end;
SetLength(bullets, 0);

for i := 0 to high(bonuses) do
  if bonuses[i] <> nil then
    begin
      bonuses[i].Free;
      bonuses[i] := nil;
    end;
SetLength(bonuses, 0);
end;

procedure TGame.StageParse(_stage : integer);
const
step = 0.5;
var
h, i, size : integer;
x, d : single;
begin
stage_number := _stage;

case stage_number of
  1 : text := e_level1.GetMap;
  2 : text := e_level2.GetMap;
end;

h := 0;
x := 0;
i := 0;
repeat
  inc(i);
  d := 1;
  if (text[i] = '+') or (text[i] = '-') then
    begin
      h := strtoint(text[i + 1]);
      if text[i] = '-' then
        h := -h;

      SetLength(map, length(map) + 1);
      map[high(map)] := GetPoint(x, h, 0);
      i := i + 2;
      d := 0;
    end;
  if (text[i] = '%') then
    begin
      d := d + strtoint(text[i + 1] + text[i + 2]) - 1;
      i := i + 3;
    end;       
  if (text[i] = '!') then
    begin
      d := d + strtoint(text[i + 1] + text[i + 2] + text[i + 3]) - 1;
      i := i + 4;
    end;

  x := x + d * step * 2;
until (i >= length(text));

case stage_number of
  1 : text := e_level1.GetText;
  2 : text := e_level2.GetText;
end;

x := 0;
i := 0;
size := 1;
repeat
  inc(i);
  d := 1;

  if pos(text[i], font_text) >= 0 then
    if (text[i] <> '%') and (text[i] <> '#') then
      AddLetter(text[i], GetPoint(x, 30 + random(40), 0), size * 25 * step);

  if (text[i] = '#') then
    begin
      size := strtoint(text[i + 1]) * 1;
      i := i + 2;
      d := 0;
    end;

  if (text[i] = '%') then
    begin
      d := d + strtoint(text[i + 1] + text[i + 2]) - 1;
      i := i + 3;
    end;

  x := x + d * step * size * 5;
until (i >= length(text));

if stage_number = 2 then
  AddLetter('?', GetPoint(x - d * step * size * 5 + 0.4, 120, 0), size * 1.5 * 25 * step);

case stage_number of
  1 : text := e_level1.GetObjects;
  2 : text := e_level2.GetObjects;
end;

h := 0;
for i := 1 to length(text) do
  if (text[i] = '#') or (text[i] = '*') then
    inc(h);
h := h div 2;
SetLength(stage, h);
stage_progress := 0;

x := 0;
i := 0;
repeat
  inc(i);
  d := 1;

  if (text[i] = '#') then
    begin
      stage[stage_progress].mode := strtoint(text[i + 1]);
      stage[stage_progress].pos := x;
      inc(stage_progress);
      i := i + 2;
      d := 0;
    end;

  if (text[i] = '*') then
    begin
      stage[stage_progress].mode := 10 + strtoint(text[i + 1]);
      stage[stage_progress].pos := x;
      inc(stage_progress);
      i := i + 2;
      d := 0;
    end;

  if (text[i] = '%') then
    begin
      d := d + strtoint(text[i + 1] + text[i + 2]) - 1;
      i := i + 3;
    end;
  if (text[i] = '!') then
    begin
      d := d + strtoint(text[i + 1] + text[i + 2] + text[i + 3]) - 1;
      i := i + 4;
    end;
    
  x := x + d * step * 2;
until (i >= length(text));
stage_progress := 0;
end;

procedure TGame.StageUpdate();
begin
if stage_progress > high(stage) then
  exit;

with stage[stage_progress] do
  if pos < camera.eye.x + game_width then
    begin
      if mode < 10 then
        game.AddBonus(mode, pos);
      if stage[stage_progress].mode >= 10 then
        game.AddEnemy(mode - 10, pos);

      inc(stage_progress);
    end;
end;

function TGame.GetMap(_pos : single) : integer;
var
i : integer;
begin
result := -1;
for i := 0 to high(map) do
  if map[i].x < _pos then
    result := i
  else
    exit;
end;

function TGame.GetHeight(_map : integer; _pos : single; add_wave : boolean = false) : single;
var
d : single;
begin
result := 0;
if (_map < 0) or (_map > high(map) - 1) then
  exit;

d := (_pos - map[_map + 1].x) / (map[_map + 1].x - map[_map].x);
result := (map[_map].y * (1 - d) + map[_map + 1].y * d) - (map[_map].y - map[_map + 1].y);

if add_wave then
  result := result + map_dots_amplitude * sin(time * map_dots_speed + _pos * map_dots_length);
end;

function TGame.GetHeightWave(_map : integer; _pos : single; var _wave : single) : single;
begin
_wave  := map_dots_amplitude * sin(time * map_dots_speed + _pos * map_dots_length);
result := GetHeight(_map, _pos, false);
end;

function TGame.GetDelta(_map : integer) : TPoint;
var
d : single;
begin
result := GetPoint(0, 0, 0);
if (_map < 0) or (_map > high(map) - 1) then
  exit;

d := sqrt(sqr(map[_map + 1].x - map[_map].x) + sqr(map[_map + 1].y - map[_map].y));
result := GetPoint((map[_map].x - map[_map + 1].x) / d, (map[_map].y - map[_map + 1].y) / d, 0);
end;

function TGame.GetMapColor(_map : integer) : TColor;
begin
result := GetColor(abs(sin(_map * 10)), 0.5 + abs(cos(_map * 10)), 0.5, 1);
result := GetColor(0.8, 0.8, 0.8, 1);
end;

function CalcNormal(p1, p2 : TPoint) : TPoint;
begin
result.x :=  p1.y - p2.y;
result.y := -p1.x + p2.x;
result.z := 0;

Normalize(result);
end;

function TGame.AddLetter(_c : char; _pos : TPoint; _size : single) : TLetter;
var
i, max : integer;
begin
result := nil;

if game_over then exit;

i   := 0;
max := high(letters);
while (i <= max) and (letters[i] <> nil) do inc(i);
if i > max then SetLength(letters, i + 1);

if letters[i] <> nil then letters[i].Free;

letters[i] := TLetter.Create(_c, _pos, _size);

result := letters[i];
end;

function TGame.AddBonus(mode : byte; pos : single) : TBonus;
var
i, max : integer;
begin
result := nil;

if game_over then exit;

i   := 0;
max := high(bonuses);
while (i <= max) and (bonuses[i] <> nil) do inc(i);
if i > max then SetLength(bonuses, i + 1);

if bonuses[i] <> nil then bonuses[i].Free;

bonuses[i] := TBonus.Create(mode, pos);

result := bonuses[i];
end;

function TGame.AddBullet(mode : byte; parent : TCreature) : TChuck;
var
i, max : integer;
move : single;
begin
result := nil;

i   := 0;
max := high(bullets);
while (i <= max) and (bullets[i] <> nil) do inc(i);
if i > max then SetLength(bullets, i + 1);

if bullets[i] <> nil then bullets[i].Free;
bullets[i] := WeaponShoot(mode, parent);

result := bullets[i];
end;

function TGame.AddEnemy(mode : byte; pos : single) : TEnemy;
var
i, max : integer;     
begin
result := nil;

if game_over then exit;

i   := 0;
max := high(enemies);
while (i <= max) and (enemies[i] <> nil) do inc(i);
if i > max then SetLength(enemies, i + 1);

if enemies[i] <> nil then enemies[i].Free;

case mode of
  1 : enemies[i] := TEnemy1.Create(pos);
  2 : enemies[i] := TEnemy2.Create(pos);
  3 : enemies[i] := TEnemy3.Create(pos);
  4 : enemies[i] := TEnemyBoss1.Create(pos);
  else enemies[i] := TEnemy1.Create(pos);
end;

result := enemies[i];
end;

procedure TGame.RenderMap;
var
i, j : integer;
s, h : single;
begin
SetColor(color_dots);
glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
SetTexture(texture_circle);
for i := 0 to high(map) do
//for i := player.map - 3 to player.map + 3 do
//  if (i >= 0) and (i <= high(map)) then
  for j := round(map[i].x) * 2 to round(map[i + 1].x) * 2 - 1 do
    if (j > (camera.eye.x - game_width) * 2) and (j < (camera.eye.x + game_width) * 2) then
    begin
      //s := 1 + sqrt(sqr(player.pos.x - j/2) + sqr(player.pos.y - GetHeightWave(i, j / 2, s)));
      //glColor4f(2 / s, 2 / s, 2 / s, 1);
      glPushMatrix;
      h := GetHeightWave(i, j / 2, s);
      glTranslatef(j / 2, h + s, 0);
      s := 0.5 + s * 0.5;
      glScalef(map_dots_size.x * s, map_dots_size.y * s, 1);

      glCallList(model_quad);
      {glBlendFunc(GL_SRC_ALPHA, GL_ONE);
      glCallList(model_quad);
      glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);   }
      
      glPopMatrix;
    end;
end;

procedure TGame.Render;
var
i, j : integer;
s : single;
p, n : TPoint;
begin
glDisable(GL_DEPTH_TEST);

for i := 0 to high(letters) do
  if letters[i] <> nil then
    letters[i].Render;

RenderMap;

for i := 0 to high(enemies) do
  if enemies[i] <> nil then
    enemies[i].Render;

player.Render;

for i := 0 to high(bullets) do
  if bullets[i] <> nil then
    bullets[i].Render;

player.RenderFront;

for i := 0 to high(bonuses) do
  if bonuses[i] <> nil then
    bonuses[i].Render;

particles.RenderBack(false);
particles.RenderFront;

label_stage1.Render;
label_stage2.Render;
end;

function TGame.Wait(seconds : integer) : boolean;
begin
if script = script_waiting then
  begin
    result := timer_wait < 0;
    exit;
  end;
script_waiting := script;
timer_wait := seconds * 1000;
result := false;
end;

procedure TGame.Update(count : single);
var
i : integer;
begin
if gui_disabled then exit;

if timer_wait >= 0 then
  timer_wait := timer_wait - count;

label_stage1.Update(count);
label_stage2.Update(count);

case stage_number of
  1 : e_level1.UpdateScript(count);
  2 : e_level2.UpdateScript(count);
end;

player.Update(count);

StageUpdate();

for i := 0 to high(letters) do
  if letters[i] <> nil then
    letters[i].Update(count);

for i := 0 to high(enemies) do
  if enemies[i] <> nil then
    with enemies[i] do
      if health > 0 then
        Update(count)
      else
        begin
          if particles1 <> nil then particles1.loop := false;
          if particles2 <> nil then particles2.loop := false;
          Free;
          enemies[i] := nil;
        end;

for i := 0 to high(bullets) do
  if bullets[i] <> nil then
    with bullets[i] do
      if not dead then
        Update(count)
      else
        begin
          if particles1 <> nil then particles1.loop := false;
          if particles2 <> nil then particles2.loop := false;
          Free;
          bullets[i] := nil;
        end;

for i := 0 to high(bonuses) do
  if bonuses[i] <> nil then
    with bonuses[i] do
      if not dead then
        Update(count)
      else
        begin
          Free;
          bonuses[i] := nil;
        end;

particles.Update(count);

// colors
SqrAspire(color.r, color_target.r, count * 0.0002);
SqrAspire(color.g, color_target.g, count * 0.0002);
SqrAspire(color.b, color_target.b, count * 0.0002);
clear_color := color;

SqrAspire(color_dots.r, color_dots_target.r, count * 0.0002);
SqrAspire(color_dots.g, color_dots_target.g, count * 0.0002);
SqrAspire(color_dots.b, color_dots_target.b, count * 0.0002);
end;

procedure TGame.Save;
begin

end;

procedure TGame.GameOver(win : boolean);
begin
if game_over then exit;

game_over := true;
game_played := false;

sounds[music1].Stop;
sounds[music2].Stop;

Save;
end;

procedure TProfiles.Load;
begin
end;

procedure TProfiles.Save;
begin
end;

end.
