unit e_gui;

interface

uses windows, e_types, e_utils, opengl, e_textures, e_font, e_screen,
     e_global, e_models, e_sound, e_particles;

const
  gui_name_element_static              = 'static';
  gui_name_element_label               = 'label';
  gui_name_element_button              = 'button';
  gui_name_element_edit                = 'edit';
  gui_name_element_checkbox            = 'checkbox';
  gui_name_element_progressbar         = 'progressbar';
  gui_name_element_memo                = 'memo';
  gui_name_element_list                = 'list';
  gui_name_element_list_servers        = 'list_servers';
  gui_name_element_scroll              = 'scroll';
  gui_name_element_strings             = 'strings';

  gui_button_mode_normal               = 1;
  gui_button_mode_rollover             = 2;
  gui_button_mode_press                = 3;

  gui_checkbox_mode_checked            = 1;
  gui_checkbox_mode_unchecked          = 2;
  gui_checkbox_mode_checked_rollover   = 3;
  gui_checkbox_mode_unchecked_rollover = 4;

  gui_mouse_scroll_null                = 1;
  gui_mouse_scroll_base                = 2;
  gui_mouse_scroll_bar                 = 3;
  gui_mouse_scroll_up                  = 4;
  gui_mouse_scroll_down                = 5;

  gui_scroll_speed                     = 0.08;

  gui_button_group_null                = 1;

  gui_hint_align_top                   = 1;
  gui_hint_align_left                  = 2;

var
  textures_gui_logo            : array[1..3] of word;
  textures_gui_credits         : word;
  textures_gui_factory         : word;
  textures_gui_gradient        : array[1..2] of word;
  textures_gui_button          : array[1..3] of word;
  textures_gui_list            : word;
  textures_gui_scroll          : array[1..7] of word;

type
  PBoolean = ^Boolean;
  PGUIElement = ^TGUIElement;
  TGUIGetHintText = function : string;
  PGUIEngine = ^TGUIEngine;
  PGUIStatic = ^TGUIStatic;
  PGUILabel  = ^TGUILabel;
  PGUIButton = ^TGUIButton;

  TGUIElement = class
    pos, size, text_pos    : TPoint;
    text                   : string;
    mode, old_mode         : word;
    phase                  : single;
    visible                : PBoolean;
    hint_caption           : string;
    hint_text              : string;
    GetHintText            : TGUIGetHintText;
    alpha                  : single;
    text_align             : TFontAlign;
  public
    procedure Initialize(t : string);     virtual;
    function  OnCursor(x, y : single) : boolean; virtual;
    procedure SetMode(m : word);
    procedure OnClick;      virtual;
    procedure OnDown;       virtual;
    procedure OnUp;         virtual;
    procedure OnMouseOn;    virtual;
    procedure OnMouseOver;  virtual;
    procedure Update(count : single); virtual;
    procedure Render;       virtual;
    procedure RenderText;   virtual;
  end;

  TGUIStatic = class(TGUIElement)
    texture      : word;
  public
    procedure Initialize(t : string);     override;
    procedure Render;       override;
    procedure RenderText;   override;
  end;

  TGUILabel = class(TGUIElement)
    target  : pointer;
    string_target : PString;
    text_end : string;
  public
    procedure Initialize(t : string);     override;
    procedure RenderText;   override;
  end;

  TGUIButton = class(TGUIElement)
    textures      : array[1..3] of word;
    group, param  : word;
    active        : pointer;
    pressed       : pointer;
    string_target : PString;
    text_size     : single;
    alpha_pressed : single;
    OnClickEvent  : procedure;
    pos_delta     : single;
  public
    procedure Initialize(t : string); override;
    procedure OnClick;      override;
    procedure OnDown;       override;
    procedure OnUp;         override;
    procedure OnMouseOn;    override;
    procedure OnMouseOver;  override;
    procedure Render;       override;
    procedure RenderText;   override;
    procedure Update(count : single); override;
  end;

  TGUIEdit = class(TGUIElement)
    text_size     : single;
    target        : PString;
  public
    procedure Initialize(t : string);     override;
    procedure OnDown;       override;
    procedure OnMouseOn;    override;
    procedure OnMouseOver;  override;
    procedure Render;       override;
    procedure RenderText;   override;
  end;
  PGUIEdit = ^TGUIEdit;

  TGUICheckbox = class(TGUIElement)
    target : PBoolean;
  public
    procedure Initialize(t : string); override;
    procedure OnClick;      override;
    procedure OnMouseOn;    override;
    procedure OnMouseOver;  override;
    procedure Render;       override;
  end;

  TGUIProgressbar = class(TGUIElement)
    target : PSingle;
  public
    procedure Initialize(t : string); override;
    procedure Update(count : single); override;
    procedure Render;       override;
  end;

  TGUIMemo = class(TGUIElement)
    strings : array of string;
    scroll, scroll_speed   : single;
    max_chars, max_strings : integer;
    string_height : single;
  public
    procedure Initialize(t : string); override;
    procedure AddString(s : string);
    procedure Clear;        virtual;
    procedure Render;       override;
    procedure RenderText;   override;
  end;
  PGUIMemo = ^TGUIMemo;

  TGUIList = class(TGUIMemo)
    selected : integer;
    mouse_pos : TPoint;
  public
    function  OnCursor(x, y : single) : boolean; override;
    procedure OnClick;      override;
    procedure RenderText;   override;
  end;
  PGUIList = ^TGUIList;

  TGUIServerInfo = record
    ip, name : string;
    port, ping : integer;
    games, games_max, players, players_max, mode : byte;
  end;

  TGUIListServers = class(TGUIList)
  public
    servers : array of TGUIServerInfo;
  public
    procedure Initialize(t : string); override;
    procedure OnClick;                override;
    procedure AddServer(_ip, _name : string; _port, _ping : integer;
                        _games, _games_max, _players, _players_max, _mode : byte);
    procedure Clear;                  override;
    procedure RenderText;             override;
  end;

  TGUIString = record
    text   : string;
    pos    : single;
    target : single;
    alpha  : single;
  end;

  TGUIStrings = class(TGUIElement)
    strings : array of TGUIString;
  public
    procedure Initialize(t : string); override;
    procedure AddString(text : string);
    procedure Clear;
    procedure Render;       override;
    procedure RenderText;   override;
    procedure Update(count : single); override;
  end;
  PGUIStrings = ^TGUIStrings;
  
  TGUIScroll = class(TGUIElement)
    target         : PGUIElement;
    mouse_pos      : TPoint;
    mouse, button  : word;
    phases         : array[3..5] of single;
    delta_size     : single;
  public
    constructor Create(t : pointer);
    procedure SetTargetScroll(s : single);
    function  GetDeltaSize : single;
    function  GetBarPos : single;
    function  GetScroll : single;
    function  OnCursor(x, y : single) : boolean; override;
    procedure OnDown;       override;
    procedure OnUp;         override;
    procedure OnMouseOn;    override;
    procedure OnMouseOver;  override;
    procedure Render;       override;
    procedure Update(count : single); override;
  end;

  TGUIHint = record
    text       : string;
    caption    : string;
    visible    : boolean;
    alpha      : single;
    parent_pos : TPoint;
    pos        : TPoint;
    align      : byte;
  end;

  TGUIEngine = class
    elements       : array of TGUIElement;
    hint           : TGUIHint;
    total_elements : integer;
  public
    function  GetGUIMouse(mouse : TPoint) : TPoint;
    procedure Load(file_name : string);
    function  AddElement(element : string; target : pointer) : integer;
    function  OnDown : boolean;
    function  OnUp : boolean;
    procedure OnMove(m : TPoint);
    procedure Render;
    procedure RenderCursor;
    procedure RenderText;
    procedure RenderHintBack;
    procedure RenderHintText;
    procedure RenderLogo;
    procedure Update(count : single);
    procedure Enable;
    procedure Disable(text : string);
    function  OnKey(key : longint) : boolean;
  end;

var
  gui : TGUIEngine;
  gui_menu : TGUIEngine;
  gui_quad : gluint;
  gui_mouse          : TPoint;
  gui_mouse_system   : TPoint;
  focus_edit : TGUIEdit = nil;

  progerss_value : single = 0;
  string_nil     : string = '';
  gui_disabled       : boolean;
  gui_disabled_text  : string;
  gui_build_arrow, gui_build_heavy, gui_build_slow, gui_build_splash : boolean;

var
  boolean_true  : boolean = true;
  boolean_false : boolean = false;

procedure GUIInit;
procedure GUIInitGame;
procedure GUIDeInitGame;
procedure GUIInitMenu;
procedure GUIDeInitMenu;

implementation

uses e_gui_init, e_main;

procedure GUIEventNil;
begin

end;

function GUIGetHintEmpty : string;
begin
result := '';
end;

function PrepareQuad : gluint;
begin
result := glGenLists(1);
glNewList(result, GL_COMPILE);

glBegin(GL_QUADS);
glTexCoord2f(0, 0); glVertex2f(0, 1);
glTexCoord2f(1, 0); glVertex2f(1, 1);
glTexCoord2f(1, 1); glVertex2f(1, 0);
glTexCoord2f(0, 1); glVertex2f(0, 0);
glEnd;
glEndList;
end;

procedure GUIInit;
var
  i : integer;
begin
gui_quad := PrepareQuad;
gui_menu := TGUIEngine.Create;
gui := TGUIEngine.Create;

GUIGenerateMenu;  
gui_mouse := GetPoint(0.5, 0.5, 0);
gui_menu.OnMove(gui_mouse);
end;

procedure GUIInitGame;
begin
if gui <> nil then GUIDeInitGame;

gui := TGUIEngine.Create;
GUIGenerateGame;
gui.OnMove(gui_mouse);
end;

procedure GUIDeInitGame;
begin
gui.Free;
gui := nil;
end;

procedure GUIInitMenu;
begin
if gui_menu <> nil then GUIDeInitMenu;

gui_menu := TGUIEngine.Create;
GUIGenerateMenu;
gui_menu.OnMove(gui_mouse);
end;

procedure GUIDeInitMenu;
begin
gui_menu.Free;
gui_menu := nil;
end;

{-- TGUIElement --}
procedure TGUIElement.Initialize;
begin
inherited Create;
visible      := @boolean_true;
hint_text    := '';
hint_caption := '';
GetHintText  := GUIGetHintEmpty;
alpha        := 0;
text_align   := faLeft;
end;

function TGUIElement.OnCursor(x, y : single) : boolean;
begin

result := (x > pos.x) and (x < pos.x + size.x) and
          (y > pos.y) and (y < pos.y + size.y) and visible^;
end;

procedure TGUIElement.SetMode(m : word);
begin
if mode <> m then
  begin
    phase := 1;
    old_mode := mode;
  end;

mode := m;
end;

procedure TGUIElement.OnClick;
begin
SoundPlay(sound_shoot);
end;

procedure TGUIElement.OnDown;
begin
end;

procedure TGUIElement.OnUp;
begin
OnClick;
end;

procedure TGUIElement.OnMouseOn;
begin
//hint_text := GetHintText;
end;

procedure TGUIElement.OnMouseOver;
begin
end;

procedure TGUIElement.Update(count : single);
var
s : single;
begin
s := count * 0.005;
phase := phase - s;
if phase < 0 then phase := 0;

s := -0.3;
if visible^ then s := 1;
Aspire(alpha, s, count * 0.007 / (2 + alpha));
end;

procedure TGUIElement.Render;
begin

end;

procedure TGUIElement.RenderText;
begin
font.TextOut(0.2, text_pos.x + pos.x, text_pos.y + pos.y, text);
end;

{-- TGUIElement --}

{-- TGUIStatic --}

procedure TGUIStatic.Initialize;
var
i : integer;
begin
inherited;

i := strtoint(t);

{if (i >= 1) and (i <= 6) then
  texture := textures_gui_static[i]; }
end;

procedure TGUIStatic.Render;
begin
inherited;
glPushMatrix;

with pos do glTranslatef(x, y, z);
with size do glScalef(x, y, z);

glColor4f(1, 1, 1, alpha);
SetTexture(texture);
glCallList(gui_quad);

glPopMatrix;     
end;

procedure TGUIStatic.RenderText;
begin
end;

{-- TGUIStatic --}

{-- TGUILabel --}

procedure TGUILabel.Initialize;
begin
inherited;

target := nil;
string_target := @string_nil;
text_end := '';
end;

procedure TGUILabel.RenderText;
var
s1, s2 : string;
begin
//if not visible^ then exit;

if alpha < 0 then exit;
glColor4f(1, 1, 1, alpha);
s1 := string_target^;
if target <> nil then s2 := inttostr(integer(target^));
font.TextOut(size.x, pos.x, pos.y, text + s1 + s2 + text_end, text_align, alpha);
end;

{-- TGUILabel --}

{-- TGUIButton --}

procedure TGUIButton.Initialize(t : string);
var
i : integer;
begin
inherited;

group := gui_button_group_null;

text_size := 0.2;
OnClickEvent := GUIEventNil;
active  := @boolean_true;
pressed := @boolean_false;
string_target := @string_nil;
mode     := gui_button_mode_normal;
old_mode := mode;
//visible := @menu_main;

alpha_pressed := -0.1;

for i := 1 to 3 do
  textures[i] := textures_gui_button[i];
{
if t = 'pause'      then OnClickEvent := GUIEventPause;
if t = 'minimize'   then OnClickEvent := GUIEventMinimize;
if t = 'close'      then OnClickEvent := GUIEventClose;
if t = 'new'        then OnClickEvent := GUIEventNew;
if t = 'continue'   then
  begin
    OnClickEvent := GUIEventContinue;
    active := @game_played;
  end;
if t = 'highscores' then OnClickEvent := GUIEventHighscores;

if t = 'menu' then
  begin
    OnClickEvent := GUIEventMenu;
    visible := @menu_scores;
  end;        }
end;

procedure TGUIButton.OnClick;
begin
inherited;

if boolean(active^) then OnClickEvent;
end;

procedure TGUIButton.OnDown;
begin
inherited;

if boolean(active^) then
  SetMode(gui_button_mode_press);
end;

procedure TGUIButton.OnUp;
begin
inherited;

if boolean(active^) then
  if old_mode <> gui_button_mode_rollover then
    SetMode(gui_button_mode_normal)
  else
    SetMode(gui_button_mode_rollover);
end;

procedure TGUIButton.OnMouseOn;
begin
inherited;

if mode = gui_button_mode_normal then
  if boolean(active^) and (focus_edit = nil) then
    SoundPlay(sound_shoot);

if mode <> gui_button_mode_press then
  SetMode(gui_button_mode_rollover);
end;

procedure TGUIButton.OnMouseOver;
begin
inherited;

SetMode(gui_button_mode_normal);
end;

procedure TGUIButton.Render;
var
s : single;
begin
inherited;
                               
glPushMatrix;

with pos do glTranslatef(x + pos_delta, y, z);
with size do glScalef(x, y, z);

  if alpha_pressed > 0 then
    begin
      glColor4f(1, 1, 1, alpha * alpha_pressed);
      SetTexture(textures[gui_button_mode_press]);
      glCallList(gui_quad);
    end;

  if phase <= 0 then
    begin
      glColor4f(1, 1, 1, 1 * alpha);
      SetTexture(textures[mode]);
      glCallList(gui_quad);
    end
  else
    begin
      glColor4f(1, 1, 1, (1 - phase) * alpha);
      SetTexture(textures[mode]);
      glCallList(gui_quad);

      glColor4f(1, 1, 1, 0 + phase * alpha);
      SetTexture(textures[old_mode]);
      glCallList(gui_quad);
    end;

glPopMatrix;     
end;

procedure TGUIButton.RenderText;
var
s : string;
i : integer;
t : single;
begin
s := text + string_target^;

t := 0.85 + alpha_pressed * 0.15;
glColor4f(t, t, t, alpha);

t := text_size + alpha_pressed * 0.07;
font.TextOut(t, text_pos.x + pos.x + pos_delta, text_pos.y + pos.y, s, text_align);
end;

procedure TGUIButton.Update(count : single);
var
s : single;
begin
inherited Update(count);

s := -0.1;
if (not boolean(active^)) and boolean(visible^) then s := 1;

pos_delta := sign(pos.x) * (1 - alpha) * 0.02;

SqrAspire(alpha_pressed, s, count * 0.0025);

if (focus_edit <> nil) and boolean(visible^) then
  Aspire(alpha, 0.2, count * 0.003);
end;

{-- TGUIButton --}

{-- TGUIEdit --}

procedure TGUIEdit.Initialize;
begin
inherited;

visible := @boolean_true;
target := nil;
text_align := faLeft;
text_size := 0.2;
end;

procedure TGUIEdit.OnDown;
begin
inherited;

focus_edit := self;
end;

procedure TGUIEdit.OnMouseOn;
begin
inherited;

if mode = gui_button_mode_normal then
  if (focus_edit = nil) then
    SoundPlay(sound_shoot);

if mode <> gui_button_mode_press then
  SetMode(gui_button_mode_rollover);
end;

procedure TGUIEdit.OnMouseOver;
begin
inherited;

SetMode(gui_button_mode_normal);
end;

procedure TGUIEdit.Render;
begin
inherited;

glPushMatrix;

with pos do glTranslatef(x, y, z);
with size do glScalef(x, y, z);

if focus_edit = self then
  begin
    glColor4f(1, 1, 1, alpha);
    SetTexture(textures_gui_button[gui_button_mode_press]);
    glCallList(gui_quad);
  end
else
  begin
    glColor4f(1, 1, 1, (1 - phase) * alpha);
    SetTexture(textures_gui_button[mode]);
    //if mode <> gui_button_mode_normal then
    glCallList(gui_quad);

    glColor4f(1, 1, 1, 0 + phase * alpha);
    SetTexture(textures_gui_button[old_mode]);
    //if mode <> gui_button_mode_normal then
    glCallList(gui_quad);
  end;

glPopMatrix;
end;

procedure TGUIEdit.RenderText;
var
i : integer;
s : string;
begin
s := text;

if target <> nil then
  s := target^;

glColor4f(1, 1, 1, alpha);
if focus_edit = self then
  font.TextOut(text_size, text_pos.x + pos.x, text_pos.y + pos.y, s + '|', text_align)
else
  font.TextOut(text_size, text_pos.x + pos.x, text_pos.y + pos.y, s, text_align);
end;

{-- TGUIEdit --}

{-- TGUICheckbox --}

procedure TGUICheckbox.Initialize(t : string);
begin
inherited;

target := @boolean_true;
end;

procedure TGUICheckbox.OnClick;
begin
inherited;
                 
target^ := not target^;

if target^ then SetMode(gui_checkbox_mode_checked_rollover)
else            SetMode(gui_checkbox_mode_unchecked_rollover);
end;

procedure TGUICheckbox.OnMouseOn;
begin
inherited;

if target^ then SetMode(gui_checkbox_mode_checked_rollover)
else            SetMode(gui_checkbox_mode_unchecked_rollover);
end;

procedure TGUICheckbox.OnMouseOver;
begin
inherited;

if target^ then SetMode(gui_checkbox_mode_checked)
else            SetMode(gui_checkbox_mode_unchecked);
end;

procedure TGUICheckbox.Render;
begin
inherited;
      {
glPushMatrix;
with pos do glTranslatef(x, y, z);
with size do glScalef(x, y, z);

glColor4f(1, 1, 1, 1 - phase);
SetTexture(textures_gui_checkbox[mode]);
glCallList(gui_quad);

glColor4f(1, 1, 1, phase);
SetTexture(textures_gui_checkbox[old_mode]);
glCallList(gui_quad);
                                       
glPopMatrix;     }
end;

{-- TGUICheckbox --}

{-- TGUIProgressbar --}

procedure TGUIProgressbar.Initialize(t : string);
begin
inherited;

target := @progerss_value;
end;

procedure TGUIProgressbar.Update(count : single);
begin
inherited;

target^ := target^ + count * 0.01;
if target^ > 100 then target^ := 0;
end;

procedure TGUIProgressbar.Render;
var
s, w1, w2 : single;
begin
inherited;
  {
glPushMatrix;

with pos do glTranslatef(x, y, z);
with size do glScalef(x, y, z);

glColor4f(1, 1, 1, 1);
SetTexture(textures_gui_progressbar[1]);
glCallList(gui_quad);

w1 := 16;//Heart.TexMan.GetSize(textures_gui_progressbar[1])[0];
w2 := 16;//Heart.TexMan.GetSize(textures_gui_progressbar[2])[0];
s := round(target^ / 100 * w1 / w2) * w2 / w1;
w2 := w1 / w2;  
SetTexture(textures_gui_progressbar[2]);
glBegin(GL_QUADS);
glTexCoord2f(0, 0);      glVertex2f(0, 1);
glTexCoord2f(s * w2, 0); glVertex2f(s, 1);
glTexCoord2f(s * w2, 1); glVertex2f(s, 0);
glTexCoord2f(0, 1);      glVertex2f(0, 0);
glEnd;

glPopMatrix;  }
end;

{-- TGUIProgressbar --}

{-- TGUIMemo --}

procedure TGUIMemo.Initialize(t : string);
var
i, a, m, c : integer;
s : string;
begin
inherited;

max_chars   := 50;
max_strings := 16;

a := 0;
m := 0;
s := '';
c := 0;

scroll := 100;
scroll_speed := 0;

string_height := 0.031;
end;

procedure TGUIMemo.AddString(s : string);
begin
inherited;

SetLength(strings, high(strings) + 2);
strings[high(strings)] := s;

scroll_speed := 0;
if high(strings) > max_strings then scroll_speed := 15 / high(strings);
scroll := 100;
end;

procedure TGUIMemo.Clear;
begin
inherited;

SetLength(strings, 0);
scroll_speed := 0;
scroll := 100;
end;

procedure TGUIMemo.Render;
begin
inherited;

glPushMatrix;

with pos do glTranslatef(x, y, z);
with size do glScalef(x, y, z);

glColor4f(1, 1, 1, alpha);
SetTexture(textures_gui_list);
glCallList(gui_quad);

glPopMatrix;
end;

procedure TGUIMemo.RenderText;
var
i, n, m : integer;
begin
m := high(strings);
n := round(scroll / 100 * (m - max_strings));
if n < 0 then n := 0;

glColor4f(1, 1, 1, alpha);

for i := n to n + max_strings do
  if i <= m then
    font.TextOut(0.15, pos.x + text_pos.x , pos.y + text_pos.y + (i - n) * string_height, strings[i]);
end;

{-- TGUIMemo --}

{-- TGUIStrings --}

procedure TGUIStrings.Initialize(t : string);
begin
inherited;
end;

procedure TGUIStrings.AddString(text : string);
var
i, n : integer;
begin
inherited;

n := -1;
for i := 0 to high(strings) do
  if strings[i].alpha > 0 then
    strings[i].target := strings[i].target - 0.035
  else
    if n < 0 then n := i;

if n < 0 then
  begin
    SetLength(strings, high(strings) + 2);
    n := high(strings);
  end;


strings[n].text   := text;
strings[n].pos    := 0;
strings[n].target := strings[n].pos;
strings[n].alpha  := 4;
end;

procedure TGUIStrings.Clear;
begin
inherited;

SetLength(strings, 0);
end;

procedure TGUIStrings.Render;
begin
inherited Render;

{glPushMatrix;

with pos do glTranslatef(x, y, z);
with size do glScalef(x, y, z);

glColor4f(1, 1, 1, alpha);
SetTexture(textures_gui_list);
glCallList(gui_quad);

glPopMatrix;}
end;

procedure TGUIStrings.RenderText;
var
i : integer;
begin
for i := 0 to high(strings) do
  if strings[i].alpha > 0 then
    begin
      glColor4f(1, 1, 1, strings[i].alpha);
      font.TextOut(0.15, pos.x + text_pos.x , pos.y + size.y - text_pos.y + strings[i].pos, strings[i].text);
    end;
end;

procedure TGUIStrings.Update(count : single);
var
i : integer;
begin
inherited Update(count);

for i := 0 to high(strings) do
  if strings[i].alpha > 0 then
    with strings[i] do
      begin
        if alpha < 1 then
          target := target - count * 0.00005;
        SqrAspire(pos, target, count * 0.002);
        alpha := alpha - count * 0.001;
      end;
end;

{-- TGUIStrings --}

{-- TGUIList --}

function TGUIList.OnCursor(x, y : single) : boolean;
begin
inherited OnCursor(x, y);

mouse_pos := GetPoint(x, y, 0);

result := (x > pos.x) and (x < pos.x + size.x) and
          (y > pos.y) and (y < pos.y + size.y) and visible^;
end;

procedure TGUIList.OnClick;
var
n, m : integer;
begin
inherited OnClick;

m := high(strings);
n := round(scroll / 100 * (m - max_strings));
if n < 0 then n := 0;

if mouse_pos.x < 1 - 0.06 then
  begin
    selected := round(n + (mouse_pos.y - pos.y - text_pos.y) / string_height);
    if selected < 0 then selected := 0;  
    if selected > high(strings) then selected := high(strings);    
  end;
end;

procedure TGUIList.RenderText;
var
i, n, m : integer;
begin
m := high(strings);
n := round(scroll / 100 * (m - max_strings));
if n < 0 then n := 0;

for i := n to n + max_strings do
  if i <= m then
    begin
      glColor4f(1, 1, 1, 0.5 * alpha);
      if i = selected then glColor4f(1, 1, 1, alpha);       
      font.TextOut(0.15, pos.x + text_pos.x , pos.y + text_pos.y + (i - n) * string_height, strings[i]);
    end;
end;

{-- TGUIList --}

{-- TGUIListServers --}

procedure TGUIListServers.Initialize(t : string);
begin
inherited Initialize(t);

max_strings := 4;
string_height := 0.11;
end;

procedure TGUIListServers.OnClick;
var
n, m : integer;
begin
inherited OnClick;

m := high(servers);
n := round(scroll / 100 * (m - max_strings));
if n < 0 then n := 0;

if mouse_pos.x < 1 - 0.06 then
  begin
    selected := round(n + (mouse_pos.y - pos.y - text_pos.y) / string_height);
    if selected < 0 then selected := 0;  
    if selected > high(servers) then selected := high(servers);    
  end;
end;

procedure TGUIListServers.AddServer;
begin
SetLength(servers, high(servers) + 2);

scroll_speed := 0;
if high(servers) > max_strings then scroll_speed := 15 / high(servers);
//scroll := 0;

with servers[high(servers)] do
  begin
    ip            := _ip;
    name          := _name;
    port          := _port;
    ping          := _ping;
    games         := _games;
    games_max     := _games_max;
    players       := _players;
    players_max   := _players_max;
    mode          := _mode;
  end;
end;

procedure TGUIListServers.Clear;
begin
inherited;

SetLength(servers, 0);
end;

procedure TGUIListServers.RenderText;

  function AddZero(number : integer) : string;
  begin
    result := inttostr(number);
    if number < 10 then
      result := '0' + result;
  end;

var
i, n, m : integer;
x, y : single;
begin
m := high(servers);
n := round(scroll / 100 * (m - max_strings));
if n < 0 then n := 0;
end;

{-- TGUIListServers --}

{-- TGUIScroll --}

constructor TGUIScroll.Create(t : pointer);
var
i : integer;
begin
inherited Create;           

target := t;
size := GetPoint(0.05 / screen.aspect, 0.05 * screen.aspect, 1);

pos.x := 1 - size.x * 1.5;
pos.y := target^.pos.y;
pos.z := 0;

visible := target^.visible;
alpha := 0;

for i := 3 to 5 do phases[i] := 0;
mouse := gui_mouse_scroll_null;

delta_size := GetDeltaSize;
end;

procedure TGUIScroll.SetTargetScroll(s : single);
begin
if s < 0 then s := 0; if s > 100 then s := 100;
                           
TGUIMemo(target^).scroll := s;
end;

function TGUIScroll.GetDeltaSize : single;
begin
result := (target^.size.y - size.y * 3) / 100;
end;

function TGUIScroll.GetBarPos : single;
begin
result := size.y + TGUIMemo(target^).scroll * delta_size;
end;

function TGUIScroll.GetScroll : single;
begin
result := (mouse_pos.y - pos.y - size.y * 1.5) / delta_size;
end;

function TGUIScroll.OnCursor(x, y : single) : boolean;
var
s : single;
old_mouse : word;

begin
result := false;
old_mouse := mouse;

mouse_pos := GetPoint(x, y, 0);

mouse := gui_mouse_scroll_null;
if (x < pos.x) or (x > pos.x + size.x) or
   (y < pos.y) or (y > pos.y + target^.size.y) then exit;

result := true;
mouse := gui_mouse_scroll_base;
              
s := pos.y + GetBarPos;
if (y >= s) and (y <= s + size.y) then       mouse := gui_mouse_scroll_bar;
if y <= pos.y + size.y then                  mouse := gui_mouse_scroll_up;
if y >= pos.y + target^.size.y - size.y then mouse := gui_mouse_scroll_down;  
end;

procedure TGUIScroll.OnDown;
begin
inherited;

button := mouse;
if button <> gui_mouse_scroll_base then exit;
SetTargetScroll(GetScroll);
button := gui_mouse_scroll_null;
end;

procedure TGUIScroll.OnUp;
begin
inherited;

button := gui_mouse_scroll_null;
end;

procedure TGUIScroll.OnMouseOn;
begin
inherited;

if not mouse_down then button := gui_mouse_scroll_null;
if button = gui_mouse_scroll_bar then SetTargetScroll(GetScroll);
end;

procedure TGUIScroll.OnMouseOver;
begin
inherited;

button := gui_mouse_scroll_null;
end;

procedure TGUIScroll.Render;
begin
inherited;

glPushMatrix;

with pos do glTranslatef(x, y, z);
with size do glScalef(x, 1, z);

glColor4f(1, 1, 1, alpha);

glPushMatrix;
glScalef(1, target^.size.y, 1);
SetTexture(textures_gui_scroll[1]);

glCallList(gui_quad);
glPopMatrix;

glPushMatrix;
glTranslatef(0, GetBarPos, 0);
glScalef(1, size.y, 1);             

if phases[3] < 1 then
  begin
    glColor4f(1, 1, 1, (1 - phases[3]) * alpha);
    SetTexture(textures_gui_scroll[2]);
    glCallList(gui_quad);
  end;

if phases[3] > 0 then
  begin
    glColor4f(1, 1, 1, phases[3] * alpha);
    SetTexture(textures_gui_scroll[5]);
    glCallList(gui_quad);
  end;

glPopMatrix;

glScalef(1, size.y, 1);

if phases[4] < 1 then
  begin
    glColor4f(1, 1, 1, (1 - phases[4]) * alpha);
    SetTexture(textures_gui_scroll[3]);
    glCallList(gui_quad);
  end;

if phases[4] > 0 then
  begin
    glColor4f(1, 1, 1, phases[4] * alpha);
    SetTexture(textures_gui_scroll[6]);
    glCallList(gui_quad);
  end;

glTranslatef(0, (target^.size.y - size.y) / size.y, 0);

if phases[5] < 1 then
  begin
    glColor4f(1, 1, 1, (1 - phases[5]) * alpha);
    SetTexture(textures_gui_scroll[4]);
    glCallList(gui_quad);
  end;

if phases[5] > 0 then
  begin
    glColor4f(1, 1, 1, phases[5] * alpha);
    SetTexture(textures_gui_scroll[7]);
    glCallList(gui_quad);
  end;

glPopMatrix;       
end;

procedure TGUIScroll.Update(count : single);
var
s : single;
i : integer;
begin
inherited Update(count);

s := count * 0.007;
for i := 3 to 5 do
  if i = mouse then
    begin
      phases[i] := phases[i] + s;
      if phases[i] > 1 then phases[i] := 1;
    end
  else
    begin
      phases[i] := phases[i] - s;
      if phases[i] < 0 then phases[i] := 0;
    end;

s := 0;
if button = gui_mouse_scroll_up   then s := -1;
if button = gui_mouse_scroll_down then s := 1;
SetTargetScroll(TGUIMemo(target^).scroll + s * count * gui_scroll_speed * TGUIMemo(target^).scroll_speed);
end;

{-- TGUIScroll --}

{-- TGUIEngine --}

function TGUIEngine.GetGUIMouse(mouse : TPoint) : TPoint;
begin
result.x := mouse.x / screen.width;
result.y := mouse.y / screen.height * screen.aspect;
//result.y := result.y * Heart.RenderAspect;
end;

procedure TGUIEngine.Load(file_name : string);
var
f : textfile;
i : integer;
e, t, s : string;
n1, n2, n3 : single;
begin
assignfile(f, file_name);
reset(f);

SetLength(elements, 0);
total_elements := 0;

while not eof(f) do
  begin
    readln(f, e);
    readln(f, t);
    i := AddElement(e, nil);
    with elements[i] do
      begin
        readln(f, n1, n2);
        pos                 := GetPoint(n1 / 1024, n2 / 768 * screen.aspect, 0);

        readln(f, n1, n2);
        size                := GetPoint(n1 / 1024, n2 / 768 * screen.aspect, 1);

        readln(f, n1, n2);
        text_pos            := GetPoint(n1 / 1024, n2 / 1024, 0);

        readln(f, s);
        text                := s;

        readln(f, s);
      end;
    if e = gui_name_element_memo then AddElement(gui_name_element_scroll, @elements[i]);
    elements[i].Initialize(t);
  end;  
closefile(f);
end;

function TGUIEngine.AddElement(element : string; target : pointer) : integer;
var
i : integer;
begin
inc(total_elements);
SetLength(elements, total_elements); 
i := high(elements);
result := i;

if element = gui_name_element_static         then elements[i] := TGUIStatic.Create;
if element = gui_name_element_label          then elements[i] := TGUILabel.Create;
if element = gui_name_element_button         then elements[i] := TGUIButton.Create;
if element = gui_name_element_edit           then elements[i] := TGUIEdit.Create;
if element = gui_name_element_checkbox       then elements[i] := TGUICheckbox.Create;
if element = gui_name_element_progressbar    then elements[i] := TGUIProgressbar.Create;
if element = gui_name_element_memo           then elements[i] := TGUIMemo.Create;
if element = gui_name_element_list           then elements[i] := TGUIList.Create;
if element = gui_name_element_list_servers   then elements[i] := TGUIListServers.Create;
if element = gui_name_element_scroll         then elements[i] := TGUIScroll.Create(target);
if element = gui_name_element_strings        then elements[i] := TGUIStrings.Create;
end;

function TGUIEngine.OnDown : boolean;
var
i : integer;
begin
result := false;

if gui_disabled then
  exit;

for i := 0 to high(elements) do
  if elements[i].OnCursor(gui_mouse.x, gui_mouse.y) then
    if (not (elements[i] is TGUILabel)) and (not (elements[i] is TGUIStatic)) and (not (elements[i] is TGUIStrings)) then
      begin
        focus_edit := nil;
        result := true;
        elements[i].OnDown;
      end
    else
      if elements[i] is TGUIStatic then
        if menu_active then
          easter := true;
end;

function TGUIEngine.OnUp : boolean;
var
i : integer;
begin
result := false;

if gui_disabled then
  exit;

for i := 0 to high(elements) do
  if elements[i].OnCursor(gui_mouse.x, gui_mouse.y) then
    if not (elements[i] is TGUILabel) then
      begin
        result := true;
        elements[i].OnUp;
        OnMove(gui_mouse_system);
        exit;
      end;
end;

procedure TGUIEngine.OnMove(m : TPoint);
var
i : integer;
begin
gui_mouse_system := m;
gui_mouse := GetGUIMouse(m);
if focus_edit = nil then hint.visible := false;

if gui_disabled then
  exit;

for i := 0 to high(elements) do
  if elements[i].OnCursor(gui_mouse.x, gui_mouse.y) then
    begin
      elements[i].OnMouseOn;
      {if focus_edit = nil then
        begin
          hint.visible := true;
          hint.text := elements[i].hint_text;
          hint.caption := elements[i].hint_caption;
          hint.parent_pos := elements[i].pos;
          hint.align := gui_hint_align_left;
          if elements[i] is TGUIButtonSmall then hint.align := gui_hint_align_top;
        end;  }
    end
  else
    elements[i].OnMouseOver;
end;

procedure TGUIEngine.Render;
var
i : integer;
begin
glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
glEnable(GL_BLEND);
glEnable(GL_TEXTURE_2D);

if menu_active then
  RenderLogo;

for i := 0 to high(elements) do
  if {elements[i].visible^} elements[i].alpha > 0 then
    elements[i].Render;

RenderText;
if menu_active or game_pause then
  RenderCursor;

if gui_disabled then
  begin
    glDisable(GL_TEXTURE_2D);
    glPushMatrix;
    glTranslatef(0.5, 0.5 * screen.aspect, 0);
    glScalef(1.5, 1.5 * screen.aspect, 1);
    glColor4f(0, 0, 0, 0.8);
    glCallList(model_quad);
    glPopMatrix;
    glEnable(GL_TEXTURE_2D);

    glColor4f(1, 1, 1, 1);
    font.gui_mode := true;
    font.flip := true;
    font.TextOut(0.4, 0.3, 0.5 * screen.aspect, gui_disabled_text);
    font.flip := false;
  end;

glDisable(GL_TEXTURE_2D);
glDisable(GL_BLEND);
end;

procedure TGUIEngine.RenderCursor;
begin
glPushMatrix;
with gui_mouse do glTranslatef(x, y, 0);
glScalef(0.05 / screen.aspect, 0.05 * screen.aspect, 1);
glTranslatef(-0.5, -0.5, 0);
SetTexture(texture_circle);
glColor4f(1, 1, 1, 1);
glCallList(gui_quad);
glPopMatrix;  
end;

procedure TGUIEngine.RenderText;
var
i : integer;
begin
font.gui_mode := true;
font.flip := true;
glColor4f(1, 1, 1, 1);
for i := 0 to high(elements) do
  if {elements[i].visible^} elements[i].alpha > 0 then
    elements[i].RenderText;

RenderHintBack;
RenderHintText;

font.flip := false;
end;

procedure TGUIEngine.RenderHintBack;
var
i : integer;
const
size_x = 0.32;
size_y = 0.255;
begin
if (hint.text = '') and (focus_edit = nil) then exit;

if hint.align = gui_hint_align_top then
  hint.pos := GetPoint(hint.parent_pos.x, hint.parent_pos.y - size_y * screen.aspect, 0);
if hint.align = gui_hint_align_left then
  hint.pos := GetPoint(hint.parent_pos.x - size_x / screen.aspect, hint.parent_pos.y, 0);

if hint.pos.x < 0 then hint.pos.x := 0;
if hint.pos.x > 1 - size_x / screen.aspect then hint.pos.x := 1 - size_x / screen.aspect;
if hint.pos.y < 0 then hint.pos.y := 0;
if hint.pos.y > screen.aspect - size_y * screen.aspect then hint.pos.y := screen.aspect - size_y * screen.aspect;

glPushMatrix;
with hint.pos do glTranslatef(x, y, 0);
glScalef(size_x / screen.aspect, size_y * screen.aspect, 1);
SetTexture(textures_gui_button[1]);
glColor4f(1, 1, 1, hint.alpha);
glCallList(gui_quad);
glPopMatrix;  
end;

procedure TGUIEngine.RenderHintText;
var
i : integer;
begin
if (hint.text = '') and (focus_edit = nil) then exit;

glColor4f(255/255, 200/255, 31/255, hint.alpha);
font.TextOut(0.23, hint.pos.x + 0.025, hint.pos.y + 0.045, hint.caption);

glColor4f(1, 1, 1, hint.alpha);
font.TextOut(0.14, hint.pos.x + 0.025, hint.pos.y + 0.1, hint.text);
end;

procedure TGUIEngine.RenderLogo;
var
i : integer;
const
size = 600;
size1_x = 168 / size;
size1_y = 169 / size;
size2_x = 182 / size;
size2_y = 183 / size;
size3_x = 456 / size;
size3_y = 151 / size;
begin
glColor4f(0, 0, 0, 1);
if easter then glColor4f(1, 1, 0, 1);

glPushMatrix;
glTranslatef(0.6 / screen.aspect, 0.15 * screen.aspect, 0);
glScalef(size3_x / screen.aspect, -size3_y * screen.aspect, 1);
SetTexture(textures_gui_logo[3]);
glCallList(model_quad);
glPopMatrix;

glPushMatrix;
glTranslatef(0.07 / screen.aspect, 0.13 * screen.aspect, 0);
glScalef(size2_x / screen.aspect, -size2_y * screen.aspect, 1);
glRotatef(-time * 0.03, 0, 0, 1);
SetTexture(textures_gui_logo[2]);
glCallList(model_quad);
glPopMatrix;

glPushMatrix;
glTranslatef(0.07 / screen.aspect, 0.13 * screen.aspect, 0);
glScalef(size1_x / screen.aspect, -size1_y * screen.aspect, 1);
SetTexture(textures_gui_logo[1]);
glCallList(model_quad);
glPopMatrix;
end;

procedure TGUIEngine.Update(count : single);
var
i : integer;
begin
for i := 0 to high(elements) do
  elements[i].Update(count);
                      {
if focus_edit <> nil then
  begin
    hint.visible := true;
    hint.text := TGUIEdit(focus_edit).hint_text;
    hint.caption := TGUIEdit(focus_edit).hint_caption;
    hint.parent_pos := TGUIEdit(focus_edit).pos;
    hint.align := gui_hint_align_left;
  end;

i := 0;
if hint.visible then i := 1;
Aspire(hint.alpha, i, count * 0.004);      }
end;

procedure TGUIEngine.Enable;
begin
gui_disabled := false;
end;

procedure TGUIEngine.Disable(text : string);
begin
gui_disabled := true;
gui_disabled_text := text;
end;

function TGUIEngine.OnKey(key : longint) : boolean;
begin
result := false;

if focus_edit = nil then
  exit;

with focus_edit do
  begin
    if system.pos(UpperCase(chr(key)), font_text) > 0 then
      if length(text) < 15 then
        text := text + chr(key);

    if key = vk_back then
      if length(text) > 0 then
        delete(text, length(text), 1);


    if (target <> nil) then
      PString(target)^ := text;

    if key = vk_return then
      focus_edit := nil;
  end;

result := true;
end;
{-- TGUIEngine --}

end.
