unit e_main;

interface

uses Windows, OpenGL, e_render, e_global, e_sound, e_types, e_models, e_particles,
     e_utils, e_screen, e_game, e_textures, e_font, e_gui, e_gui_init, e_logo, e_camera, 
     e_console, e_gui_utils;

procedure GameFinish;     
function  GameStart(number : integer = 1) : boolean;
procedure Initialize;
procedure Deinitialize;
procedure Update(count : single);
procedure Render;

implementation

procedure ResetTime;
begin
count_fps := 0;
count_last := GetTickCount;
count_sec := count_last;
time := 0;
time_scale := 1;
end;

procedure GameFinish;
begin
GUIDeInitGame;
game.Save;
game_started := false;
game.Free;
particles.Free;
Console.Write('Game finished');
//GameStart;
end;

function GameStart(number : integer = 1) : boolean;
begin
result := false;

if game_started then GameFinish;

particles    := TParticlesEngine.Create;
game         := TGame.Create(number);

GUIInitGame;

ResetTime;
game_started := true;

result := true;

Console.Write('Game started');
end;

procedure Initialize;
begin
menu_active := true;
ModelsInitialize;
Console.Write('Models');
TexturesInitialize;
Console.Write('Textures');
SoundInitialize;
Console.Write('Sound');
LogoInitialize;
Console.Write('Intro');
font := TFont.Create;
Console.Write('Font');
clear_color := GetColorB(234, 227, 115, 255);
clear_color := GetColorB(128, 128, 128, 255);
//clear_color := GetColorB(0, 0, 0, 255);

profiles := TProfiles.Create;
profiles.Load;
Console.Write('Profiles');

GameStart;

GUIInit;
GUIEventMenu;
GUIGenerateGame;
Console.Write('Interface');
{if menu_first then
  begin
    menu_main   := false;
    menu_scores := true;
  end;            }    

menu_active := true;
menu_active := false;
end;

procedure Deinitialize;
begin
Console.Write('');
Console.Write('Deinitialization');
profiles.Save;
GameFinish;
SoundDeinitialize;
end;

procedure Update(count : single);
var
a : integer;
begin
console.Update(count);
//count := count * time_scale;
time := time + count;

texture_time := time * 0.0005;

SoundUpdate(count);


if logo_count < logo_time then
  begin
    LogoUpdate(count);
    exit;
  end;

if menu_active then
  begin
    Aspire(camera_menu_pos.y, -5, count * 0.001);
    gui_menu.Update(count);
  end;

if (not menu_active) or (game.game_over) then
  //if game_started then
    begin
      if (not game_pause) then
        begin
          Aspire(camera_offset_size, 0, count * 0.008);
          if camera_offset_size > 3.5 then camera_offset_size := 3.5;
          
          camera_offset := GetPoint(GetRandom * camera_offset_size, GetRandom * camera_offset_size, 0);
          game.Update(count);

          if game.game_over then
            begin
              a := game.stage_number;
              GameStart(a);
            end;
        end;
      gui.Update(count);
    end;
end;

procedure CountFPS;
begin
count_new := GetTickCount;

inc(count_fps);
if (count_new - count_sec) >= 1000 then
  begin
    count_sec := count_sec + 1000;
    fps := count_fps;
    count_fps := 0;
  end;
end;

procedure RenderStart;
begin
CountFPS;

glMatrixMode(GL_PROJECTION);
glLoadIdentity;

glViewPort(0, 0, screen.width, screen.height);

with clear_color do glClearColor(r, g, b, a);
glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
end;

procedure RenderFinish;
begin
glMatrixMode(GL_MODELVIEW);
glLoadIdentity;
SwapBuffers(DC);
count_last := count_new;
end;

procedure RenderMenu;
begin
glMatrixMode(GL_PROJECTION);
glLoadIdentity;
glOrtho(0, 1, screen.aspect, 0, -1, 1);
glMatrixMode(GL_MODELVIEW);
glLoadIdentity;

gui_menu.Render;
end;

procedure Render;
begin
RenderStart;

if logo_count < logo_time then
  LogoRender
else  
  begin
    e_render.Render;
    if menu_active then
      RenderMenu;
  end;

Console.Render;

RenderFinish;
end;

end.
