unit e_player;

interface

uses OpenGL, Windows, e_types, e_textures, e_models, e_global, e_utils, e_camera, e_sound,
     e_particles, e_font, e_weapons;

type
  TPlayer = class(TCreature)
    score        : integer;
    score_next_life : integer;
    life         : integer;
    timer_life   : single;

    pos_i        : integer;
    jump_height  : single;
    jump_start   : boolean;
    jump_timer   : single;
    jump         : single;
    radius       : single;

    crouch       : boolean;
    crouch_active : boolean;

    weapon       : integer;
    timer_shoot  : single;

    timer_hit    : single;
    map          : integer;
    map_min      : single;
    map_max      : single;
    trails       : integer;

    turbo_count  : byte;

    control      : boolean;
    particles_trail : array of TParticlesEmitter;
    visible      : boolean;
  public
    constructor Create;
    procedure   GetDamage;
    procedure   GetScore(_score : integer);
    procedure   TrailsSet(_trails : integer);
    procedure   Render;
    procedure   RenderFront;
    procedure   Shoot(turbo : boolean);
    procedure   Update(count : single);
  end;

implementation

uses e_game;

constructor TPlayer.Create;
begin
color_base    := GetColor(1, 1, 1, 1);
color         := GetColor(1, 1, 1, 1);

pos           := GetPoint(30, 0, 0);
radius        := 0.8;                                  
trails        := 0;
weapon        := 0;

weapon        := 1;
timer_shoot   := 0;

map_max       := -1;

score         := 0;
score_next_life := 100;
turbo_count   := 0;
life          := 3;
timer_life    := 0;
end;

procedure TPlayer.GetDamage;
begin
if timer_life > 0 then
  exit;

timer_life := 3000;
life := life - 1;

particles.AddEmitter(particles_blood, pos, 30, color);
SoundPlay(sound_damage);

if life <= 0 then
  game.GameOver(false);
end;

procedure TPlayer.GetScore(_score : integer);
begin
score := score + _score;

if score > score_next_life then
  begin
    life := life + 1;
    score_next_life := score_next_life + 100;
  end;
end;

procedure TPlayer.TrailsSet(_trails : integer);
var
i : integer;
begin
if trails = _trails then
  exit;

if _trails < 0 then _trails := 0;

// delete
for i := _trails to trails - 1 do
  begin
    particles_trail[i * 2].loop := false;
    particles_trail[i * 2 + 1].loop := false;
  end;

SetLength(particles_trail, (_trails + 1) * 2);
for i := trails to _trails - 1 do
  begin
    particles_trail[i * 2] := particles.AddEmitter(particles_fire, pos, 40, color_base);
    particles_trail[i * 2 + 1] := particles.AddEmitter(particles_fire, pos, 40, GetColor(1, 1, 1, 1));
    particles_trail[i * 2 + 1].multiply := true;
  end;

trails := _trails;
end;

procedure TPlayer.Render;
begin
end;

procedure TPlayer.RenderFront;
var
p : TPoint;
begin
if (not visible) or (trails > 0) then
  exit;
  
glColor4f(1, 1, 1, 1);

SetTexture(texture_circle);
glPushMatrix;
with pos do glTranslatef(x, y, 0);
glScalef(radius * 2, radius * 2, 1);
glRotatef(angle, 0, 0, 1);
glCallList(model_quad);
glPopMatrix;    
end;

procedure TPlayer.Shoot(turbo : boolean);
const
TURBO_MAX = 5;
TURBO_SPEED = 0.3;
var
i : integer;
begin
if weapon = 0 then
  exit;

if timer_shoot > 0 then
  exit;

if turbo and (turbo_count < TURBO_MAX) then
  begin
    timer_shoot := weapons_params[weapon].delay * TURBO_SPEED;
    inc(turbo_count);
  end
else
  begin
    timer_shoot := weapons_params[weapon].delay * (1 + turbo_count * TURBO_SPEED);
    turbo_count := 0;
  end;

SoundPlay(sound_shoot);

if weapon = 1 then
  for i := 0 to 4 do
    game.AddBullet(weapon, self).angle := i * 0.2;

if weapon = 2 then
  game.AddBullet(weapon, self);

if weapon = 3 then
  game.AddBullet(weapon, self);
end;

procedure TPlayer.Update(count : single);
var
p, n : TPoint;
a, d : single;
i : integer;
c, b : TColor;
begin
pos_i := round(pos.x);

if timer_life >= 0 then
  timer_life := timer_life - count;

i := 0;
if control then
  begin
    if keys[vk_left] or keys[ord('A')] then
      i := -1;
    if keys[vk_right] or keys[ord('D')] then
      i :=  1;

    {if keys[ord('Q')] then
      begin
        TrailsSet(random(5) + 1);
        keys[ord('Q')] := false;
      end;
    if keys[ord('E')] then
      begin
        color := GetColor(random, random, random, 1);
        color_base := GetColor(0, 0, 0, 1);
        keys[ord('E')] := false;
      end;    }

    if keys[ord('K')] then
      Shoot(false);

    if keys[ord('L')] then
      Shoot(true);

    if pos.y < game.GetHeight(map, pos.x) then
      speed.x := 0.11 * i
    else
      speed.x := 0.07 * i;
    speed.x := 0.11 * i;
  end;        

//if pos.y >= jump_height then
//  jump_start := false;

if jump_start then
  begin
    jump := jump + count * 0.05;
    if jump > 30 then
      jump_start := false;
  end
else
  if jump > 0 then
    begin
      a := 1;
      if control then
        if keys[vk_down] or keys[ord('S')] then
          a := abs(jump) * 0.5 + 1;
      jump := jump - count * 0.02 * a;
    end
  else
    if control then
      if keys[vk_up] or keys[ord('W')] or keys[vk_space] then
        jump_start := true;

a := sqr(pos.y - game.GetHeight(map, pos.x));
speed.y := (jump - a) * count * 0.0005;

//if jump > 0 then
//  jump := jump - count * 0.00001;
//speed.y := speed.y - (abs(speed.y) + 0.5) * count * 0.00005;
{if not jump_start then
  begin              
    if jump_timer > 0 then
      begin
        speed.y := speed.y - count * 0.00003;
        jump_timer := jump_timer - count;
      end
    else
      speed.y := speed.y - count * 0.0001;
  end;  }

if speed.x >  0.2 then speed.x :=  0.2;
if speed.x < -0.2 then speed.x := -0.2;

angle := PointAngle(speed);

GetSide(speed.x);

a := game.GetHeight(map, pos.x);

if control then
  if keys[vk_down] or keys[ord('S')] then
    begin
      if pos.y < a + 0.5 then
        SqrAspire(pos.y, a - 5, count * 0.002);
      if pos.y < a + 0.2 then
        crouch := true;
    end;

if not crouch then
  begin
    pos.x := pos.x + speed.x * count * 0.1;
    pos.y := pos.y + speed.y * count * 0.1;
  end
else
  if not (keys[vk_down] or keys[ord('S')]) then
    begin
      SqrAspire(pos.y, a + 1, count * 0.002);
      if pos.y > a then
        crouch := false;
    end;           

map := game.GetMap(pos.x);

if map_min < pos.x then
  map_min := pos.x;

if map_max > 0 then
  if map_min > map_max then
    map_min := map_max;
    
if pos.x < map_min - game_width / 2 then
  pos.x := map_min - game_width / 2;

if map_max > 0 then
  if pos.x > map_max + game_width / 2 then
    pos.x := map_max + game_width / 2;

if not crouch then
  begin
    a := game.GetHeight(map, pos.x);
    if pos.y < a then
      pos.y := a;
  end;

a := trails / 4 - jump / 15;
if a < 0.25 then a := 0.25;
SqrAspire(radius, a, count * 0.001);

c := color;
if crouch then
  c := GetColor(0, 0, 0, 1);
if timer_hit > 0 then
  begin
    timer_hit := timer_hit - count;
    c := GetColor(1, 1, 1, 1);
  end;

b := color_base;
if timer_life > 0 then
  b := GetColor(1, 1, 1, 1);
               
if trails > 0 then
  for i := 0 to trails - 1 do
    begin
      a := -(side * time / 300 + i * pi * 2 / trails);
      if sign(speed.x) <> 0 then
        a := a * sign(speed.x);
      particles_trail[i * 2].pos.x := pos.x + sin(a) * (radius - 0.25);
      particles_trail[i * 2].pos.y := pos.y + cos(a) * (radius - 0.25);
      particles_trail[i * 2 + 1].pos := particles_trail[i * 2].pos;

      SqrAspire(particles_trail[i * 2].color.r, b.r, count * 0.002);
      SqrAspire(particles_trail[i * 2].color.g, b.g, count * 0.002);
      SqrAspire(particles_trail[i * 2].color.b, b.b, count * 0.002);

      SqrAspire(particles_trail[i * 2 + 1].color.r, c.r, count * 0.002);
      SqrAspire(particles_trail[i * 2 + 1].color.g, c.g, count * 0.002);
      SqrAspire(particles_trail[i * 2 + 1].color.b, c.b, count * 0.002);
    end;

if timer_shoot >= 0 then
  timer_shoot := timer_shoot - count;

camera.eye.x := pos.x;
if camera.eye.x < map_min then
  camera.eye.x := map_min;

if map_max > 0 then
  if camera.eye.x > map_max then
    camera.eye.x := map_max;

{if map_max > 0 then
  if camera.eye.x > map_max then
    camera.eye.x := map_max;    }

camera.eye.x := camera.eye.x + camera_shift;
camera.eye.y := 0;
camera.eye.z := 50;

camera.target := camera.eye;
camera.target.z := 0;
end;

end.
