unit e_screen;

interface

uses Windows, OpenGL, e_global, e_types, e_utils;

type
  TScreenResolution = record
    width, height : integer;
  end;

  TScreen = record
    width, height, color_depth : integer;
    resolutions : TSelect;
    resolution_string : TString;
    fullscreen : boolean;
    aspect : single;
    vsync : boolean;
  end;

  PScreen = ^TScreen;

var
  screen : TScreen = (color_depth : 32; fullscreen : true; vsync : true;);

  screen_resolutions : array of TScreenResolution;
  screen_frequency   : integer = 75;

  wglSwapIntervalEXT       : function (interval: GLint): Boolean; stdcall;
  wglGetSwapIntervalEXT    : function: GLint; stdcall;

procedure ScreenPrepare(scr : PScreen);
procedure ScreenSet(scr : PScreen);
function  ScreenSetMode(scr : PScreen) : boolean;
//procedure ScreenSetVSync(scr : PScreen; scr_vsync : boolean);

implementation

procedure ScreenPrepare(scr : PScreen);
var
b, m : boolean;
i, a, n, w, h : integer;
mode : TDevMode;
begin
i := 0;
n := 0;
b := false;
repeat
b := not EnumDisplaySettings(nil, i, mode);

SetLength(screen_resolutions, n + 1);
m := true;

for a := 0 to n do
  if (screen_resolutions[a].width  = mode.dmPelsWidth) and
     (screen_resolutions[a].height = mode.dmPelsHeight) then m := false;

if (mode.dmPelsWidth < 640) or (mode.dmPelsHeight < 480) or (mode.dmBitsPerPel < 32) then m := false;
if ChangeDisplaySettings(mode, CDS_TEST or CDS_FULLSCREEN) <> DISP_CHANGE_SUCCESSFUL then m := false;

if m then
  with screen_resolutions[n] do
    begin
      width  := mode.dmPelsWidth;
      height := mode.dmPelsHeight;
      SetLength(scr^.resolutions.elements, n + 1);
      scr^.resolutions.elements[n] := inttostr(width) + 'x' + inttostr(height);
      inc(n);
    end;

inc(i);
until b;

w := GetSystemMetrics(SM_CXSCREEN);
h := GetSystemMetrics(SM_CYSCREEN);
w := 800;
h := 600;

scr^.resolutions.number := n;
for i := 1 to n - 1 do
  if (screen_resolutions[i].width = w) and (screen_resolutions[i].height = h) then
    begin
      scr^.resolutions.number := i + 1;
      break;
    end;      
end;

procedure ScreenVSync(enabled : boolean);
var
  wglSwapIntervalEXT : function(interval : GLint) : GLboolean; stdcall;
  wglGetSwapIntervalEXT : function: GLint; stdcall;
  interval : GLint;
begin
  interval := 0;
  if enabled then interval := 1;
  
  wglSwapIntervalEXT := wglGetProcAddress('wglSwapIntervalEXT');
  wglGetSwapIntervalEXT := wglGetProcAddress('wglGetSwapIntervalEXT');

  if addr(wglGetSwapIntervalEXT) <> nil then
    if wglGetSwapIntervalEXT <> interval then
    begin
      if addr(wglSwapIntervalEXT) <> nil then
        wglSwapIntervalEXT(interval);
    end;
end;

procedure ScreenSet(scr : PScreen);
var
x, y : integer;
Style : DWORD;
Rect  : TRect;
s : string;
begin
with scr^ do
  begin
    width  := screen_resolutions[resolutions.number - 1].width;
    height := screen_resolutions[resolutions.number - 1].height;

    s := inttostr(width) + 'x' + inttostr(height);
    for x := length(s) to 15 do
      s := ' ' + s;
    screen.resolution_string := s;

    //mouse_system := GetPoint(0, 0, 0);

    aspect := width / height;
    //PrepareFont(width, height);
    x := GetSystemMetrics(SM_CXSCREEN);
    y := GetSystemMetrics(SM_CYSCREEN);
    if fullscreen then Style := WS_POPUP
    else               Style := WS_POPUP or WS_CAPTION or WS_SYSMENU or WS_MINIMIZEBOX;
    SetWindowLong(handle, GWL_STYLE, Style or WS_VISIBLE);
    Rect.Left   := 0;
    Rect.Top    := 0;
    Rect.Right  := width;
    Rect.Bottom := height;
    AdjustWindowRect(Rect, Style, False);
    if not fullscreen then
      with Rect do
        SetWindowPos(handle, 0, round((x - width) / 2), round((y - height) / 2), Right - Left, Bottom - Top, 0)
    else
      SetWindowPos(handle, 0, 0, 0, width, height, 0);

    ShowWindow(handle, SW_SHOW);
    //glViewport(0, 0, width, height);

    ScreenSetMode(scr);
    ScreenVSync(vsync);
    //VSync(scr, vsync_enabled);
    //SetCursorPos(0, 0);
  end;
end;

function ScreenSetMode(scr : PScreen) : boolean;
var
DeviceMode : TDevMode;
begin
if scr^.fullscreen then
  with DeviceMode do
    begin
      dmBitsPerPel := scr^.color_depth;
      dmPelsWidth := scr^.width;
      dmPelsHeight := scr^.height;
      dmDisplayFrequency := screen_frequency;
      dmFields := DM_BITSPERPEL or DM_PELSWIDTH or DM_PELSHEIGHT or DM_DISPLAYFREQUENCY;
      dmSize := SizeOf(DeviceMode);
      Result := ChangeDisplaySettings(DeviceMode, CDS_FULLSCREEN) = DISP_CHANGE_SUCCESSFUL;
    end
else ChangeDisplaySettings(TDevMode(nil^), CDS_FULLSCREEN);
end;

end.
