unit e_sound;

interface

uses
  bass, windows, e_types, e_global, e_utils;

const
  sound_shoot          = 0;
  sound_damage         = 1;
  sound_bonus          = 2;
  music1               = 3;
  music2               = 4;

type
  TSoundFade = (sf_none, sf_in, sf_out);

  TSound = class
    handle : HSTREAM;
    loaded : boolean;
    fade   : TSoundFade;
    volume : single;
    volume_base : single;
    no_fade : boolean;
    target : PByte;
    target_old : byte;
  public
    constructor Create(filename : string; _volume : single = 1; loop : boolean = false);
    procedure   Update(count : single);
    procedure   SetTarget(_target : PByte);
    procedure   Play;
    procedure   Stop;
  end;

procedure SoundInitialize;
procedure SoundDeinitialize;
procedure SoundUpdate(count : single);
procedure SoundPlay(id : integer; wait : boolean = false);
procedure SoundSetVolume;
procedure SoundMusicSetVolume;

var
  sound_on : boolean = true;
  music_on : boolean = true;
  volume_sound : integer = 10;
  volume_music : integer = 10;
  volume_music_fade1 : single = -0.2;
  volume_music_fade2 : single = -0.2;
  sound_initialized : boolean = false;

var
  music_playing : boolean = false;
  sounds : array[0..4] of TSound;

implementation

uses e_main;

var
  sound_frequency : integer = 44100;
  sound_volume : single = 1;
  sound_no_fade : boolean = true;

constructor TSound.Create(filename : string; _volume : single = 1; loop : boolean = false);
begin
handle := BASS_StreamCreateFile(False, PChar(filename), 0, 0, 0);
if loop then BASS_ChannelFlags(handle, BASS_SAMPLE_LOOP, BASS_SAMPLE_LOOP);

no_fade := sound_no_fade;
fade := sf_none;
volume := 0;
volume_base := sound_volume * _volume;

if no_fade then
  BASS_ChannelSetAttribute(handle, BASS_ATTRIB_VOL, volume_base)
else
  BASS_ChannelSetAttribute(handle, BASS_ATTRIB_VOL, 0);
BASS_ChannelSetAttribute(handle, BASS_ATTRIB_FREQ, sound_frequency);

target := nil;
target_old := 1;

loaded := true;
end;

procedure TSound.Update(count : single);
var
s : single;
begin
if target <> nil then
  if target^ <> target_old then
     begin
       case target^ of
         1, 3 : Stop;
         2    : Play;
       end;
       target_old := target^;
     end;

if no_fade then exit;

case fade of
  sf_in  : s := 1;
  sf_out : s := 0;
end;

if fade = sf_none then
  begin
    if volume < 0.5 then
      BASS_ChannelPause(handle);
    exit;
  end;

if Aspire(volume, s, count * 0.0002) then
  fade := sf_none;

if loaded then
  BASS_ChannelSetAttribute(handle, BASS_ATTRIB_VOL, volume_base * volume);
end;

procedure TSound.SetTarget(_target : PByte);
begin
target := _target;
target_old := target^;
end;

procedure TSound.Play;
begin
{if no_fade then
  if loaded then
    BASS_ChannelSetAttribute(handle, BASS_ATTRIB_VOL, volume_base);  }

if loaded then
  BASS_ChannelPlay(handle, true);
fade := sf_in; 
end;

procedure TSound.Stop;
begin
fade := sf_out;
if no_fade and loaded then
  BASS_ChannelPause(handle);
end;

procedure SoundInitialize;
var
i : integer;
s : string;
begin
if sound_initialized then exit;

if (HIWORD(BASS_GetVersion) <> BASSVERSION) then
  exit;

if not BASS_Init(-1, sound_frequency, 0, Handle, nil) then
	exit;

sound_frequency := 44100;
sound_volume := volume_music / 10;
sound_no_fade := false;

sounds[music1] := TSound.Create('music/music1.ogg', 1, true);
sounds[music2] := TSound.Create('music/music2.ogg', 1, true);
//BASS_ChannelSetAttribute(music1.handle, BASS_ATTRIB_VOL, 0);
//BASS_ChannelFlags(music1.handle, BASS_SAMPLE_LOOP, BASS_SAMPLE_LOOP);


//sound_frequency := 22050;
sound_volume := volume_sound / 10;
sound_no_fade := true;

sounds[sound_shoot]   := TSound.Create('sounds/shoot.wav');
sounds[sound_damage]  := TSound.Create('sounds/damage.wav');
sounds[sound_bonus]   := TSound.Create('sounds/bonus.wav');

sound_initialized := true;
end;

procedure SoundDeinitialize;
var
i : integer;
begin
sound_initialized := false;

{for i := 0 to high(music) do
  if music[i] <> nil then
    if music[i].loaded then
	    BASS_SampleFree(music[i].handle);

for i := 0 to high(sounds) do
  if sounds[i] <> nil then
    if sounds[i].loaded then
	    BASS_SampleFree(sounds[i].handle);  }

BASS_Free();
end;

procedure SoundUpdate(count : single);
var
i : integer;
begin
for i := 0 to high(sounds) do
  if sounds[i] <> nil then
    if sounds[i].loaded then
      sounds[i].Update(count);


{if not Aspire(volume_music_fade1, 1, count * 0.00005) then
  BASS_ChannelSetAttribute(music1.handle, BASS_ATTRIB_VOL, volume_music_fade1 * volume_music * 0.1);  }
{for i := 0 to high(music) do
  if music[i] <> nil then
    if music[i].loaded then
      music[i].Update(count);  }
end;

procedure SoundPlay(id : integer; wait : boolean = false);
begin
if sounds[id] <> nil then
  if sounds[id].loaded then
    if (not wait) or (BASS_ChannelIsActive(sounds[id].handle) <> BASS_ACTIVE_PLAYING) then
      sounds[id].Play;
end;

procedure SoundSetVolume;
begin

end;

procedure SoundMusicSetVolume;
begin

end;

end.
