unit e_utils;

interface

uses windows, OpenGL, e_types, e_camera;

function  CollideCircles(p1, p2 : TPoint; r1 : single; r2 : single = 0) : boolean;
function  CollideLines(a1, a2, b1, b2 : TPoint) : boolean;
function  PointsAngle(p1, p2 : TPoint) : single;
function  PointAngle(p : TPoint) : single;
function  PointVisible(p : TPoint) : boolean;
function  CircleVisible(p : TPoint; r : single) : boolean;
function  Sign(s : single) : integer;
function  UpperCase(const S: string): string;
function  IntToStr(Num : Integer) : string;
function  StrToInt(Str : String) : integer;
function  StrToFloat(s:string; delemiter : String = '.'):Single;
function  BoolToStr(b : boolean) : string;
function  FileExists(file_name: string): boolean;
function  DirectoryExists(const Directory: string): Boolean;
function  ExtractPath(const link: string): string;
function  GetRandom : single;
function  Aspire(var s : single; aim, speed : single) : boolean;
procedure AspirePoint(var p : TPoint; aim : TPoint; speed : single);
procedure AspireColor(var c : TColor; aim : TColor; speed : single);
function SqrAspire(var s : single; aim, speed : single) : boolean;
function SqrtAspire(var s : single; aim, speed : single) : boolean;
procedure SqrAspirePoint(var p : TPoint; aim : TPoint; speed : single);
procedure ViewOrtho;
procedure ViewPerspective(c : TCamera);
function  ArcTan2(Y, X: Extended): Extended;
function  ArcSin(X: Extended): Extended;

const
//  camera_pos : TPoint = (x : 9.5; y : 9.5; z : 0);
  camera_pos : TPoint = (x : 0; y : 0; z : 0);
var
  game_size : single = 20;
  game_width : single = 33;

implementation

uses e_screen, e_main;

function CollideCircles(p1, p2 : TPoint; r1 : single; r2 : single = 0) : boolean;
begin
  result := sqr(p1.x - p2.x) + sqr(p1.y - p2.y) < sqr(r1 + r2);
end;

function CollideLines(a1, a2, b1, b2 : TPoint) : boolean;
var
t1, t2, a, b, c : single;
begin
result := false;
a := b1.x - a1.x;
b := b2.x - b1.x;
c := a2.x - a1.x;
t1 := (c * (a1.y - b1.y) + (a2.y - a1.y) * a) / ((b2.y - b1.y) * c - b * (a2.y - a1.y));
if (t1 <= 0) or (t1 >= 1) then exit;
t2 := (a + t1 * b) / c;
result := (t2 > 0) and (t2 < 1);
end;

function PointsAngle(p1, p2 : TPoint) : single;
var
s : single;
begin
s := sqrt(sqr(p1.x - p2.x) + sqr(p1.y - p2.y));
if s < 0.001 then s := 0.001;
s := arcsin((p1.y - p2.y) / s) * 180 / pi;
if (p1.x - p2.x) < 0 then s := 180 - s;

result := s;
end;

function PointAngle(p : TPoint) : single;
begin
result := PointsAngle(p, Getpoint(0, 0, 0));
end;

function PointVisible(p : TPoint) : boolean;
var
width, height : integer;
begin
width := round(17 * sqr(screen.aspect));
height := 18;

result := false;
if p.x < camera.target.x - width  then exit;
if p.x > camera.target.x + width  then exit;
if p.y < camera.target.y - height then exit;
if p.y > camera.target.y + height then exit;
result := true;
end;

function CircleVisible(p : TPoint; r : single) : boolean;
var
width, height : integer;
begin
width := round(13 * sqr(screen.aspect));
height := 18;

result := false;
if p.x < camera.target.x - width  - r then exit;
if p.x > camera.target.x + width  + r then exit;
if p.y < camera.target.y - height - r then exit;
if p.y > camera.target.y + height + r then exit;
result := true;
end;

function  Sign(s : single) : integer;
begin
result := -1;
if s > 0 then result := 1;
end;

function UpperCase(const S: string): string;
asm
       push  ebx
       push  esi
       push  edi
       mov   esi, eax          // s
       mov   eax, edx
       test  esi, esi
       jz    @Nil
       mov   edx, [esi-4]      // Length(s)
       mov   edi, eax          // @Result
       test  edx, edx
       jle   @Nil
       mov   ecx, [eax]
       mov   ebx, edx
       test  ecx, ecx
       jz    @Realloc          // Jump if Result not allocated
       test  edx, 3
       jnz   @Length3
       xor   edx, [ecx-4]
       cmp   edx, 3
       jbe   @TestRef
       jmp   @Realloc
@Length3:
       or    edx, 2
       xor   edx, [ecx-4]
       cmp   edx, 1
       ja    @Realloc
@TestRef:
       cmp   [ecx-8], 1
       je    @LengthOK         // Jump if Result RefCt=1
@Realloc:
       mov   edx, ebx
       or    edx, 3
       call  System.@LStrSetLength
@LengthOK:
       mov   edi, [edi]        // Result
       mov   [edi-4], ebx      // Correct Result length
       mov   byte ptr [ebx+edi], 0
       add   ebx, -1
       and   ebx, -4
       mov   eax, [ebx+esi]

@Loop: mov   ecx, eax
       or    eax, $80808080    // $E1..$FA
       mov   edx, eax
       sub   eax, $7B7B7B7B    // $66..$7F
       xor   edx, ecx          // $80
       or    eax, $80808080    // $E6..$FF
       sub   eax, $66666666    // $80..$99
       and   eax, edx          // $80
       shr   eax, 2            // $20
       xor   eax, ecx          // Upper
       mov   [ebx+edi], eax
       mov   eax, [ebx+esi-4]
       sub   ebx, 4
       jge   @Loop

       pop   edi
       pop   esi
       pop   ebx
       ret

@Nil:  pop   edi
       pop   esi
       pop   ebx
       jmp    System.@LStrClr   // Result:=''
end;

function IntToStr(Num : Integer) : string;
begin
  Str(Num, result);
end;

function StrToInt(Str : String) : integer;
var
i : integer;
begin
  Val(Str, result, i);
end;

function  StrToFloat(s:string; delemiter : String = '.'):Single;
  function DoThouthens(n:Integer):Integer;
  var
    i:Integer;
  begin
    Result := 1;
    for i := 0 to n-1 do
      Result := Result * 10;
  end;
var
  f,p:string;
  fi,ps:Integer;
begin
  if pos(delemiter, s)<>0 then
  begin
    f := Copy(s, 1, pos(delemiter, s)-1);
    delete(s, 1, pos(delemiter, s));
    p := s;
    fi := strtoint(f);
    ps := strtoint(p);
    Result := fi + ps/DoThouthens(length(p));
  end else
    Result := strtoint(s);
end;

function BoolToStr(b : boolean) : string;
begin
result := 'off';
if b then result := 'on';
end;

function FileExists(file_name: string): boolean;
var
 f: file;
begin
if file_name = '' then begin Result := false; exit; end;
FileMode := 64;
AssignFile(f, file_name);
{$I-}
Reset(f);
{$I+}
Result:=IOResult=0;
if Result then CloseFile(f);
end;

function DirectoryExists(const Directory: string): Boolean;
{$IFDEF LINUX}
var
  st: TStatBuf;
begin
  if stat(PChar(Directory), st) = 0 then
    Result := S_ISDIR(st.st_mode)
  else
    Result := False;
end;
{$ENDIF}
{$IFDEF MSWINDOWS}
var
  Code: Integer;
begin
  Code := GetFileAttributes(PChar(Directory));
  Result := (Code <> -1) and (FILE_ATTRIBUTE_DIRECTORY and Code <> 0);
end;
{$ENDIF}

function ExtractPath(const link: string): string;
var i,n: integer;
    l: string;
begin
l:=link;
if link[Length(link)]='\' then Delete(l,Length(link),1);
if Pos('\',l)= 0 then begin result:=l+'\'; Exit; end;
result:=l;
for i:=Length(l) downto 1 do
    begin
    if l[i] = '\' then begin result:=l+'\'; Exit; end;
    if l[i] = '.' then
       for n:=Length(l) downto 1 do
           if l[n] = '\' then begin result:=copy(l,1,n); Exit; end;
    end;
end;

function GetRandom : single;
begin
result := random - random;
end;

function Aspire(var s : single; aim, speed : single) : boolean;
var
b : boolean;
begin
b := true;

if (s + speed < aim) then begin s := s + speed; b := false; end;
if (s - speed > aim) then begin s := s - speed; b := false; end;

if b then s := aim;

result := b;
end;

procedure AspirePoint(var p : TPoint; aim : TPoint; speed : single);
begin
Aspire(p.x, aim.x, speed);
Aspire(p.y, aim.y, speed);
Aspire(p.z, aim.z, speed);
end;

procedure AspireColor(var c : TColor; aim : TColor; speed : single);
begin
Aspire(c.r, aim.r, speed);
Aspire(c.g, aim.g, speed);
Aspire(c.b, aim.b, speed);
end;

function SqrAspire(var s : single; aim, speed : single) : boolean;
begin
result := Aspire(s, aim, speed * (abs(s - aim)));
end;

function SqrtAspire(var s : single; aim, speed : single) : boolean;
var
t : single;
begin
t := abs(s - aim);
if t < 0.0001 then
  t := 1;
result := Aspire(s, aim, speed / t);
end;

procedure SqrAspirePoint(var p : TPoint; aim : TPoint; speed : single);
begin
SqrAspire(p.x, aim.x, speed);
SqrAspire(p.y, aim.y, speed);
SqrAspire(p.z, aim.z, speed);
end;

procedure ViewOrtho;
var
x, y : single;
begin
glMatrixMode(GL_PROJECTION);
glLoadIdentity;
//glOrtho(-1, 1, -1, 1, -1, 1);
x := 0;
y := 0;
with camera_pos do
  glOrtho(-game_size + x, game_size + x, (-game_size + y) / screen.aspect, (game_size + y) / screen.aspect, -10, 10);
glMatrixMode(GL_MODELVIEW);
glLoadIdentity;
end;

procedure ViewPerspective(c : TCamera);
begin
glMatrixMode(GL_PROJECTION);
glLoadIdentity;
with c do gluPerspective(fov, screen.aspect, cull_near, cull_far);
glMatrixMode(GL_MODELVIEW);
glLoadIdentity;
end;

function ArcTan2(Y, X: Extended): Extended;
asm
        FLD     Y
        FLD     X
        FPATAN
        FWAIT
end;

function ArcSin(X: Extended): Extended;
begin
  Result := ArcTan2(X, Sqrt(1 - X*X))
end;

end.
