unit e_weapons;

interface

uses OpenGL, e_types, e_models, e_textures, e_utils, e_particles, e_sound,
     e_camera, e_global, e_font, e_console;

type
  TWeaponParams = record
    damage       : single;
    delay        : single;
    speed        : single;
  end;

const
  weapons_params : array[1..3] of TWeaponParams = (
    (damage : 20; delay : 1500; speed : 0.003),
    (damage : 15; delay : 400; speed : 0.07),
    (damage : 15; delay : 500; speed : 0.055)
    );

type
  TChuck = class(TCreature)
    mode         : byte;
    map          : integer;
    dead         : boolean;
    parent       : TCreature;
    particles1   : TParticlesEmitter;
    particles2   : TParticlesEmitter;
  public
    constructor Create(_mode : byte; _parent : TCreature);    virtual;
    procedure   Render;                           virtual;
    procedure   Update(count : single);           virtual;
  end;

  TChuckBlade = class(TChuck)
    radius : single;
  public
    constructor Create(_mode : byte; _parent : TCreature);    override;
    procedure   Render;                           override;
    procedure   Update(count : single);           override;
  end;

  TChuckBullet = class(TChuck)
  public
    constructor Create(_mode : byte; _parent : TCreature);    override;
    procedure   Render;                           override;
    procedure   Update(count : single);           override;
  end;

  TChuckFirework = class(TChuck)
    fall             : boolean;
    height           : single;
  public
    constructor Create(_mode : byte; _parent : TCreature);    override;
    procedure   Render;                           override;
    procedure   Update(count : single);           override;
  end;

function WeaponShoot(mode : byte; parent : TCreature) : TChuck;
function GetWeaponName(mode : byte) : TString;

implementation

uses e_game;

function WeaponShoot(mode : byte; parent : TCreature) : TChuck;
begin
case mode of
  1   : result := TChuckBlade.Create(mode, parent);
  2   : result := TChuckBullet.Create(mode, parent);
  3   : result := TChuckFirework.Create(mode, parent);
  else   result := TChuckBullet.Create(1, parent);
end;  
end;

function GetWeaponName(mode : byte) : TString;
begin
case mode of
  1  : result := TString('machine gun');
  2  : result := TString('shotgun');
  3  : result := TString('minigun');
  4  : result := TString('sniper rifle');
  5  : result := TString('rocket launcher');
  6  : result := TString('berserk rockets');
  7  : result := TString('cannonfire');
  8  : result := TString('plasma gun');
  9  : result := TString('plasma rifle');
  10 : result := TString('neonation');
  11 : result := TString('flamethrower');
  12 : result := TString('igniter');
  else result := TString('undefined');
end;
end;

constructor TChuck.Create(_mode : byte; _parent : TCreature);
begin
parent := _parent;
mode   := _mode;
dead   := false;
end;

procedure TChuck.Render;
begin
end;


procedure TChuck.Update(count : single);
begin
end;



constructor TChuckBlade.Create(_mode : byte; _parent : TCreature);
var
  s : single;
begin
inherited Create(_mode, _parent);

pos             := parent.pos;
side            := parent.side;
speed.x         := 0;
speed.y         := 0;
angle           := 0;
dead            := false;
radius          := 0;

particles1 := particles.AddEmitter(particles_blade, pos, 10, parent.color_base);
particles2 := particles.AddEmitter(particles_blade, pos, 1, parent.color);
particles2.multiply := true;

particles1.parent := self;
particles2.parent := self;
end;

procedure TChuckBlade.Render;
var
s : single;
i : integer;
begin

end;

procedure TChuckBlade.Update(count : single);
const
FULL_PATH = 3.5 * pi;
var
i : integer;
s : single;
begin
if dead then exit;

angle := angle + count * weapons_params[mode].speed;
if angle > FULL_PATH then
  dead := true;

if angle < FULL_PATH - pi * 0.5 then
  SqrAspire(radius, 5, count * 0.002)
else
  SqrAspire(radius, 0, count * 0.005); 

if side > 0 then
  begin
    pos.x := parent.pos.x + cos(angle) * radius;
    pos.y := parent.pos.y + sin(angle) * radius;
  end
else
  begin
    pos.x := parent.pos.x + cos(pi - angle) * radius;
    pos.y := parent.pos.y + sin(pi - angle) * radius;
  end;

map := game.GetMap(pos.x);
s := game.GetHeight(map, pos.x);
if pos.y < s then
  begin
    pos.y := s;
    angle := angle + count * 0.003;
  end;

particles1.pos := pos;
particles2.pos := pos;

for i := 0 to high(game.enemies) do
  if game.enemies[i] <> nil then
    if (game.enemies[i].health > 0) then
      if sqr(pos.x - game.enemies[i].pos.x) + sqr(pos.y - game.enemies[i].pos.y) < sqr(game.enemies[i].size / 2) then
        begin
          game.enemies[i].GetDamage(weapons_params[mode].damage);
          dead := true;
          break;
        end;
end;



constructor TChuckBullet.Create(_mode : byte; _parent : TCreature);
var
  s : single;
begin
inherited Create(_mode, _parent);

pos             := parent.pos;
side            := parent.side;
speed.x         := 0;
speed.y         := 0;
angle           := random * 360;
dead            := false;

particles1 := particles.AddEmitter(particles_bullet, pos, 15, parent.color_base);
particles2 := particles.AddEmitter(particles_bullet, pos, 15, parent.color);
particles2.multiply := true;

particles1.parent := self;
particles2.parent := self;
end;

procedure TChuckBullet.Render;
var
s : single;
i : integer;
begin

end;

procedure TChuckBullet.Update(count : single);
var
i : integer;
s : single;
begin
if dead then exit;

if abs(speed.x) < weapons_params[mode].speed then
  speed.x := speed.x + count * 0.0001 * side;
pos.x := pos.x + speed.x * count * 0.3;
map := game.GetMap(pos.x);  
s := game.GetHeight(map, pos.x);
if pos.y < s then
  pos.y := s
else
  SqrAspire(pos.y, s, count * 0.004);
  
particles1.pos := pos;
particles2.pos := pos;

angle := PointsAngle(game.map[map + 1], game.map[map]);

for i := 0 to high(game.enemies) do
  if game.enemies[i] <> nil then
    if (game.enemies[i].health > 0) then
      if sqr(pos.x - game.enemies[i].pos.x) + sqr(pos.y - game.enemies[i].pos.y) < sqr(game.enemies[i].size / 2) then
        begin
          game.enemies[i].GetDamage(weapons_params[mode].damage);
          dead := true;
          break;
        end;

if (pos.x < camera.eye.x - game_width) or (pos.x > camera.eye.x + game_width) then
  dead := true;
end;



constructor TChuckFirework.Create(_mode : byte; _parent : TCreature);
var
  s : single;
begin
inherited Create(_mode, _parent);

pos             := parent.pos;
side            := parent.side;
speed.x         := side * (0.1 + random * 0.05);
speed.y         := random * 5;
angle           := random * 360;
dead            := false;
fall            := false;
color_base      := parent.color_base;
color           := parent.color;
height          := 9 + GetRandom * 2;

particles1 := particles.AddEmitter(particles_firework, pos, 10, color_base);
particles2 := particles.AddEmitter(particles_firework, pos, 10, color);
particles2.multiply := true;
end;

procedure TChuckFirework.Render;
var
s : single;
i : integer;
begin

end;

procedure TChuckFirework.Update(count : single);
var
i : integer;
s : single;
begin
if dead then exit;

if not fall then
  begin
    SqrAspire(speed.x, -side * 0.025, count * 0.003);
    //if abs(speed.x) < weapons_params[mode].speed then
    //  speed.x := speed.x + count * 0.00005 * side;
  end
else
  if abs(speed.x) < 0.0001 then
    speed.x := GetRandom * weapons_params[mode].speed;

pos.x := pos.x + speed.x * count * 0.3;

map := game.GetMap(pos.x);
s := game.GetHeight(map, pos.x);

if not fall then
  begin
    if pos.y < s then
      pos.y := s;

    if Aspire(pos.y, s + height, count * 0.015) then
      begin
        fall := true;
        speed.x := 0;
        SoundPlay(sound_damage);
        for i := 1 to 14 do                   
          with TChuckFirework(game.AddBullet(mode, self)) do
            begin
              fall := true;
              speed.x := 0;
            end;
      end;
  end
else
  begin
    speed.y := speed.y - count * 0.015;
    pos.y := pos.y + count * speed.y * 0.002;

    if pos.y < s then
      dead := true;
  end;
  
particles1.pos := pos;
particles2.pos := pos;

for i := 0 to high(game.enemies) do
  if game.enemies[i] <> nil then
    if (game.enemies[i].health > 0) then
      if sqr(pos.x - game.enemies[i].pos.x) + sqr(pos.y - game.enemies[i].pos.y) < sqr(game.enemies[i].size / 2) then
        begin
          game.enemies[i].GetDamage(weapons_params[mode].damage);
          dead := true;
          break;
        end;

if (pos.x < camera.eye.x - game_width) or (pos.x > camera.eye.x + game_width) then
  dead := true;
end;

end.
