#ifndef ONCE_ANIMATORS
#define ONCE_ANIMATORS

#include "Globals.h"

enum AnimatorState
{
	ANIM_STATE_READY,
	ANIM_STATE_RUN,
	ANIM_STATE_END
};

class Animator abstract
{
	protected :	
		float TimeFull;
		float TimeLoop;
		float TimeStarted;
		
	public :
		Animator();

		AnimatorState State;

		int LoopIndex;
		int LoopsCount;

		float Period;
		float Interval;
		
		float DeltaStart;
		float DeltaLoop;

		float PeriodVariation;

		float Delay0;
		float Delay1;

		bool Delayed;
		bool Running;
		bool Standby;
		bool Ended;
		
		void Start(float Time);
		void Stop();

		void Update(float Time);
		void SetState(AnimatorState s);

		// Note, dT is time from TimeStarted to current time
		virtual void Calculate(float Time, float dT) abstract;
		virtual void Initialize(float Time) abstract;
};

class LinearAnimator : public Animator
{
	public :
		LinearAnimator();

		float Value;
		float ValueNorm;

		float Value0;
		float Value1;
		float ValueRange;

		void SetBounds(float A, float B);

		void Initialize(float Time);
		void Calculate(float Time, float dT);
};

class AccelerationAnimator : public Animator
{
	protected :
		Vector2f Position0;
		Vector2f Velocity0;
		Vector2f Acceleration0;

		float VelocityHit;
		float PositionHit;

		void UpdateVelocity(float dT0);
		void UpdatePosition(float dT0);

		void CalculateVelocity(Vector2f& v, float dT0);
		void CalculatePosition(Vector2f& p, float dT0);

	public :
		float Time0;

		AccelerationAnimator();

		Vector2f Position;
		Vector2f Velocity;
		Vector2f Acceleration;

		virtual void SetPosition(float X, float Y);
		virtual void SetVelocity(float X, float Y);
		virtual void SetAcceleration(float X, float Y);

		virtual void Initialize(float Time);
		virtual void Calculate(float Time, float dT);
};

class GravityAnimator : public AccelerationAnimator
{
	protected :
		void UpdateState(float Time);
		bool CalculateHitGround(Vector2f& ht);

	public :
		GravityAnimator();

		int HitIndex;
		int HitsCount;

		float Throttle;
		float Friction;
		float Breaking;
		Vector2f Speed;

		float Ground;
		bool Landed;
		bool GroundHit;
		bool MoveActive;

		void Initialize(float Time);
		void Calculate(float Time, float dT);

		void GetStartVelocity(float Speed, Vector2f& Target, Vector2f& Velocity1, Vector2f& Velocity2);
};

#endif