
#include "Bullets.h"

BulletsClass::BulletsClass()
{
	BulletID = 0;
	HitsCount = 0;
}

// WARNING!!! Impossible physics!
void BulletsClass::CheckHit(Bullet* b0, Bullet* b1)
{
	Bullet *b2 = NULL;

	bool SameHit = (b0 -> HitID == b1 -> ID) && (b1 -> HitID == b0 -> ID);

	float dx = b1 -> Position[0] - b0 -> Position[0];
	float dy = b1 -> Position[1] - b0 -> Position[1];
	float d = dx * dx + dy * dy;

	float s = b0 -> Weapon -> BulletRadius / b0 -> Scale * 
		b1 -> Weapon -> BulletRadius / b1 -> Scale;

	if ((d <= s) && (!SameHit))
	{
		if (((b0 -> Scale == 1.0f) || (b1 -> Scale == 1.0f)) &&
			((b0 -> Enemy && !b1 -> Enemy) || (!b0 -> Enemy && b1 -> Enemy)))
			HitsCount++;

		b0 -> Rotate = true;
		b1 -> Rotate = true;

		b0 -> Processed = true;
		b1 -> Processed = true;

		b0 -> HitID = b1 -> ID;
		b1 -> HitID = b0 -> ID;

		b0 -> Scale = 0.75f;
		b1 -> Scale = 0.75f;

		b0 -> Damage = b0 -> Weapon -> WeaponDamage * 0.5f;
		b1 -> Damage = b1 -> Weapon -> WeaponDamage * 0.5f;

		b0 -> EffectGravity.Acceleration[1] = 2.0f * b0 -> Weapon -> BulletGravity;
		b1 -> EffectGravity.Acceleration[1] = 2.0f * b1 -> Weapon -> BulletGravity;

		Vector2f& v0 = b0 -> EffectGravity.Velocity;
		Vector2f& v1 = b1 -> EffectGravity.Velocity;
/*
		Vector2f v;
		v[0] = (v0[0] + v1[0]) * 0.5f;
		v[1] = (v0[1] + v1[1]) * 0.5f;

		// Simple non-elastic hit
		b0 -> EffectGravity.Velocity[0] = v[0];
		b0 -> EffectGravity.Velocity[1] = v[1];

		b1 -> EffectGravity.Velocity[0] = v[0];
		b1 -> EffectGravity.Velocity[1] = v[1];	
*/
		// Crazy unknown hit
		float vl0 = VectorLength(v0);
		float vl1 = VectorLength(v1);
		float vl = (vl0 + vl1) * 0.5f;

		Vector2f dv;
		dv[0] = v0[0] - v1[0];
		dv[1] = v0[1] - v1[1];
		VectorNormalize(dv);

		if (Trigger)
		{	
			b2 = b0; 
			b0 = b1; 
			b1 = b2;
		}
		
		Trigger = !Trigger;

		b0 -> EffectGravity.Velocity[0] = -dv[1] * vl;
		b0 -> EffectGravity.Velocity[1] =  dv[0] * vl;

		b1 -> EffectGravity.Velocity[0] =  dv[1] * vl;
		b1 -> EffectGravity.Velocity[1] = -dv[0] * vl;		
	}
}

void BulletsClass::Update(float Time, float dT)
{
	Bullet *b0, *b1;
	Vector2f Direction;

	// Update position and angle
	for (int i = 0; i < List.Count; i++)
	{
		b0 = List[i];
		if (!b0) continue;

		b0 -> Processed = false;

		if (b0 -> EffectGravity.Standby)
			b0 -> EffectGravity.Start(Time);

		b0 -> EffectGravity.Update(Time);

		if (b0 -> EffectGravity.Landed)
			b0 -> Rotate = false;

		b0 -> Position[0] = b0 -> EffectGravity.Position[0];
		b0 -> Position[1] = b0 -> EffectGravity.Position[1];

		if (b0 -> Weapon -> Kind == WEAPON_BALL)
		{
			b0 -> Angle += b0 -> EffectGravity.Speed[0] * dT / 
				(b0 -> Weapon -> BulletRadius * b0 -> Scale);

			if (b0 -> Angle > M_PI) b0 -> Angle -= PI_2;
			if (b0 -> Angle < -M_PI) b0 -> Angle += PI_2;
		}
		else
			if (b0 -> Rotate)
			{
				b0 -> Angle += dT * b0 -> AngleSpin * M_PI * 5.0f;
	
				if (b0 -> Angle > M_PI) 
					b0 -> Angle -= M_PI;
			}
			else
			{
				Direction[0] = b0 -> EffectGravity.Velocity[0];
				Direction[1] = b0 -> EffectGravity.Velocity[1];
				VectorNormalize(Direction);

				if (Direction[0] == 0.0f) b0 -> Angle = Direction[1] * M_PI_2;
				else b0 -> Angle = atan2(Direction[1], Direction[0]);
			}

		switch (b0 -> Weapon -> Kind)
		{
			case WEAPON_BALL :
			case WEAPON_MINI :
			case WEAPON_RIFLE :
			{
				float n = VectorLength(b0 -> EffectGravity.Velocity);
				if ((b0 -> EffectGravity.Landed) && (n < 10.0f))
					List.Remove(i);

				break;
			}
		}
	}

	for (int i = 0; i < List.Count; i++)
	{
		b0 = List[i];
		if (!b0) continue;
		if (b0 -> Processed) continue;

		for (int j = i + 1; j < List.Count; j++)
		{
			b1 = List[j];
			if (!b1) continue;	
			if (b1 -> Processed) continue;

			CheckHit(b0, b1);
		}
	}
}

void BulletsClass::Render()
{
	Bullet *b = NULL;
	hgeSprite *s = NULL;

	for (int i = 0; i < List.Count; i++)
	{
		b = List[i];
		if (!b) continue;
		
		s = b -> Weapon -> BulletSprite;
		if (!s) continue;

		if (b -> Scale == 1.0f) s -> SetColor(0xFF9F9F9F);
		else s -> SetColor(0xFFFF9F9F);

		s -> RenderEx(b -> Position[0], b -> Position[1], 
			b -> Angle, b -> Scale, b -> Scale);
	}
}

void BulletsClass::AddBullet(WeaponParams& Weapon)
{
	Weapon.Time = 0.0f;

	Bullet *b = new Bullet();
	List.Add(b);
	
	b -> Scale = 1.0f;
	b -> Enemy = false;
	b -> Weapon = Weapon.Info;
	b -> Damage = Weapon.Info -> WeaponDamage;
	b -> Rotate = false;
	b -> HitID = BulletID;
	b -> ID = BulletID;
	BulletID++;

	if (random(1)) b -> AngleSpin = 1;
	else b -> AngleSpin = -1;

	b -> Position[0] = Weapon.Position[0] + Weapon.Direction[0] * Weapon.Info -> WeaponLength;
	b -> Position[1] = Weapon.Position[1] + Weapon.Direction[1] * Weapon.Info -> WeaponLength;

	float vx = Weapon.Direction[0] * Weapon.Info -> BulletSpeed;
	float vy = Weapon.Direction[1] * Weapon.Info -> BulletSpeed;
	
	b -> Angle = Weapon.Angle;

	b -> EffectGravity.HitsCount = Weapon.Info -> BulletHits;
	b -> EffectGravity.Interval = 0.0f;
	b -> EffectGravity.Throttle = Weapon.Info -> BulletThrottle;
	b -> EffectGravity.Friction = Weapon.Info -> BulletFriction;
	b -> EffectGravity.Breaking = Weapon.Info -> BulletBreaking;
	b -> EffectGravity.Speed[0] = vx * Weapon.Info -> BulletTangential;
	b -> EffectGravity.Ground = (float) SCREEN_HEIGHT;
	b -> EffectGravity.SetPosition(b -> Position[0], b -> Position[1]);
	b -> EffectGravity.SetVelocity(vx, vy);
	b -> EffectGravity.SetAcceleration(0.0f, Weapon.Info -> BulletGravity);
	b -> EffectGravity.MoveActive = false;
}

void BulletsClass::AddEnemyBullet(WeaponParams& Weapon, Vector2f& Target, Vector2f& Direction)
{
	Bullet *b = new Bullet();
	List.Add(b);

	b -> Scale = 1.0f;
	b -> Enemy = true;
	b -> Weapon = Weapon.Info;
	b -> Damage = Weapon.Info -> WeaponDamage;
	b -> Rotate = false;
	b -> HitID = BulletID;
	b -> ID = BulletID;
	BulletID++;
	
	if (random(1)) b -> AngleSpin = 1;
	else b -> AngleSpin = -1;

	float vx = Direction[0];
	float vy = Direction[1];

	VectorNormalize(Direction);
	if (Direction[0] == 0.0f) b -> Angle = Direction[1] * M_PI_2;
	else b -> Angle = atan2(Direction[1], Direction[0]);

	b -> Position[0] = Weapon.Position[0] + Weapon.Direction[0] * Weapon.Info -> WeaponLength;
	b -> Position[1] = Weapon.Position[1] + Weapon.Direction[1] * Weapon.Info -> WeaponLength;

	b -> EffectGravity.HitsCount = Weapon.Info -> BulletHits;
	b -> EffectGravity.Interval = 0.0f;
	b -> EffectGravity.Throttle = Weapon.Info -> BulletThrottle;
	b -> EffectGravity.Friction = Weapon.Info -> BulletFriction;
	b -> EffectGravity.Breaking = Weapon.Info -> BulletBreaking;
	b -> EffectGravity.Speed[0] = vx * Weapon.Info -> BulletTangential;
	b -> EffectGravity.Ground = (float) SCREEN_HEIGHT;
	b -> EffectGravity.SetPosition(b -> Position[0], b -> Position[1]);
	b -> EffectGravity.SetAcceleration(0.0f, Weapon.Info -> BulletGravity);
	b -> EffectGravity.SetVelocity(vx, vy);
	b -> EffectGravity.MoveActive = false;

	Weapon.Time = 0.0f;
}