
#include "Enemies.h"

EnemiesClass::EnemiesClass()
{
	Font = NULL;
}

EnemiesClass::~EnemiesClass()
{
	for (int i = 0; i < Textures.Count; i++)
		hge -> Texture_Free(*Textures[i]);
}

void EnemiesClass::Update(Vector2f& Target, float dT)
{
	Enemy *e = NULL;
	for (int i = 0; i < List.Count; i++)
	{
		e = List[i];
		if (!e) continue;

		e -> Update(Target, dT);
		
		if (e -> Fire)	
		{
			bool Fire = e -> CheckFire(Target);

			if (Fire)
			{
				Bullets -> AddEnemyBullet(e -> Weapon, Target, e -> Weapon.Direction);
				e -> Fire = false;
			}
			else
			{
				e -> Weapon.Direction[1] = -1.0f;
				e -> Weapon.Direction[0] = 0.0f;
				e -> Weapon.Angle = -M_PI_2;
			}

		}
	}
}

void EnemiesClass::Render()
{
	Enemy *e = NULL;
	for (int i = 0; i < List.Count; i++)
	{
		e = List[i];
		if (!e) continue;

		e -> Render();
	}
}

bool EnemiesClass::LoadResources()
{
	HTEXTURE Tex = 0;
	HTEXTURE *TexPointer = NULL;

	// Static turrel. INDEX 0
	Tex = hge -> Texture_Load(".\\gfx\\turrel_01.png");
	if (!Tex) return false;

	TexPointer = new HTEXTURE(Tex);
	Textures.Add(TexPointer);

	// Static turrel. INDEX 1
	Tex = hge -> Texture_Load(".\\gfx\\turrel_02.png");
	if (!Tex) return false;

	TexPointer = new HTEXTURE(Tex);
	Textures.Add(TexPointer);

	// Static turrel. INDEX 2
	Tex = hge -> Texture_Load(".\\gfx\\turrel_03.png");
	if (!Tex) return false;

	TexPointer = new HTEXTURE(Tex);
	Textures.Add(TexPointer);

	return true;
}

void EnemiesClass::AddEnemy(EnemiesKinds Kind, Vector2f& Position)
{
	switch (Kind)
	{
		case TURREL_STATIC_RIFLE :
		case TURREL_STATIC_MINI :
		case TURREL_STATIC_BALL :
		{
			AddEnemyStaticTurrel(Kind, Position);
			break;
		}
	}
}

void EnemiesClass::AddEnemyStaticTurrel(EnemiesKinds Kind, Vector2f& Position)
{
	EnemyStaticTurrel *e = new EnemyStaticTurrel();
	HTEXTURE Tex = *Textures[Kind];

	
	float w = (float) hge -> Texture_GetWidth(Tex);
	float h = (float) hge -> Texture_GetHeight(Tex);
	
	e -> Image = new hgeSprite(Tex, 0.0f, 0.0f, w, h);
	e -> Image -> SetHotSpot(w / 2.0f, h / 2.0f);

	switch (Kind)
	{
		case 0 :
			Position[0] += 0.0f;
			Position[1] += -20.0f;

			e -> Weapon.Offset[0] = 0.0f;
			e -> Weapon.Offset[1] = -2.0f;

			e -> Life = 60.0f;
			break;

		case 1 :
			Position[0] += 0.0f;
			Position[1] += -21.0f;

			e -> Weapon.Offset[0] = 0.0f;
			e -> Weapon.Offset[1] = -3.0f;

			e -> Life = 50.0f;
			break;

		case 2 :
			Position[0] += 0.0f;
			Position[1] += -5.0f;

			e -> Weapon.Offset[0] = 0.0f;
			e -> Weapon.Offset[1] = -14.0f;

			e -> Life = 40.0f;
			break;
	}

	e -> Position[0] = Position[0];
	e -> Position[1] = Position[1];

	e -> Weapon.Position[0] = Position[0] + e -> Weapon.Offset[0];
	e -> Weapon.Position[1] = Position[1] + e -> Weapon.Offset[1];

	e -> Weapon.Direction[0] = 0.0f;
	e -> Weapon.Direction[1] = -1.0f;

	e -> Weapon.Info = &WeaponsInfo[Kind];
	e -> Weapon.Time = 0.0f;
	e -> Weapon.Angle = -M_PI_2;

	e -> Font = Font;

	List.Add(e);
}

/* --- Abstract enemy class --- */

Enemy::Enemy()
{ 
	Image = NULL; 
	Fire = false; 
	Life = 100.0f;

	Velocity[0] = 0.0f;
	Velocity[1] = 0.0f;

	Font = NULL;
}
Enemy::~Enemy()
{ 
	if (Image) delete Image; 
}

bool Enemy::CheckFire(Vector2f& Target)
{
	// It is not right, but this will make some probability...
	float x = Weapon.Position[0] + Weapon.Direction[0] * Weapon.Info -> WeaponLength;
	float y = Weapon.Position[1] + Weapon.Direction[1] * Weapon.Info -> WeaponLength;

	GravityAnimator g;
	g.SetPosition(x, y);
	g.SetAcceleration(0.0f, Weapon.Info -> BulletGravity);

	Vector2f v1 = {0.0f, 0.0f};
	Vector2f v2 = {0.0f, 0.0f};
	Vector2f *v = NULL;

	// Test if bullet can reach the target
	g.GetStartVelocity(Weapon.Info -> BulletSpeed, Target, v1, v2);

	float a;
	float a1 = -VectorAngle(v1);
	float a2 = -VectorAngle(v2);

	// Check if in sector
	bool Reach = false;
	bool InSectors1 = false;
	bool InSectors2 = false;
		
	for (int i = 0; i < Weapon.Info -> WeaponSectors.Count; i++)
	{
		WeaponSector *s = Weapon.Info -> WeaponSectors[i];

		if ((a1 >= s -> Min) && (a1 <= s -> Max)) InSectors1 = true;
		if ((a2 >= s -> Min) && (a2 <= s -> Max)) InSectors2 = true;
	}

	bool Reach1 = ((v1[0] != 0.0f) || (v1[1] != 0.0f)) && InSectors1;
	bool Reach2 = ((v2[0] != 0.0f) || (v2[1] != 0.0f)) && InSectors2;

	// Check if can reach
	switch (Weapon.Info -> WeaponStraight)
	{
		case -1 : {	Reach = Reach1; a = a1; v = &v1; break; }
		case  1 : {	Reach = Reach2; a = a2; v = &v2; break; }

		case  0 : 
		{ 
			Reach = Reach1 || Reach2; 
			
			if (Reach2) { a = a2; v = &v2; }
			else {a = a1; v = &v1; }

			if ((abs(a - M_PI_2) < M_PI / 120.0f) && 
				(Reach1 && !Reach2)) Reach = false;
			
			break; 
		}
	}

	if (Reach)
	{
		Weapon.Direction[0] = (*v)[0];
		Weapon.Direction[1] = (*v)[1];
		Weapon.Angle = -a;
	}

	return Reach;
}

void Enemy::Update(Vector2f& Target, float dT)
{
	Weapon.Time += dT;

	if (Weapon.Time > Weapon.Info -> EnemyPeriod)
		Weapon.Time = Weapon.Info -> EnemyPeriod;

	if (Weapon.Time == Weapon.Info -> EnemyPeriod)
	{
		Fire = true;
		Weapon.Time = 0.0f;
	}
}

/* --- Static Turrel --- */

bool EnemyStaticTurrel::CheckHit(Vector2f& Point)
{
	float w, h, cx, cy;

	switch (Weapon.Info -> Kind)
	{
		case 0 :
			w = 50.0f; h = 44.0f;
			cx = 0.0f; cy = 0.0f;

			break;

		case 1 :
			w = 54.0f; h = 54.0f;
			cx = -1.0f; cy = -2.0f;

			break;

		case 2 :
			w = 60.0f; h = 44.0f;
			cx = 0.0f; cy = -8.0f;

			break;
	}

	w *= 0.5f;
	h *= 0.5f;

	float x0 = (Position[0] + cx) - w;
	float x1 = (Position[0] + cx) + w;

	float y0 = (Position[1] + cy) - h;
	float y1 = (Position[1] + cy) + h;

	bool Hit = (Point[0] >= x0) && (Point[0] <= x1) &&
		(Point[1] >= y0) && (Point[1] <= y1);

	return Hit;
}

void EnemyStaticTurrel::Update(Vector2f& Target, float dT)
{
	Enemy::Update(Target, dT);
	
	// Some other stuff...
}

void EnemyStaticTurrel::Render()
{
	if (Image) 
	{
		Image -> SetColor(0xFFEFEFEF);		
		Image -> Render(Position[0], Position[1]);
	}

	if (Weapon.Info -> WeaponSprite) 
	{
		Weapon.Info -> WeaponSprite -> SetColor(0xFFAFAFAF);
		Weapon.Info -> WeaponSprite -> RenderEx(Weapon.Position[0], 
			Weapon.Position[1], Weapon.Angle, 1.1f, 1.1f);
	}

	if (Font)
	{
		Font -> SetColor(0x7F9F9F9F);
		Font -> SetScale(0.5f);
		Font -> printf(Position[0] + Weapon.Offset[0], 
			Position[1] + Weapon.Offset[1] - 52.0f, 
			HGETEXT_CENTER, "%.0f", Life);
	}

/*
	Vector2f HitCenter;
	HitCenter[0] = Position[0];
	HitCenter[1] = Position[1] - 8.0f;

	float w = 60.0f;
	float h = 44.0f;

	float x1 = HitCenter[0] - w / 2.0f;
	float y1 = HitCenter[1] - h / 2.0f;

	hge -> Gfx_RenderLine(x1, y1, 
		x1 + w, y1, 0xFFFF0000);

	hge -> Gfx_RenderLine(x1, y1, 
		x1, y1 + h, 0xFFFF0000);

	hge -> Gfx_RenderLine(x1 + w, y1 + h, 
		x1 + w, y1, 0xFFFF0000);

	hge -> Gfx_RenderLine(x1 + w, y1 + h, 
		x1, y1 + h, 0xFFFF0000);
*/
}