
#include "Globals.h"
#include "LevelClass.h"

const float SCORE_OFFSET_X = 10.0f;
const float SCORE_OFFSET_Y = 0.0f;

static hgeQuad q = 
{
	{
		{0.0f, 0.0f, 0.0f, 0xFFFFFFFF, 0.0f, 0.0f},
		{0.0f, 0.0f, 0.0f, 0xFFFFFFFF, 0.0f, 0.0f},
		{0.0f, 0.0f, 0.0f, 0xFFFFFFFF, 0.0f, 0.0f},
		{0.0f, 0.0f, 0.0f, 0xFFFFFFFF, 0.0f, 0.0f}
	},
	0,
	BLEND_DEFAULT
};

void MakeQuad(float x, float y, float w, float h, DWORD Color, hgeQuad& q)
{
	for (int i = 0; i < 4; i++)
		q.v[i].col = Color;

	q.v[0].x = x - w;
	q.v[1].x = x + w;
	q.v[2].x = x + w;
	q.v[3].x = x - w;

	q.v[0].y = y - h;
	q.v[1].y = y - h;
	q.v[2].y = y + h;
	q.v[3].y = y + h;
}

LevelClass::LevelClass()
{
	Font = NULL;
	
	BackgroundTex = 0;
	Background = NULL;
	
	ForegroundTex = 0;
	Foreground = NULL;

	BordersTex = 0;
	Borders = NULL;

	State = LEVEL_STATE_STARTUP;

	EffectFade.SetBounds(255.0f, 0.0f);
	EffectFade.Interval = 1.0f;
	EffectFade.Value = 255.0f;

	EffectWait.SetBounds(1.0f, 0.0f);
	EffectWait.Interval = 5.0f;

	LevelScore = 0.0f;
	LevelLoaded = false;
	LevelLength = 0.0f;
	LevelPosition = 0.0f;
	LevelWave = 0.0f;

	SpacePressed = false;
}

LevelClass::~LevelClass()
{
    if (ForegroundTex) hge -> Texture_Free(ForegroundTex);
	if (BackgroundTex) hge -> Texture_Free(BackgroundTex);
	if (BordersTex) hge -> Texture_Free(BordersTex);

	if (Background) delete Background;
	if (Foreground) delete Foreground;
	if (Borders) delete Borders;

	if (Font) delete Font;
}

void LevelClass::SetLevelPosition(float PlayerX)
{
	PlayerX = floor(PlayerX);

	if (PlayerX > LevelPosition + (float) SCREEN_WIDTH * 0.75f)
	{
		LevelPosition = PlayerX - (float) SCREEN_WIDTH * 0.75f;

		if (LevelPosition > LevelLength - (float) SCREEN_WIDTH)
			LevelPosition = LevelLength - (float) SCREEN_WIDTH;
	}
	else
		if (PlayerX < LevelPosition + 800.0f * 0.25f)
		{
			LevelPosition = PlayerX - 800.0f * 0.25f;

			if (LevelPosition < 0.0f)
				LevelPosition = 0.0f;
		}
}

void LevelClass::Update(float Time, float X, float Y, float dT)
{
	switch (State)
	{
		case LEVEL_STATE_STARTUP :
		{			
			if (SpacePressed)
			{
				if (EffectFade.Standby) 
				{
					EffectFade.SetBounds(255.0f, 0.0f);
					EffectFade.Start(Time);
				}
				
				if (EffectFade.Running)
					EffectFade.Update(Time);

				if (EffectFade.Ended) 
				{
					State = LEVEL_STATE_GAME;		
					EffectFade.Value = 0.0f;
				}
			}

			break;
		}

		case LEVEL_STATE_GAME :
		{
			if (Player -> EffectGravity.Standby)
			{
				Player -> EffectGravity.Start(Time);
				LevelStarted = Time;
			}

			Player -> Update(Time, dT);
			Player -> SetWeaponAngle(dT, X + LevelPosition, Y);

			Player -> Position[0] = Player -> EffectGravity.Position[0];
			Player -> Position[1] = Player -> EffectGravity.Position[1];

			Player -> Velocity[0] = Player -> EffectGravity.Velocity[0];
			Player -> Velocity[1] = Player -> EffectGravity.Velocity[1];

			Player -> Speed[0] = Player -> EffectGravity.Speed[0];
			Player -> Speed[1] = Player -> EffectGravity.Speed[1];

			if (Player -> Position[0] < 0.0f) 
			{
				Player -> Position[0] = 0.0f;
				Player -> EffectGravity.Position[0] = 0.0f;
			}

			if (Player -> Position[0] > LevelLength) 
			{
				Player -> Position[0] = LevelLength;
				Player -> EffectGravity.Position[0] = LevelLength;
			}

			Player -> Weapon.Position[0] = Player -> Position[0] + Player -> Weapon.Offset[0];
			Player -> Weapon.Position[1] = Player -> Position[1] + Player -> Weapon.Offset[1];

			// Delayed death... 
			if (Player -> Position[1] >= (float) SCREEN_HEIGHT) 
			{
				Player -> Life -= dT * 15.0f;
				if (Player -> Life < 0.0f) Player -> Life = 0.0f;
			}

			if (Enemies)
			{
				Vector2f Target;
				Target[0] = Player -> Position[0];
				Target[1] = Player -> Position[1] - Player -> BodyRadius;

				Enemies -> Update(Target, dT);
			}

			if (Bullets) Bullets -> Update(Time, dT);

			if ((Bullets -> HitsCount >= 20) && (Player -> RedPlate) && 
				(Player -> EffectGravity.Landed) || (Player -> Life <= 0.0f))
			{
				State = LEVEL_STATE_FINISH;
				EffectFade.SetState(ANIM_STATE_READY);
				LevelTime = Time - LevelStarted;
			}

			break;
		}

		case LEVEL_STATE_FINISH :
		{
			if (EffectFade.Standby)
			{
				EffectFade.SetBounds(0.0f, 255.0f);
				EffectFade.Start(Time);
			}

			if (EffectFade.Running)
				EffectFade.Update(Time);

			if (EffectFade.Ended)
			{
				if (EffectWait.Standby)
				{
					EffectWait.Start(Time);
					{
						Player -> EffectGravity.Stop();
						Player -> EffectGravity.SetState(ANIM_STATE_READY);
					}
				}

				if (EffectWait.Running)
					EffectWait.Update(Time);

				if (EffectWait.Ended)
				{
					EffectFade.Value = 255.0f;
					EffectFade.SetState(ANIM_STATE_READY);
					EffectWait.SetState(ANIM_STATE_READY);
					State = LEVEL_STATE_STARTUP;
					
					if (Player -> Life > 0.0f)
						LoadLevel("level_02.xml", 2);
					else
						if (LevelID == 1)
							LoadLevel("level_01.xml", 1);
						else 
							LoadLevel("level_02.xml", 2);

					Player -> Life = 100.0f;
					LevelScore = Player -> Life;
					Player -> RedPlate = false;
					
					Player -> Velocity[0] = 0.0f;
					Player -> Velocity[1] = 0.0f;

					Player -> Speed[0] = 0.0f;
					Player -> Speed[1] = 0.0f;
					
					Player -> EffectGravity.Speed[0] = Player -> Speed[0];
					Player -> EffectGravity.Speed[1] = Player -> Speed[1];
					Player -> EffectGravity.SetVelocity(Player -> Velocity[0], Player -> Velocity[1]);
					Player -> EffectGravity.Ground = (float) SCREEN_HEIGHT;
				}
			}
		}
	}

	UpdateScore();
	UpdateGrounds();
	UpdatePlayerHits();
	UpdateEnemiesHits();					
}

void LevelClass::UpdatePlayerHits()
{
	float d, ds;
	Vector2f dp, dv;
	Bullet *b = NULL;

	float dm = Player -> BodyRadius * Player -> BodyRadius;

	for (int i = 0; i < Bullets -> List.Count; i++)
	{
		b = Bullets -> List[i];
		if (!b) continue;

		dp[0] = Player -> Position[0] - b -> Position[0];
		dp[1] = Player -> Position[1] - b -> Position[1] - Player -> BodyRadius;

		d = dp[0] * dp[0] + dp[1] * dp[1];

		dv[0] = b -> EffectGravity.Velocity[0];
		dv[1] = b -> EffectGravity.Velocity[1];

		VectorNormalize(dp);
		VectorNormalize(dv);

		ds = dp[0] * dv[0] + dp[1] * dv[1];

		if ((d <= dm) && (ds >= 0.0f) && (ds <= 1.0f))
		{
			Player -> Life -= b -> Damage;
			if (Player -> Life < 0.0f) Player -> Life = 0.0f;

			Bullets -> List.Remove(i);
		}
	}
}

void LevelClass::UpdateEnemiesHits()
{
	float ds;
	Vector2f dp, dv;
	
	Bullet *b = NULL;
	Enemy *e = NULL;

	for (int i = 0; i < Bullets -> List.Count; i++)
	{
		b = Bullets -> List[i];
		if (!b) continue;

		for (int j = 0; j < Enemies -> List.Count; j++)
		{	
			e = Enemies -> List[j];
			if (!e) continue;

			bool Hit = e -> CheckHit(b -> Position);
			if (!Hit) continue;

			dp[0] = e -> Position[0] - b -> Position[0];
			dp[1] = e -> Position[1] - b -> Position[1];

			dv[0] = b -> EffectGravity.Velocity[0];
			dv[1] = b -> EffectGravity.Velocity[1];

			VectorNormalize(dp);
			VectorNormalize(dv);

			ds = dp[0] * dv[0] + dp[1] * dv[1];

			if ((ds >= 0.0f) && (ds <= 1.0f))
			{
				e -> Life -= b -> Damage;		
				if (e -> Life <= 0.0f) 
					Enemies -> List.Remove(j);
				
				Bullets -> List.Remove(i);
				break;
			}
		}
	}
}


void LevelClass::UpdateGrounds()
{
	// Stage 1. Update ground for player
	float g = (float) SCREEN_HEIGHT;
	PlateStruct *pm = NULL;

	// Find new ground level for player
	for (int i = 0; i < Plates.Count; i++)
	{
		PlateStruct *p = Plates[i];

		if ((Player -> Position[0] < p -> Corner[0]) ||
			(Player -> Position[0] > p -> Corner[0] + p -> Width))
			continue;
		
		if (Player -> FallDown)
		{
			if ((p -> Corner[1] > Player -> Position[1]) &&	
				(p -> Corner[1] < g)) {	g = p -> Corner[1];	pm = p;	}
		}
		else
		{
			if ((p -> Corner[1] >= Player -> Position[1]) && 
				(p -> Corner[1] <= g)) { g = p -> Corner[1]; pm = p; }
		}
	}

	// Set new ground value
	Player -> EffectGravity.Ground = g;
	
	if (pm)
	{
		if (pm -> Blue && (Player -> EffectGravity.Landed))
		{
			pm -> Blue = false;
			PlatesCountBlue--;
		}

		if (PlatesCountBlue == 0)
			Player -> RedPlate = pm -> Red;
	}

	// Reset move down flag if a bit moved
	if (Player -> FallGround != Player -> Position[1])
		Player -> FallDown = false;

	// Stage 2. Update ground for bullets
	PlateStruct *p = NULL;
	Bullet *b = NULL;

	// For each bullet and plate do...
	for (int i = 0; i < Bullets -> List.Count; i++)
	{
		b = Bullets -> List[i];
		if (!b) continue;

		g = (float) SCREEN_HEIGHT;

		for (int j = 0; j < Plates.Count; j++)
		{
			p = Plates[j];

			if ((b -> Position[0] > p -> Corner[0] + p -> Width) ||
				(b -> Position[0] < p -> Corner[0])) continue;
	
			if ((p -> Corner[1] >= b -> Position[1]) && 
				(p -> Corner[1] <= g)) g = p -> Corner[1];
		}

		b -> EffectGravity.Ground = g;
	}
}

void LevelClass::UpdateScore()
{
	float ScoreDelta = Player -> Life - LevelScore;

	if (ScoreDelta < 0)
	{
		float dt = hge -> Timer_GetDelta();

		dt = dt / (dt + 0.5f);
		LevelScore += ScoreDelta * dt;
	}
}

void LevelClass::RenderBack()
{
	if (Background) Background -> Render(0.0f, 0.0f);
}

void LevelClass::RenderFront()
{
	if (Foreground)
	{
		DWORD color = ARGB((int) EffectFade.Value, 0, 0, 0);

		Foreground -> SetColor(color);
		Foreground -> Render(0.0f, 0.0f);
	}
}

void LevelClass::RenderPlates()
{
	float x, y;
	PlateStruct *p = NULL;

	for (int i = 0; i < Plates.Count; i++)
	{
		p = Plates[i];
		if (!p) continue;

		if (p -> Blue) Borders -> SetColor(0x7F0000FF);
		else
			if ((p -> Red) && (PlatesCountBlue == 0)) 
				Borders -> SetColor(0x7FFF0000);
			else 
				Borders -> SetColor(0xFF9F9F9F);

		// Disable flip
		Borders -> SetFlip(false, false);

		// Select line texture
		Borders -> SetTextureRect(32.0f, 0.0f, 32.0f, 32.0f, false);
		for (int j = 1; j < p -> Repeat; j++)
		{
			x = p -> Corner[0] + 32.0f * (float) j - 16.0f;
			y = p -> Corner[1] - 16.0f;

			Borders -> Render(x, y);
		}

		// Select corner texture
		Borders -> SetTextureRect(0.0f, 0.0f, 32.0f, 32.0f, false);
		y = p -> Corner[1] - 16.0f;

		// Left corder
		x = p -> Corner[0] - 16.0f;
		Borders -> Render(x, y);

		// Right corner
		x = p -> Corner[0] + p -> Width - 16.0f;
		Borders -> SetFlip(true, false);
		Borders -> Render(x, y);
	}
}

void LevelClass::RenderScore()
{
	// Render player life
	Font -> SetScale(0.75f);
	if (LevelScore <= 20.0f) 
		Font -> SetColor(0xFFDF0000);
	else 
		if (LevelScore <= 50.0f)
			Font -> SetColor(0xFFDF6F00);
		else
			Font -> SetColor(0xFF9F9F9F);
	Font -> printf(SCORE_OFFSET_X + 1.0f, SCORE_OFFSET_Y + 1.0f,
		HGETEXT_LEFT, "%d", round(LevelScore));

	if (LevelScore <= 20.0f) 
		Font -> SetColor(0xFF7F0000);
	else
		if (LevelScore <= 50.0f)
			Font -> SetColor(0xFF7F3F00);
		else
			Font -> SetColor(0xFF000000);
	Font -> printf(SCORE_OFFSET_X, SCORE_OFFSET_Y,
		HGETEXT_LEFT, "%d", round(LevelScore));

	// Render hits count
	Font -> SetScale(0.5f);
	if (Bullets -> HitsCount >= 20) 
		Font -> SetColor(0xFF00DF00);
	else 
		Font -> SetColor(0xFF9F9F9F);
	Font -> printf(SCORE_OFFSET_X + 1.0f, SCORE_OFFSET_Y + 76.0f,
		HGETEXT_LEFT, "%d", Bullets -> HitsCount);

	if (Bullets -> HitsCount >= 20) 
		Font -> SetColor(0xFF005F00);
	else 
		Font -> SetColor(0xFF000000);
	Font -> printf(SCORE_OFFSET_X, SCORE_OFFSET_Y + 75.0f,
		HGETEXT_LEFT, "%d", Bullets -> HitsCount);

	// Render weapon name
	Font -> SetScale(0.5f);
	Font -> SetColor(0xFF9F9F9F);
	Font -> printf(SCORE_OFFSET_X + 1.0f, SCORE_OFFSET_Y + 51.0f,
		HGETEXT_LEFT, "%s", Player -> Weapon.Info -> Name);

	Font -> SetColor(0xFF000000);
	Font -> printf(SCORE_OFFSET_X, SCORE_OFFSET_Y + 50.0f,
		HGETEXT_LEFT, "%s", Player -> Weapon.Info -> Name);
}

void LevelClass::RenderInfo()
{
	Font -> SetScale(0.5f);
	
	float x = (float) SCREEN_WIDTH - 10.0f + 1.0f;

	Font -> SetColor(0xFF9F9F9F);
	Font -> printf(x, 90.0f, HGETEXT_RIGHT, "%d : FPS", hge -> Timer_GetFPS());
	Font -> printf(x, 120.0f, HGETEXT_RIGHT, "%d : Bullets", Bullets -> List.Alive);

	x -= 1.0f;
	Font -> SetColor(0xFF000000);
	Font -> printf(x, 90.0f, HGETEXT_RIGHT, "%d : FPS", hge -> Timer_GetFPS());
	Font -> printf(x, 120.0f, HGETEXT_RIGHT, "%d : Bullets", Bullets -> List.Alive);
}

DWORD LevelClass::MakeColor(float t, float dt)
{
	t += dt;

	// Use this if you want glamour :)
	// int r = round(127.5f * (cos(t) + 1.0f));
	// int g = round(127.5f * (cos(t + PI_2 / 3.0f) + 1.0f));
	// int b = round(127.5f * (cos(t + PI_2 / 1.5f) + 1.0f));
	// return ARGB(0xFF, r, g, b);

	int r = round(255.0f * (cos(t) / 4.0f + 0.75f));
	return ARGB(0xFF, r, r, r);
}

void LevelClass::RenderColored(float x, float& y, float t, float& dt, char *Text)
{	
	DWORD c = MakeColor(t, dt);

	int r = GETR(c) >> 2;
	Font -> SetColor(ARGB(0xFF, r, r, r));	
	Font -> Render(x + 1.0f, y + 1.0f, HGETEXT_LEFT, Text);

	Font -> SetColor(c);
	Font -> Render(x, y, HGETEXT_LEFT, Text);

	dt += 0.3f;
	y += 30.0f;
}

void LevelClass::Render()
{
	if (!LevelLoaded) return;

	RenderBack();

	hge -> Gfx_SetTransform(0.0f, 0.0f, -LevelPosition, 0.0f, 0.0f, 1.0f, 1.0f);

	RenderPlates();

	if (Enemies) Enemies -> Render();
	if (Player) Player -> Render();
	if (Bullets) Bullets -> Render();

	hge -> Gfx_SetTransform();

	// Render map
	float MapWidth = 70.0f;
	float MapHeight = 40.0f;
	float xoff = (float) SCREEN_WIDTH - MapWidth * 2.0f - 10.0f;
	float yoff = 10.0f;
	float x = MapWidth + xoff;
	float y = MapHeight + yoff;
	
	// Render map background
	MakeQuad(x, y, MapWidth, MapHeight, 0x2F000000, q);
	hge -> Gfx_RenderQuad(&q);

	// Render player position on map
	x = xoff + Player -> Position[0] / LevelLength * MapWidth * 2.0f;
	y = yoff + Player -> Position[1] / (float) SCREEN_HEIGHT * MapHeight * 2.0f;
	MakeQuad(x, y, 2.0f, 2.0f, 0x9FFF0000, q);
	hge -> Gfx_RenderQuad(&q);

	RenderScore();

	Font -> SetScale(0.5f);
	Font -> SetColor(0x2F000000);
	Font -> printf((float) SCREEN_WIDTH - 10.0f, (float) SCREEN_HEIGHT - 30.0f,
		HGETEXT_RIGHT, "%s", "impossible physics");

	// Fadein and fadeout effect
	if (State != LEVEL_STATE_GAME)
		RenderFront();

	// Stupid game over screen
	if ((State == LEVEL_STATE_FINISH) &&
		(EffectFade.Ended))
	{
		Font -> SetScale(1.0f);

		float x = (float) SCREEN_WIDTH * 0.5f;
		float y = ((float) SCREEN_HEIGHT - Font -> GetHeight()) * 0.5f;

		if (Player -> Life > 0.0f)
		{
			// Non-balanced score formula
			int score = round(Player -> Life / LevelTime * (1.0f + float(Bullets -> HitsCount - 20) / 20.0f) * 1000.0f);

			Font -> SetColor(0xFF4F3F00);
			Font -> printf(x + 1.0f, y + 1.0f, HGETEXT_CENTER, "Victory! Your score is: %d", score);

			Font -> SetColor(0xFFDF9F00);
			Font -> printf(x, y, HGETEXT_CENTER, "Victory! Your score is: %d", score);
		}
		else
		{
			Font -> SetColor(0xFF4F0000);
			Font -> Render(x + 1.0f, y + 1.0f, HGETEXT_CENTER, "Mission failed!");

			Font -> SetColor(0xFFDF0000);
			Font -> Render(x, y, HGETEXT_CENTER, "Mission failed!");
		}

		Font -> SetScale(0.5f);
		Font -> SetColor(0xFF4F4F4F);
		Font -> Render(x + 1.0f, y + 51.0f, HGETEXT_CENTER, "please wait for 5 seconds");

		Font -> SetColor(0xFFDFDFDF);
		Font -> Render(x, y + 50.0f, HGETEXT_CENTER, "please wait for 5 seconds");
	}

	if ((State == LEVEL_STATE_STARTUP) && !SpacePressed)
	{
		Font -> SetColor(0xFFDFDFDF);
		Font -> SetScale(0.5f);

		float x = 10.0f;
		float y = 10.0f;
		
		LevelWave += hge -> Timer_GetDelta();
		float t = LevelWave / 10.0f * M_PI * 2.0f;
		float dt = 0.0f;
		float n = 0.0f;

		RenderColored(x, y, t, dt, ":");
		RenderColored(x, y, t, dt, "    WASD   ");
		RenderColored(x, y, t, dt, "     S     ");
		RenderColored(x, y, t, dt, "     ");
		RenderColored(x, y, t, dt, "     ");
		RenderColored(x, y, t, dt, "      ");
		 
		y += 30.0f;
		RenderColored(x, y, t, dt, " ?");
		RenderColored(x, y, t, dt, "         .   ");	
		RenderColored(x, y, t, dt, "  .     20      ");
		RenderColored(x, y, t, dt, "  .        .");

		y += 15.0f;
		RenderColored(x, y, t, dt, "         . ");	
		RenderColored(x, y, t, dt, "  ,    ,    .");
		RenderColored(x, y, t, dt, "        ,    .");
		RenderColored(x, y, t, dt, "       ,   .");
		RenderColored(x, y, t, dt, "   ,      ,  ");
		RenderColored(x, y, t, dt, "      .");

		y += 30.0f;
		Font -> SetColor(MakeColor(t, dt));
		Font -> Render((float) SCREEN_WIDTH * 0.5f, y, HGETEXT_CENTER, " Enter...");
	}

	if (SHOW_INFO)
		RenderInfo();
}

bool LevelClass::LoadResources()
{
	float w = (float) SCREEN_WIDTH;
	float h = (float) SCREEN_HEIGHT;

	// Create fade-in and fade-out texture 
	Foreground = new hgeSprite(0, 0.0f, 0.0f, w, h);
	Foreground -> SetColor(0xFF000000);

	// White background
	Background = new hgeSprite(0, 0.0f, 0.0f, w, h);
	Background -> SetColor(0xFFFFFFFF);

	// Plate borders
	BordersTex = hge -> Texture_Load(".\\gfx\\borders.png");
	if (!BordersTex) return false;

	Borders = new hgeSprite(BordersTex, 0.0f, 0.0f, 32.0f, 32.0f);

	// Create font resource
	Font = new hgeFont(".\\gfx\\fonts\\main.fnt");
	if (Font -> GetHeight() == 0.0f) return false;

	Enemies -> Font = Font;

	// Loaded = Field -> LoadLevel(".\\levels\\roa_egypt2.xml");
	// if (!Loaded) return false;

	return true;
}

bool LevelClass::LoadLevel(const char *LevelName, int ID)
{
	// Remove
	Plates.Clear();
	Enemies -> List.Clear();
	Bullets -> List.Clear();
	Bullets -> HitsCount = 0;
	PlatesCountBlue = 0;
	LevelPosition = 0.0f;
	LevelID = ID;

	const char *FilePath = ".\\levels\\";
	size_t FileNameLen = strlen(FilePath) + strlen(LevelName) + 1;
	char *FileName = new char [FileNameLen];
	ZeroMemory(FileName, FileNameLen);

	// Construct file name
	strncat(FileName, FilePath, strlen(FilePath));
	strncat(FileName, LevelName, strlen(LevelName));

	// Create xml parsers
	TiXmlDocument doc = TiXmlDocument();
	TiXmlHandle xml = TiXmlHandle(&doc);

	// Load xml file
	bool Loaded = doc.LoadFile(FileName);
	delete [] FileName;

	if (!Loaded) return false;
	TiXmlElement* element = NULL;

	// Load plates
	element = xml.FirstChild("Level").FirstChild("Plates").
		FirstChild("plate").Element();

	if (!element) return false;

	while (element)
	{

		int x = -1;
		int y = -1;
		int m = -1;
		int r = 0;
		int b = 0;

		element -> Attribute("x", &x);
		element -> Attribute("y", &y);
		element -> Attribute("m", &m);
		element -> Attribute("r", &r);
		element -> Attribute("b", &b);

		if ((x == -1) || (y == -1) || (m == -1)) 
		{
			Loaded = false;
			break;
		}

		PlateStruct *p = new PlateStruct();
		p -> Corner[0] = (float) x;
		p -> Corner[1] = (float) y;
		p -> Width = (float) m * 32.0f;
		p -> Red = (r == 1);
		p -> Blue = (b == 1);
		p -> Repeat = m;

		PlatesCountBlue += b;

		Plates.Add(p);

		if (LevelLength < p -> Corner[0] + p -> Width)
			LevelLength = p -> Corner[0] + p -> Width;

		element = element -> NextSiblingElement("plate");
	}

	// Load player params
	element = xml.FirstChild("Level").FirstChild("Player").
		FirstChild("info").Element();

	{
		int x = -1;
		int y = -1;
		int w = -1;

		element -> Attribute("x", &x);
		element -> Attribute("y", &y);
		element -> Attribute("w", &w);

		if ((x == -1) || (y == -1) || (w == -1)) 
			return false;

		StartPosition[0] = (float) x;
		StartPosition[1] = (float) y;
		
		if ((w < 0) || (w > 2)) w = 0;
		StartWeapon = (WeaponsKinds) w;
		
		Player -> Position[0] = StartPosition[0];
		Player -> Position[1] = StartPosition[1];
		Player -> Weapon.Info = &WeaponsInfo[StartWeapon];

		Player -> EffectGravity.SetPosition(Player -> Position[0], Player -> Position[1]);
	}

	// Load enemies
	element = xml.FirstChild("Level").FirstChild("Enemies").
		FirstChild("enemy").Element();

	if (!element) return false;

	while (element)
	{

		int x = -1;
		int y = -1;
		int t = -1;

		element -> Attribute("x", &x);
		element -> Attribute("y", &y);
		element -> Attribute("t", &t);

		if ((x == -1) || (y == -1) || (t == -1)) 
		{
			Loaded = false;
			break;
		}
		
		if ((t >= 0) && (t <= 2))
		{
			Vector2f p = { (float) x, (float) y };
			Enemies -> AddEnemy((EnemiesKinds) t, p);
		}

		element = element -> NextSiblingElement("enemy");
	}

	LevelLoaded = Loaded;
	return Loaded;
}