#ifndef ONCE_LISTS
#define ONCE_LISTS

#include <windows.h>

// Base class for two lists. This lists are arrays of pointers 
// to T (i.e. T*). Parameter Grow sets number of elements, which 
// will be allocated when current size needs to be enlarged

template <class T, int Grow = 8>
class CustomList abstract
{
	protected :
		int Size;

	public :
		int Alive;
		int Count;
		T** Items;

		CustomList()
		{
			Size = 0;
			Count = 0;
			Alive = 0;

			Items = NULL;
		}

		~CustomList()
		{
			Clear();
		}

		virtual void Add(T* Item) abstract;

		void Clear()
		{
			T* Item = NULL;
			for (int i = 0; i < Count; i++)
			{
				Item = Items[i];
				if (Item) delete Item;
			}

			if (Items)
			{
				free(Items);
				Items = NULL;	
			}

			Count = 0;
			Alive = 0;
			Size = 0;
		}

		void Remove(int Index)
		{
			delete Items[Index];
			Items[Index] = NULL;

			Alive -= 1;
		}

		T* operator [] (int Index)
		{
			return Items[Index];
		}
};

// Use this list to create a list, whose items could be used many 
// times. I.e. new added item will be placed in the first empty 
// slot. This list is not index-safe. All indexes must be valid

template <class T, int Grow = 8>
class SieveList : public CustomList <T, Grow>
{
	public :
		void Add(T* Item)
		{
			int Index = Count;

			for (int i = 0; i < Count; i++)
			if (!Items[i])
			{
				Index = i;
				break;
			}

			if (Index == Size)
			{
				Size += Grow;
				Items = (T**) realloc(Items, sizeof(T*) * Size);
			}
	
			Alive += 1;
			Items[Index] = Item;
			if (Index == Count)	Count++;			
		}
};

// Use this list to create a solid list. Each new item will be 
// added to the end of the list. This list is not index-safe. 
// All indexes must be valid

template <class T, int Grow = 8>
class SolidList : public CustomList <T, Grow>
{
	public :
		void Add(T* Item)
		{
			if (Count == Size)
			{
				Size += Grow;
				Items = (T**) realloc(Items, sizeof(T*) * Size);
			}
	
			Alive += 1;
			Items[Count] = Item;
			Count++;
		}
};

#endif