
#include <float.h>
#include <windows.h>

#include "Globals.h"
#include "LevelClass.h"
#include "PlayerClass.h"
#include "Enemies.h"
#include "Weapons.h"

LevelClass *Level = NULL;
PlayerClass *Player = NULL;
BulletsClass *Bullets = NULL;
EnemiesClass *Enemies = NULL;

bool WasPaused = false;

bool RenderFunc()
{
	hge -> Gfx_BeginScene();

	if (Level) Level -> Render();

	hge -> Gfx_EndScene();
	return false;
}

bool FrameFunc()
{
	float mx;
	float my;

	float Time = (float) GetTickCount() / 1000.0f;
	float hdt = hge -> Timer_GetDelta();

	if (WasPaused)
	{
		float PreviousFrame = Time - hdt;
		Player -> EffectGravity.Time0 = PreviousFrame;

		Bullet *b = NULL;
		for (int i = 0; i < Bullets -> List.Count; i++)
		{
			b = Bullets -> List[i];
			if (!b) continue;

			b -> EffectGravity.Time0 = PreviousFrame;
		}

		WasPaused = false;
	}

	hge -> Input_GetMousePos(&mx, &my);
	bool MouseButton = hge -> Input_GetKeyState(HGEK_LBUTTON); 
	bool MouseClick = hge -> Input_KeyDown(HGEK_LBUTTON); 

	if (Level) 
	{
		Level -> Update(Time, mx, my, hdt);
		Level -> SetLevelPosition(Player -> Position[0]);
	}

	// Toggle show info about game and data
	if (hge -> Input_KeyDown(HGEK_I))
		SHOW_INFO = !SHOW_INFO;

	bool MoveLeft = hge -> Input_GetKeyState(HGEK_LEFT) || hge -> Input_GetKeyState(HGEK_A);
	bool MoveRight = hge -> Input_GetKeyState(HGEK_RIGHT) || hge -> Input_GetKeyState(HGEK_D);

	Player -> EffectGravity.MoveActive = (MoveLeft || MoveRight);

	if (Player -> EffectGravity.MoveActive)
	{
		float dir = -1.0f;
		if (MoveRight) dir = 1.0f;

		// Calculate new speed value
		Player -> Speed[0] += dir * 400.0f * hdt;
		if (abs(Player -> Speed[0]) > 200.0f)
			Player -> Speed[0] = dir * 200.0f;
	}

	// Set velocity only if landed
	if (Player -> EffectGravity.Landed)
		Player -> EffectGravity.Velocity[0] = Player -> Speed[0];
	else
		Player -> EffectGravity.Speed[0] = Player -> Speed[0];

	if (hge -> Input_GetKeyState(HGEK_UP) || hge -> Input_GetKeyState(HGEK_W))
	{
		Player -> Speed[1] = -450.0f;

		if ((Player -> EffectGravity.Landed) ||
			(Player -> EffectGravity.GroundHit))
		{
			Player -> EffectGravity.Velocity[1] = Player -> Speed[1];
			
			Player -> EffectGravity.Speed[1] = 0.0f;
			Player -> Speed[1] = 0.0f;		
		}
	}

	if (hge -> Input_KeyDown(HGEK_ENTER))
	{
		Level -> SpacePressed = true;
	}

	if (hge -> Input_GetKeyState(HGEK_DOWN) || hge -> Input_GetKeyState(HGEK_S))
	{
		if (Player -> EffectGravity.Landed)
		{
			Player -> FallGround = Player -> EffectGravity.Ground;
			Player -> FallDown = true;
		}
	}

	if (hge -> Input_GetKeyState(HGEK_LBUTTON))
	{
		if (Player -> Weapon.Time == Player -> Weapon.Info -> PlayerPeriod)
			Bullets -> AddBullet(Player -> Weapon);
	}

	if (hge -> Input_KeyDown(HGEK_RBUTTON))
	{
		int k = Player -> Weapon.Info -> Kind + 1;
		if (k > 2) k = 0;
		
		Player -> Weapon.Info = &WeaponsInfo[k];
	}

	return false;
}

bool FocusLostFunc()
{
	return false;
}

bool FocusGainFunc()
{
	WasPaused = true;
	return false;
}

// This is the entry point to the application
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInst, LPSTR CmdString, int CmdShow)
{
	srand(GetTickCount());

	hge = hgeCreate(HGE_VERSION);

	hge -> System_SetState(HGE_TITLE, "HGE Wheel Tank");
	hge -> System_SetState(HGE_WINDOWED, true);

	hge -> System_SetState(HGE_FRAMEFUNC, FrameFunc);
	hge -> System_SetState(HGE_RENDERFUNC, RenderFunc);
	hge -> System_SetState(HGE_FOCUSLOSTFUNC, FocusLostFunc);
	hge -> System_SetState(HGE_FOCUSGAINFUNC, FocusGainFunc);

	hge -> System_SetState(HGE_SCREENWIDTH, SCREEN_WIDTH);
	hge -> System_SetState(HGE_SCREENHEIGHT, SCREEN_HEIGHT);
	hge -> System_SetState(HGE_SCREENBPP, 32);

	hge -> System_SetState(HGE_HIDEMOUSE, false);
	hge -> System_SetState(HGE_USESOUND, false);
	
	if (hge -> System_Initiate())
	{
		InitializeWeapons();

		Level = new LevelClass();
		Player = new PlayerClass();
		Bullets = new BulletsClass();
		Enemies = new EnemiesClass();

		Level -> Player = Player;
		Level -> Bullets = Bullets;
		Level -> Enemies = Enemies;
		Enemies -> Bullets = Bullets;

		bool Ok = Level -> LoadResources();
		Ok = Ok && Player -> LoadResources();
		Ok = Ok && Enemies -> LoadResources();
		
		Ok = Ok && Level -> LoadLevel("level_01.xml", 1);
		Level -> LevelScore = Player -> Life;
		
		if (Ok) 
		{
			hge -> System_Start();
		}
		else
		{
			char *ErrorMessage = "Some resources are not loaded!    ";
			MessageBox(NULL, ErrorMessage, "Resources Error", 
				MB_OK | MB_ICONINFORMATION | MB_APPLMODAL);
		}

	}
	else
	{
		char *ErrorMessage = hge -> System_GetErrorMessage(); 
		MessageBox(NULL, ErrorMessage, "HGE Error", 
			MB_OK | MB_ICONERROR | MB_APPLMODAL);
	}

	if (Player)
	{
		delete Player;
		Player = NULL;
	}

	if (Level)
	{
		delete Level;
		Level = NULL;
	}

	if (Bullets)
	{
		delete Bullets;
		Bullets = NULL;
	}

	if (Enemies)
	{
		delete Enemies;
		Enemies = NULL;
	}

	hge -> System_Shutdown();
	hge -> Release();

	return 0;
}
