
#include "PlayerClass.h"

PlayerClass::PlayerClass()
{
	RedPlate = false;

	Position[0] = 0.0f;
	Position[1] = 0.0f;

	Velocity[0] = 0.0f;
	Velocity[1] = 0.0f;

	FallDown = false;
	FallGround = 0.0f;
	
	Speed[0] = 0.0f;
	Speed[1] = 0.0f;

	BodyRadius = 32.0f;
	BodyAngle = -M_PI;

	BodyTex = 0;
	Body = NULL;

	EffectGravity.HitsCount = -1;
	EffectGravity.Interval = 0.0f;
	
	EffectGravity.Throttle = 2.0f;
	EffectGravity.Friction = 1.5f;
	EffectGravity.Breaking = 1.2f;
	EffectGravity.Ground = (float) SCREEN_HEIGHT;
	EffectGravity.SetPosition(0.0f, 0.0f);
	EffectGravity.SetVelocity(0.0f, 0.0f);
	EffectGravity.SetAcceleration(0.0f, 800.0f);

	Weapon.Info = &WeaponsInfo[2];
	Weapon.Time = Weapon.Info -> PlayerPeriod;
	
	Weapon.Offset[0] = 0.0f;
	Weapon.Offset[1] = -BodyRadius;
	Weapon.Direction[0] = 1.0f;
	Weapon.Direction[1] = 0.0f;
	Weapon.Angle = 0.0f;

	Life = 100.0f;
}

PlayerClass::~PlayerClass()
{
	if (BodyTex) hge -> Texture_Free(BodyTex);
	if (Body) delete Body;

	for (int i = 0; i < 3; i++)
	{
		WeaponInfo& wp = WeaponsInfo[i];

		if (wp.BulletTex) hge -> Texture_Free(wp.BulletTex);
		if (wp.BulletSprite) delete wp.BulletSprite;

		if (wp.WeaponTex) hge -> Texture_Free(wp.WeaponTex);
		if (wp.WeaponSprite) delete wp.WeaponSprite;
	}
}

void PlayerClass::Render()
{
	float s = 2 * BodyRadius / Body -> GetWidth();

	// Render main body background
	Body -> SetColor(0xFFEFEFEF);
	Body -> RenderEx(Position[0], Position[1] - BodyRadius, BodyAngle, s, s);

	// Render additional parts of body
	// Body -> RenderEx(Position[0], Position[1] - BodyRadius, BodyAngle * 5.0f, s / 2.0f, s / 2.0f);
	
	if (Weapon.Info -> WeaponSprite)
	{
		Weapon.Info -> WeaponSprite -> SetColor(0xFFAFAFAF);
		Weapon.Info -> WeaponSprite -> RenderEx(Position[0] + Weapon.Offset[0], 
			Position[1] + Weapon.Offset[1], Weapon.Angle, 1.1f, 1.1f);
	}
}

void PlayerClass::Update(float Time, float dT)
{
	EffectGravity.Update(Time);

	BodyAngle += Speed[0] * dT / BodyRadius;
	if (BodyAngle > M_PI) BodyAngle -= PI_2;
	if (BodyAngle < -M_PI) BodyAngle += PI_2;

	Weapon.Time += dT;
	if (Weapon.Time > Weapon.Info -> PlayerPeriod)
		Weapon.Time = Weapon.Info -> PlayerPeriod;
}

void PlayerClass::SetWeaponAngle(float dT, float X, float Y)
{
	Weapon.Direction[0] = X - Position[0];
	Weapon.Direction[1] = Y - (Position[1] - BodyRadius);

	VectorNormalize(Weapon.Direction);

	float a;

	if (Weapon.Direction[0] == 0.0f) a = Weapon.Direction[1] * M_PI_2;
	else a = atan2(Weapon.Direction[1], Weapon.Direction[0]);

/*
	float da = a - Weapon.Angle;
	if (abs(da) > M_PI)
		da -= da / abs(da) * M_PI;
*/

	Weapon.Angle = a;
}

bool PlayerClass::LoadWeapon(int Index, char *WeaponName, char *BulletName)
{
	WeaponInfo *wp = &WeaponsInfo[Index];
	wp -> WeaponTex = hge-> Texture_Load(WeaponName);
	if (!wp -> WeaponTex) return false;

	float w = (float) hge -> Texture_GetWidth(wp -> WeaponTex);
	float h = (float) hge -> Texture_GetHeight(wp -> WeaponTex);

	wp -> WeaponSprite = new hgeSprite(wp -> WeaponTex, 0.0f, 0.0f, w, h);
	switch(Index)
	{
		case 0: wp -> WeaponSprite -> SetHotSpot(26.0f, 32.0f); break;
		case 1: wp -> WeaponSprite -> SetHotSpot(26.0f, 34.0f); break;
		case 2: wp -> WeaponSprite -> SetHotSpot(27.0f, 33.0f); break;
	}

	wp -> BulletTex = hge-> Texture_Load(BulletName);
	if (!wp -> BulletTex) return false;

	w = (float) hge -> Texture_GetWidth(wp -> BulletTex);
	h = (float) hge -> Texture_GetHeight(wp -> BulletTex);

	wp -> BulletSprite = new hgeSprite(wp -> BulletTex, 0.0f, 0.0f, w, h);
	wp -> BulletSprite -> SetHotSpot(w / 2.0f, h / 2.0f);

	return true;
}

bool PlayerClass::LoadResources()
{
	float w, h;
	
	BodyTex = hge -> Texture_Load(".\\gfx\\body.png");
	if (!BodyTex) return false;

	w = (float) hge -> Texture_GetWidth(BodyTex);
	h = (float) hge -> Texture_GetHeight(BodyTex);

	Body = new hgeSprite(BodyTex, 0.0f, 0.0f, w, h);
	Body -> SetHotSpot(w / 2.0f, h / 2.0f);

	// Weapon 0!
	bool Ok = LoadWeapon(0, ".\\gfx\\weapon_01.png", ".\\gfx\\bullet_01.png");
	if (!Ok) return false;

	Ok = LoadWeapon(1, ".\\gfx\\weapon_02.png", ".\\gfx\\bullet_02.png");
	if (!Ok) return false;

	Ok = LoadWeapon(2, ".\\gfx\\weapon_03.png", ".\\gfx\\bullet_03.png");
	if (!Ok) return false;

	return true;
}