unit Camera;

interface

uses Windows,
     openGl,L_Math,eXgine;

Type
  TCamera = object
    private
      proj   : TMatrix4;
      m       : TMatrix4;
      Plane  : array [0..3, 0..3] of single;
    public
      Pos,ang : TVector;
      procedure Init;
      procedure Update;
      procedure SetRender;
      procedure SetProj;
      procedure BuildFrustum;
      function inFrustum(var p: TVector; Radius: single): boolean;
  end;


implementation

procedure TCamera.Init;
begin

  //tFOV := 90;
end;

procedure TCamera.Update;
var
 A, B, C, D, E, F : single;
 cx, cy, cz       : TVector;
begin
// pos:=vec.Add(pos,vec.Create(0,6,0));

 with Ang do
  begin
    A := cos(X);
    B := sin(X);
    C := sin(Y + pi);
    D := cos(Y + pi);
    E := cos(Z);
    F := sin(Z);
  end;
 cx := Vec.Create(C*E+B*D*F, A*F, B*C*F-E*D);
 cy := Vec.Create(B*D*E-C*F, A*E, D*F+B*C*E);
 cz := Vec.Create(A*D,        -B,       A*C);

 m[0][0] := cx.X; m[0][1] := cy.X; m[0][2] := cz.X; m[0][3] := 0;
 m[1][0] := cx.Y; m[1][1] := cy.Y; m[1][2] := cz.Y; m[1][3] := 0;
 m[2][0] := cx.Z; m[2][1] := cy.Z; m[2][2] := cz.Z; m[2][3] := 0;
 m[3][0] := -vec.Dot(cx, Pos);
 m[3][1] := -vec.Dot(cy, Pos);
 m[3][2] := -vec.Dot(cz, Pos);
 m[3][3] := 1;
 BuildFrustum;
end;

procedure TCamera.SetRender;
begin
  glLoadMatrixf(@m);
end;

procedure TCamera.SetProj;
begin
  glGetFloatv(GL_PROJECTION_MATRIX, @proj);
end;

procedure TCamera.BuildFrustum;

 procedure Normalize;
 var
  len : single;
  i   : integer;
 begin
 for i := 0 to 3 do
  begin
  len := 1/sqrt(sqr(Plane[i][0]) + sqr(Plane[i][1]) + sqr(Plane[i][2]));
  Plane[i][0] := Plane[i][0]*len;
  Plane[i][1] := Plane[i][1]*len;
  Plane[i][2] := Plane[i][2]*len;
  Plane[i][3] := Plane[i][3]*len; 
  end;
 end;

var
 f : TMatrix4;
begin
f := M_Mult(m, proj);
//    //
// 
Plane[0][0] := f[0][3] - f[0][0];
Plane[0][1] := f[1][3] - f[1][0];
Plane[0][2] := f[2][3] - f[2][0];
Plane[0][3] := f[3][3] - f[3][0];
// 
Plane[1][0] := f[0][3] + f[0][0];
Plane[1][1] := f[1][3] + f[1][0];
Plane[1][2] := f[2][3] + f[2][0];
Plane[1][3] := f[3][3] + f[3][0];
// 
Plane[2][0] := f[0][3] - f[0][1];
Plane[2][1] := f[1][3] - f[1][1];
Plane[2][2] := f[2][3] - f[2][1];
Plane[2][3] := f[3][3] - f[3][1];
// 
Plane[3][0] := f[0][3] + f[0][1];
Plane[3][1] := f[1][3] + f[1][1];
Plane[3][2] := f[2][3] + f[2][1];
Plane[3][3] := f[3][3] + f[3][1];
//    
Normalize;
end;

function TCamera.inFrustum(var p: TVector; Radius: single): boolean;
var
 i : integer;
begin
Result := false;
for i := 0 to 3 do
 if Plane[i][0] * p.X +
    Plane[i][1] * p.Y +
    Plane[i][2] * p.Z +
    Plane[i][3] < -Radius then
  Exit;
Result := true;
end;


end.
