unit Editor;

interface

uses Windows, opengl, eXgine, GUI, sys_main, camera; //sysutils;

type
  TEditorMode = (emNone, emBlock, emDelete, emEdit);
  TBlockType = (btTile, btBase, btSolid);

  TBlockData = record
    BType   : TBlockType;
    pos     : TVector;
    W, H    : integer;
    Texture : TTexture;
  end;

  TBlock = object
    Used : boolean;
    LBD  : TBlockData;
    BDID : integer;
  public
    procedure Create(T: TBlockType;Pos: TVector; W,H : integer; Tex: TTexture);
    procedure render;
  end;



  procedure EditorUpdate;
  procedure EditorRender;
  procedure EditorInit;
  procedure EditorFree;


  procedure SaveMap(FileName: string);
  procedure LoadMap(FileName: string);

const
  B_MAX = 256;

var
  MapFile: TFile;
  Blocks : array of TBlockData;

  Count  : Integer = 1;
  ShowInf: boolean;
  Scroll : integer;

  BD     : array[1..B_MAX] of TBlockData;
  MapObj : array[1..B_MAX] of TBlock;

  em : TEditorMode = emNone;
  bt : TBlockType  = btTile;

  cam: TCamera;

  TexSet : Integer = 1;

  Ramka  : TBlock;

  SysAxis,
  SysQuad,
  BTile,
  BBase,
  BSolid : TTexture;
implementation

procedure TBlock.Create(T: TBlockType;Pos: TVector; W,H : integer; Tex: TTexture);
begin
  BDID:=Count;
  BD[BDID].BType:=T;
  BD[BDID].pos:=Pos;
  BD[BDID].W:=W;
  BD[BDID].H:=H;
  BD[BDID].Texture:=Tex;
  LBD:=BD[BDID];
end;

procedure TBlock.render;
begin
  glPushMatrix;
  glTranslatef(BD[BDID].pos.x, BD[BDID].pos.y, 0);
  With BD[BDID] do
  begin
    glColor4f(0.5,0.5,0.5, 1);
    if texture<>0 then tex.Enable(Texture)
    else glColor3f(0.5, 0.75, 0.5);
    glBegin(GL_QUADS);
      glTexCoord2f(0,1);
      glVertex2f(-w/2, +h/2);
      glTexCoord2f(0,0);
      glVertex2f(+w/2, +h/2);
      glTexCoord2f(1,0);
      glVertex2f(+w/2, -h/2);
      glTexCoord2f(1,1);
      glVertex2f(-w/2, -h/2);
    glEnd;
  end;
  tex.Disable();
  glPopMatrix;
end;

procedure EditorInit;
begin
  SysAxis:=tex.Load('Data\Axis.jpg');
  SysQuad:=tex.Load('Data\SysQuad.jpg')  ;
  BTile  :=tex.Load('Data\Set_1\Block1.jpg');
  BBase  :=tex.Load('Data\Set_1\Wall.jpg');
  BSolid :=tex.Load('Data\Set_1\Floor.jpg');

  InitGUI;

  cam.Pos:=vec.Create(0,0,-302);
  cam.ang:=vec.Create(0, 1.57 , 0);

  count:=128;
  Ramka.Create(btTile, vec.Create(0,0,0),64,64, SysQuad);
  count:=1;

  MapObj[Count].Create(btTile,vec.Create(0, 0, 0), 100, 100, SysAxis);
  MapObj[Count].Used:=True;
  inc(count);
end;

procedure EditorFree;
begin

end;

procedure EditorRender;
var i: integer;
    U: string;
begin
  ogl.Clear(True, True);
  ogl.Set3D(90, 0.1, 1024);
  cam.SetRender;

  ramka.render;

  for I := 1 to count do
    if MapObj[i].Used then MapObj[i].render;


  ogl.Set2D(0,0,wnd.Width,wnd.Height);
  RenderGui(0);

  if ShowInf then
  for i := 1 to count do
  begin
    if MapObj[i].Used then U:='+' else U:='-';
    ogl.TextOut(0, 210, scroll+10+i*10, PChar(IntToStr(Sizeof(TBlockData)*Count)));
    ogl.TextOut(0, 230, scroll+10+i*10, PChar(U));
 //   ogl.TextOut(0, 250, scroll+10+i*10, PChar(floatToStr(bd[i].pos.x)));
//    ogl.TextOut(0, 350, scroll+10+i*10, PChar(floatToStr(bd[i].pos.y)));
//   ogl.TextOut(0, 450, scroll+10+i*10, PChar(floatToStr(bd[i].pos.z)));
//    ogl.TextOut(0, 550, scroll+10+i*10, PChar(floatToStr(bd[i].Texture)));
  end;
end;

procedure EditorUpdate;
var i : integer;
    m : TPoint;
begin
  UpdateGUI(0);

  cam.Update;
  cam.SetProj;

  //  
  if inp.Down(VK_UP)    then cam.pos.Y:=cam.Pos.Y+1;
  if inp.Down(VK_DOWN)  then cam.pos.Y:=cam.Pos.Y-1;
  if inp.Down(VK_LEFT)  then cam.pos.x:=cam.pos.x+1;
  if inp.Down(VK_RIGHT) then cam.pos.x:=cam.pos.x-1;

  //     
  if inp.LastKey=(VK_F1) then
     if ShowInf then showInf:=false else ShowInf:=true;
  if showinf then
     begin
       if inp.Down(VK_HOME) then inc(scroll, 5);
       if inp.Down(VK_END)  then dec(scroll, 5);
     end;

  if inp.LastKey=(VK_F5) then LoadMap('Test');
  

  GetCursorPos(m);
  BD[128].pos.X:= cam.Pos.X+400-m.X;
  BD[128].pos.Y:= cam.Pos.Y+300-m.Y;

if inp.LastKey=M_BTN_1 then
begin
  if m.X<600 then
  begin
    case em of
      emNone  : ; //   8-)
      emBlock :   //   
        begin
          case bt of
            btTile:
              MapObj[Count].Create(bt,vec.Add(
                                   vec.Create(cam.Pos.x,
                                              cam.Pos.Y,
                                              0),
                                   vec.Create(400-m.X, 300-m.Y,0)),
                                  64, 64, BTile);
            btBase:
              MapObj[Count].Create(bt,vec.Add(
                                   vec.Create(cam.Pos.x,
                                              cam.Pos.Y,
                                              0),
                                   vec.Create(400-m.X, 300-m.Y, 0)),
                                   64, 32, BBase);
            btSolid:
              MapObj[Count].Create(bt,vec.Add(
                                   vec.Create(cam.Pos.x,
                                              cam.Pos.Y,
                                              0),
                                   vec.Create(400-m.X, 300-m.Y, 0)),
                                   64, 32, BSolid);
          end;
          MapObj[Count].Used:=True;
          inc(count);
        end;
      emDelete:  //   
        begin
            for I := 1 to count do
              with BD[I] do
                if (cam.Pos.x+400-m.X > pos.x-w/2)and
                   (cam.Pos.x+400-m.X < pos.x+w/2)and
                   (cam.Pos.y+300-m.Y > pos.y-h/2)and
                   (cam.Pos.y+300-m.Y < pos.y+h/2)then
                   MapObj[i].Used:=false;
        end;
    end;
  end;


end;

end;


procedure SaveMap(FileName: string);
var i , j , cn  : integer;
    freeB : array[1..B_MAX] of integer;
begin
  try
  j:=1;
  for I := 1 to count do
    begin
      //   
      if not mapObj[i].Used then
        begin
          freeB[j]:=i;
          inc(j);
        end else inc(cn);
    end;
  except
    LogOut('    !!');
  end;

  try
      //   
    j:=0;
    if cn<>count then
      while count-cn>0 do
        if mapObj[count].Used then
        begin
          BD[freeB[j]]:=BD[count];
          dec(count);
          inc(j);
        end else dec(count);
  except
    LogOut('   ');
  end;

  try
     //    
    SetLength(Blocks, count);
    for i := 1 to Count do Blocks[i]:=BD[i];
  except
    LogOut('    ');
  end;

  try
    //  ...
   MapFile:=FileOpen(PChar(FileName+'.mdt'), true);
   FileWrite(MapFile, Blocks, Sizeof(TBlockData)*count);
   FileClose(MapFile);

   MapFile:=FileOpen(PChar(FileName+'.mnf'), true);
   FileWrite(MapFile, TexSet, 1);
   FileClose(MapFile);

  except
    LogOut('   ');
  end;


end;

procedure LoadMap(FileName: string);
var s : cardinal;
    i : integer;
begin
  MapFile:=FileOpen(PChar(FileName+'.mdt'), false);
  s:=FileSize(MapFile);
  SetLength(Blocks, 0);
  SetLength(Blocks, trunc(s/SizeOf(TBlockData)));
  FileRead(MapFile, Blocks, s);
  FileClose(MapFile);

  for I := 1 to trunc(s/SizeOf(TBlockData)) do
    begin
      BD[i]:=Blocks[i];
      mapObj[i].LBD:=BD[i];
      mapObj[i].BDID:=i;
      mapObj[i].Used:=true;
    end;

  count:= trunc(s/SizeOf(TBlockData));

end;


end.
