unit Enemies;

interface

uses Windows, eXgine, openGL, guns, level, player;

type
  TEnemyRecord = record
    Pos           : TVector;
    W, H, Texture, TP : integer;
    Ang, A, Spd   : Single;
    HP, param, fallTime, coolTime     : Integer;
  end;

  TEnemy = class
    constructor Create(_Etype, _i: integer; _pos: TVector);
  private
    EType: integer;
    ERec : TEnemyRecord;
    id   : integer;
  public
    procedure Render;
    procedure Update;
    function  PCollision() : integer;
  end;

procedure AddEnemy(_EType: Integer; _Pos: TVector3f);
procedure EnemiesInit;
procedure EnemiesRender;
procedure EnemiesUpdate;
procedure EnemiesFree;

var
  En     : array [1..128] of TEnemy;
  EExist : array [1..128] of boolean;

  EnRunner,
  EnSphere,
  EnFly,
  EnRocket,
  EnLuncher: TTexture;
  ECount   : integer;
implementation

constructor TEnemy.Create(_Etype, _i: Integer; _pos: TVector3f);
begin
  EType:=_EType;
  ERec.Pos:=_Pos;
  ID:=_i;
  ERec.TP:=EType;
  case EType of
    1: begin
         ERec.W:=64;
         ERec.H:=64;
         ERec.HP:=30;
         ERec.Texture:=EnSphere;
       end;
    2: begin
         ERec.W :=96;
         ERec.H :=64;
         ERec.HP:=40;
         ERec.Texture:=EnFly;
         ERec.Ang:=3.14;
       end;
    3: begin
         ERec.W:=48;
         ERec.H:=100;
         ERec.HP:=80;
         ERec.Texture:=EnRunner;
         ERec.Ang:=3.14;
       end;
    4: begin
         ERec.W:=64;
         ERec.H:=64;
         ERec.HP:=50;
         ERec.param:=0;
         ERec.Texture:=EnLuncher;
       end;
    5: begin
         ERec.W:=64;
         ERec.H:=16;
         ERec.HP:=5;
         ERec.param:=3;
         ERec.Texture:=EnRocket;
       end;
  end;
end;

procedure TEnemy.Render;
begin
  glPushMatrix;
  glColor4f(1, 1, 1, 1);
  glTranslatef(ERec.Pos.X, ERec.Pos.Y, 0);
  glRotatef((ERec.Ang+3.14)*180/3.14, 0, 0, 1);
  tex.Enable(ERec.Texture);
  with ERec do
  begin
    glBegin(GL_QUADS);
      glTexCoord2f(0,0);
      glVertex2f(-w/2, +h/2);
      glTexCoord2f(1,0);
      glVertex2f(+w/2, +h/2);
      glTexCoord2f(1,1);
      glVertex2f(+w/2, -h/2);
      glTexCoord2f(0,1);
      glVertex2f(-w/2, -h/2);
    glEnd;
  end;
  tex.Disable();
  glPopMatrix;
end;

procedure TEnemy.Update;
var i, b: integer;
    _x, _y: integer;
    tmp: single;
begin
  {   
                                    }
  case EType of

{=================================================================
    =====================================================}
    1: begin
         if (abs(plr.Pos.X-ERec.Pos.X)<600)and
            (PRec.unvis=false) then
            if plr.Pos.X>ERec.Pos.X then ERec.Spd:=2 else ERec.Spd:=-2;

         //   ))
         ERec.Ang:=ERec.Ang-0.05;

         ERec.Pos:=vec.Add(ERec.Pos,
                    vec.Create(ERec.Spd*cos(45),
                               ERec.Spd*sin(45),
                               0));

  {      
                         }

         collision(ERec.Pos, ERec.param, 64);

         if ERec.param=3 then
           begin
             inc(ERec.FallTime);
             if ERec.fallTime>20 then ERec.fallTime:=20;
             ERec.Pos:=vec.Add(ERec.Pos, vec.Create(0,-0.15*ERec.fallTime,0));
           end;

         if ERec.param=0 then ERec.fallTime:=0;

       end;

{=================================================================
     =========================================}
    2: begin
         if (abs(plr.Pos.X-ERec.Pos.X)<600)and
            (PRec.unvis=false) then
            begin
              if plr.Pos.X>ERec.Pos.X then  _x:= 4 else _x:= -4;
              if plr.Pos.Y+400>ERec.Pos.Y then _y:=4 else _y:=-4;

              if (ERec.Pos.X>Plr.Pos.X-30)and
                 (ERec.Pos.X<Plr.Pos.X+30)then
                 begin
                   if (ERec.coolTime>25) then
                   begin
                     ERec.coolTime:=0;
                     AddBullet(10, vec.Add(ERec.Pos,
                                           vec.Create(0, -48, 0)),
                                           -1.57);
                   end;
                   _x:=0;
                   _y:=0;
                 end;

              ERec.Pos:=vec.Add(ERec.Pos, vec.Create(_x, _y, 0));
              inc(ERec.CoolTime);
              collision(ERec.Pos, ERec.param, 64);
            end;
       end;

{==================================================================
   - =) ============================================}
    3: begin
         if (abs(plr.Pos.X-ERec.Pos.X)<600)and
            (PRec.unvis=false)then
            begin
               if plr.Pos.X>erec.Pos.X then ERec.Spd:=2;
               if plr.Pos.X<ERec.Pos.X then ERec.Spd:=-2;
               if ERec.coolTime>100 then
                 begin
                    AddBullet(12, vec.Add(ERec.Pos,
                                          vec.Create(0, 25, 0)), 0);
                    ERec.coolTime:=0;
                 end;

            end;

            collision(ERec.Pos, ERec.param, 100);

            if ERec.param=3 then
            begin
              inc(ERec.FallTime);
              if ERec.fallTime>20 then ERec.fallTime:=20;
              ERec.Pos:=vec.Add(ERec.Pos, vec.Create(0,-0.15*ERec.fallTime,0));
            end;

            if ERec.param=0 then ERec.fallTime:=0;

            inc(ERec.coolTime);
       end;
{=================================================================
     =========================================}

    4: begin
         if (abs(plr.Pos.X-ERec.Pos.X)<600)and
            (PRec.unvis=false)and
            (ERec.coolTime>75)then
            begin
              AddEnemy(5, self.ERec.Pos);
              ERec.coolTime:=0;
            end;

         collision(ERec.Pos, ERec.param, 64);

         if ERec.param=3 then
           begin
             inc(ERec.FallTime);
             if ERec.fallTime>20 then ERec.fallTime:=20;
             ERec.Pos:=vec.Add(ERec.Pos, vec.Create(0,-0.15*ERec.fallTime,0));
           end;

         if ERec.param=0 then ERec.fallTime:=0;

         inc(ERec.coolTime);
       end;
{=================================================================
    =====================================================}

    5: begin
         if (abs(plr.Pos.X-ERec.Pos.X)<600)and
            (PRec.unvis=false) then
            begin
              ERec.Spd:=4;
              tmp:=arctan((ERec.Pos.Y-plr.Pos.Y)/
                               (ERec.Pos.x-plr.Pos.X));
              if ERec.Pos.X<plr.Pos.x then tmp:=tmp+3.14;
                              // ERec.Ang:=tmp;
              if ERec.Ang-tmp>3.14 then ERec.Ang:=ERec.Ang-0.1;
              if ERec.Ang-tmp<3.14 then ERec.Ang:=ERec.Ang+0.1;

  {           if tmp-ERec.Ang>3.14 then ERec.Ang:=ERec.Ang+0.1;
              if tmp-ERec.Ang<3.14 then ERec.Ang:=ERec.Ang-0.1;        }
            end;

         ERec.Pos:=vec.Add(ERec.Pos,
                           vec.Create(ERec.Spd*cos(ERec.Ang),
                                      ERec.Spd*sin(ERec.Ang),
                                      0)
                           ); 
         inc(ERec.coolTime);

         if ERec.coolTime/10 = ERec.coolTime div 10 then AddBullet(11, ERec.Pos, ERec.Ang);
         
         if ERec.coolTime>1500 then EExist[id]:=false;
         if ERec.param<>3 then EExist[id]:=false;
       end;
  end;

  {   
                          }

  for I := 1 to 128 do
    if BExist[i] then b:=Bullets[i].Collision(ERec.Pos, ERec.W, ERec.H);

  case b of
    1: ERec.HP:=ERec.HP-10;
  end;

  if ERec.HP<=0 then EExist[id]:=false;


end;

function TEnemy.PCollision: integer;
begin
  if (ERec.Pos.X>plr.pos.X-15)and
     (ERec.Pos.X<plr.pos.X+15)and
     (ERec.Pos.Y>plr.pos.Y-40)and
     (ERec.Pos.Y<plr.pos.Y+40)then
     begin
       result:=ERec.Tp;
       if (ERec.TP=1) or (ERec.TP=5) then BExist[id]:=false;
     end else result:=0;
end;

procedure Enemiesinit;
begin
  EnSphere:=tex.Load('Data\SPR\ESphere.tga');
  EnFly:=tex.Load('Data\SPR\EFly.tga');
  EnRunner:=tex.Load('Data\SPR\ERunner.tga');
  EnRocket:=tex.Load('Data\SPR\ERocket.tga');
  EnLuncher:=Tex.Load('Data\SPR\ErUNNER.tga');
end;

procedure EnemiesRender;
var i: integer;
begin
  for i := 1 to 128 do
    if EExist[i] then En[i].Render;
end;

procedure EnemiesUpdate;
var i,j: integer;
begin
  ECount:=0;
  for i := 1 to 128 do
    if EExist[i] then
      begin
    {    for j := 1 to B_MAX do
          if Blocks[j].BType=3 then Bullets[i].Collision(Blocks[j].pos, blocks[j].W, blocks[j].H);   }
        En[i].Update;
        inc(ECount);
      end;
end;

procedure EnemiesFree;
var i: integer;
begin
  for i := 1 to 128 do
    if EExist[i] then En[i].Free;
end;

procedure AddEnemy(_EType: Integer; _Pos: TVector3f);
var i: integer;
begin
  for I := 1 to 128 do
    if not EExist[i] then
       begin
         En[i]:=TEnemy.Create(_eType, i,_Pos);
         EExist[i]:=true;
         LogOut('Enemy created in slot # '+IntToStr(i));
         LogOut('Debug enemy id check. ID: '+inttostr(En[i].id));
         Exit;
       end;
end;

end.
