unit Guns;

interface

uses Windows, eXgine, openGL, player, level, sound;

type
  TBulletDataRecord = record
    Pos         : TVector;
    W, H        : integer;
    Spd, A, Ang : single;
    Tp          : integer;
    Texture     : integer;
  end;

  TBullet = class
    constructor Create(BType, _ID: integer; _Pos: TVector; _Ang: single);
  private
    Blt   : TBulletDataRecord;
    lTime : integer;
    id    : integer;
  public
    procedure Render;
    procedure Update;
    function  Collision(pos: TVector; W,H : integer):integer;
  end;

procedure InitBullets;
procedure RenderBullets;
procedure UpdateBullets;
procedure FreeBullets;
procedure AddBullet(BType: Integer; _Pos: TVector3f; _Ang: Single);

var Bullets: array[1..256] of TBullet;                
    BExist : array[1..256] of Boolean;

    StartScreen1, //    ,    =)
    SimpBullet,
    EnergyShot,
    cross,
    fire,
    LazerShot       : TTexture;

    BCount: integer;
implementation

uses Game;

constructor TBullet.Create(BType, _ID: Integer; _Pos: TVector3f; _Ang: Single);
begin
  Blt.Pos:=_Pos;
  Blt.Ang:=_Ang;
  Blt.Tp:=BType;
  id:=_ID;
  case BType of
    1: begin   //   .
         Blt.Spd:=5;
         Blt.A:=0.25;
         blt.W:=8;
         blt.H:=8;
         blt.Texture:=SimpBullet;
         snd.Play(PistSFX, plr.Pos.X-cam.Pos.X, plr.Pos.Y-cam.pos.Y+150, 0, false)
       end;
  10: begin
         Blt.Spd:=7;
         Blt.A:=0.1;
         blt.W:=30;
         blt.H:=30;
         blt.Texture:=EnergyShot;
       end;
  11: begin
        Blt.Spd:=1;
        Blt.A:=-0.1;
        blt.W:=8;
        blt.H:=8;
        blt.Texture:=Fire;
      end;
  12: begin
         Blt.Spd:=4;
         Blt.A:=0.2;
         blt.W:=30;
         blt.H:=8;
         blt.Texture:=EnergyShot;
         LogOut('----------=================FIRE!!!================---------------');
      end;
  end;
end;

procedure TBullet.Render;
begin
  glPushMatrix;
  glColor4f(1, 1, 1, 1);
  glTranslatef(blt.Pos.X, blt.Pos.Y, 0);
  glRotatef(blt.Ang, 0, 0, 1);
  tex.Enable(Blt.Texture);
  with blt do
  begin
    glBegin(GL_QUADS);
      glTexCoord2f(0,0);
      glVertex2f(-w/2, +h/2);
      glTexCoord2f(1,0);
      glVertex2f(+w/2, +h/2);
      glTexCoord2f(1,1);
      glVertex2f(+w/2, -h/2);
      glTexCoord2f(0,1);
      glVertex2f(-w/2, -h/2);
    glEnd;
  end;
  tex.Disable();
  glPopMatrix;
end;


procedure TBullet.Update;
begin
  blt.Spd:=blt.Spd+blt.A;
  blt.Pos:=vec.Add(blt.Pos,
                   vec.Create(blt.Spd*cos(blt.Ang),
                              blt.Spd*sin(blt.Ang),
                              0));
  inc(lTime);
  if (Blt.Tp=1)and(lTime>200) then
    begin
      BExist[id]:=false;
      LogOut('Enemy deleted from slot # '+IntToStr(id));
    end;
  if (Blt.Tp=10)and(lTime>200) then
    begin
      BExist[id]:=false;
      LogOut('Enemy deleted from slot # '+IntToStr(id));
    end;
  if (Blt.Tp=12)and(lTime>200) then
    begin
      BExist[id]:=false;
      LogOut('Enemy deleted from slot # '+IntToStr(id));
    end;
  if (Blt.Tp=11)and(lTime>150) then
    begin
      BExist[id]:=false;
      LogOut('Enemy deleted from slot # '+IntToStr(id));
    end;
end;

function TBullet.Collision(pos: TVector3f; W: Integer; H: Integer): integer;
begin
  if (blt.Pos.X>pos.X-w/2)and
     (blt.Pos.X<pos.X+w/2)and
     (blt.Pos.Y>pos.Y-H/2)and
     (blt.Pos.Y<pos.Y+H/2)then
     begin
       result:=blt.Tp;
       BExist[id]:=false;
     end else result:=0;
end;

procedure initBullets;
begin
  cross:=tex.Load('Data\GFX\Cross.tga');
  simpBullet:=tex.Load('Data\GFX\Bullet.tga');
  EnergyShot:=tex.Load('Data\GFX\Energy.tga');
  Fire:=tex.Load('Data\GFX\Fire.tga');
  LazerShot:=tex.Load('Data\GFX\Lazer.tga');
  StartScreen1:=tex.Load('Data\GFX\StartScreen1.tga');
end;

procedure RenderBullets;
var i: integer;
    p: tpoint;
begin
  for i := 1 to 256 do
    if BExist[i] then bullets[i].Render;
end;

procedure UpdateBullets;
var i,j: integer;
begin
  BCount:=0;
  for i := 1 to 256 do
    if BExist[i] then
      begin
        for j := 1 to B_MAX do
          if Blocks[j].BType=3 then Bullets[i].Collision(Blocks[j].pos, blocks[j].W, blocks[j].H);
        bullets[i].Update;
        inc(BCount);
      end;
end;

procedure FreeBullets;
var i: integer;
begin
  for i := 1 to 256 do
    if BExist[i] then bullets[i].Free;
end;

procedure AddBullet(BType: Integer; _Pos: TVector3f; _Ang: Single);
var i: integer;
begin
  for I := 1 to 256 do
    if not BExist[i] then
       begin
         bullets[i]:=TBullet.Create(BType, i,_Pos,_Ang);
         BExist[i]:=true;
         LogOut('Bullet created in slot # '+IntToStr(i));
         Exit;
       end;
end;

end.
