unit Level;

interface

uses Windows, opengl, eXgine, GUI, sys_main, camera,
     scriptProcesses; //sysutils;

type
  TEditorMode = (emNone, emBlock, emDelete, emEdit);
  TBlockType = (btTile, btBase, btSolid);

  TBlockData = record
    BType, SubType   : integer;
    pos     : TVector;
    W, H    : integer;
    Texture : TTexture;
  end;

  TBlock = object
    Used : boolean;
    LBD  : TBlockData;
    BDID : integer;

    SCRPROC  :  procedure;    // script
  public
    procedure Create(T, S: integer;Pos: TVector; W,H : integer; Tex: TTexture);
    procedure render;
  end;

procedure LoadMap(FileName: string);
procedure LevelRender;
procedure Collision(var lpos: TVector; var Param: integer; const lH: integer);  overload;

const
  B_MAX = 768;

var
  MapFile: File of TBlockData;
  Blocks : array of TBlockData;

  Count  : Integer = 1;
  ShowInf: boolean;
  Scroll : integer;

  BD     : array[1..B_MAX] of TBlockData;
  MapObj : array[1..B_MAX] of TBlock;

  em : TEditorMode = emNone;
  bt : TBlockType  = btTile;

  fl : integer;

  LvL: integer = 1;
implementation

uses Game, Player, Enemies;

procedure TBlock.Create(T, S: integer;Pos: TVector; W,H : integer; Tex: TTexture);
begin
  BDID:=Count;
  BD[BDID].BType:=T;
  BD[BDID].SubType:=s;
  BD[BDID].pos:=Pos;
  BD[BDID].W:=W;
  BD[BDID].H:=H;
  BD[BDID].Texture:=Tex;
  LBD:=BD[BDID];
end;

procedure TBlock.render;
begin
  glPushMatrix;
  glTranslatef(BD[BDID].pos.x, BD[BDID].pos.y, 0);
  With BD[BDID] do
  begin
    glColor4f(0.5,0.5,0.5, 1);
    if texture<>0 then tex.Enable(Texture)
    else glColor3f(0.5, 0.75, 0.5);
    glBegin(GL_QUADS);
      glTexCoord2f(0,0);
      glVertex2f(-w/2, +h/2);
      glTexCoord2f(1,0);
      glVertex2f(+w/2, +h/2);
      glTexCoord2f(1,1);
      glVertex2f(+w/2, -h/2);
      glTexCoord2f(0,1);
      glVertex2f(-w/2, -h/2);
    glEnd;
  end;
  tex.Disable();
  glPopMatrix;
end;

procedure LoadMap(FileName: string);
var s : cardinal;
    i : integer;
begin
  AssignFile(MapFile, FileName+'.mdt');
  Reset(mapFile);
  i:=0;
  //read (MapFile, count);
  SetLength(Blocks, B_MAX);
    while not EOF(MapFile) do
      begin
        Read (mapFile, Blocks [i]);
        inc(i);
      end;
  CloseFile(MapFile);

  count:=B_MAX;

  for I := 1 to count do
    begin
      BD[i]:=Blocks[i];
      mapObj[i].LBD:=BD[i];
      mapObj[i].BDID:=i;
      mapObj[i].Used:=true;
    end;

  for I := 1 to count do
    case bd[i].BType of
      4: begin
           mapObj[i].Used:=false;
           MapObj[i].SCRPROC:=@TestScript;
           LogOut(' ');
         end;
      5: begin

          end;
      6: begin
           mapObj[i].Used:=false;
           case BD[i].SubType of
             1: begin
                  AddEnemy(1, bd[i].Pos);
                end;
             2: begin
                  AddEnemy(2, bd[i].Pos);
                end;
             3: begin
                  AddEnemy(3, bd[i].pos);
                end;
             4: begin
                  AddEnemy(4, bd[i].pos);
                end;
           end;
         end;
    end;
  AddEnemy(1, vec.Create(100,100,0));
end;

procedure LevelRender;
var i: integer;
begin
  cam.SetRender;
  for I := 1 to count do
    if (MapObj[i].Used){and
       (cam.inFrustum(BD[i].pos, sqrt(sqr(BD[i].W)+sqr(BD[i].H))))  }
        then MapObj[i].render;
end;

procedure Collision(var lpos: TVector; var Param: integer; const lH: integer);
var i: integer;
    c: boolean;
begin
  inc(fl);
  if (fl>10)and(param=4)and not (inp.Down(ORD('S'))) then param:=0;
  
  if (fl>10)and(param=0) then
    begin
      param:=3;
      fl:=0;
    end;

  for i := 1 to count do
    begin
      case BD[i].BType of
        2 :
          begin
            //     8-)
            With BD[i] do
              if (lpos.Y-lh/2<pos.Y+H/2)and
                 (lpos.Y-lh/2>pos.Y-H/2)and
                 (lpos.X>pos.X-W/2)and
                 (lpos.X<pos.X+W/2)then
                 begin
                   if (param<>1)and
                      (param<>2)then
                      begin
                        lpos.Y:=pos.Y+lh/2+h/2;
                        param:=0;
                        Exit;
                      end;
                 end;
          end;
        3:
          begin
             //    
            With BD[i] do
              if (lpos.X+10>pos.X-w/2)and
                 (lpos.X+10<pos.X+w/2)and
                 (lpos.Y+lH/2>pos.Y)and
                 (lPos.Y-lH/2<pos.Y)then
                 begin
                   lpos.X:=pos.X-w/2-15;
                   PRec.spd:=0;
                 end;
            //   
            With BD[i] do
              if (lpos.X-10<pos.X+w/2)and
                 (lpos.X-10>pos.X-w/2)and
                 (lpos.Y+lH/2>pos.Y)and
                 (lPos.Y-lH/2<pos.Y)then
                 begin
                   lpos.X:=pos.X+w/2+15;
                   PRec.spd:=0;
                 end;
            //     8-)
            With BD[i] do
              if (lpos.Y-lh/2<pos.Y+H/2)and
                 (lpos.Y-lh/2>pos.Y-H/2)and
                 (lpos.X>pos.X-W/2)and
                 (lpos.X<pos.X+W/2)then
                 begin
                   lpos.Y:=pos.Y+lh/2+h/2;
                   param:=0;
                   Exit;
                 end;
            //     8-)
            With BD[i] do
              if (lpos.Y+lh/2<pos.Y+H/2)and
                 (lpos.Y+lh/2>pos.Y-H/2)and
                 (lpos.X>pos.X-W/2)and
                 (lpos.X<pos.X+W/2)then
                 begin
                   if (param=1)then
                      begin
                        lpos.Y:=pos.Y-lh/2-h/2;
                        param:=3;
                        Exit;
                      end;
                 end;
          end;
        4:
          With BD[i] do
              if (lpos.Y<pos.Y+H/2)and
                 (lpos.Y>pos.Y-H/2)and
                 (lpos.X>pos.X-W/2)and
                 (lpos.X<pos.X+W/2)then
                 begin
                   if @mapObj[i].SCRPROC<>nil
                      then mapObj[i].SCRPROC;
                   LogOut('  !!!');
                   Exit;
                 end;
      end;
    end;
end;





end.
