unit uGui;

interface

uses
  Windows,
  OpenGL,
  uSysUtils,
  eXgine;

type
  TPanel = object
    x, y : Single;
    w, h : word;
    color: array [0..3] of Single;
    Visible: boolean;
  public
    procedure Create(ex, ey: Single; eW, eH: word; eVisible: Boolean = True);
    Procedure Render;
  end;

  TButton = object
    X, Y    : Single;
    w, h    : Word;
    Over    : Boolean;
    Down    : Boolean;
    Color   : array[0..3] of Single;
    Caption : PChar;
    Visible : Boolean;
    OnClick : procedure;
    procedure Create(ex, ey: Single; eW, eH: word; eCaption: PChar; proc: Pointer; eVisible: Boolean = True);
    procedure Render;
    procedure Update;
  end;

  TGCursor = object
    Visible : boolean;
    Color   : array[0..3] of Single;
    procedure Render;
  end;

  TCheckBox = object
    X, Y : single;
    w, h : word;
    over, down, checked : boolean;
    color: array[0..3] of Single;
    Caption: PChar;
    Visible: boolean;
    procedure Create(ex, ey: single;eW,eH: word; eCaption: PChar; eChecked: boolean =false; eVisible: boolean = true);
    procedure Render;
    procedure Update;
  end;

  TEdit = object
    X, Y : single;
    w, h : word;
    over, down, checked : boolean;
    color: array[0..3] of Single;
    Caption: PChar;
    Visible: boolean;
    procedure Create(ex, ey: single;eW,eH: word; eCaption: PChar; eChecked: boolean =false; eVisible: boolean = true);
    procedure Render;
    procedure Update;
  end;


var
 cpos : TPoint;

implementation

{================================================================
 BUTTON ========================================================}

procedure TButton.Create(ex, ey: Single; eW,eH: Word; eCaption: PChar; proc: Pointer; eVisible: Boolean = True);
begin
  x        := ex;
  y        := ey;
  Over     := False;
  Down     := False;
  Caption  := eCaption;
  w        := eW;
  h        := eH;
  Color[0] := 0.8;
  Color[1] := 0.8;
  Color[2] := 0.8;
  Color[3] := 0.5;
  Visible  := eVisible;
  OnClick  := proc;
end;

procedure TButton.Render;
begin
  if not Visible then Exit;
  glPushMatrix;
  glTranslatef(x, y, 0);
    Ogl.Blend(BT_SUB);
    glColor4fv(@Color);
    glBegin(GL_QUADS);
      glVertex2f(-w/2, +h/2);
      glVertex2f(+w/2, +h/2);
      glVertex2f(+w/2, -h/2);
      glVertex2f(-w/2, -h/2);
    glEnd;
    Ogl.Blend(BT_NONE);
    glColor3f(0, 0, 0);
    Ogl.TextOut(0, -w/2+5, -h/2+4, Caption);
  glPopMatrix;
end;

procedure TButton.Update;
begin
  GetCursorPos(CPos);
  if not Visible then Exit;
  if (CPos.X > x-w/2) and (CPos.X < x+w/2) and
     (CPos.Y > y-h/2) and (CPos.Y < y+h/2) then Over:=True else Over:=False;
  if Over then
  begin
    Color[0] := 1;
    Color[1] := 1;
    Color[2] := 1;
  end else
  begin
    Color[0] := 0.8;
    Color[1] := 0.8;
    Color[2] := 0.8;
  end;

  if Over and Inp.Down(M_BTN_1) then
  begin
    Down:=True;
  end;

  if (not Inp.Down(M_BTN_1)) and Down then
  begin
    Down:=False;
    if Assigned(OnClick) then OnClick;
  end;

  if Down and (not Over) then Down:=False;
end;

{===============================================================
 CURSOR ========================================================}

procedure TGCursor.Render;
begin
  GetCursorPos(CPos);
  if not Visible then exit;
  Color[0] := 0.8;
  Color[1] := 0.8;
  Color[2] := 0.8;
  Color[3] := 1;
  glColor4fv(@Color);
    glBegin(GL_TRIANGLES);
      glVertex2f(CPos.X, CPos.Y);
      glVertex2f(CPos.X+10, CPos.Y+5);
      glVertex2f(CPos.X+5, CPos.Y+10);
    glEnd;
end;

{===================================================================
 CHECK BOX ========================================================}

procedure TCheckBox.Create(ex, ey: Single;eW,eH: word; eCaption: PChar; eChecked: Boolean = false; eVisible: Boolean = True);
begin
  x        := ex;
  y        := ey;
  Over     := False;
  Down     := False;
  Caption  := eCaption;
  w        := eW;
  h        := eH;
  Color[0] := 0.8;
  Color[1] := 0.8;
  Color[2] := 0.8;
  Color[3] := 0.5;
  Visible  := eVisible;
end;

procedure TCheckBox.Render;
begin
  if not Visible then Exit;
  glPushMatrix;
  glTranslatef(x, y, 0);
    Ogl.Blend(BT_SUB);
    glColor4fv(@Color);
    glBegin(GL_QUADS);
      glVertex2f(-w/2, +h/2);
      glVertex2f(+w/2, +h/2);
      glVertex2f(+w/2, -h/2);
      glVertex2f(-w/2, -h/2);
    glEnd;
    if Checked then
     begin
       glColor3f(1,0,0);
     end else
     begin
       glColor3f(0,0,1);
     end;
    glBegin(GL_QUADS);
      glVertex2f(-w/2, +h/2);
      glVertex2f(-w/2+H, +h/2);
      glVertex2f(-w/2+h, -h/2);
      glVertex2f(-w/2, -h/2);
    glEnd;
    Ogl.Blend(BT_NONE);
    glColor3f(0, 0, 0);
    Ogl.TextOut(0, -w/2+h+5, -h/2+1, Caption);
  glPopMatrix;
end;

procedure TCheckBox.Update;
begin
  GetCursorPos(CPos);
  if not Visible then Exit;
  if (CPos.X > x-w/2) and (CPos.X < x+w/2) and
     (CPos.Y > y-h/2) and (CPos.Y < y+h/2) then Over:=True else Over:=False;
  if Over then
  begin
    Color[0] := 1;
    Color[1] := 1;
    Color[2] := 1;
  end else
  begin
    Color[0] := 0.8;
    Color[1] := 0.8;
    Color[2] := 0.8;
  end;

  if Over and Inp.Down(M_BTN_1) then
  begin
    Down:=True;
  end;

  if (not Inp.Down(M_BTN_1)) and Down then
  begin
    Down:=False;
    if Checked then Checked:=false else checked:= true;

  end;

  if Down and (not Over) then Down:=False;
end;


{================================================================
 PANEL =========================================================}

procedure TPanel.Create(ex, ey: Single; eW,eH: Word; eVisible: Boolean = True);
begin
  x        := ex;
  y        := ey;
  w        := eW;
  h        := eH;
  Visible  :=eVisible;
  Color[0] := 0.2;
  Color[1] := 0.2;
  Color[2] := 0.1;
  Color[3] := 0.2;
end;

procedure TPanel.Render;
begin
  if not Visible then Exit;
  glPushMatrix;
  glTranslatef(x, y, 0);
    Ogl.Blend(BT_SUB);
    glColor4fv(@Color);
    glBegin(GL_QUADS);
      glVertex2f(-w/2, +h/2);
      glVertex2f(+w/2, +h/2);
      glVertex2f(+w/2, -h/2);
      glVertex2f(-w/2, -h/2);
    glEnd;
    Ogl.Blend(BT_NONE);
  glPopMatrix;
end;


{=================================================================
  EDIT ==========================================================}

procedure TEdit.Create(ex, ey: Single;eW,eH: word; eCaption: PChar; eChecked: Boolean = false; eVisible: Boolean = True);
begin
  x        := ex;
  y        := ey;
  Over     := False;
  Down     := False;
  Caption  := eCaption;
  w        := eW;
  h        := eH;
  Color[0] := 0.8;
  Color[1] := 0.8;
  Color[2] := 1;
  Color[3] := 0.5;
  Visible  := eVisible;
end;

procedure TEdit.Render;
begin
  if not Visible then Exit;
  glPushMatrix;
  glTranslatef(x, y, 0);
    Ogl.Blend(BT_SUB);
    glColor4fv(@Color);
    glBegin(GL_QUADS);
      glVertex2f(-w/2, +h/2);
      glVertex2f(+w/2, +h/2);
      glVertex2f(+w/2, -h/2);
      glVertex2f(-w/2, -h/2);
    glEnd;
    Ogl.Blend(BT_NONE);
    if Checked then
      begin
        glColor3f(0, 0, 0);
        Ogl.TextOut(0, -w/2+h+5, -h/2+4, PChar(Caption+'|'));
      end else
      begin
        glColor3f(0.1, 0.1, 0.1);
        Ogl.TextOut(0, -w/2+h+5, -h/2+4, PChar(Caption));
      end;
  glPopMatrix;
end;

procedure TEdit.Update;
begin
  GetCursorPos(CPos);
  if not Visible then Exit;
  if (CPos.X > x-w/2) and (CPos.X < x+w/2) and
     (CPos.Y > y-h/2) and (CPos.Y < y+h/2) then Over:=True else Over:=False;
  if Over then
  begin
    Color[0] := 0.85;
    Color[1] := 0.85;
    Color[2] := 0.65;
  end else
  begin
    Color[0] := 0.7;
    Color[1] := 0.7;
    Color[2] := 0.5;
  end;

  if Over and Inp.Down(M_BTN_1) then
  begin
    Down:=True;
  end;

  if (not Inp.Down(M_BTN_1)) and Down then
  begin
    Down:=False;
    if Checked then Checked:=false else checked:= true;
  end;

  if Inp.Down(M_BTN_1) and (not Over) then
  begin
    Down:=False;
    Checked:=false;
  end;

  if Checked then
    begin
      if inp.Down(VK_Return) then
        begin
          checked:=false;
          exit;
        end;
    end;
end;


end.
